Package: JLPM
Type: Package
Title: Joint Latent Process Models
Version: 1.0.4
Date: 2026-04-30
Authors@R: c(person("Viviane", "Philipps", role=c("aut", "cre"), email="Viviane.Philipps@u-bordeaux.fr"),
             person("Tiphaine", "Saulnier", role="aut"),
	     person("Cecile", "Proust-Lima", role="aut"))
Description: Estimation of extended joint models with shared random effects. Longitudinal data are handled in latent process models for continuous (Gaussian or curvilinear) and ordinal outcomes while proportional hazard models are used for the survival part. We propose a frequentist approach using maximum likelihood estimation. See Saulnier et al, 2022 <doi:10.1016/j.ymeth.2022.03.003>.
License: GPL (>= 2.0)
Depends: R (>= 3.5.0), lcmm
Imports: survival (>= 2.37-2), spacefillr, stringr, marqLevAlg (>=
        2.0.6)
URL: https://VivianePhilipps.github.io/JLPM/
BugReports: https://github.com/VivianePhilipps/JLPM/issues
LazyLoad: yes
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-04 08:18:52 UTC; vp3
Author: Viviane Philipps [aut, cre],
  Tiphaine Saulnier [aut],
  Cecile Proust-Lima [aut]
Maintainer: Viviane Philipps <Viviane.Philipps@u-bordeaux.fr>
Repository: CRAN
Date/Publication: 2026-05-04 13:00:13 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-04 17:02:41 UTC; unix
Archs: JLPM.so.dSYM

Package: JLPM
Type: Package
Title: Joint Latent Process Models
Version: 1.0.4
Date: 2026-04-30
Authors@R: c(person("Viviane", "Philipps", role=c("aut", "cre"), email="Viviane.Philipps@u-bordeaux.fr"),
             person("Tiphaine", "Saulnier", role="aut"),
	     person("Cecile", "Proust-Lima", role="aut"))
Description: Estimation of extended joint models with shared random effects. Longitudinal data are handled in latent process models for continuous (Gaussian or curvilinear) and ordinal outcomes while proportional hazard models are used for the survival part. We propose a frequentist approach using maximum likelihood estimation. See Saulnier et al, 2022 <doi:10.1016/j.ymeth.2022.03.003>.
License: GPL (>= 2.0)
Depends: R (>= 3.5.0), lcmm
Imports: survival (>= 2.37-2), spacefillr, stringr, marqLevAlg (>=
        2.0.6)
URL: https://VivianePhilipps.github.io/JLPM/
BugReports: https://github.com/VivianePhilipps/JLPM/issues
LazyLoad: yes
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-05-04 08:18:52 UTC; vp3
Author: Viviane Philipps [aut, cre],
  Tiphaine Saulnier [aut],
  Cecile Proust-Lima [aut]
Maintainer: Viviane Philipps <Viviane.Philipps@u-bordeaux.fr>
Repository: CRAN
Date/Publication: 2026-05-04 13:00:13 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-04 17:21:18 UTC; unix
Archs: JLPM.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(coef,jointLPM)
S3method(print,jointLPM)
S3method(summary,jointLPM)
S3method(vcov,jointLPM)
export(IRT4Sselecting)
export(IRT4Sstaging)
export(REconddensity)
export(WaldMult)
export(convert)
export(createFargs)
export(createX0)
export(diffSojournTime)
export(jointLPM)
export(loglik)
export(predictRE)
export(removeNA)
export(sojournTime)
importFrom(grDevices,col2rgb)
importFrom(grDevices,n2mfrow)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,axis)
importFrom(graphics,hist)
importFrom(graphics,lines)
importFrom(graphics,matlines)
importFrom(graphics,matplot)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,segments)
importFrom(lcmm,Jointlcmm)
importFrom(lcmm,multlcmm)
importFrom(lcmm,predictYcond)
importFrom(marqLevAlg,mla)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,clusterSetRNGStream)
importFrom(parallel,makeCluster)
importFrom(parallel,parApply)
importFrom(parallel,stopCluster)
importFrom(spacefillr,generate_sobol_owen_set)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,delete.response)
importFrom(stats,dnorm)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,get_all_vars)
importFrom(stats,integrate)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.action)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(stringr,str_detect)
importFrom(survival,Surv)
importFrom(survival,untangle.specials)
importFrom(utils,as.roman)
useDynLib(JLPM, .registration=TRUE, .fixes="C_")

## Installation


To install the CRAN version, use :

``` r
install.packages("JLPM")
```

To get the most recent update, install it from github :

``` r
remotes::install_github("VivianePhilipps/JLPM")
```


## Documentation


Some vignettes are available on the JLPM website <https://VivianePhilipps.github.io/JLPM/>.

#' @export
vcov.jointLPM <- function(object, ...)
{
    res <- matrix(0, length(object$best), length(object$best))

    res[upper.tri(res, diag = TRUE)] <- object$V
    res <- t(res)
    res[upper.tri(res, diag = TRUE)] <- object$V

    noms <- sub("varcov", "cholesky", names(object$best))
    rownames(res) <- colnames(res) <- noms

    return(res)
}

# internal function for the 4S method

#### STEP 4 - SELECTING
## function to integrate : (P'im)^2 / Pim between boundaries of stage s
.integrate_info <- function(lambda,
                              i,thres,discrim,
                              m,nl){
  # arguments : - lamba = dimension process value
  #             - i = item
  #             - thresh = item threshold parameters
  #             - discrim = item discrimination parameters
  #             - m = modality
  #             - nl = nb of levels
  res <- .Pim_prime(lambda,i,thres,discrim,m,nl)^2
  denom <- .Pim(lambda,i,thres,discrim,m,nl)
  res <- ifelse(denom == 0,
                res <- 0,
                res <- res / denom)
  return(res)
}

derivMat <- function(fixed, random, data, var.time)
{
    h <- sapply(data[, var.time], function(x) { max(1E-7, 1E-4 * abs(x)) })

    ## t + h
    dplus <- data
    dplus[, var.time] <- dplus[, var.time] + h
    mat_ef <- model.matrix(fixed, data = dplus)
    mat_ef <- as.data.frame(mat_ef[, -1])
    mat_ea <- NULL
    if(!is.null(random)) mat_ea <- model.matrix(random, data = dplus)
    Xplus <- cbind(mat_ef, mat_ea)
    
    ## t - h
    dmoins <- data
    dmoins[, var.time] <- dmoins[, var.time] - h
    mat_ef <- model.matrix(fixed, data = dmoins)
    mat_ef <- as.data.frame(mat_ef[, -1])
    mat_ea <- NULL
    if(!is.null(random)) mat_ea <- model.matrix(random, data = dmoins)
    Xmoins <- cbind(mat_ef, mat_ea)
    
    ## derivee par rapport a var.time
    Xdt <- as.matrix((Xplus - Xmoins) / (2 * h))

    return(Xdt)
}

#' Brief summary of a joint latent process model
#' 
#' This function provides a brief summary of model estimated with the \code{jointLPM} function.
#' 
#' @param x an object inheriting from class \code{jointLPM} for a joint latent process model.
#' @param ... further arguments to be passed to or from other methods. They are ignored in this function.
#' @return the function is used for its side effects, no value is returned.
#' 
#' @author Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
#' 
#' @export
#'
print.jointLPM <- function(x,...)
{
    cat("Joint latent process model with shared random effects", "\n")
    cat("     fitted by maximum likelihood method", "\n")

    cl <- x$call
    cl$B <- NULL
    if(is.data.frame(cl$data))
        {
            cl$data <- NULL
            x$call$data <- NULL    
        }
    cat(" \n")
    dput(cl)
    cat(" \n")

    posfix <- x$posfix

    cat("Statistical Model:", "\n")
    cat(paste("     Dataset:", as.character(as.expression(x$call$data))),"\n")
    cat(paste("     Number of subjects:", x$ns),"\n")

    cat(paste("     Number of observations:", x$N[13]),"\n")
    cat(paste("     Number of parameters:", length(x$best))," \n")
    if(length(posfix)) cat(paste("     Number of estimated parameters:", length(x$best)-length(posfix))," \n")


    nbevt <- x$N[14]
    if(nbevt>0)
    {
        nprisq <- rep(NA,nbevt)
        nrisq <- rep(NA,nbevt)
        for(ke in 1:nbevt)
        {
            if(x$typrisq[ke]==1) nprisq[ke] <- x$nz[ke]-1
            if(x$typrisq[ke]==2) nprisq[ke] <- 2
            if(x$typrisq[ke]==3) nprisq[ke] <- x$nz[ke]+2

            
            cat(paste("     Event",ke,": \n"))
            cat(paste("        Number of events: ", x$nevent[ke],"\n",sep=""))
            
            if (x$typrisq[ke]==2)
            {
                cat("        Weibull baseline risk function \n")
            }
            if (x$typrisq[ke]==1)
            {
                cat("        Piecewise constant baseline risk function with nodes \n")
                cat("        ",x$hazardnodes[1:x$nz[ke],ke]," \n")
            }
            if (x$typrisq[ke]==3)
            {
                cat("        M-splines baseline risk function with nodes \n")
                cat("        ",x$hazardnodes[1:x$nz[ke],ke]," \n")
            }
                        
        }
    }

    
    ny <- x$N[12]
    ntr <- rep(NA,ny)
    numSPL <- 0
    cat("     Link functions: ")
    for (yk in 1:ny)
        {
            if (x$linktype[yk]==0)
                {
                    ntr[yk] <- 2
                    if (yk>1) cat("                     ")
                    cat("Linear for",x$Names$Ynames[yk]," \n")
                }
            if (x$linktype[yk]==1)
                {
                    ntr[yk] <- 4
                    if (yk>1) cat("                     ")
                    cat("Standardised Beta CdF for",x$Names$Ynames[yk]," \n")
                }
            if (x$linktype[yk]==2) 
                {
                    numSPL <- numSPL+1
                    ntr[yk] <- x$nbnodes[numSPL]+2
                    if (yk>1) cat("                     ")
                    cat("Quadratic I-splines with nodes", x$linknodes[1:x$nbnodes[numSPL],yk],"for",x$Names$Ynames[yk], "\n")
                }
            if (x$linktype[yk]==3) 
                {
                    ntr[yk] <- x$nbmod[yk]-1
                    if (yk>1) cat("                     ")
                    cat("Thresholds for",x$Names$Ynames[yk], "\n")
                }
        }

    cat(" \n")
    cat("Iteration process:", "\n")

    if(x$conv==1) cat("     Convergence criteria satisfied")
    if(x$conv==2) cat("     Maximum number of iteration reached without convergence")
    if(x$conv==3) cat("     Convergence with restrained Hessian matrix")
    if(x$conv==4|x$conv==12)
        {
            cat("     The program stopped abnormally. No results can be displayed.\n")
        }
    else
        {
            cat(" \n")
            cat("     Number of iterations: ", x$niter, "\n")
            cat("     Convergence criteria: parameters=", signif(x$gconv[1],2), "\n")
            cat("                         : likelihood=", signif(x$gconv[2],2), "\n")
            cat("                         : second derivatives=", signif(x$gconv[3],2), "\n")
            cat(" \n")
            cat("Goodness-of-fit statistics:", "\n")
            cat(paste("     maximum log-likelihood:", round(x$loglik,2))," \n")
            cat(paste("     AIC:", round(x$AIC,2))," \n")
            cat(paste("     BIC:", round(x$BIC,2))," \n")
            cat(" \n")

        }
}

# internal function for the 4S method

#### STEP 4 - SELECTING
## first derivate of probability function P' : P'im(lambda) = dPim(lambda) / dlambda
.Pim_prime <- function(lambda,
                       i,thres,discrim,
                       m,nl){
  # arguments : - lamba = dimension process value
  #             - i = item
  #             - thresh = item threshold parameters
  #             - discrim = item discrimination parameters
  #             - m = modality
  #             - nl = nb of levels
  res <- NA
  discrim_i <- discrim[i]
  # m=0, lower modality
  if(m == 0){
    thres.im <- thres[1,i]
    res <- -discrim_i * dnorm(-discrim_i*(lambda - thres.im))
  }
  # m=nl-1, upper modality
  if(m == (nl-1)){
    thres.im <- thres[nl-1,i]
    res <- discrim_i * dnorm(-discrim_i * (lambda - thres.im))
  }
  # other modalities
  if(m != 0 && m!=(nl-1)){
    thres.im.inf <- thres[m,i]
    thres.im.sup <- thres[m+1,i]
    res <- -discrim_i * dnorm(-discrim_i * (lambda - thres.im.sup)) + discrim_i * dnorm(-discrim_i * (lambda - thres.im.inf))
  }
  return(res)  
}

subroutine REconddensity(ui0,Y0,X0,Tentr0,Tevt0,Devt0,ind_survint0 &
     ,idea0,idg0,idcor0,idcontr0,idsurv0,idtdv0 &
     ,typrisq0,nz0,zi0,nbevt0,idtrunc0,logspecif0 &
     ,ny0,nv0,nobs0,nmes0,idiag0,ncor0,nalea0&
     ,npm0,b0,epsY0,idlink0,nbzitr0,zitr0,uniqueY0,indiceY0,nvalSPLORD0&
     ,idst0,nXcl0,Xcl_Ti0,Xcl_GK0,Xcs_Ti0,Xcs_GK0,nonlin0,centerpoly0 &
     ,loglik)

  use modirtsre

  IMPLICIT NONE

  !Declaration des variables en entree
  integer,intent(in)::nv0,ny0
  integer, intent(in)::nobs0,idiag0,npm0,ncor0,nalea0
  integer,intent(in)::idtrunc0,logspecif0,nbevt0
  double precision, dimension(1),intent(in)::Tentr0,Tevt0
  integer, dimension(1),intent(in)::ind_survint0,Devt0
  integer, dimension(nv0),intent(in)::idtdv0,idsurv0
  integer,dimension(nbevt0),intent(in)::typrisq0,nz0
  double precision,dimension(maxval(nz0),nbevt0),intent(in)::zi0    
  double precision,dimension(ny0),intent(in)::epsY0
  integer, dimension(ny0),intent(in)::idlink0,nbzitr0,nvalSPLORD0
  double precision,dimension(maxval(nbzitr0),ny0),intent(in)::zitr0
  integer,dimension(nobs0),intent(in)::indiceY0
  double precision,dimension(sum(nvalSPLORD0(:))),intent(in)::uniqueY0
  integer, dimension(nv0),intent(in)::idea0,idg0,idcor0,idcontr0
  double precision, dimension(sum(idea0)), intent(in)::ui0
  integer,dimension(1,ny0)::nmes0   
  double precision,dimension(nobs0),intent(in)::Y0
  double precision,dimension(nobs0*nv0),intent(in)::X0
  integer,dimension(nbevt0),intent(in)::idst0   
  integer,dimension(2),intent(in)::nXcl0    
  double precision,dimension(2),intent(in)::centerpoly0
  double precision,dimension(1,nXcl0(1)),intent(in)::Xcl_Ti0,Xcs_Ti0
  double precision,dimension(15*1,nXcl0(2)),intent(in)::Xcl_GK0,Xcs_GK0
  integer,intent(in),dimension(3*nbevt0)::nonlin0
  double precision, dimension(npm0), intent(in) :: b0
  
  !Declaration des variables en sortie
  double precision,intent(out)::loglik

  !Variables locales
  integer::jtemp,i,j,ier,k,ktemp,yk,k1,k2,p
  integer::ke,it
  double precision::eps
  double precision,external::conddensity

  maxmes=sum(nmes0(1,:))
  ns = 1


  allocate(rangeY(ny0),minY(ny0),maxY(ny0),idlink(ny0),ntr(ny0),epsY(ny0))

  nySPL=0
  nyORD=0
  rangeY=0
  epsY=epsY0
  do k=1,ny0
     idlink(k)=idlink0(k)
     minY(k)=zitr0(1,k)
     maxY(k)=zitr0(nbzitr0(k),k)
     if (idlink(k).eq.2) then
        nySPL=nySPL+1
     end if
     if (idlink(k).eq.3) then
        nyORD=nyORD+1
     end if
  end do
  
  if(nySPL.gt.0) then 
     allocate(nvalSPL(nySPL))
     nvalSPL=0
  else
     allocate(nvalSPL(1))
     nvalSPL(1) = 0
  end if

  if(nyORD.gt.0) then
     allocate(nvalORD(nyORD))
     nvalORD=0
  else
     allocate(nvalORD(1))
     nvalORD(1) = 0
  end if

  k1=0
  k2=0
  do k=1,ny0
     if(idlink(k).eq.2) then
        k1=k1+1
        nvalSPL(k1)=nvalSPLORD0(k)
     else if (idlink(k).eq.3) then
        k2=k2+1
        nvalORD(k2)=nvalSPLORD0(k)
     end if
  end do
  ntotvalSPL=sum(nvalSPL(:))
  ntotvalORD=sum(nvalORD(:))

  
  if(all(idlink.ne.2)) then
     allocate(zitr(1,1))
     allocate(mm(1),mm1(1),mm2(1),im(1),im1(1),im2(1))
     mm(1)=0.d0
     mm1(1)=0.d0
     mm2(1)=0.d0
     im(1)=0.d0
     im1(1)=0.d0
     im2(1)=0.d0
  else
     allocate(zitr(-1:(maxval(nbzitr0)+2),nySPL))
     allocate(mm(ntotvalSPL),mm1(ntotvalSPL),mm2(ntotvalSPL),im(ntotvalSPL),im1(ntotvalSPL),im2(ntotvalSPL))
  end if


  zitr=0.d0  
  k1=0
  k2=0
  do k=1,ny0
     if (idlink(k).eq.0) ntr(k)=2     
     if (idlink(k).eq.1) ntr(k)=4
     if (idlink(k).eq.2) then
        k1=k1+1
        ntr(k)=nbzitr0(k)+2

        zitr(1:nbzitr0(k),k1)=zitr0(1:nbzitr0(k),k)
        zitr(-1,k1)=zitr(1,k1)
        zitr(0,k1)=zitr(1,k1)
        zitr(ntr(k)-1,k1)=zitr(ntr(k)-2,k1)
        zitr(ntr(k),k1)=zitr(ntr(k)-1,k1)
     end if
     if (idlink(k).eq.3) then
        k2 = k2+1
        ntr(k) = nvalORD(k2)-1
     end if
  end do

  ntrtot = sum(ntr)
  allocate(Y(nobs0),X(nobs0,nv0),uniqueY(ntotvalSPL+ntotvalORD) &
       ,idea(nv0),idg(nv0),idcor(nv0),idcontr(nv0),nmes(1,ny0) &
       ,indiceY(nobs0))

  allocate(Tsurv0(ns),Tsurv(ns),Tsurvint(ns),ind_survint(ns),Devt(ns))
  allocate(Tsurv0_st2(ns,15),Tsurv_st2(ns,15))
  allocate(typrisq(nbevt0),nz(nbevt0),nprisq(nbevt0),nevtparx(nv0),nxcurr(nv0))
  allocate(idsurv(nv0),idtdv(nv0))
  allocate(idst(nbevt0), nonlinRE(nbevt0), nonlinCL(nbevt0), nonlinCS(nbevt0))
  allocate(nassoREevt(nbevt0), nassoCLevt(nbevt0), nassoCSevt(nbevt0))

  ! zi : contient noeuds pour hazard (ou min et max si Weibull)
  if(any(typrisq0.eq.3)) then
     allocate(zi(-2:maxval(nz0)+3,nbevt0))
  else
     allocate(zi(maxval(nz0),nbevt0))
  end if
  
  allocate(Xcl_Ti(ns,nXcl0(1)),Xcl_GK(15*ns,nXcl0(1)),Xcl0_GK(15*ns,nXcl0(1)))
  allocate(Xcs_Ti(ns,nXcl0(1)-1),Xcs_GK(15*ns,nXcl0(1)-1),Xcs0_GK(15*ns,nXcl0(1)-1))

  eps=1.d-20

  ! enregistrement pour les modules
  nbevt=nbevt0    
  typrisq=typrisq0
  idtrunc=idtrunc0
  Tsurv0=Tentr0   
  Tsurv=Tevt0    
  devt=devt0    
  ind_survint=ind_survint0
  logspecif=logspecif0 
  Tsurvint=Tsurv
  ny=ny0
  nv=nv0
  nobs=nobs0
  ncor=ncor0
  nalea=nalea0
  idiag=idiag0
  nXcl=nXcl0
  centerpoly = centerpoly0
  
  do ke = 1,nbevt
     ! association(s) sur l'evenement ke
     idst(ke) = idst0(ke)
     nonlinRE(ke) = nonlin0(ke)
     nonlinCL(ke) = nonlin0(nbevt + ke)
     nonlinCS(ke) = nonlin0(2 * nbevt + ke)

     ! var expl aux 15 points de quadrature pour niveau courant
     if(idst(ke).ne.1) then ! besoin de Xcl_GK[,1] pour fct_risq_base
        Xcl_Ti=Xcl_Ti0
        Xcl_GK=Xcl_GK0(:,1:nXcl(1))
        if (idtrunc.eq.1) then
           Xcl0_GK=Xcl_GK0(:,(nXcl(1)+1):nXcl(2))
        end if

        ! temps pour les integrales 0->Ti et 0->T0i
        do i=1,ns  
           do p=1,15
              Tsurv_st2(i,p) = Xcl_GK(15*(i-1)+p,1)
              if (idtrunc.eq.1) then
                 Tsurv0_st2(i,p) = Xcl0_GK(15*(i-1)+p,1) 
              end if
           end do
        end do
     end if

     ! var expl aux 15 points de quadrature pour pente courante
     if(idst(ke).eq.3 .or. idst(ke).eq.4) then
        Xcs_Ti=Xcs_Ti0
        Xcs_GK=Xcs_GK0(:,1:(nXcl(1)-1))
        if (idtrunc.eq.1) then
           Xcs0_GK=Xcs_GK0(:,nXcl(1):(nXcl(2)-2))
        end if
     end if
     
  end do
  
  !     if (verbose==1) write(*,*)'ntotvalSPL',ntotvalSPL

  if (ntotvalSPL+ntotvalORD.gt.0) uniqueY(1:ntotvalSPL+ntotvalORD)=uniqueY0(1:ntotvalSPL+ntotvalORD)

  nmes=0
  Y=0.d0
  X=0.d0
  idea=0
  idg=0
  idcor=0
  idcontr=0
  idsurv=0
  idtdv=0
  ktemp=0

  do k=1,nv
     idsurv(k)=idsurv0(k)
     idtdv(k)=idtdv0(k)
     idea(k)=idea0(k)
     idg(k)=idg0(k)
     idcor(k)=idcor0(k)
     idcontr(k)=idcontr0(k)

     jtemp=0
     DO i=1,ns
        do yk=1,ny            
           if (k.eq.1) then
              nmes(i,yk)=nmes0(i,yk)   !dim(nmes)=ns*ny    
              do j=1,nmes(i,yk)
                 jtemp=jtemp+1
                 Y(jtemp)=Y0(jtemp)
                 indiceY(jtemp)=indiceY0(jtemp)
                 ktemp=ktemp+1
                 X(jtemp,k)=X0(ktemp)
              end do
           else
              do j=1,nmes(i,yk)
                 ktemp=ktemp+1
                 jtemp=jtemp+1
                 X(jtemp,k)=X0(ktemp)
              end do
           end if
        end do
     end do
  end do
          
  id_nXcl(1)=sum(idg)-1 !sans intercept
  id_nXcl(2)=sum(idea)
  
  ! definition Tsurvint 
  nvdepsurv=0
  if(sum(ind_survint).gt.0) then
     nvdepsurv=1
     do k=1,nv
        if (idtdv(k).eq.1) then
           it=0
           do i=1,ns
              Tsurvint(i)=X(it+1,k)
              it=it+maxval(nmes(i,:))
           end do
        end if
     end do
  end if


  ! creation des parametres

  nprisq=0
  nrisqtot=0

  do ke=1,nbevt

     nz(ke)=nz0(ke) ! nb de noeuds pour hazard (ou 2 pr Weibull)

     if (typrisq(ke).eq.1) then
        nprisq(ke)=nz(ke)-1
     end if
     if (typrisq(ke).eq.2) then
        nprisq(ke)=2
     end if
     if (typrisq(ke).eq.3) then
        nprisq(ke)=nz(ke)+2
     end if

     nrisqtot = nrisqtot+nprisq(ke)  ! nb total de prm pour hazards
     zi(1:nz(ke),ke)=zi0(1:nz(ke),ke)
  end do

  ! nvarxevt = nombre total de coef pour survie (sans prm hazard)
  nxevt=0
  nevtparx=0
  do j=1,nv

!     if(idtdv(j).ne.1) then

        if(idsurv(j).eq.1) then
           nevtparx(j) = 1
           nxevt = nxevt + 1
        end if
        if(idsurv(j).eq.2) then 
           nevtparx(j) = nbevt
           nxevt = nxevt + 1
        end if
!     end if

  end do

  nvarxevt = sum(nevtparx) !+ nvdepsurv

  nea=0
  nef=0
  ncontr=0
  do k=1,nv
     if (idg(k).eq.1) then
        nef = nef + 1
     end if
     nea=nea+idea(k)
     ncontr=ncontr+idcontr(k)*(ny-1) 
  end do
  nef = nef - 1 !intercept pas estime

 nasso = 0
 nassoRE = 0
 nassoCL = 0
 nassoCS = 0
 do ke=1,nbevt
    nassoREevt(ke) = 0
    nassoCLevt(ke) = 0
    nassoCSevt(ke) = 0
    
    if(idst(ke).eq.1) then
       if(nonlinRE(ke).eq.0) then
          nassoRE = nassoRE + nea
          nassoREevt(ke) = nea
       else
          nassoRE = nassoRE + 3 * nea
          nassoREevt(ke) = 3 * nea
       end if
       
    else
       if(idst(ke).eq.2 .or. idst(ke).eq.4) then
          if(nonlinCL(ke).eq.0) then
             nassoCL = nassoCL + 1
             nassoCLevt(ke) = 1
          else
             nassoCL = nassoCL + 3
             nassoCLevt(ke) = 3
          end if
       end if
       
       if(idst(ke).eq.3 .or. idst(ke).eq.4) then
          if(nonlinCS(ke).eq.0) then
             nassoCS = nassoCS + 1
             nassoCSevt(ke) = 1
          else
             nassoCS = nassoCS + 3
             nassoCSevt(ke) = 3
          end if
       end if
    end if
    nasso = nasso + nassoREevt(ke) + nassoCLevt(ke) + nassoCSevt(ke)
 end do

 if (idiag.eq.1) then
     nvc=nea-1
  else if(idiag.eq.0) then
     nvc=(nea+1)*nea/2-1
  end if

  npmtot = nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+ny

  ! creer base de splines si au moins un hazard splines
  if(any(typrisq.eq.3)) then
     allocate(Tmm(ns*nbevt),Tmm1(ns*nbevt),Tmm2(ns*nbevt),Tmm3(ns*nbevt),        &
              Tim(ns*nbevt),Tim1(ns*nbevt),Tim2(ns*nbevt),Tim3(ns*nbevt),        &
              Tmm0(ns*nbevt),Tmm01(ns*nbevt),Tmm02(ns*nbevt),Tmm03(ns*nbevt),    &
              Tim0(ns*nbevt),Tim01(ns*nbevt),Tim02(ns*nbevt),Tim03(ns*nbevt),    &
              Tmmt(ns*nbevt),Tmmt1(ns*nbevt),Tmmt2(ns*nbevt),Tmmt3(ns*nbevt),    &
              Timt(ns*nbevt),Timt1(ns*nbevt),Timt2(ns*nbevt),Timt3(ns*nbevt),    &
              Tmm_st2(15,ns*nbevt),Tmm1_st2(15,ns*nbevt),Tmm2_st2(15,ns*nbevt),Tmm3_st2(15,ns*nbevt),        &
              Tmm0_st2(15,ns*nbevt),Tmm01_st2(15,ns*nbevt),Tmm02_st2(15,ns*nbevt),Tmm03_st2(15,ns*nbevt))

     do ke=1,nbevt
        if(typrisq(ke).eq.3) then
           call splines_irtsre(ke)
           call splines_irtsre2(ke)
        end if
     end do

  end if



  ! base de splines transfos
  if (any(idlink.eq.2)) then 
     call design_splines_irtsre(ier)
     if (ier.eq.-1) then
        loglik=-1.d9
        go to 1589
     end if
  end if

!  print*,'appel conddensity'
  ! calcul de la densite conditionnelle
  loglik = conddensity(ui0, b0)

  
1589 continue

  
  if (any(typrisq.eq.3)) then
     deallocate(Tmm,Tmm1,Tmm2,Tmm3,Tim,Tim1,Tim2,Tim3,Tmm0,    &
          Tmm01,Tmm02,Tmm03,Tim0,Tim01,Tim02,Tim03,Tmmt,Tmmt1,     &
          Tmmt2,Tmmt3,Timt,Timt1,Timt2,Timt3,    &
          Tmm_st2,Tmm1_st2,Tmm2_st2,Tmm3_st2,    &   
          Tmm0_st2,Tmm01_st2,Tmm02_st2,Tmm03_st2)
  endif

  deallocate(Tsurv0,Tsurv,Tsurvint &
       ,Tsurv0_st2, Tsurv_st2 & 
       ,ind_survint,zi,devt,typrisq,nz,nprisq,idsurv,idtdv &
       ,nevtparx,nxcurr)

  deallocate(Y,X,idea,idg,idcor,idcontr,nmes,uniqueY,indiceY,ntr)


  deallocate(zitr,mm,mm1,mm2,im,im1,im2,minY,maxY,rangeY,idlink,nvalSPL,nvalORD,epsY)


  deallocate(Xcl_Ti,Xcl_GK,Xcl0_GK, Xcs_Ti,Xcs_GK,Xcs0_GK)
  deallocate(idst, nonlinRE, nonlinCL, nonlinCS, nassoREevt, nassoCLevt, nassoCSevt)
  
  return
  


  
end subroutine REconddensity







!-----------------------------------------------------------
!                       conddensity
!------------------------------------------------------------


double precision function conddensity(ui, b) 

  use modirtsre

  IMPLICIT NONE
  integer ::i,j,k,l,m,jj,ll,ii,numSPL,ykord
  integer ::ier,kk,j1,j2,sumMesYk,yk,sumntr,ke,sumnrisq
  integer::nevtxcurr,nxevtcurr,sumnassoCL,sumnassoCS
  double precision,dimension(maxmes,nv) ::X00
  double precision,dimension(maxmes,nea) ::Z
  double precision,dimension(maxmes,(ncontr+sum(idcontr)))::X01
  double precision,dimension(ncontr+sum(idcontr))::b01
  double precision,dimension(maxmes,maxmes) ::VC,Corr
  double precision,dimension(npmtot) :: b
  double precision,dimension(maxmes*(maxmes+1)/2) ::Vi
  double precision,dimension(nv) :: b0
  double precision,dimension(npmtot)::b1
  double precision,dimension(nxevt)::Xevt,bevt
  double precision,dimension(nbevt)::bevtint
  double precision,dimension(maxval(nprisq))::brisq
  double precision,dimension(3)::bassoCL
  double precision,dimension(3)::bassoCS
  double precision,dimension(nef)::beta_ef 

  double precision :: eps,det,som,eta0
  double precision ::Y4,jacobien,beta_densite,ytemp
  double precision,dimension(maxmes) :: mu,Y1,Y2,Y3,tcor
  double precision,dimension(-1:maxval(ntr)-3)::splaa
  double precision::aa1,bb1,dd1,aa,bb,betai,cc1
  double precision,dimension(nea)::ui
!  double precision,dimension(maxmes)::wi,wsim
!  double precision,dimension(ny)::asim
  double precision::binf,bsup ! , ai
  double precision,dimension(nbevt)::risq,surv,surv0,survint
!  double precision::SX,x22
  double precision::div,vrais_Y,vrais_surv,varexpsurv
  double precision::surv0_glob,surv_glob,fevt,easurv
  double precision,external::alnorm, poly, logit, sigmoid
  double precision::pred_cl_Ti
  ! pour f(b) :
  double precision::vrais_b 
  double precision,dimension(nea)::Y3_b 
  double precision::det_b,Y4_b,div_b 
  double precision,dimension(nea,nea)::Ut 
  double precision,dimension(nea*(nea+1)/2)::Ut_v 
  
  i = 1
  nmescur = 0
  ! definir le nombre total de mesures pour le sujet i : nmestot (valable que pour cette fonction)

  b1=0.d0
  eps=1.D-20
  do k=1,npmtot
     b1(k)=b(k)
  end do
!print*,"ui=", ui
!print*,"b=", b

  ! -------- creation de Vi = ZiGZi'+se*seIni ----------
  ! creation de Zi

  Z=0.d0
  l=0
  do k=1,nv
     if (idea(k).eq.1) then
        l=l+1
        do j=1,sum(nmes(i,:))
           Z(j,l)=dble(X(nmescur+j,k))  
        end do
     end if
  end do

  !matrice Corr variance de BM/AR

  Corr=0.d0
  tcor=0.d0
  if (ncor.gt.0) then
     do k=1,nv
        if (idcor(k).eq.1) then
           do j=1,sum(nmes(i,:))
              tcor(j) = X(nmescur+j,k)
           end do
        end if
     end do
     do j1=1,sum(nmes(i,:))
        do j2=1,sum(nmes(i,:))
           if (ncor.eq.1) then 
              Corr(j1,j2) = Corr(j1,j2)+b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)*min(tcor(j1),tcor(j2))
           else if (ncor.eq.2) then
              Corr(j1,j2) = Corr(j1,j2)+b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   exp(-b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1)*abs(tcor(j1)-tcor(j2)))
           end if
        end do
     end do

     ! passer en cholesky si on a de l ordinal
     if(any(idlink.eq.3)) then
        jj=0
        Vi=0.d0
        do j=1,sum(nmes(i,:))
           do k=j,sum(nmes(i,:))
              jj=j+k*(k-1)/2
              Vi(jj)=Corr(j,k)
           end do
        end do

        CALL DMFSD(Vi,sum(nmes(i,:)),EPS,IER)

        Corr=0.d0
        do j=1,sum(nmes(i,:))
           do k=1,j
              Corr(j,k)=Vi(k+j*(j-1)/2)
           end do
        end do
     end if
  end if

  ! creation de Y1
  Y1=0.d0
  splaa=0.d0
  jacobien = 0.d0
  sumMesYk = 0
  sumntr=0
  numSPL=0
  do yk=1,ny

     if (idlink(yk).eq.0) then  ! Linear link

        do j=1,nmes(i,yk)
           Y1(sumMesYk+j)=(dble(Y(nmescur+sumMesYk+j))-b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)) &
                /abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))
            
           jacobien = jacobien - log(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))
        end do

     else if (idlink(yk).eq.1) then  ! Beta link


        aa1=exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1))/ &
             (1+exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)))
        bb1=exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))/ &
             (1+exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2)))
        bb1=aa1*(1.d0-aa1)*bb1

        cc1=abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+3))

        dd1=abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+4))

        aa=aa1*aa1*(1-aa1)/bb1-aa1
        bb=aa*(1-aa1)/aa1

        do j=1,nmes(i,yk)

           ytemp=(dble(Y(nmescur+sumMesYk+j))-minY(yk)+epsY(yk))/(maxY(yk)-minY(yk)+2*epsY(yk))
           Y1(sumMesYk+j)=(betai(aa,bb,ytemp)-cc1)/dd1


           if (Y1(sumMesYk+j).eq.999.d0) then
              conddensity=-1.d9
              !print*,"-1.d9 Y1=999"
              goto 654
           end if

           jacobien = jacobien + log(abs(beta_densite(ytemp,aa,bb))/dd1)
           jacobien=jacobien-log(abs(maxY(yk)-minY(yk)+2*epsY(yk)))
        end do

     else if (idlink(yk).eq.2) then ! Splines link
        numSPL=numSPL+1

        splaa=0.d0
        eta0=0.d0
        eta0=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)

        do kk=2,ntr(yk)
           splaa(kk-3)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+kk)&
                *b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+kk)
        end do
        !if(i==1 .and. id==0 .and. jd==0) print*,"eta0=",eta0,"splaa=",sqrt(splaa)
        do j=1,nmes(i,yk)
           ll=0
           !if(i==1 .and. id==0 .and. jd==0) print*,"Y=",Y(nmescur+sumMesYk+j)
           if (Y(nmescur+sumMesYk+j).eq.zitr(ntr(yk)-2,numSPL)) then
              ll=ntr(yk)-3
           end if

           som=0.d0
           do kk = 2,ntr(yk)-2
              if ((Y(nmescur+sumMesYk+j).ge.zitr(kk-1,numSPL)).and. &
                   (Y(nmescur+sumMesYk+j).lt.zitr(kk,numSPL))) then
                 ll=kk-1
              end if
           end do

           if (ll.lt.1.or.ll.gt.ntr(yk)-3) then          
              conddensity=-1.d9
              !print*,"-1.d9 ll<1 ou ll>ntr-3",ll!," ntr=",ntr(yk)," numSPL=",numSPL," y=",Y(nmescur+sumMesYk+j)
              goto 654
           end if
           if (ll.gt.1) then
              do ii=2,ll
                 som=som+splaa(ii-3)
              end do
           end if



           Y1(sumMesYk+j)=eta0+som +splaa(ll-2)*im2(indiceY(nmescur+sumMesYk+j)) &
                +splaa(ll-1)*im1(indiceY(nmescur+sumMesYk+j))&
                + splaa(ll)*im(indiceY(nmescur+sumMesYk+j))

           jacobien = jacobien + log(splaa(ll-2)*mm2(indiceY(nmescur+sumMesYk+j)) &
                +splaa(ll-1)*mm1(indiceY(nmescur+sumMesYk+j))&
                +splaa(ll)*mm(indiceY(nmescur+sumMesYk+j)))

           !print*,"jac=",jacobien
           !print*,"ll =",ll
           !print*,"splaa =",splaa(ll-2)
           !print*,"nmescur+sumMesYk+j =",nmescur+sumMesYk+j
           !print*,"indiceY= ",indiceY(nmescur+sumMesYk+j)
           !print*,"mm =",mm(nmescur+sumMesYk+j)
           !print*,"mm1 =",mm1(nmescur+sumMesYk+j)
           !print*,"mm2 =",mm2(nmescur+sumMesYk+j)
           !print*,"Y =",Y(nmescur+sumMesYk+j)

           !                write(*,*)'Y',Y1(sumMesYk+j),sumMesYk,yk,j,jacobien
        end do
     else if (idlink(yk).eq.3) then  ! Threshold link
        do j=1,nmes(i,yk)
           Y1(sumMesYk+j)=Y(nmescur+sumMesYk+j)
           !if(nmescur.lt.15) then
           !   print*,"nmescur=",nmescur," sumMesYk=",sumMesYk," j=",j
           !   print*,"Y=",Y(nmescur+sumMesYk+j), "  Y1=",Y1(sumMesYk+j)
           !end if
        end do
     end if
     sumMesYk=sumMesYk+nmes(i,yk)
     sumntr=sumntr+ntr(yk)
     
  end do !fin boucle yk

  !         if (i.lt.3)then
  !            write(*,*)'nmes',nmes(i),b1((nef+ncontr+nvc+1):npm),nef+ncontr
  !            write(*,*)'Y1',Y1
  !         end if




  ! contribution individuelle a la vraisemblance
  ! print*,"i=",i," -ni*log(2pi)=",-sum(nmes(i,:))*dlog(dble(2*3.14159265)), " log(det)=",det
  ! print*,"Vi=",VC
  ! sans classes latentes : ng=1
  conddensity=0.d0


  b0=0.d0
  b01=0.d0
  l=0
  m=0
  X00=0.d0
  X01=0.d0
  do k=1,nv
     if (idg(k).ne.0) then
        l=l+1
        do j=1,sum(nmes(i,:))
           X00(j,l)=dble(X(nmescur+j,k))  
        end do
        ! idg ne 0 pour l'intercept forcement donc on met le parm a 0
        if (k.eq.1) then
           b0(l)=0.d0
        else
           b0(l)=b1(nrisqtot+nvarxevt+nasso+l-1)
        end if
     end if

     !contrast : 
     if (idcontr(k).ne.0) then
        m=m+1
        sumMesYk=0
        do yk=1,ny
           ! creation matrice design des contrastes: X01
           do j=1,nmes(i,yk)
              X01(sumMesYk+j,(m-1)*ny+yk) = dble(X(nmescur+sumMesYk+j,k))
           end do
           sumMesYk=sumMesYk+nmes(i,yk)
           ! creation vecteur parms des contrastes: b01
           if (yk<ny) THEN
              b01((m-1)*ny+yk)=b1(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+yk)
           else
              b01((m-1)*ny+ny) =-sum(b1((nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+1) &
                    :(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+(ny-1))))
                   !-sum(b1(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+1 &
                   !:nef+(m-1)*(ny-1)+ny-1))
           end if
        end do
     end if
  end do


     vrais_Y=1.d0
     mu=0.d0

     ! esperance conditionnelle
     mu = matmul(X00,b0)+matmul(X01,b01)+matmul(Z,ui)

     sumMesYk=0
     sumntr=0
     ykord=0
     do yk =1,ny

        if(idlink(yk).eq.3) then
           !! yk est ordinal
           ykord = ykord + 1

           do j=1,nmes(i,yk)
              !! trouver binf et bsup tq binf < lambda + epsilon < bsup

              !! initialiser au premier seuil
              binf = b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)
              bsup = binf

              !! si Y>minY ajouter b1(..)^2
              if(indiceY(nmescur+sumMesYk+j).gt.1) then
                 do ll=2,min(indiceY(nmescur+sumMesYk+j),ntr(yk))
                    bsup = bsup + b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+ll)**2
                    if(ll.lt.indiceY(nmescur+sumMesYk+j)) then
                       binf = binf + b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+ll)**2
                    end if
                 end do
              end if
              !if(i.lt.2)print*,"y=",Y1(sumMesYk+j)," indiceY=",indiceY(nmescur+sumMesYk+j),  &
              !     " Y=",Y(nmescur+sumMesYk+j)," mu=",mu(sumMesYk+j)
              !if(i.lt.2)print*," binf=",binf," bsup=",bsup
              !if(i.lt.2)print*,"se=",abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))
              !if(i.lt.2)print*,nrisqtot, nvarxevt, nasso, nef, ncontr, nvc, ncor, ntrtot, nalea, yk
              !! centrer et standardiser
              binf = (binf - mu(sumMesYk+j))/abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))
              bsup = (bsup - mu(sumMesYk+j))/abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))

              !if(i.lt.2) print*,"yk=",yk,"j=",j," nvalORD=",nvalORD(ykord)
              
              !if(i.lt.2) print*,"indice = ", indiceY(nmescur+sumMesYk+j)," binf=",binf," bsup=",bsup
              if(indiceY(nmescur+sumMesYk+j).eq.1) then
                 !! si Y=minY
                 !if(i.lt.2) print*,"min"
                 vrais_Y = vrais_Y * alnorm(binf,.false.)
              else if(indiceY(nmescur+sumMesYk+j).eq.nvalORD(ykord)) then
                 !! si Y=maxY
                    vrais_Y = vrais_Y * (1.d0-alnorm(bsup,.false.))
              else
                 !! minY < Y < maxY
                 !if(i.lt.2) print*,"entre min et max"
                    vrais_Y = vrais_Y * (alnorm(bsup,.false.)-alnorm(binf,.false.))
              end if
              !if(i.lt.2) print*,"vrais_Y=",vrais_Y
           end do


        else
           
           !! yk est continu
           !print*,"MC continu"
           if(nmes(i,yk).gt.0) then
              !! variance de Y|ui,wi
              VC=0.d0
              do j1=1,nmes(i,yk)
                 VC(j1,j1) = b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk)**2 !variance de l'erreur yk
                 if (nalea.eq.ny) then ! intercept aleatoire de yk
                    do j2=1,nmes(i,yk)
                       VC(j1,j2) = VC(j1,j2) + &
                            b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk)**2
                    end do
                 end if
              end do
              
              ! Vi en vecteur
              jj=0
              Vi=0.d0
              do j1=1,nmes(i,yk)
                 do j2=j1,nmes(i,yk)
                    jj=j1+j2*(j2-1)/2
                    Vi(jj)=VC(j1,j2)
                 end do
              end do
              
              ! inversion
              CALL dsinv(Vi,nmes(i,yk),eps,ier,det)
              if (ier.eq.-1) then
                 conddensity=-1.d9
                 !print*,"-1.d9 dsinv continu MC"
                 !print*,"b=",b
                 !print*,"bfix=",bfix
                 !print*,"fix=",fix
                 goto 654
              end if
              
              ! retransformation du vecteur Vi en matrice :
              VC=0.d0
              do j1=1,nmes(i,yk)
                 do j2=1,nmes(i,yk)
                    if (j2.ge.j1) then
                       VC(j1,j2)=Vi(j1+j2*(j2-1)/2)
                    else
                       VC(j1,j2)=Vi(j2+j1*(j1-1)/2)
                    end if
                 end do
              end do
           
              ! calcul de la vrais
              Y2=0.d0
              Y3=0.d0
              Y4=0.d0
              do j=1,nmes(i,yk)
                 Y2(j) = Y1(sumMesYk+j)-mu(sumMesYk+j)
              end do
              Y3=matmul(VC,Y2)
              Y4=DOT_PRODUCT(Y2,Y3)
              
              div = (dble(2*3.14159265)**(dble(nmes(i,yk))/2))*sqrt(exp(det))
              vrais_Y = vrais_Y * exp(-Y4/2.d0)/div

           end if
        end if
        
        sumMesYk = sumMesYk+nmes(i,yk)
        sumntr = sumntr + ntr(yk)
     end do ! fin boucle yk
!print*,"vrais_Y = ", vrais_Y

     ! partie survie

     if (nbevt.ne.0) then

        ! calcul de brisq en chaque composante et risq, surv et surv0 pour chaque evt
        risq=0.d0
        surv=0.d0
        surv0=0.d0
        survint=0.d0

        sumnrisq=0
        sumnassoCL=0
        sumnassoCS=0
        do ke=1,nbevt

           brisq=0.d0
           if (logspecif.eq.1) then
              do k=1,nprisq(ke)
                 brisq(k)=exp(b1(sumnrisq+k))
              end do
           else
              do k=1,nprisq(ke)
                 brisq(k)=b1(sumnrisq+k)*b1(sumnrisq+k)
              end do
           end if
           
           bassoCL=0.d0 
           do k=1,nassoCLevt(ke)
              bassoCL(k) = b1(nrisqtot+nvarxevt+sumnassoCL+k) !basso = eta = prm estime des EAs partages
           end do

           bassoCS=0.d0 
           do k=1,nassoCSevt(ke)
              bassoCS(k) = b1(nrisqtot+nvarxevt+nassoCL+sumnassoCS+k)
           end do
           
           beta_ef=0.d0  !beta_ef = prms des EFs necessaires pr pred curlev
           do k=1,nef
              beta_ef(k) = b1(nrisqtot+nvarxevt+nasso+k)
           end do

           if (idst(ke).eq.1) then  !bi
              call fct_risq_irtsre(i,ke,brisq,risq,surv,surv0,survint)  
           else !if(idst.eq.2) then   !niv.courant du processus latent
              call fct_risq_irtsre_2(i,ke,brisq,bassoCL,bassoCS,beta_ef,ui,risq,surv,surv0)  
           end if

           sumnrisq = sumnrisq + nprisq(ke)
           sumnassoCL = sumnassoCL + nassoCLevt(ke)
           sumnassoCS = sumnassoCS + nassoCSevt(ke)
        end do

!  print*,"fct_risq ok"
        ! variables explicatives de la survie
        Xevt=0.d0
        bevt=0.d0
        bevtint=0.d0
        if (nxevt.ne.0) then    ! si varexpsurv

           m=0
           do ke=1,nbevt
              nevtxcurr=0
              do k=1,nv 

                 if (idtdv(k).ne.1) then

                    if (idsurv(k).eq.1) then  
                       m=m+1
                       bevt(m)=b1(nrisqtot+nevtxcurr+1)
                       Xevt(m)=X(nmescur+1,k)
                    else
                       if (idsurv(k).eq.2) then   
                          m=m+1
                          bevt(m)=b1(nrisqtot+nevtxcurr+ke)
                          Xevt(m)=X(nmescur+1,k)
                       end if
                    end if

                 else ! i.e timedepvar

                    if (idsurv(k).eq.1) then  
                       bevtint(ke)=b1(nrisqtot+nevtxcurr+1)
                    else 
                       if (idsurv(k).eq.2) then
                          bevtint(ke)=b1(nrisqtot+nevtxcurr+ke)
                       end if
                    end if
                 end if
                 nevtxcurr=nevtxcurr+nevtparx(k)
              end do
           end do

        end if

        ! vrais survie
        surv_glob=0.d0
        surv0_glob=0.d0
        varexpsurv=0.d0
        nxevtcurr=0
        fevt=0.d0
        pred_cl_Ti=0.d0
        easurv=0.d0
        m=0
        do ke=1,nbevt
        
           ! calculer Xevt * bevt
           varexpsurv=0.d0
           if (nxevt.ne.0) then   !si varexpsurv
              varexpsurv=DOT_PRODUCT(Xevt((nxevtcurr+1):(nxevtcurr+nxevt))&
                   ,bevt((nxevtcurr+1):(nxevtcurr+nxevt)))
           end if
           
           ! effets aleatoires partages 
           if (idst(ke).eq.1) then
              easurv=0.d0
              if(nea.gt.0) then      !si EA
                 if(nonlinRE(ke).eq.0) then ! asso lineaire
                    easurv=DOT_PRODUCT(ui,b1((nrisqtot+nvarxevt+m+1)&
                         :(nrisqtot+nvarxevt+m+nea)))
                    m = m+nea
                 else if(nonlinRE(ke).eq.1) then ! asso polynomiale
                    do k = 1,nea
                       easurv = easurv + poly(ui(k), b1((nrisqtot+nvarxevt+m+1)&
                            :(nrisqtot+nvarxevt+m+3)), 0.d0)
                       m = m+3
                    end do
                 else if (nonlinRE(ke).eq.2) then !  asso logistique
                    do k = 1,nea
                       easurv = easurv + sigmoid(ui(k), b1((nrisqtot+nvarxevt+m+1)&
                         :(nrisqtot+nvarxevt+m+3)))
                       m = m+3
                    end do
                 end if
              end if
           else
              m = m + nassoCLevt(ke) + nassoCSevt(ke)
           end if

           ! avoir evt au temps Ti si Devt=1
           if (Devt(i).eq.ke) then     !si sujet i a evt ke
              if (idst(ke).eq.1) then
                fevt=risq(ke)*exp(varexpsurv+easurv)   !fct de risq         
             else                 
                fevt=risq(ke)*exp(varexpsurv) !*pred_cl_Ti
             end if
              if (ind_survint(i).eq.1) then
                 fevt=fevt*exp(bevtint(ke))
              end if
           end if

           ! risque cumule jusque Ti
           if (idst(ke).eq.1) then
            Surv_glob=surv_glob + survint(ke)*exp(varexpsurv+easurv) + &
                  exp(bevtint(ke)+varexpsurv+easurv)*(surv(ke)-survint(ke))     
           else !if (idst(ke).eq.2) then
            Surv_glob=surv_glob + surv(ke)*exp(varexpsurv)
         end if
    
           ! troncature : risque cumule au temps T0
           if (idtrunc.eq.1) then
            if (idst(ke).eq.1) then
              surv0_glob=surv0_glob+surv0(ke)*exp(varexpsurv+easurv)    
            else !if (idst(ke).eq.2) then
              surv0_glob=surv0_glob+surv0(ke)*exp(varexpsurv)  
            end if
           end if
           
           nxevtcurr=nxevtcurr+nxevt
        end do

        ! vraisemblance de la partie survie
        vrais_surv = exp(-Surv_glob)
        !print*,"vrais_surv ok", vrais_surv
        !print*, "fevt=", fevt
       
        if(Devt(i).gt.0) vrais_surv = vrais_surv * fevt

        !if (idtrunc.eq.1) then
        !   vrais_surv = vrais_surv / exp(-surv0_glob)
        !end if
               
        ! vrais totale 
        conddensity = log(vrais_Y) + log(vrais_surv)

     else !  pas de survie

        conddensity = log(vrais_Y)

     end if

   conddensity = conddensity + jacobien

  if (idtrunc.eq.1) then    !delayedentry
     !print*,"surv0_glob = ", surv0_glob
      conddensity = conddensity + surv0_glob
  end if  
  !print*,"trunc ok condensity=", conddensity



  !!!! compute f(b), a Gaussian multivariate density !!!! 

  Ut=0.d0 !matrice varcov des EAs
  Ut(1,1)=1.d0
  if (nea>1) then 
     if (idiag.eq.1) then
        do j=2,nea
           do k=2,nea
              if (j.eq.k) then
                 Ut(j,k)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+j-1)
              else
                 Ut(j,k)=0.d0
              end if
           end do
        end do
     end if
     if (idiag.eq.0) then
        do j=2,nea
           do k=1,j
              Ut(j,k)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+k-1+j*(j-1)/2)
           end do
        end do
     end if
  end if

  !Ut en vecteur
  jj=0
  Ut_v=0.d0
  do j1=1,nea
     do j2=j1,nea
        jj=j1+j2*(j2-1)/2
        Ut_v(jj)=Ut(j2,j1)
     end do
  end do

  ! inversion
  CALL dsinv(Ut_v,nea,eps,ier,det_b)
  if (ier.eq.-1) then
     conddensity=-1.d9
     !print*,"-1.d9 dsinv Ut_v"
     goto 654
  end if

  ! retransformation du vecteur Ut_v en matrice :
  Ut=0.d0
  do j1=1,nea
     do j2=1,nea
        if (j2.ge.j1) then
           Ut(j1,j2)=Ut_v(j1+j2*(j2-1)/2)
        else
           Ut(j1,j2)=Ut_v(j2+j1*(j1-1)/2)
        end if
     end do
  end do

  ! calcul de la densite des EAs
  Y3_b=0.d0
  Y4_b=0.d0
  Y3_b=matmul(Ut,ui)
  Y4_b=DOT_PRODUCT(ui,Y3_b)

  div_b = 0.d0        
  div_b = dble(nea / 2.d0) * log(dble(2 * 3.14159265)) + 0.5 * det_b
  vrais_b = -Y4_b / 2.d0 - div_b

  !print*,"vrais_b = ", vrais_b
  conddensity = conddensity + vrais_b
  654 continue
!print*,"fin, condensity = ", conddensity
  return

end function conddensity

#' WaldMult 
#' Performs multivariate Wald tests.
#' 
#' Given a vector of estimated parameters (coef = (theta_1, ... theta_n)') and
#' their associated variance matrix (vcov = Var(coef)), the WaldMult function
#' performs either the test of the null hypothesis :
#'
#' 1. H0 : theta_m = (theta_j1, ..., theta_jm)' = 0
#'    ie, tests if a subset of m parameters are simultaneously equal to zero.
#'    The Wald statistic is w = theta_m' * Var(theta_m)^(-1) * theta_m
#'    and the associated p-value is obtained by p = P(chi2_(dof=m) > w)
#'
#' 2. H0 : theta = c1*theta_j1 + ... + cm*theta_jm = 0
#'    ie, tests if a combination of m parameters is equal to zero.
#'    The Wald statistic is w = theta^2 / Var(theta)
#'                            = (c1*theta_j1 + ... + cm*theta_jm)^2 / Var(c1*theta_j1 + ... + cm*theta_jm)
#'    and the associated p-value is obtained by p = P(chi2_(dof=1) > w)
#'
#' 3. H0 : Theta = A*coef = 0 with A a matrix with m lines and n columns.
#'    ie, tests if multiple combinations of parameters are simultaneously equal to zero.
#'    The Wald statistic is w = Theta * Var(Theta)^(-1) * Theta'
#'                            = (A*coef)' * (A*vcov*A')^(-1) * (A*coef)
#'    and the associated p-value is obtained by p = P(chi2_(dof=m) > w)
#'
#' The function can be applied to any object having a \code{coef} 
#'
#' @param Mod an object for which the \code{coef} and \code{vcov} methods are diefined.
#' @param coef if \code{Mod} is missing, the vector of estimated parameters
#' @param vcov if \code{Mod} is missing, the matrix of variance of coef
#' @param pos vector containing the indices of the parameters involved in
#' the test. Required if A is not specified.
#' @param contrasts optional vector containing the the c1, ..., cm value in
#' case 2 (see details). Should be of same lengtht as pos.
#' @param A matrix A of case 3 (see details). A should have as many columns as
#' parameters in coef.
#' @param name characters string giving the name of the test printed in the
#' output (the row names of the output). By default, the name's test is the
#' null hypothesis.
#' @param value the value against which to test. By default, value=0.
#'
#' @return
#' If contrasts is NULL, the function returns a matrix with 1 row and
#' 2 columns containing the value of the Wald test's statistic and
#' the associated p-value.
#'
#' If contrasts is not NULL, the function returns a matrix with 1 row
#' and 4 columns containing the value of the coefficient (dot product
#' of pos and contrasts), his standard deviation, the value of the
#' Wald test's statistic and the associated p-value.
#'
#' If A is specified, the function returns a matrix with m rows and
#' 2 columns containing the value of the Wald test's statistic and
#' the associated p-value.
#'
#'@examples
#' require(lcmm)
#' m <- hlme(Y ~ Time + X2, random = ~ 1, subject = "ID", data = data_hlme)
#' summary(m)
#'
#' mm <- hlme(Y ~ -1 + Time + factor(X2), random = ~ 1, subject = "ID", data = data_hlme)
#' summary(mm)
#'
#' ## Retrieve from model mm the difference between X2 levels as in model m :
#' WaldMult(Mod = mm, pos = c(2, 3), contrasts = c(-1, 1))
#' ## or
#' WaldMult(coef = coef(mm), vcov = vcov(mm), A = matrix(c(0, -1, 1, 0, 0), 1, 5))
#'
#' @export
WaldMult <- function(Mod, coef, vcov, pos = NULL, contrasts = NULL, A = NULL, name = NULL, value = 0)
{
    if(missing(Mod))
    {
        if(missing(coef)) stop("coef should be specified")
        if(missing(vcov)) stop("vcov should be specified")
        if(!is.matrix(vcov)) stop("vcov should be a matrix")
    }
    else
    {
##        if(!inherits(Mod, "jointLPM")) stop("Mod should be a jointLPM model")
        coef <- stats::coef(Mod)
        vcov <- stats::vcov(Mod)
    }
    
    npm <- length(coef)
    if((nrow(vcov) != npm) | (ncol(vcov) != npm)) stop("the dimension of vcov does not match with the length of coef")

    if(is.null(names(coef))) names(coef) <- paste("coef", 1:npm, sep = "")
    
    if(is.null(A)) ## cas 1. ou 2.
    {
        ## On teste si pos est un vecteur
        if (is.null(pos))
        {
            stop("Either A or pos must be specified")
        }
        else
        {
            if (!is.vector(pos)) stop("pos must be a numeric vector")

            ## subset of parameters and their variance
            Vect <- coef[pos]
            Mat <- vcov[pos, pos]
            
            
            if (is.null(contrasts)) ## cas 1.
            { 
                
                if (!missing(value))
                {
                    if (!is.vector(value)) stop("value must be a numeric vector")
                    if (length(value) != length(pos)) stop("value must have the same length as the vector pos")
                    
                    Vect <- coef[pos] - value
                }
                
                
                Wald <- t(Vect) %*% solve(Mat) %*% Vect
                
                ddl <- length(pos)
                p_value <- 1 - pchisq(Wald, df = ddl)
                
                Results <- matrix(NA, nrow = 1, ncol = 2)
                colnames(Results) <- c("Wald Test", "p_value")
                if (is.null(name)) 
                {
                    if (!is.null(value))
                    {
                        rownames(Results) <- paste(names(coef[pos]), " = ", value, collapse = " and ", sep = "")
                    }
                    else
                    {
                        rownames(Results) <- paste(paste(names(coef[pos]), collapse = " = "), "= 0")
                    }
                }
                else
                {
                    rownames(Results) <- name
                }
                
                Results[, 1] <- round(Wald, 5)
                Results[, 2] <- round(p_value, 5)
            }
            else ## cas 2.
            {
                if (length(contrasts) != length(pos))  stop("contrasts must have the same length as the vector pos")
                if (sum(abs(contrasts)) == 0) stop("The absolute value of the sum of contratsts components must be different from 0")
                
                ## linear combination of the parameters
                Scalaire <- sum(Vect * contrasts)
                
                if (value != 0)
                {
                    if (!is.vector(value)) stop("value must be a numeric vector")
                    if (length(value) != 1) stop("value must be a vector with a unique argument")
                    
                    Scalaire <- sum(Vect * contrasts) - value
                }

                ## variance of the combination
                Var <- t(contrasts) %*% Mat %*% contrasts
                

                Wald <- Scalaire^2 / Var
                p_value <- 1 - pchisq(Wald, df = 1) #2 * (1 - pnorm(abs(Wald)))
                
                Results <- matrix(NA, nrow = 1, ncol = 4)
                colnames(Results) <- c("coef", "Se", "Wald Test", "p_value")
                if (is.null(name)) 
                {
                    if(is.null(value)) value <- 0
                    rownames(Results) <- paste(paste(names(coef[pos]), "*", contrasts, collapse = " + "), "= ", value)
                } 
                else
                {
                    rownames(Results) <- name
                }
                
                
                Results[, 1] <- round(sum(Vect * contrasts), 5)
                Results[, 2] <- round(sqrt(Var), 5)
                Results[, 3] <- round(Wald, 5)
                Results[, 4] <- round(p_value, 5)
            }
            
        }
    }
    else ## cas 3.
    {
        if(ncol(A) != npm) stop("A should have as many columns as parameters in coef")
        ddl <- nrow(A)

        if(!missing(value))
        {
            if(length(value) == 1)
            {
                value <- rep(value, ddl)
            }
            else
            {
                if(length(value) != ddl) stop(paste("Vector value should be of length", ddl))
            }
        }
        else
        {
            value <- rep(0, ddl)
        }

        Theta <- A %*% coef - value
        Wald <- t(Theta) %*% solve(A %*% vcov %*% t(A)) %*% Theta
        p_value <- 1 - pchisq(Wald, df = ddl)
        
        Results <- matrix(NA, nrow = 1, ncol = 2)
        colnames(Results) <- c("Wald Test", "p_value")   
        if (is.null(name)) 
        {
            rownames(Results) <- paste("A*coef = (", paste(value, collapse = ", "), ")'", sep = "")
        }
        else
        {
            rownames(Results) <- name
        }            
        Results[, 1] <- round(Wald, 5)
        Results[, 2] <- round(p_value, 5)
    }
    
    return(Results)
}

#' Projection function, used in Step 3 _ Staging from the 4Smethod
#'
#' This function determines the stage thresholds in the latent scale of the
#' dimension process. Specifically, this function predicts the dimension scores
#' corresponding to stage changes and deducts the corresponding thresholds in the
#' dimension scale based on a correspondence between the predicted scores and
#' the expected sum of the items. The output is a vector of the estimated stage transition
#' thresholds. 
#' 
#' @param D an object of class \code{jointLPM} representing the estimated 
#' dimension model with the items
#' @param Dss an object of class \code{jointLPM} representing the estimated proxy 
#' model between the dimension scores and the stages
#' @param nsim number of points used in the numerical integration (Monte-Carlo). 
#' nsim should be relatively important (nsim=1000 by default).
#' @param bounds dimension scale boundaries between which are computed a grid of
#' the dum of the items
#'
#' @return a vector of the estimated stage transition thresholds
#' 
#' ###@examples
#' 
#' @author Tiphaine Saulnier, Viviane Philipps and Cecile Proust-Lima
#' 
#' @references
#' preprint :  arXiv:2407.08278
#' 
#' @export

IRT4Sstaging <- function(D,Dss,nsim=1000,bounds=c(-10,10))
{
  if(missing(D)) stop("the model (argument D) is missing")
  if(!inherits(D,"jointLPM")) stop("use only with jointLPM model")
  if(missing(Dss)) stop("the model (argument Dss) is missing")
  if(!inherits(Dss,"jointLPM")) stop("use only with jointLPM model")
  
  
  # convert JLPM -> multlcmm for predictions
  dss <- convert(object = Dss, to = "multlcmm")
  
  
  ### item score predictions  
  Ypred <- predictYcond(x = dss,
                        lprocess = Dss$thres[,1],
                        nsim=nsim)$pred$Ypred[1:length(Dss$thres[,1])]
  
  ### conditional expectation of the sum of items
  tmp <- data.frame(lambda = seq(from = bounds[1], to = bounds[2], by = 0.01))
  tmp$sum <- sapply(X = tmp$lambda, 
                    FUN = .expect, 
                    ni=D$N[12], nl=max(D$nbmod), discrim = D$discrim, thres = D$thres)

  ### deduct lambda, such that score = sum of items
  res <- rep(NA,length(Dss$thres[,1]))
  for(j in 1:length(Dss$thres[,1]))
    res[j] <- max(tmp$lambda[which(abs(tmp$sum - Ypred[j]) == min(abs(tmp$sum - Ypred[j])))]) 
  
  
  return(res)
}

#' removeNA
#'
#' Remove missing values
#' 
#' Caution : if several response variables are included in 'form', then the resulting data frame will contain one column (named outcome) grouping all these response variables. The irst lines will refer to the first response variables, a second block to the second, etc. All covariates will be repeated for each response variable.
#' 
#' @param form a list of formula
#' @param data a data frame containing all variables mentionned in form
#' @return a list containing
#'  \item{newdata}{a data frame created from data, containing only the variables used in the formula, and without any missing values}
#'  \item{Y}{the names of the response variables}
#'  \item{YlinesNA}{the lines removed from each response variables}
#'  \item{nmes}{the number of measures for each response variable}
#'  \item{X}{the names of the covariates}
#'  \item{XlinesNA}{the lines removed from data because of missing values in the covariates}
#'
#' @examples
#'  d <- data.frame(y1=c(3,6,4,2), y2=c(6.5,NA,9.5,4.5), x=c(0,0,1,0))
#'  removeNA(form=list(y1+y2~x), data=d)
#' ## 1      3.0 0   | first block referring to y1
#' ## 2      6.0 0   |
#' ## 3      4.0 1   |
#' ## 4      2.0 0   |
#' ## 11     6.5 0     | second block referring to y2
#' ## 31     9.5 1     |
#' ## 41     4.5 0     |
#' 
#' @export
removeNA <- function(form, data)
{
    nform <- length(form)
    vars <- vector("list", length=nform) #covariates names
    nas <- vector("list", length=nform) # NA in covariates
    outcomes <- NULL # outcomes names
    naoutcomes <- list() # NA in each outcome
    term <- vector("list", length=nform)

    ## loop on the formulas : keep variables and lines with NA
    for(k in 1:nform)
    {
        ## covariates
        termsk <- delete.response(terms(form[[k]]))
        dk <- model.frame(termsk, data=data)
        vars[[k]] <- all.vars(termsk)
        nas[[k]] <- na.action(dk)
        term[[k]] <- terms(dk)

        ## outcomes
        if(attr(terms(form[[k]]), "response")==1)
        {
            y <- all.vars(form[[k]][[2]])
            outcomes <- c(outcomes, y)
            
            ny <- length(y)
            for(m in 1:ny)
            {
                naoutcomes <- c(naoutcomes, list(which(is.na(data[,y[m]]))))
            }           
        }
    }
    
    ## create newdata
    allvars <- unique(unlist(vars))
    linesNA <- unique(unlist(nas))
    
    if(is.null(outcomes))
    {
        ## keep same form as data
        
        if(is.null(linesNA))
        {
            newdata <- data[, allvars, drop=FALSE]
        }
        else
        {
            newdata <- data[-linesNA, allvars, drop=FALSE]
        }
        nmes <- nrow(newdata)
    }
    else
    {
        ## rbind the data, with one block per outcome
        ## a block has nrow(data)-length(linesNA) rows
        
        newdata <- NULL
        nmes <- rep(NA,length(outcomes))
        for(k in 1:length(outcomes))
        {
            nak <- unique(c(linesNA, naoutcomes[[k]]))
            if(!length(nak))
            {
                newdk <- data[, c(outcomes[k],allvars), drop=FALSE]
            }
            else
            {
                newdk <- data[-nak, c(outcomes[k],allvars), drop=FALSE]
            }
            colnames(newdk) <- c("outcome", allvars)
            nmes[k] <- nrow(newdk)

            newdata <- rbind(newdata, newdk)
        }
    }
    
    return(list(newdata=newdata, Y=outcomes, YlinesNA=naoutcomes, nmes=nmes, X=allvars, XlinesNA=linesNA, terms=term))    
}

#' Estimation of joint latent process models
#' 
#' Functions for the estimation of joint latent process models (JLPM).
#' Continuous and ordinal outcomes are handled for the longitudinal part,
#' whereas the survival part considers multiple competing events.
#' The likelihood is computed using Monte-Carlo integration. Estimation is achieved
#' by maximizing the log-likelihood using a robust iterative algorithm.
#'
#' Please report to the JLPM-team any question or suggestion regarding the package
#' via github only (https://github.com/VivianePhilipps/JLPM/issues).
#' 
#' @name JLPM-package
#' @author Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
#' 
#' @references
#' Saulnier, Philipps, Meissner, Rascol, Pavy-Le-Traon, Foubert-Samier, Proust-Lima (2022).
#' Joint models for the longitudinal analysis of measurement scales in the presence 
#' of informative dropout, Methods; 203:142-51. 

#' 
#' Philipps, Hejblum, Prague, Commenges, Proust-Lima (2021).
#' Robust and efficient optimization using a Marquardt-Levenberg algorithm with 
#' R package marqLevAlg, The R Journal 13:2.
#'
#' @keywords package
#' @importFrom graphics axis hist lines matlines matplot mtext par plot points segments polygon
#' @importFrom grDevices rainbow rgb col2rgb n2mfrow
#' @importFrom stats as.formula formula get_all_vars integrate median model.frame model.matrix delete.response na.action na.fail na.omit na.pass pchisq dnorm pnorm qnorm quantile rnorm sd terms residuals vcov fitted coef update
#' @importFrom survival Surv untangle.specials
#' @importFrom spacefillr generate_sobol_owen_set
#' @importFrom stringr str_detect
#' @importFrom utils as.roman
#' @importFrom parallel clusterEvalQ clusterExport clusterSetRNGStream makeCluster parApply stopCluster
#' @importFrom marqLevAlg mla
#' @importFrom lcmm multlcmm Jointlcmm predictYcond
#' @useDynLib JLPM, .registration=TRUE, .fixes="C_"
NULL

#' Estimation of latent process joint models for multivariate longitudinal 
#' outcomes and time-to-event data.
#' 
#' This function fits extended joint models with shared random effects.
#' The longitudinal submodel handles multiple continuous longitudinal outcomes
#' (Gaussian or  non-Gaussian, curvilinear) as well as 
#' ordinal longitudinal outcomes in a mixed effect framework. The model assumes 
#' that all the outcomes 
#' measure the same underlying latent process defined as their common
#' factor, and each outcome is related to this latent common factor by a
#' outcome-specific measurement model whose nature depends on the type of the 
#' outcome (linear model for Gaussian outcome, curvilinear model for 
#' non-Gaussian outcome, probit model for binary outcome, cumulative probit 
#' model for ordinal outcome).
#' At the latent process level, the model assumes a linear mixed model.
#' The survival submodel handles right-censored (possibly left-truncated) 
#' time-to-event with competing causes
#' The association between the longitudinal and the survival data is captured 
#' by including 
#' the random effect from the mixed model or the predicted current level 
#' of the underlying process as linear predictors in the cause-specific 
#' proportional hazard survival model.
#' Parameters of the measurement models, of the latent process mixed model and of the 
#' survival model are simultaneously estimated using a maximum likelihood method,
#' through a Marquardt-Levenberg optimization algorithm.
#' 
#' 
#' 
#' A. THE MEASUREMENT MODELS
#' 
#' \code{jointLPM} function estimates one measurement model per outcome to link
#' each outcome Y_k(t) with the underlying latent common factor L(t) they measure.
#' To fix the latent process dimension, we chose to constrain the latent 
#' process dimension with the intercept of the mixed model fixed at 0 and the 
#' standard error of the first random effect fixed at 1. The nature of each 
#' measurement
#' model adapts to the type of the outcome it models. 
#' 
#' 1. For continuous Gaussian outcomes, linear models are used and require 2 parameters for 
#' the transformation (Y(t) - b1)/b2
#' 
#' 2. For continuous non-Gaussian outcomes, curvilinear models use 
#' parameterized link functions to link outcomes to the latent process. 
#' With the "beta" link function, 4 parameters are required for the
#' following transformation: [ h(Y(t)',b1,b2) - b3]/b4 where h is the Beta CDF
#' with canonical parameters c1 and c2 that can be derived from b1 and b2 as
#' c1=exp(b1)/[exp(b2)*(1+exp(b1))] and c2=1/[exp(b2)*(1+exp(b1))], and Y(t)'
#' is the rescaled outcome i.e. Y(t)'= [ Y(t) - min(Y(t)) + epsY ] / [
#' max(Y(t)) - min(Y(t)) +2*epsY ].
#' With the "splines" link function, n+2 parameters are required for the
#' following transformation b_1 + b_2*I_1(Y(t)) + ... + b_(n+2)*I_(n+1)(Y(t)),
#' where I_1,...,I_(n+1) is the basis of quadratic I-splines. To constrain the
#' parameters to be positive, except for b_1, the program estimates b_k^* (for
#' k=2,...,n+2) so that b_k=(b_k^*)^2.
#' 
#' 3. For ordinal outcomes (and binary outcomes), cumulative probit models are used. For a
#' (n+1)-level outcome, the model consist of determining n thresholds t_k in the 
#' latent process scale which correspond to the outcome level changes. Then,
#' Y(t) = n' <=> t_n' < L(t) + e <= t_(n'+1) with e the standard error of the outcome.
#' To ensure that t_1 < t_2 < ... < t_n, the program estimates t'_1, t'_2, ..., t'_n
#' such that t_1=t'_1, t_2=t_1+(t'_2)^2, t_3=t_2+(t'_3)^2, ...
#' 
#' 
#' B. THE SURVIVAL MODEL
#' 
#' a. BASELINE RISK FUNCTIONS
#' 
#' For the baseline risk functions, the following parameterizations are considered. 
#' 
#' 1. With the "Weibull" function: 2 parameters are necessary w_1 and w_2 so that 
#' the baseline risk function a_0(t) = w_1^2*w_2^2*(w_1^2*t)^(w_2^2-1) if logscale=FALSE  
#' and \cr
#' a_0(t) = exp(w_1)*exp(w_2)(t)^(exp(w_2)-1) if logscale=TRUE.
#' 
#' 2. with the "piecewise" step function and nz nodes (y_1,...y_nz), nz-1 parameters 
#' are necesssary p_1,...p_nz-1 so that the baseline risk function a_0(t) = p_j^2 
#' for y_j < t =< y_j+1 if logscale=FALSE and a_0(t) = exp(p_j) for y_j < t =< y_j+1 if logscale=TRUE.
#' 
#' 3. with the "splines" function and nz nodes (y_1,...y_nz), nz+2 parameters 
#' are necessary s_1,...s_nz+2 so that the baseline risk function a_0(t) = sum_j s_j^2 M_j(t) 
#' if logscale=FALSE and a_0(t) = sum_j exp(s_j) M_j(t) if logscale=TRUE where M_j is the basis of cubic M-splines.
#' Two parameterizations of the baseline risk function are proposed (logscale=TRUE or FALSE) 
#' because in some cases, especially when the instantaneous risks are very close to 0, 
#' some convergence problems may appear with one parameterization or the other. 
#' As a consequence, we recommend to try the alternative parameterization 
#' (changing logscale option) 
#' when a model does not converge (maximum number of iterations reached) although
#'   convergence criteria based on the parameters and likelihood are small.
#' 
#' 
#' b. ASSOCIATION BETWEEN LONGITUDINAL AND SURVIVAL DATA
#' 
#' The association between the longitudinal and the survival data is captured by including 
#' a function of the elements from the latent process mixed model as a predictor in the survival model.
#'  We implemented so far two association structures,
#' that should be specified through \code{sharedtype} argument.
#' 
#' 1. the random effect from the latent process linear mixed model (\code{sharedtype='RE'}):
#' the q random effects modeling the individual deviation in the longitudinal model
#'  are also included
#' in the survival model, so that a q-vector of parameters measures the association
#' between the risk of event and the longitudinal outcome(s).
#' 
#' 2. the predicted current level of the underlying process (\code{sharedtype='CL'}):
#' the predicted latent process defined by the mixed model at time t is included as
#' time-dependent covariate in the risk of event model for time t.
#' The association between the longitudinal process and the risk of event
#' is then quantified by a unique parameter.
#' 
#'  
#' C. THE VECTOR OF PARAMETERS B
#' 
#' The parameters in the vector of initial values \code{B} or in the vector of
#' maximum likelihood estimates \code{best} are included in the following
#' order:
#' (1) parameters for the baseline risk function: 2 parameters for each Weibull, 
#' nz-1 for each piecewise constant risk and nz+2 for each splines risk. In the 
#' presence of competing events, the number of parameters should be adapted to 
#' the number of causes of event;
#' (2) for all covariates in survival, one parameter 
#' is required. Covariates parameters should be included in the same order as in survival.
#' In the presence of cause-specific effects, the number of parameters should be
#' multiplied by the number of causes;
#' (3) parameter(s) of association between the longitudinal 
#' and the survival process: for \code{sharedtype='RE'}, one parameter per random effect
#' and per cause of event is 
#' required; for \code{sharedtype='CL'}, one parameter per cause of event is required;
#' (4) for all covariates  in fixed, one parameter is required. Parameters should
#' be included in the same  order as in fixed;
#' (5)for all covariates included with \code{contrast()} in \code{fixed}, one
#' supplementary parameter per outcome is required excepted for the last
#' outcome for which the parameter is not estimated but deduced from the others;
#' (6) the variance of each random-effect specified in random 
#' (excepted the intercept which is constrained to 1) 
#' if idiag=TRUE and the inferior triangular variance-covariance matrix of all 
#' the random-effects if idiag=FALSE;
#' (7) if \code{cor} is specified, the standard error of the Brownian motion or 
#' the standard error and the correlation parameter of the autoregressive process;
#' (8) parameters of each measurement model: 2 for "linear", 4 for "beta", 
#' n+2 for "splines" with n nodes, n for "thresholds" for a (n+1)-level outcome; 
#' (9) if \code{randomY=TRUE}, the standard 
#' error of the outcome-specific random intercept (one per outcome); 
#' (10) the outcome-specific standard errors (one per outcome)
#' 
#' 
#' 
#' C. CAUTIONS REGARDING THE USE OF THE PROGRAM
#' 
#' Some caution should be made when using the program. Convergence criteria are
#' very strict as they are based on the derivatives of the log-likelihood in
#' addition to the parameter and log-likelihood stability. In some cases, the
#' program may not converge and reach the maximum number of iterations fixed at
#' 100 by default. In this case, the user should check that parameter estimates at the
#' last iteration are not on the boundaries of the parameter space.
#' 
#' If the parameters are on the boundaries of the parameter space, the
#' identifiability of the model is critical. This may happen especially with
#' splines parameters that may be too close to 0 (lower boundary). When
#' identifiability of some parameters is suspected, the program can be run
#' again from the former estimates by fixing the suspected parameters to their
#' value with option posfix. This usually solves the problem. An alternative is
#' to remove the parameters of the Beta of Splines link function from the
#' inverse of the Hessian with option partialH. If not, the program should be 
#' run again with other initial values, with a higher maximum number of iterations 
#' or less strict convergence tolerances.
#' 
#' To reduce the computation time, this program can be carried out in parallel mode,
#' ie. using multiple cores which number can be specified with argument \code{nproc}.
#' 
#' 
#' 
#' @param fixed a two-sided linear formula object for specifying the
#' fixed-effects in the linear mixed model at the latent process level. The
#' response outcomes are separated by \code{+} on the left of \code{~} and the
#' covariates are separated by \code{+} on the right of the \code{~}. For
#' identifiability purposes, the intercept specified by default should not be
#' removed by a \code{-1}. Variables on which a contrast above the different
#' outcomes should also be estimated are included with \code{contrast()}.
#' @param random a one-sided formula for the random-effects in the
#' latent process mixed model. Covariates with a random-effect are separated
#' by \code{+}. An intercept should always be included for identifiability.
#' @param subject name of the covariate representing the grouping structure.
#' @param idiag optional logical for the variance-covariance structure of the
#' random-effects. If \code{FALSE}, a non structured matrix of
#' variance-covariance is considered (by default). If \code{TRUE} a diagonal
#' matrix of variance-covariance is considered.
#' @param randomY optional logical for including an outcome-specific random
#' intercept. If \code{FALSE} no outcome-specific random intercept is added
#' (default). If \code{TRUE} independent outcome-specific random intercepts
#' with parameterized variance are included.
#' @param link optional vector of families of parameterized link functions defining
#' the measurement models (one by outcome). Option "linear" (by default) specifies a linear
#' link function. Other possibilities include "beta" for estimating a link
#' function from the family of Beta cumulative distribution functions, "Splines" 
#' for approximating the link function by I-splines and "thresholds" for ordinal
#' outcomes modelled by cumulative probit models. For splines case, the number of 
#' nodes and the nodes location should be also specified. The number of nodes is 
#' first entered followed by \code{-}, then the location is specified with "equi", 
#' "quant" or "manual" for respectively equidistant
#' nodes, nodes at quantiles of the marker distribution or interior nodes
#' entered manually in argument \code{intnodes}. It is followed by \code{-splines}.
#' For example, "7-equi-splines" means
#' I-splines with 7 equidistant nodes, "6-quant-splines" means I-splines with 6
#' nodes located at the quantiles of the marker distribution and
#' "9-manual-splines" means I-splines with 9 nodes, the vector of 7 interior
#' nodes being entered in the argument \code{intnodes}.
#' @param intnodes optional vector of interior nodes. This argument is only
#' required for a I-splines link function with nodes entered manually.
#' @param epsY optional positive real used to rescale the marker in
#' (0,1) when the beta link function is used. By default, epsY=0.5.
#' @param cor optional indicator for inclusion of an autocorrelated Gaussian
#' process in the linear mixed model at the latent process level. Option
#' \code{BM(time)} indicates a brownian motion with parameterized variance while
#' option \code{AR(time)} specifies an autoregressive process in continuous time
#' with parameterized variance and correlation intensity. In both cases, \code{time}
#' is the variable representing the measurement times. By default, no autocorrelated
#' Gaussian process is added.
#' @param survival two-sided formula object. The left side of the formula corresponds 
#' to a Surv() object for right-censored (\code{Surv(EntryTime,Time,Indicator)})
#' and possibly left-truncated (\code{Surv(EntryTime,Time,Indicator)}).
#' Multiple causes of event can be considered 
#' in the Indicator (0 for censored, k for event of cause k). The right side of the 
#' formula specifies the covariates to include in the survival model.
#' Cause-specific covariate effect are specified with \code{cause()}.For example,  
#' Surv(Time,Indicator) ~ X1 + cause(X2) indicates a common effect of X1 and a cause-specific effect of X2. 
#' @param hazard optional family of hazard function assumed for the survival model. 
#' By default, "Weibull" specifies a Weibull baseline risk function. Other possibilities 
#' are "piecewise" for a piecewise constant risk function or "splines" for a cubic M-splines 
#' baseline risk function. For these two latter families, the number of nodes and the 
#' location of the nodes should be specified as well, separated by -. The number of 
#' nodes is entered first followed by -, then the location is specified with "equi", 
#' "quant" or "manual" for respectively equidistant nodes, nodes at quantiles of the 
#' times of event distribution or interior nodes entered manually in argument hazardnodes. 
#' It is followed by - and finally "piecewise" or "splines" indicates the family of 
#' baseline risk function considered. Examples include "5-equi-splines" for M-splines 
#' with 5 equidistant nodes, "6-quant-piecewise" for piecewise constant risk over 5 
#' intervals and nodes defined at the quantiles of the times of events distribution 
#' and "9-manual-splines" for M-splines risk function with 9 nodes, the vector of 7 
#' interior nodes being entered in the argument hazardnodes. In the presence of competing 
#' events, a vector of hazards should be provided such as hazard=c("Weibull","5-quant-splines") 
#' with 2 causes of event, the first one modelled by a Weibull baseline cause-specific 
#' risk function and the second one by splines.
#' @param hazardnodes optional vector containing interior nodes if splines or piecewise 
#' is specified for the baseline hazard function in hazard.
#' @param TimeDepVar optional vector containing an intermediate time
#' corresponding to a change in the risk of event. This time-dependent
#' covariate can only take the form of a time variable with the assumption that
#' there is no effect on the risk before this time and a constant effect on the
#' risk of event after this time (example: initiation of a treatment to account
#' for).
#' @param logscale optional boolean indicating whether an exponential
#' (logscale=TRUE) or a square (logscale=FALSE -by default) transformation is
#' used to ensure positivity of parameters in the baseline risk functions. See
#' details section
#' @param startWeibull optional numeric with Weibull hazard functions only.
#' Indicates the shift in the Weibull distribution.
#' @param hazardrange optional vector indicating the range of the survival times 
#' (that is the minimum and maximum). By default, the range is defined according 
#' to the minimum and maximum observed values of the survival times. The option 
#' should be used only for piecewise constant and Splines hazard functions.
#' @param data data frame containing all variables named in \code{fixed},
#'  \code{random}, \code{cor}, \code{survival} and \code{subject}.
#' @param B optional specification for the initial values for the parameters.
#' Initial values should be entered in the order detailed in \code{details} section.
#' @param convB optional threshold for the convergence criterion based on the
#' parameter stability. By default, convB=0.0001.
#' @param convL optional threshold for the convergence criterion based on the
#' log-likelihood stability. By default, convL=0.0001.
#' @param convG optional threshold for the convergence criterion based on the
#' derivatives. By default, convG=0.0001.
#' @param maxiter optional maximum number of iterations for the Marquardt
#' iterative algorithm. By default, maxiter=100.
#' @param nsim number of points used to plot the estimated link functions. By
#' default, nsim=100.
#' @param range optional vector indicating the range of the outcomes (that is
#' the minimum and maximum). By default, the range is defined according to the
#' minimum and maximum observed values of the outcome. The option should be
#' used only for Beta and Splines transformations.
#' @param subset optional vector giving the subset of observations in
#' \code{data} to use. By default, all lines.
#' @param na.action Integer indicating how NAs are managed. The default is 1
#' for 'na.omit'. The alternative is 2 for 'na.fail'. Other options such as
#' 'na.pass' or 'na.exclude' are not implemented in the current version.
#' @param posfix Optional vector giving the indices in vector B of the
#' parameters that should not be estimated. Default to NULL, all parameters are
#' estimated.
#' @param partialH optional vector giving the indices in vector B of parameters that
#' can be dropped from the Hessian matrix to define convergence criteria.
#' @param verbose logical indicating if information about computation should be
#' reported. Default to TRUE.
#' @param returndata logical indicating if data used for computation should be
#' returned. Default to FALSE, data are not returned.
#' @param methInteg character indicating the type of integration to compute the
#' log-likelihood. 'MCO' for ordinary Monte Carlo, 'MCA' for antithetic Monte Carlo,
#' 'QMC' for quasi Monte Carlo. Default to "QMC".
#' @param nMC integer, number of Monte Carlo simulations. Default to 1000.
#' @param sharedtype indicator of shared random function type. \code{'RE'} indicates
#' an association through the random effects included in the linear mixed model.
#' \code{'CL'} defines a association through the predicted current level of the latent process.
#' @param centerpoly for polynomial associations only, value(s) to center degree 2 and 3 polynomial terms.
#' @param var.time name of the variable representing the measurement times.
#' @param nproc number of cores for parallel computation.
#' @param clustertype the type of cluster that should internally be created.
#' See \code{parallel::makeCluster} for possible values.
#' 
#' @return An object of class "jointLPM" is returned containing some internal information
#' used in related functions. Users may investigate the following elements : 
#' \item{ns}{number of grouping units in the dataset} 
#' \item{loglik}{log-likelihood of the model} 
#' \item{best}{vector of parameter estimates in the same order as specified in 
#' \code{B} and detailed in section \code{details}}
#' \item{V}{vector containing the upper triangle matrix of variance-covariance
#' estimates of \code{best} with exception for variance-covariance parameters
#' of the random-effects for which \code{V} contains the variance-covariance
#' estimates of the Cholesky transformed parameters displayed in \code{cholesky}} 
#' \item{gconv}{vector of convergence criteria: 1. on the parameters, 2. on the 
#' likelihood, 3. on the derivatives} 
#' \item{conv}{status of convergence: =1 if the convergence criteria were satisfied, 
#' =2 if the maximum number of iterations was reached, =4 or 5 if a problem occured 
#' during optimisation} 
#' \item{call}{the matched call} 
#' \item{niter}{number of Marquardt iterations} 
#' \item{nevent}{number of occured event}
#' \item{pred}{table of individual predictions and residuals in the underlying
#' latent process scale; it includes marginal predictions (pred_m), marginal
#' residuals (resid_m), subject-specific predictions (pred_ss) and
#' subject-specific residuals (resid_ss) and finally the transformed
#' observations in the latent process scale (obs).}
#' \item{predRE}{table containing individual predictions of the random-effects : 
#' a column per random-effect, a line per subject.}
#' \item{predRE_Y}{table containing individual predictions of the outcome-specific
#' random intercept}
#' \item{predSurv}{table containing the predicted baseline risk function and
#' the predicted cumulative baseline risk function }
#' \item{cholesky}{vector containing the estimates of the Cholesky transformed
#' parameters of the variance-covariance matrix of the random-effects}
#' \item{estimlink}{table containing the simulated values of each outcome and
#' the corresponding estimated link function} 
#' \item{epsY}{definite positive reals used to rescale the markers in (0,1) when 
#' the beta link function is used. By default, epsY=0.5.} 
#' \item{AIC}{the Akaike's information criterion}
#' \item{BIC}{the Bayesian information criterion}
#' \item{CPUtime}{the runtime in seconds}
#' \item{data}{the original data set (if returndata is TRUE)}
#' @author Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
#' 
#' @references
#' Saulnier, Philipps, Meissner, Rascol, Pavy-Le-Traon, Foubert-Samier, Proust-Lima (2022).
#' Joint models for the longitudinal analysis of measurement scales in the presence 
#' of informative dropout, Methods; 203:142-51. 
#' 
#' Philipps, Hejblum, Prague, Commenges, Proust-Lima (2021).
#' Robust and efficient optimization using a Marquardt-Levenberg algorithm with 
#' R package marqLevAlg, The R Journal 13:2.  
#' 
#' @examples
#' #### Examples with paquid data from R-package lcmm
#' library(lcmm)
#' paq <- paquid[which(paquid$age_init<paquid$agedem),]
#' paq$age65 <- (paq$age-65)/10
#'
#' #### Example with one Gaussian marker :
#' ## We model the cognitive test IST according to age, sexe and eduction level. We assume
#' ## a Weibull distribution for the time to dementia and link the longitudinal and survival
#' ## data using the random effects.
#' ## We provide here the call to the jointLPM function without optimization (maxiter=0). The
#' ## results should therefore not be interpreted.
#' M0 <- jointLPM(fixed = IST~age65*(male+CEP),
#'                 random=~age65,
#'                 idiag=FALSE,
#'                 subject="ID",
#'                 link="linear",
#'                 survival=Surv(age_init,agedem,dem)~male,
#'                 sharedtype='RE',
#'                 hazard="Weibull",
#'                 data=paq,
#'                 var.time="age65",
#'                 maxiter=0)
#' M0$best ## these are the initial values of each of the 15 parameters
#' 
#' \donttest{ 
#' ## Estimation with one Gaussian marker
#' ## We remove the maxiter=0 option to estimate the model. We specify initial values
#' ## to reduce the runtime, but this can take several minutes.
#' binit1 <- c(0.1039, 5.306, -0.1887, -1.0355, -4.3817, -1.0543, -0.1161, 0.8588,
#' 0.0538, -0.1722, -0.2224, 0.3296, 30.7768, 4.6169, 0.7396)
#' M1 <- jointLPM(fixed = IST~age65*(male+CEP),
#'                 random=~age65,
#'                 idiag=FALSE,
#'                 subject="ID",
#'                 link="linear",
#'                 survival=Surv(age_init,agedem,dem)~male,
#'                 sharedtype='RE',
#'                 hazard="Weibull",
#'                 data=paq,
#'                 var.time="age65",
#'                 B=binit1)
#' ## Optimized the parameters to be interpreted :
#' summary(M1)
#'
#' 
#' #### Estimation with one ordinal marker :
#' ## We consider here the 4-level hierarchical scale of dependence HIER and use "thresholds"
#' ## to model it as an ordinal outcome. We assume an association between the current level
#' ## of dependency and the risk of dementia through the option sharedtype="CL".
#' ## We use a parallel optimization on 2 cores to reduce computation time.
#' binit2 <- c(0.0821, 2.4492, 0.1223, 1.7864, 0.0799, -0.2864, 0.0055, -0.0327, 0.0017,
#' 0.3313, 0.9763, 0.9918, -0.4402)
#' M2 <- jointLPM(fixed = HIER~I(age-65)*male,
#'                 random = ~I(age-65),
#'                 subject = "ID", 
#'                 link = "thresholds",
#'                 survival = Surv(age_init,agedem,dem)~male,
#'                 sharedtype = 'CL',
#'                 var.time = "age",
#'                 data = paq, 
#'                 methInteg = "QMC", 
#'                 nMC = 1000,
#'                 B=binit2,
#'                 nproc=2)
#' summary(M2)
#' 
#' }
#' 
#' @export
#' 
jointLPM <- function(fixed,random,subject,idiag=FALSE,cor=NULL,link="linear",intnodes=NULL,epsY=0.5,randomY=FALSE, var.time,
                survival=NULL,hazard="Weibull",hazardrange=NULL,hazardnodes=NULL,TimeDepVar=NULL,logscale=FALSE,startWeibull=0, sharedtype='RE', centerpoly = 0,
                methInteg="QMC",nMC=1000,data,subset=NULL,na.action=1,
                B,posfix=NULL,maxiter=100,convB=0.0001,convL=0.0001,convG=0.0001,partialH=NULL,
                nsim=100,range=NULL,verbose=TRUE,returndata=FALSE,
                nproc=1, clustertype=NULL)
{
    ptm <- proc.time()
    
    cl <- match.call()
    
    nom.subject <- as.character(subject)
    
    if(missing(random)) stop("At least one random effect is required")
    if(random==~-1) stop("At least one random effect is required")
    if(missing(fixed)) stop("The argument fixed must be specified in any model")
    if(!inherits(fixed,"formula")) stop("The argument fixed must be a formula")
    if(!inherits(random,"formula")) stop("The argument random must be a formula")
    if(missing(data)){ stop("The argument data should be specified and defined as a data.frame")}
    if(nrow(data)==0) stop("Data should not be empty")
    if(missing(subject)){ stop("The argument subject must be specified")}
    if(!is.numeric(data[,subject])) stop("The argument subject must be numeric")
    if(all(link %in% c("linear","beta","thresholds")) & !is.null(intnodes)) stop("Intnodes should only be specified with splines links")
    if(!(na.action%in%c(1,2)))stop("only 1 for 'na.omit' or 2 for 'na.fail' are required in na.action argument")
    #if(!(sharedtype%in%c('RE','CL')))stop("The value of argument sharedtype must be 'RE' (for shared random effects) or 'CL' (for shared latent process current level)")
    if(missing(var.time) | length(var.time)!=1)stop("The argument var.time is missing or is not of length 1")
    if(!(var.time %in% colnames(data))) stop("Unable to find variable 'var.time' in 'data'")

    shtypes <- c("RE", "REpoly", "RElogit", "CL", "CLpoly", "CLlogit", "CS", "CSpoly", "CSlogit",
                 "CL+CS", "CLpoly+CS", "CLlogit+CS", "CL+CSpoly", "CL+CSlogit", "CLpoly+CSpoly", "CLpoly+CSlogit", "CLlogit+CSpoly", "CLlogit+CSlogit") # associations possibles
    sharedtype <- gsub("[[:space:]]", "", sharedtype)
    if(!all(sharedtype %in% shtypes)) stop(paste("sharedtype should be among", paste(shtypes, collapse = ", ")))

    if(any(sharedtype %in% shtypes[-c(1:3)]) & (!missing(cor))) print("WARNING : wi not computed on current level prediction 'cause considered not shared")
    if(any(sharedtype %in% shtypes[-c(1:3)]) & (!missing(TimeDepVar))) stop("model with sharedtype='CL' not yet programmed with time dependent effect on survival")

    
    #    if(length(posfix) & missing(B)) stop("A set of initial parameters must be specified if some parameters are not estimated")
    
    
    ## garder data tel quel pour le renvoyer
    if(returndata==TRUE)
    {
        datareturn <- data
    }
    else
    {
        datareturn <- NULL
    }
    
    ## test de l'argument cor
    ncor <- 0
    cor.type <- cl$cor[1]
    cor.time <- cl$cor[2]
    cor <- paste(cor.type,cor.time,sep="-")
    if (!isTRUE(all.equal(cor,character(0))))
    {
        if (substr(cor,1,2)=="AR") { ncor <- 2 }
        else if (substr(cor,1,2)=="BM") { ncor <- 1  }
        else { stop("The argument cor must be of type AR or BM") }
        
        if(!(strsplit(cor,"-")[[1]][2] %in% colnames(data))) stop("Unable to find time variable from argument 'cor' in 'data'")
        else { cor.var.time <- strsplit(cor,"-")[[1]][2] }
    }
    
    ##pour acces aux attributs des formules
    afixed <- terms(fixed, specials=c("factor","contrast"))
    if(attr(afixed,"intercept")==0) stop("An intercept should appear in fixed for identifiability purposes")
    
    arandom <- terms(random, specials=c("factor"))
    ##fixed sans contrast
    fixed2 <- gsub("contrast","",fixed)
    fixed2 <- formula(paste(fixed2[2],fixed2[3],sep="~"))   
    afixed2 <- terms(fixed2)
    
    ##verifier si toutes les varialbes sont dans data
    variables <- c(attr(afixed,"variables"),attr(arandom,"variables"))
    variables <- unlist(lapply(variables,all.vars))  
    if(!all(variables %in% colnames(data))) stop(paste("Data should contain the variables",paste(unique(variables),collapse=" ")))
    
    
    ##contrast
    contr <- ~-1
    if(!is.null(attr(afixed,"specials")$contrast))
    {
        vcontr <- attr(afixed,"term.labels")[setdiff(attr(afixed,"specials")$contrast-1,untangle.specials(afixed,"contrast",2)$terms)]
        vcontr <- gsub("contrast","",vcontr)
        contr <- as.formula(paste("~-1+",paste(vcontr,collapse="+")))
    }
    acontr <- terms(contr)
    
    
    ##liste des outcomes
    nomsY <- as.character(attr(afixed,"variables")[2])
    nomsY <- strsplit(nomsY,split=" + ",fixed=TRUE)
    nomsY <- as.vector(nomsY[[1]])
    ny <- length(nomsY)
    
    ##pas de contrast ni randomY si un seul Y
    if(ny<2 & length(attr(afixed,"specials")$contrast)) stop("No contrast can be included with less than two outcomes")
    if(ny<2 & randomY==TRUE) stop("With less than 2 outcomes randomY should be FALSE")
    
    ##liste des variables utilisees  (sans les interactions et sans les Y)
    ttesLesVar <- colnames(get_all_vars(afixed,data=data[1,]))
    ttesLesVar <- c(ttesLesVar, colnames(get_all_vars(arandom,data=data[1,])))
    if (ncor>0) ttesLesVar <- unique(c(ttesLesVar,cor.var.time))
    else ttesLesVar <- unique(ttesLesVar)
    ttesLesVar <- setdiff(ttesLesVar, nomsY)
    
    ## argument subset
    form1 <- paste(c(nom.subject,nomsY,ttesLesVar),collapse="+")
    if(!isTRUE(all.equal(as.character(cl$subset),character(0))))
    {
        cc <- cl
        cc <- cc[c(1,which(names(cl)=="subset"))]
        cc[[1]] <- as.name("model.frame")
        cc$formula <- formula(paste("~",form1))
        cc$data <- data
        cc$na.action <- na.pass
        data <- eval(cc)
    }
    
    attributes(data)$terms <- NULL
    
    ## si subject est un factor
    if(is.factor(data[,nom.subject]))
    {
        data[,nom.subject] <- as.numeric(data[,nom.subject])
    }
    
    ## partie survie
    if(is.null(survival))
    {
        nbevt <- 0
        idtrunc <- 0
        nprisq <- 0
        nrisqtot <- 0
        nvarxevt <- 0
        nvarxevt2 <- 0
        typrisq <- 0
        noms.surv <- NULL
        nom.timedepvar <- NULL
        form.commun <- ~-1
        form.cause <- ~-1
        survival <- ~-1
        nz <- 0
        zi <- 0
        minT <- 0
        maxT <- 0
    }
    else
    {
        ## objet Surv
        surv <- cl$survival[[2]]
        
        if(length(surv)==3) #censure droite sans troncature gauche
        {
            idtrunc <- 0 
            
            Tevent <- getElement(object=data,name=as.character(surv[2]))
            Event <- getElement(object=data,name=as.character(surv[3]))  
            Tentry <- rep(0,length(Tevent)) #si pas de troncature, Tentry=0
            
            noms.surv <-  c(as.character(surv[2]),as.character(surv[3]))
            
            surv <- do.call("Surv",list(time=Tevent,event=factor(Event)))  
        }
        
        if(length(surv)==4) #censure droite et troncature
        {
            idtrunc <- 1
            
            Tentry <- getElement(object=data,name=as.character(surv[2]))
            Tevent <- getElement(object=data,name=as.character(surv[3]))
            Event <- getElement(object=data,name=as.character(surv[4]))  
            
            noms.surv <-  c(as.character(surv[2]),as.character(surv[3]),as.character(surv[4]))   
            
            surv <- do.call("Surv",list(time=Tentry,time2=Tevent,event=factor(Event)))   
        }  
        
        ## nombre d'evenement concurrents
        nbevt <- length(setdiff(unique(surv[, ncol(surv)]), c(NA, 0)))
        if(nbevt < 1)
        {
            warning("No observed event in the data")
            nbevt <- 1
        }
        
        
        ## pour la formule pour survivial, creer 3 formules : 
        ## une pour les covariables en mixture, une pour les covariables avec effet specifique a la cause, et une pour les effets communs.  
        form.surv <- cl$survival[3]
        
        noms.form.surv <- all.vars(attr(terms(formula(paste("~",form.surv))),"variables"))
        if(length(noms.form.surv)==0)
        {
            form.cause <- ~-1
            form.commun <- ~-1
            asurv <- terms(~-1)
        }
        else
        {
            ##creer la formula pour cause
            form1 <- gsub("mixture","",form.surv)       #TS: pas de classes latentes
            form1 <- formula(paste("~",form1))
            asurv1 <- terms(form1,specials="cause")  
            ind.cause <- attr(asurv1,"specials")$cause
            if(length(ind.cause))
            {
                form.cause <- paste(labels(asurv1)[ind.cause],collapse="+")
                form.cause <- gsub("cause","",form.cause)
                form.cause <- formula(paste("~",form.cause))
            }
            else
            {
                form.cause <- ~-1 
            }
            
            
            ## creer la formule pour ni cause ni mixture
            asurv <- terms(formula(paste("~",form.surv)),specials=c("cause"))
            ind.commun <- setdiff(1:length(labels(asurv)),unlist(attr(asurv,"specials")))
            if(length(ind.commun))
            {
                form.commun <- paste(labels(asurv)[ind.commun],collapse="+")
                form.commun <- gsub("cause","",form.commun)   # si X1:cause(X2)
                form.commun <- formula(paste("~",form.commun))  
            }
            else
            {
                form.commun <- ~-1 
            }
        }
    }
    
    nom.timedepvar <- NULL
    if(!missing(TimeDepVar))
    {
        if(!is.null(TimeDepVar))
        {  
            nom.timedepvar <- as.character(TimeDepVar)
            if(!isTRUE(nom.timedepvar %in% colnames(data))) stop(paste("Data should contain variable",nom.timedepvar))
        }
    }
    
    
    ##verifier si toutes les variables sont dans data
    varSurv <- unique(all.vars(terms(survival)))
    if(!is.null(nom.timedepvar)){if(!(nom.timedepvar %in% all.vars(terms(survival)))) stop("Variable in 'TimeDepVar' should also appear as a covariate in the 'survival' argument")}  
    if(!all(varSurv %in% colnames(data))) stop(paste("Data should contain the variables",paste(varSurv,collapse=" ")))
    varSurv <- setdiff(varSurv, noms.surv)
    ttesLesVar <- unique(c(ttesLesVar,varSurv))
    
    
    ##subset de data avec les variables utilisees
    newdata <- data[,c(nom.subject,nomsY,noms.surv,ttesLesVar),drop=FALSE]
    
    
    ## remplacer les NA de TimeDepVar par Tevent
    if((nbevt>0)){
      Tint <- Tevent
      nvdepsurv <- 0  
      if(!is.null(nom.timedepvar))
      {
        Tint <- newdata[,nom.timedepvar]
        Tint[(is.na(Tint))] <- Tevent[(is.na(Tint))]
        Tint[Tint>Tevent] <- Tevent[Tint>Tevent]
        Tint[Tint<Tentry] <- Tentry[Tint<Tentry]
        nvdepsurv <- 1
        if (length(Tint[Tint<Tevent])==0)
        {
          stop("TimeDepVar is always greater than Time of Event. \n")  
          nvdepsurv <- 0
        }
        if (length(Tint[Tint>Tentry])==0)
        {
          Tint <- Tevent
          stop("TimeDepVar is always lower than Time of Entry (0 by default). \n")
          nvdepsurv  <- 0
        }
        
        newdata[,nom.timedepvar] <- Tint 
      }
    }
    
    
    dataSurv <- NULL
    if((nbevt>0))
    {
        dataSurv <- data.frame(getElement(object=data,name=nom.subject),Tentry,Tevent,Event,Tint)
    }
    
    
    ##un data frame par outcome et creation Y0
    dataY <- paste("data",nomsY,sep=".")
    Y0 <- NULL
    IND <- NULL
    outcome <- NULL
    data0 <- NULL
    nayk <- vector("list",ny)
    for (k in 1:ny)
    {
        dtemp <- newdata[,c(nom.subject,nomsY[k],noms.surv,ttesLesVar)]
        ##enlever les NA
        linesNA <- apply(dtemp,2,function(v) which(is.na(v)))
        linesNA <- unique(unlist(linesNA))
        if(length(linesNA)) nayk[[k]] <- linesNA
        if(na.action==1 & length(linesNA)>0) dtemp <- dtemp[-linesNA,]
        if(na.action==2 & length(linesNA)>0) stop("Data contains missing values")
        assign(dataY[k],dtemp)
        Y0 <- c(Y0, dtemp[,nomsY[k]])
        IND <- c(IND, dtemp[,nom.subject])
        outcome <- c(outcome,rep(nomsY[k],nrow(dtemp)))
        data0 <- rbind(data0, dtemp[,setdiff(colnames(dtemp),nomsY[k]),drop=FALSE])   #dataset sans NA avec les covariables utilisees; obs ordonnees par outcome
    }

    rangeTlong <- NULL
    try(rangeTlong <- range(data0[, var.time]), silent = TRUE)

    ##creation de X0 (ttes les var + interactions)
    Xfixed <- model.matrix(fixed2[-2], data=data0)
    Xrandom <- model.matrix(random, data=data0)
    Xcontr <- model.matrix(contr,data=data0)
    Xsurv <- model.matrix(form.commun,data=data0)
    Xsurvcause <- model.matrix(form.cause,data=data0)
    
    
    z.fixed <- strsplit(colnames(Xfixed),split=":",fixed=TRUE)
    z.fixed <- lapply(z.fixed,sort)
    
    z.random <- strsplit(colnames(Xrandom),split=":",fixed=TRUE)
    z.random <- lapply(z.random,sort)
    
    if(contr != ~-1)
    {
        z.contr <- strsplit(colnames(Xcontr),split=":",fixed=TRUE)
        z.contr <- lapply(z.contr,sort)
    }
    else
    {
        z.contr <- list()
    }
    
    if(form.commun != ~-1)
    {
        z.surv <- strsplit(colnames(Xsurv),split=":",fixed=TRUE)
        z.surv <- lapply(z.surv,sort)
    }
    else
    {
        z.surv <- list() 
    }
    
    if(form.cause != ~-1)
    {
        z.survcause <- strsplit(colnames(Xsurvcause),split=":",fixed=TRUE)
        z.survcause <- lapply(z.survcause,sort)
    }
    else
    {
        z.survcause <- list() 
    }
    
    
    if(!all(z.contr %in% z.fixed))  stop("The covariates in contrast should also appear in fixed")
    
    X0 <- cbind(Xfixed, Xrandom, Xsurv, Xsurvcause)
    
    nom.unique <- unique(colnames(X0))
    X0 <- X0[,nom.unique,drop=FALSE]
    
    form.cor <- ~-1
    if (ncor>0)
    {
        if(!(cor.var.time %in% colnames(X0)))
        {
            X0 <- cbind(X0, data0[,cor.var.time])
            colnames(X0) <- c(nom.unique, cor.var.time)
            nom.unique <- c(nom.unique,cor.var.time)
            form.cor <- formula(paste("~-1+",cor.var.time))
        }
    }
    
    X0 <- as.matrix(X0)
    ##X0 fini
    
    formulas <- list(form.subject = formula(paste("~", nom.subject)),
                     fixed.left = fixed2[-3],
                     fixed.right = fixed2[-2],
                     random = random,
                     contrast = contr,
                     surv.left = formula(paste("~", paste(noms.surv, collapse = "+"))),
                     surv.commun = form.commun,
                     surv.cause = form.cause,
                     form.cor = form.cor)
    z <- removeNA(formulas, data = data)
    term <- z$terms
    names(term) <- names(formulas)
    
    ## term <- list(form.subject = formula(paste("~", nom.subject)),
    ##              fixed.left = terms(model.frame(terms(fixed2[-3]), data = data0)),
    ##              fixed.right = terms(model.frame(terms(fixed2[-2]), data = data0)),
    ##              random = terms(model.frame(terms(random), data = data0)),
    ##              contrast = terms(model.frame(terms(contr), data = data0)),
    ##              surv.left = formula(paste("~", paste(noms.surv, collapse = "+"))),
    ##              surv.commun = terms(model.frame(terms(form.commun), data = data0)),
    ##              surv.cause = terms(model.frame(terms(form.cause), data = data0)),
    ##              form.cor = form.cor)
    
    ##test de link
    if (length(link)!=1 & length(link)!=ny) stop("One link per outcome should be specified")
    if(any(link %in% c("splines","Splines")))
    {
        link[which(link %in% c("splines","Splines"))] <- "5-quant-splines"
    }
    if(length(link)==1 & ny>1)
    {
        link <- rep(link, ny)
    }
    
    idlink <- rep(2,ny)
    idlink[which(link=="linear")] <- 0
    idlink[which(link=="beta")] <- 1
    idlink[which(link=="thresholds")] <- 3
    
    spl <- strsplit(link[which(idlink==2)],"-")
    if(any(sapply(spl,length)!=3)) stop("Invalid argument 'link'")
    
    nySPL <- length(spl)
    nybeta <- sum(idlink==1)
    nyORD <- sum(idlink==3)
    ##remplir range si pas specifie
    if(!is.null(range) & length(range)!=2*(nySPL+nybeta)) stop("Length of vector range is not correct.")
    if((length(range)==2*(nySPL+nybeta)) & (nySPL+nybeta>0))
    {
        ind12 <- which(idlink==1 | idlink==2)
        for (i in 1:(nySPL+nybeta))
        {
            rg <- range(get(dataY[ind12[i]])[,nomsY[ind12[i]]])
            if(rg[1]<range[2*(i-1)+1] | rg[2]>range[2*(i-1)+2]) stop("The range specified do not cover the entire range of the data")
        }
    }
    if((is.null(range) & (nybeta+nySPL)>0) | length(range)!=2*(nySPL+nybeta))
    {
        range <- NULL
        for(k in which(idlink!=0))
        {
            min1 <- min(get(dataY[k])[,nomsY[k]])
            min2 <- round(min1,3)
            if(min1<min2) min2 <- min2-0.001
            
            max1 <- max(get(dataY[k])[,nomsY[k]])
            max2 <- round(max1,3)
            if(max1>max2) max2 <- max2+0.001
            
            range <- c(range, min2, max2)
        }
    }
    
    
    ## epsY
    if (any(idlink==1))
    {
        if (any(epsY<=0))
        {
            stop("Argument 'epsY' should be positive.")
        }
        
        if(length(epsY)==1) epsY <- rep(epsY,nybeta)
        
        if(length(epsY)!=nybeta) stop(paste("Argument 'epsY' should be of length",nybeta))
        if(nybeta!=ny)
        {
            epsY2 <- rep(0,ny)
            epsY2[which(idlink==1)] <- epsY
            epsY <- epsY2
        }
        
    }
    
    
    nbzitr <- rep(2,ny) #nbzitr = nb de noeuds si splines, 2 sinon
    nbnodes <- NULL  #que pour les splines
    spltype <- NULL
    if(nySPL>0)
    {
        for (i in 1:nySPL)
        {
            nbnodes <- c(nbnodes, spl[[i]][1])
            spltype <- c(spltype, spl[[i]][2])
            if(spl[[i]][3] != "splines") stop("Invalid argument link")
        }
    }
    nbnodes <- as.numeric(nbnodes)
    nbzitr[which(idlink==2)] <- nbnodes
    
    ##test splines
    if(!(all(spltype %in% c("equi","quant","manual")))) stop("The location of the nodes should be 'equi', 'quant' or 'manual'")
    
    ##tester longueur de intnodes
    if(!is.null(intnodes))
    {
        if(length(intnodes) != sum(nbnodes[which(spltype=="manual")]-2)) stop(paste("Vector intnodes should be of length",sum(nbnodes[which(spltype=="manual")]-2)))
    }
    
    ##intnodes2 : contient tous les noeuds interieurs (pas seulement ceux de manual)
    intnodes2 <- rep(NA,sum(nbnodes-2))
    nb <- 0
    nbspl <- 0
    for (k in 1:ny)
    {
        if (idlink[k]!=2) next
        else
        {
            nbspl <- nbspl+1
            
            if(spltype[nbspl]=="manual")
            {
                nodes <- intnodes[(nb+1):(nb+nbnodes[nbspl]-2)]
                if(!length(nodes)) stop("The length of intnodes is not correct")
                intnodes2[(sum(nbnodes[1:nbspl]-2)-(nbnodes[nbspl]-2)+1):sum(nbnodes[1:nbspl]-2)] <-  nodes
                nb <- nb+nbnodes[nbspl]-2
                
                idrg <- length(which(idlink[1:k] != 0))
                if(any(nodes <= range[2*(idrg-1)+1]) | any(nodes >= range[2*idrg])) stop("Interior nodes must be in the range of the outcome")
            }
            
            if(spltype[nbspl]=="equi")
            {
                nodes <- seq(range[2*(nbspl-1)+1], range[2*nbspl], length.out=nbnodes[nbspl])
                nodes <- nodes[-nbnodes[nbspl]]
                nodes <- nodes[-1]
                intnodes2[(sum(nbnodes[1:nbspl]-2)-(nbnodes[nbspl]-2)+1):sum(nbnodes[1:nbspl]-2)] <- nodes
            }
            
            if(spltype[nbspl]=="quant")
            {
                nodes <- quantile(get(dataY[k])[,nomsY[k]], probs=seq(0,1,length.out=nbnodes[nbspl]))
                if(length(unique(nodes)) != length(nodes)) stop(paste("Some nodes are equal for link number",k,"; Please try to reduce the number of nodes or use manual location."))
                nodes <- nodes[-nbnodes[nbspl]]
                nodes <- nodes[-1]
                intnodes2[(sum(nbnodes[1:nbspl]-2)-(nbnodes[nbspl]-2)+1):sum(nbnodes[1:nbspl]-2)] <- as.vector(nodes)
            }
        }
    }
    
    if(nb != length(intnodes)) stop(paste("The vector intnodes should be of length",nb))
    
    ##remplir zitr
    m <- 0
    if(nySPL>0) m <- max(nbnodes)
    zitr <- matrix(0,max(m,2),ny)
    nb12 <- 0
    nbspl <- 0
    for (k in 1:ny)
    {
        if((idlink[k]==0) | (idlink[k]==3)) zitr[1:2,k] <- c(min(get(dataY[k])[,nomsY[k]]),max(get(dataY[k])[,nomsY[k]]))
        
        if(idlink[k]==1)
        {
            nb12 <- nb12 + 1
            zitr[1:2,k] <- range[2*(nb12-1)+1:2]
        }
        
        if(idlink[k]==2)
        {
            nb12 <- nb12+1
            nbspl <- nbspl+1
            zitr[2:(nbzitr[k]-1),k] <- intnodes2[ifelse(nbspl==1,0,1)*sum(nbnodes[1:(nbspl-1)]-2) + 1:(nbnodes[nbspl]-2)]
            zitr[1,k] <- range[2*(nb12-1)+1]
            zitr[nbnodes[nbspl],k]  <- range[2*nb12]
            
            ##verifier s'il y a des obs entre les noeuds
            hcounts <- hist(get(dataY[k])[,nomsY[k]],breaks=zitr[1:nbnodes[nbspl],k],plot=FALSE,include.lowest=TRUE,right=TRUE)$counts
            if(any(hcounts==0)) stop(paste("Link function number",k,"can not be estimated. Please try other nodes such that there are observations in each interval."))
        }
    }
    
    ##uniqueY0 et indiceY0
    uniqueY0 <- NULL
    indiceY0 <- NULL
    nvalSPLORD <- rep(0,ny)
    nbmod <- rep(0,ny)
    modalites <- vector("list",ny)
    nb <- 0
    for (k in 1:ny)
    {
        if((idlink[k]!=2) & (idlink[k]!=3))
        {
            indiceY0 <- c(indiceY0, rep(0,length(get(dataY[k])[,nomsY[k]])))
            next
        }
        
        yk <- get(dataY[k])[,nomsY[k]]
        uniqueTemp <- sort(unique(yk))
        permut <- order(order(yk))  # sort(y)[order(order(y))] = y
        if(length(as.vector(table(yk)))==length(uniqueTemp))
        {
            indice <- rep(1:length(uniqueTemp), as.vector(table(yk)))
            if(idlink[k]==2)
            {
                indiceTemp <- nb + indice[permut]
            }
            else
            {
                indiceTemp <- indice[permut]
            }
            
            nb <- nb + length(uniqueTemp)
            
            uniqueY0 <- c(uniqueY0, uniqueTemp)
            indiceY0 <- c(indiceY0, indiceTemp)
            nvalSPLORD[k] <- length(uniqueTemp)
        }
        else
        {
            uniqueY0 <- c(uniqueY0, yk)
            indiceY0 <- c(indiceY0, ifelse(idlink[k]==2,nb,0)+c(1:length(yk)))
            nb <- nb + length(yk)
            nvalSPLORD[k] <- length(yk)
        }
        if(idlink[k]==3)
        {
            nbmod[k] <- length(na.omit(uniqueTemp))
            modalites[[k]] <- uniqueTemp
        }
    }
    indiceYinf0 <- indiceY0 - 1
    
    ##ordonner les mesures par individu
    matYX <- cbind(IND,Y0,indiceY0,indiceYinf0,outcome,X0)
    matYXord <- matYX[order(IND),]
    Y0 <- as.numeric(matYXord[,2])
    X0 <- apply(matYXord[,-c(1:5),drop=FALSE],2,as.numeric)
    IND <- matYXord[,1]
    outcome <- matYXord[,5]
    indiceY0 <- as.numeric(matYXord[,3])
    indiceYinf0 <- as.numeric(matYXord[,4])
    
    if((nbevt>0)){
      
      dataSurv <- dataSurv[which(dataSurv[,1] %in% IND),]
      dataSurv <- dataSurv[order(dataSurv[,1]),]
      nmes <- as.vector(table(dataSurv[,1]))
      data.surv <- apply(dataSurv[cumsum(nmes),-1],2,as.numeric)
      tsurv0 <- data.surv[,1]
      tsurv <- data.surv[,2]
      devt <- data.surv[,3]
      tsurvint <- data.surv[,4]
      ind_survint <- (tsurvint<tsurv) + 0

        ## confirm nbevt for the selected sample (IND)
        nbevt <- max(1, length(setdiff(unique(devt), 0)))
      
      ## test de hazard
      arghaz <- hazard
      hazard <- rep(hazard,length.out=nbevt)
      if(any(hazard %in% c("splines","Splines")))
      {
        hazard[which(hazard %in% c("splines","Splines"))] <- "5-quant-splines"
      }
      if(any(hazard %in% c("piecewise","Piecewise")))
      {
        hazard[which(hazard %in% c("piecewise","Piecewise"))] <- "5-quant-piecewise"
      }
      
      haz13 <- strsplit(hazard[which(!(hazard=="Weibull"))],"-")
      if(any(sapply(haz13,length)!=3)) stop("Invalid argument hazard")
      
      nz <- rep(2,nbevt)
      locnodes <- NULL
      typrisq <- rep(2,nbevt)
      nprisq <- rep(2,nbevt)
      
      nznodes <- 0 #longueur de hazardnodes
      ii <- 0
      if(any(hazard!="Weibull"))
      {
        
        for (i in 1:nbevt)
        {
          if(hazard[i]=="Weibull") next;
          
          ii <- ii+1
          
          nz[i] <- as.numeric(haz13[[ii]][1])
          if(nz[i]<3) stop("At least 3 nodes are required")
          typrisq[i] <- ifelse(haz13[[ii]][3] %in% c("splines","Splines"),3,1)
          nprisq[i] <- ifelse(haz13[[ii]][3] %in% c("splines","Splines"),nz[i]+2,nz[i]-1)
          locnodes <- c(locnodes, haz13[[ii]][2])
          if(!(haz13[[ii]][3] %in% c("splines","Splines","piecewise","Piecewise"))) stop("Invalid argument hazard")
          
          if((haz13[[ii]][2]=="manual"))
          {
            nznodes <- nznodes + nz[i]-2
          }
          
          if(!all(locnodes %in% c("equi","quant","manual"))) stop("The location of the nodes should be 'equi', 'quant' or 'manual'")
        }
        
        if(!is.null(hazardnodes))
        {
          if(!any(locnodes == "manual"))  stop("hazardnodes should be NULL if the nodes are not chosen manually")
          
          if(length(hazardnodes) != nznodes) stop(paste("Vector hazardnodes should be of length",nznodes))
        }
      }
      else
      {
        if(!is.null(hazardnodes)) stop("hazardnodes should be NULL if Weibull baseline risk functions are chosen")
      }
      
      
      if(nbevt>1 & length(arghaz)==1 & nznodes>0)
      {
        hazardnodes <- rep(hazardnodes,length.out=nznodes*nbevt)
      }
      
      nrisqtot <- sum(nprisq)
      
      zi <- matrix(0,nrow=max(nz),ncol=nbevt)
      nb <- 0
      
      minT1 <- 0
      maxT1 <- max(tsurv)
      tsurvevt <- tsurv
      
      if(idtrunc==1)
      {
        minT1 <- min(tsurv,tsurv0)
        maxT1 <- max(tsurv,tsurv0)
    }
    
    ## arrondir
    minT2 <- round(minT1,3)
    if(minT1<minT2) minT2 <- minT2-0.001
    minT <- minT2
    
    maxT2 <- round(maxT1,3)
    if(maxT1>maxT2) maxT2 <- maxT2+0.001
    maxT <- maxT2
    
    if(length(hazardrange)){
      if(hazardrange[1]>minT) stop(paste("hazardrange[1] should be <=",minT))
      if(hazardrange[2]<maxT) stop(paste("hazardrange[2] should be >=",maxT))
      minT <- hazardrange[1]
      maxT <- hazardrange[2]
    }
    
    startWeib <- rep(0,nbevt)
    startWeib[which(typrisq==2)] <- rep(startWeibull, length.out=length(which(typrisq==2)))
    ii <- 0
    for(i in 1:nbevt)
    {
        if(typrisq[i]==2)
        {
          if(minT < startWeib[i]) stop("Some entry or event times are bellow startWeibull")
          zi[1:2,i] <- c(startWeib[i],maxT)
        }
        else
        {
          ii <- ii+1
          
          if(locnodes[ii]=="manual")
          {
            zi[1:nz[i],i] <- c(minT,hazardnodes[nb+1:(nz[i]-2)],maxT)
            nb <- nb + nz[i]-2
          }
          if(locnodes[ii]=="equi")
          {
            zi[1:nz[i],i] <- seq(minT,maxT,length.out=nz[i])
          }
          if(locnodes[ii]=="quant")
          {
            pi <- c(1:(nz[i]-2))/(nz[i]-1)
            qi <- quantile(tsurvevt,prob=pi)
            zi[1,i] <- minT
            zi[2:(nz[i]-1),i] <- qi
            zi[nz[i],i] <- maxT
          }
        }
      }

        if((length(unique(zi[1,])) > 1) & any(sharedtype %in% shtypes[-c(1:3)])) stop("All the hazards don't start from the same value. Check hazardrange and startWeibull.")
      
        ## non linear associations
        nonlin <- rep(0, 3 * nbevt)
        nonlin[1:nbevt] <- sapply(1:nbevt, function(ke) { as.numeric(length(grep("REpoly", sharedtype[ke]))) }) + sapply(1:nbevt, function(ke) { 2 * as.numeric(length(grep("RElogit", sharedtype[ke]))) }) #1 pour REpoly, 2 pour RElogit
        nonlin[nbevt + 1:nbevt] <- sapply(1:nbevt, function(ke) { as.numeric(length(grep("CLpoly", sharedtype[ke]))) }) + sapply(1:nbevt, function(ke) { 2 * as.numeric(length(grep("CLlogit", sharedtype[ke]))) }) # 1 pour CLpoly, 2 pour CLlogit
        nonlin[2 * nbevt + 1:nbevt] <- sapply(1:nbevt, function(ke) { as.numeric(length(grep("CSpoly", sharedtype[ke]))) }) + sapply(1:nbevt, function(ke) { 2 * as.numeric(length(grep("CSlogit", sharedtype[ke]))) }) # 1 pour CSpoly, 2 pour CSlogit

      ###TS: cas ou gi(bi,t)=niv.courant -> construction des matrices design pr predire lambda a chq pnt de quadrature GK
      if(all(sharedtype %in% shtypes[1:3])){  #gi(bi,t)=bi
        Xpred <- 0
        Xpred_Ti <- 0
        nbXpred <- 0
        Xpred_cs <- 0
        Xpredcs_Ti <- 0
        centerpoly0 <- 0
      }
      if(any(sharedtype %in% shtypes[-c(1:3)])){   #gi(bi,t)=niv.courant(t) ou pente
        
        # /!\ si varexp dependante du tps (autre que var.time), prediction impossible
        nom.var <- c(attr(terms(fixed2[-2]), "term.labels"),attr(terms(random), "term.labels"))  # noms des covariables EF et EA
        nom.var <- nom.var[!str_detect(nom.var,var.time)] # nom.var[!(nom.var == var.time)] # noms des variables, autre que celle incluant var.time
        if(length(nom.var)>0){
          for(v in 1:length(nom.var)){ #verif: covariables (hors var.time) independantes du temps
            tmp <- unique(na.omit(data[which(data[, nom.subject] %in% IND), c(nom.subject,nom.var[v])]))  #dataframe 2 colonnes : subject et var v, en supprimant les lignes doublons
            if(nrow(tmp) != length(unique(IND))) #var v dependante du temps
              stop(paste(nom.var[v]," variable seems to be time dependant, can't use sharedtype 'CL' or 'CS' due to impossibility to predict"))
          }  
        }
        
        ## matrices design
        if(FALSE){
        # database contenant les variables des EFs et des EAs, 1 ligne par sujet
        data_tmp <- as.data.frame(unique(na.omit(data[,c(subject,nom.var)])))
        colnames(data_tmp) <- c(subject,nom.var)
        data_tmp$tmp_name <- NA
        colnames(data_tmp)[which(colnames(data_tmp) == "tmp_name")] <- var.time
        if(nrow(data_tmp)!=length(unique(IND))) stop("Make sure at least 1 visit per subject has complete data")  # si un sujet a un valeur NA a 1 covar a chq visit, alors il n'est plus pris en compte
        if(nrow(data_tmp)!=length(tsurv)) stop("pblm")
        
        # Xpredcl_vectTi (Ti event) vecteur par patient
        data_tmp[,var.time] <- tsurv # remplacer la variable de temps par T
        #matrices design
        mat_ef <- model.matrix(object = fixed2[-2], data = data_tmp) # EF sans contraste et sans outcome, 
        mat_ef <- as.data.frame(mat_ef[,-1])  # et sans intercept car non-estime
        mat_ea <- model.matrix(object = random, data = data_tmp) # EA
        # concatenation
        Xpredcl_vectTi <- cbind(data_tmp[,var.time],mat_ef,mat_ea)  # 1ere colonne = tps de quadrature
        #nb_ind_Xpred <- c(ncol(mat_ef),ncol(mat_ea))  #nb de var des EF, EA
        Xpredcl_vectTi <- as.matrix(Xpredcl_vectTi)
        Xpred_Ti <- Xpredcl_vectTi
        nbXpred <- ncol(Xpred_Ti)
        
        # noeuds de quadrature Gauss-Kronrod
        ptGK_1 <- 0.991455371120812639206854697526329
        ptGK_2 <- 0.949107912342758524526189684047851
        ptGK_3 <- 0.864864423359769072789712788640926
        ptGK_4 <- 0.741531185599394439863864773280788
        ptGK_5 <- 0.586087235467691130294144838258730
        ptGK_6 <- 0.405845151377397166906606412076961
        ptGK_7 <- 0.207784955007898467600689403773245
        ptGK_8 <- 0.000000000000000000000000000000000
        
        # Xpredcl (Ti event)
        data_tmp[,var.time] <- tsurv # remplacer la variable de temps par T
        data_tmp[,var.time] <- (data_tmp[,var.time] - 0) / 2  #centr = centre de l intervalle 0 -> Ti
        data_tmp$mid_interv_surv <- data_tmp[,var.time]  #hlgth = demi longueur de l intervalle 0 -> Ti
        data_tmp <- data_tmp[rep(1:nrow(data_tmp),each=15),]  #1ligne/point quadrature/patient
        data_tmp$ptGK <- c(ptGK_1,-ptGK_1,ptGK_2,-ptGK_2,ptGK_3,-ptGK_3,ptGK_4,-ptGK_4,ptGK_5,-ptGK_5,ptGK_6,-ptGK_6,ptGK_7,-ptGK_7,ptGK_8) #pnts de quadrature GK
        data_tmp[,var.time] <- data_tmp[,var.time] + data_tmp$mid_interv_surv * data_tmp$ptGK  #centr+absc ou absc=hlgth*pnt
        #matrices design
        mat_ef <- model.matrix(object = fixed2[-2], data = data_tmp) # EF sans contraste et sans outcome,
        mat_ef <- as.data.frame(mat_ef[,-1])  # et sans intercept car non-estime
        mat_ea <- model.matrix(object = random, data = data_tmp) # EA
        # concatenation
        Xpredcl <- cbind(data_tmp[,var.time],mat_ef,mat_ea)  # 1ere colonne = tps de quadrature
        Xpredcl <- as.matrix(Xpredcl)
        Xpred <- Xpredcl
        
        # Xpredcl0 (Ti0, tronc gche)
        if(idtrunc==1){
          data_tmp[,var.time] <- rep(tsurv0,each=15) # remplacer la variable de temps par T0
          data_tmp[,var.time] <- (data_tmp[,var.time] - 0) / 2  #centr = centre de l intervalle 0 -> T0i
          data_tmp$mid_interv_surv <- data_tmp[,var.time]  #hlgth = demi longueur de l intervalle 0 -> T0i
          data_tmp[,var.time] <- data_tmp[,var.time] + data_tmp$mid_interv_surv * data_tmp$ptGK  #centr+absc ou absc=hlgth*pnt
          #matrices design
          mat_ef <- model.matrix(object = fixed2[-2], data = data_tmp) # EF sans contraste et sans outcome,
          mat_ef <- as.data.frame(mat_ef[,-1])  # et sans intercept car non-estime
          mat_ea <- model.matrix(object = random, data = data_tmp) # EA
          # concatenation
          Xpredcl0 <- cbind(data_tmp[,var.time],mat_ef,mat_ea)
          Xpredcl0 <- as.matrix(Xpredcl0)
          Xpred <- cbind(Xpred,Xpredcl0)
        }else{
          Xpredcl0 <- NULL
        }
        
          nbXpred <- c(nbXpred, ncol(Xpred))
        }

          ## Viviane : remplace ce qui precede
          data_tmp <- as.data.frame(unique(na.omit(data[which(data[, nom.subject] %in% IND), c(nom.subject, nom.var)])))
          colnames(data_tmp) <- c(nom.subject,nom.var)
          data_tmp <- data_tmp[order(data_tmp[, nom.subject]), , drop = FALSE]

          deriv <- ifelse(length(grep("CS", sharedtype)) > 0, TRUE, FALSE)
          X_GK <- matrixGK(data_tmp, fixed2[-2], random, var.time, idtrunc, tsurv0, tsurv, deriv, zi[1,1])
                                        # a controler : tsurv0=NULL si pas idtrunc ??
          Xpred_Ti <- X_GK$Xpred_Ti     # X(t) pour calculer lambda(Ti)
          Xpred <- X_GK$Xpred_cl        # X(t) pour calculer S(Ti)
          Xpredcs_Ti <- X_GK$Xpredcs_Ti # dX(t)/dt pour calculer lambda(Ti)
          Xpred_cs <- X_GK$Xpred_cs     # dX(t)/dt pour calculer S(Ti)
          nbXpred <- c(ncol(Xpred_Ti), ncol(Xpred))


          ## centerpoly
          centerpoly0 <- c(0, 0) # centrage pour CLpoly et CSpoly
          l <- 0 # expected length of argument centerpoly
          if(any(nonlin[nbevt + 1:nbevt] == 1)) l <- l + 1
          if(any(nonlin[2 * nbevt + 1:nbevt] == 1)) l <- l + 1
          if(l > 0) if(length(centerpoly) != l) stop(paste("centerpoly should be of length", l, "\n"))
          if(length(centerpoly) == 1)
          {
              if(any(nonlin[nbevt + 1:nbevt] == 1)) centerpoly0[1] <- centerpoly
              if(any(nonlin[2 * nbevt + 1:nbevt] == 1)) centerpoly0[2] <- centerpoly
          }
          else
          {
              centerpoly0 <- centerpoly
          }
      }
      
    }
    else{
      tsurv0 <- rep(0,length(unique(IND)))
      tsurv <- rep(0,length(unique(IND))) 
      devt <- rep(0,length(unique(IND)))
      tsurvint <- rep(0,length(unique(IND)))
      ind_survint <- rep(0,length(unique(IND)))
      #TS
      Xpred <- rep(0,length(unique(IND))*15)
      Xpred_Ti <- rep(0,length(unique(IND)))
      Xpred_cs <- rep(0,length(unique(IND))*15)
      Xpredcs_Ti <- rep(0,length(unique(IND))*15)
      nbXpred <- c(1,1)
      centerpoly0 <- c(0, 0)
      nonlin <- 0
    }
            
    
    ##parametres pour Fortran
    ns <- length(unique(IND))
    nv <- dim(X0)[2]
    nobs <- length(Y0)
    idiag0 <- ifelse(idiag==TRUE,1,0)
    nalea <- ifelse(randomY==TRUE,ny,0)
    logspecif <- as.numeric(logscale)
    #loglik <- 0
    ni <- 0
    istop <- 0
    gconv <- rep(0,3)
    resid_m <- rep(0,nobs)
    resid_ss <- rep(0,nobs)
    Yobs <- rep(0,nobs)
    time <- seq(minT,maxT,length.out=nsim)
    risq_est <- matrix(0,nrow=nsim,ncol=nbevt)
    risqcum_est <- matrix(0,nrow=nsim,ncol=nbevt)
    predRE_Y <- rep(0,ns*nalea)
    rlindiv <- rep(0,ns)
    marker <- rep(0,nsim*ny)
    transfY <- rep(0,nsim*ny)
    
    
    
    ##nmes
    nmes <- matrix(0,ns,ny)
    for (k in 1:ny)
    {
        INDpresents <- which(unique(IND) %in% get(dataY[k])[,nom.subject])
        nmes[INDpresents,k] <- as.vector(table(get(dataY[k])[,nom.subject]))
    }
    maxmes <- max(apply(nmes,1,sum))
    
    
    
    ##remplir idprob, etc
    z.X0 <- strsplit(nom.unique,split=":",fixed=TRUE)
    z.X0 <- lapply(z.X0,sort)
    
    idea <- (z.X0 %in% z.random) + 0
    idg <- (z.X0 %in% z.fixed) + 0
    idcontr <- (z.X0 %in% z.contr) + 0
    
    if (ncor>0) idcor <- colnames(X0) %in% cor.var.time +0
    else idcor <- rep(0,nv)
    
    idsurv <- z.X0 %in% z.surv + 2*(z.X0 %in% z.survcause)
    idsurv[1] <- 0 # 0 pour l'intercept
    
    idtdv <- z.X0 %in% nom.timedepvar + 0
    
    ## Si pas TimeDepVar dans formule survival
    if(length(nom.timedepvar) & all(idtdv==0))
    {
        stop("Variable in 'TimeDepVar' should also appear as a covariate in the 'survival' argument")
    }
    
    ## nb coef ppour survie
    nvarxevt <- sum(idsurv==1) + nbevt*sum(idsurv==2)
    
    nea <- sum(idea)
    predRE <- rep(0,ns*nea)
    
    ## nb parametres d'association  #TS
    if((length(sharedtype) == 1) & (nbevt > 1)) sharedtype <- rep(sharedtype, nbevt)
    if((length(sharedtype) > 1) & (length(sharedtype) != nbevt)) stop("sharedtype should be of length 1 or", nbevt, "(number of competing events) \n")

    nasso <- 0
    if((nbevt>0)){
      for(ke in 1:nbevt)
      {        
        if(sharedtype[ke] == "RE")
          nasso <- nasso + nea
        else if(sharedtype[ke] %in% c("REpoly", "RElogit"))
          nasso <- nasso + 3 * nea
        else if(sharedtype[ke] %in% c("CL", "CS"))
          nasso <- nasso + 1
        else if(sharedtype[ke] %in% c("CLpoly", "CLlogit", "CSpoly", "CSlogit"))
          nasso <- nasso + 3
        else if(sharedtype[ke] %in% c("CL+CS"))
          nasso <- nasso + 2
        else if(sharedtype[ke] %in% c("CLpoly+CS", "CLlogit+CS", "CL+CSpoly", "CL+CSlogit"))
          nasso <- nasso + 4
        else if(sharedtype[ke] %in% c("CLpoly+CSpoly", "CLlogit+CSpoly", "CLpoly+CSlogit", "CLlogit+CSlogit"))
          nasso <- nasso + 6
      }
    }
   
    
    ## prm partie long
    nef <- sum(idg==1)-1
    ncontr <- (ny-1)*sum(idcontr)
    nvc <- ifelse(idiag0==1,nea,nea*(nea+1)/2)-1
    ntr <- rep(0,ny)
    ntr[which(idlink==0)] <- 2
    ntr[which(idlink==1)] <- 4
    ntr[which(idlink==2)] <- nbzitr[which(idlink==2)]+2
    ntr[which(idlink==3)] <- nbmod[which(idlink==3)]-1
    ntrtot <- sum(ntr)
    
    ##nombre total de parametres
    NPM <- nrisqtot + nvarxevt + nasso +
        nef + ncontr + nvc + ncor + ntrtot + nalea + ny
    
    
    V <- rep(0, NPM*(NPM+1)/2)  #pr variance des parametres
    
    ## parametres MC
    methInteg <- switch(methInteg,"MCO"=1,"MCA"=2,"QMC"=3)
    seqMC <- 0
    dimMC <- 0
    if(methInteg==3)
    {
        dimMC <- nea+nalea
        if(ncor>0) dimMC <- dimMC+maxmes
        if((dimMC > 0) & (nMC > 0))
        {
            sequnif <- spacefillr::generate_sobol_owen_set(nMC, dimMC)
            seqMC <- apply(sequnif, 2, qnorm)
        }
    }
    
    

    ###valeurs initiales
    if(!(missing(B)))
    {
        if(!is.vector(B)) stop("B should be a vector")
        
        if (length(B)==NPM) b <- B
        else stop(paste("Vector B should be of length",NPM))
        
    }
    else ## B missing
    {
        b <- rep(0,NPM)
        
        if(nbevt>0){  #TS : si Weibull et prms au carre pr positivite -> valeurs par defaut = 1
            if(any(hazard!="Weibull")==FALSE & isFALSE(logscale)==TRUE){
                for(i in 1:nbevt){
                    if(typrisq[i]==2){
                        b[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- 1
                        if(idtrunc==1 & any(Tentry==0)) #TS : si entree retardee et au moins un temps vaut 0
                          b[sum(nprisq[1:i])-nprisq[i]+nprisq[i]] <- 1.25 #sinon pblm lors de recherche prms, risq instant tend vers infini qd w2 < 1 et t=0
                    }
                }
            }
            if(any(hazard %in% c("splines","Splines")) & idtrunc==1 & any(Tentry==0)){
                for(i in 1:nbevt){
                    if(typrisq[i]==3){
                        b[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- 10^-7 #sinon pgrm crash
                    }
                }
            }
        }
        
        if (nvc>0)
        {
            if(idiag==1) b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- rep(1,nvc)
            if(idiag==0)
            {
                init.nvc <- diag(nea)
                init.nvc <- init.nvc[upper.tri(init.nvc, diag=TRUE)]
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- init.nvc[-1]
            }
        }
        
        if(ncor>0) b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor] <- 1
        
        if(nalea>0) b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:nalea] <- 1
        
        b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny] <-  1
        
        for(k in 1:ny)
        {
            if(idlink[k]==0)
            {
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-1] <- mean(get(dataY[k])[,nomsY[k]])
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])] <- 1
            }
            if(idlink[k]==1)
            {
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-3] <- 0
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-2] <- -log(2)
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-1] <- 0.7
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])] <- 0.1
            }
            if(idlink[k]==2)
            {
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-ntr[k]+1] <- -2
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-ntr[k]+2:ntr[k]] <- 0.1
            }
            if(idlink[k]==3)
            {
                b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-ntr[k]+1] <- 0
                if(ntr[k]>1) b[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:k])-ntr[k]+2:ntr[k]] <- 0.1
            }
        }
    }
    
    
    
    
    ## ## faire wRandom et b0Random
    ## nef2 <- sum(idg!=0)-1 + (ny-1)*sum(idcontr)
    ## NPM2 <- nef2+ nvc+ncor+nalea+ny+ntrtot
    
    ## wRandom <- rep(0,NPM)
    ## b0Random <- rep(0,ng-1)
    
    ## l <- 0
    ## t <- 0
    ## m <- 0
    ## for (i in 1:nv)
    ## {
    ##     if(idg[i]==1)
    ##     {
    ##         if(i==1) next
    ##         l <- l+1
    ##         t <- t+1
    ##         wRandom[nprob+t] <- l
    ##     }
    ##     if(idg[i]==2)
    ##     {
    ##         if(i==1)
    ##         {
    ##             t <- t+ng-1
    ##             b0Random <- c(b0Random,rep(0,ng-1))
    ##             next
    ##         }
    ##         l <- l+1
    ##         for (g in 1:ng)
    ##         {
    ##             t <- t+1
    ##             wRandom[nprob+t] <- l
    ##         }
    ##     }
    ##     if(idcontr[i]==1)
    ##     {
    ##         wRandom[nef-ncontr+m+1:(ny-1)] <- nef2-ncontr+m+1:(ny-1)
    ##         m <- m+ny-1
    ##     }
    ## }
    
    ## if(nvc>0)
    ## {
    ##     wRandom[nef+1:nvc] <- nef2+1:nvc
    ## }
    ## if(nw>0)
    ## {
    ##     b0Random <- c(b0Random,rep(1,ng-1))
    ## }
    
    ## if(ncor>0) wRandom[nef+nvc+nw+1:ncor] <- nef2+nvc+1:ncor
    
    ## wRandom[nef+nvc+nw+ncor+1:ny] <- nef2+nvc+ncor+1:ny
    
    ## if(nalea>0)
    ## {
    ##     wRandom[nef+nvc+nw+ncor+ny+1:nalea] <- nef2+nvc+ncor+ny+1:nalea
    ## }
    
    ## wRandom[nef+nvc+nw+ncor+ny+nalea+1:ntrtot] <- nef2+nvc+ncor+ny+nalea+1:ntrtot
    ## ## wRandom et b0Random ok.
    
    
    ##------------------------------------------
    ##------nom au vecteur best
    ##--------------------------------------------
    
    nom.X0 <- colnames(X0)
    nom.X0[nom.X0=="(Intercept)"] <- "intercept"
    
    if(nbevt>0)
    {
        ##prm fct de risque
        if(isTRUE(logscale))
        {
            for(i in 1:nbevt)
            {
                nom1 <- rep(paste("event",i,sep=""),nprisq[i])
                if(typrisq[i]==2)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:2]," log(Weibull",1:2,")",sep="")
                }
                if(typrisq[i]==1)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:(nz[i]-1)]," log(piecewise",1:(nz[i]-1),")",sep="")
                }
                if(typrisq[i]==3)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:(nz[i]-1)]," log(splines",1:(nz[i]+2),")",sep="")
                }
            }
        }
        else
        {
            for(i in 1:nbevt)
            {
                nom1 <- rep(paste("event",i,sep=""),nprisq[i])
                if(typrisq[i]==2)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:2]," +/-sqrt(Weibull",1:2,")",sep="")
                }
                if(typrisq[i]==1)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:(nz[i]-1)]," +/-sqrt(piecewise",1:(nz[i]-1),")",sep="")
                }
                if(typrisq[i]==3)
                {
                    names(b)[sum(nprisq[1:i])-nprisq[i]+1:nprisq[i]] <- paste(nom1[1:(nz[i]-1)]," +/-sqrt(splines",1:(nz[i]+2),")",sep="")
                }
            }
        }
        
        
        ##prm covariables survival
        nom1 <- NULL
        for(j in 1:nv)
        {
            if(idsurv[j]==1) #X
            {
                if(idtdv[j]==1)
                {
                    nom1 <- c(nom1,paste("I(t>",nom.timedepvar,")",sep=""))
                }
                else
                {
                    nom1 <- c(nom1,nom.X0[j])
                }
            }
            
            if(idsurv[j]==2) #cause(X)
            {
                if(idtdv[j]==1)
                {
                    nom1 <- c(nom1,paste("I(t>",nom.timedepvar,") event",1:nbevt,sep=""))
                }
                else
                {
                    nom1 <- c(nom1,paste(nom.X0[j],paste("event",1:nbevt,sep="")))
                }
            }
            
        }
        
        if(nvarxevt>0) names(b)[nrisqtot+1:nvarxevt] <- nom1
        
        
    shtypes <- c("RE", "REpoly", "RElogit", "CL", "CLpoly", "CLlogit", "CS", "CSpoly", "CSlogit",
                 "CL+CS", "CLpoly+CS", "CLlogit+CS", "CL+CSpoly", "CL+CSlogit", "CLpoly+CSpoly", "CLpoly+CSlogit", "CLlogit+CSpoly", "CLlogit+CSlogit") # associations possibles


        namesh <- rep(NA, nasso)
        jj <- 0
        for(ke in 1:nbevt)
        {
            evt <- paste("event", ke, sep = "")
            if(sharedtype[ke] == "RE")
            {
                namesh[jj + 1:nea] <- paste(rep(evt, nea), paste("RE", 1:nea, sep = ""))
                jj <- jj + nea
            }
            if(sharedtype[ke] == "REpoly")
            {
                namesh[jj + 1:(3 * nea)] <- paste(evt, paste(rep(paste("RE", 1:nea, sep = ""), each = 3), "poly", 1:3, sep = ""))
                jj <- jj + 3 * nea
            }
            if(sharedtype[ke] == "RElogit")
            {
                namesh[jj + 1:(3 * nea)] <- paste(evt, paste(rep(paste("RE", 1:nea, sep = ""), each = 3), "logit", 1:3, sep = ""))
                jj <- jj + 3 * nea
            }
            if(length(grep("\\bCL\\b", sharedtype[ke])))
            {
                namesh[jj + 1] <- paste(evt, "CL")
                jj <- jj + 1
            }
            if(length(grep("\\bCLpoly\\b", sharedtype[ke])))
            {
                namesh[jj + 1:3] <- paste(evt, paste("CLpoly", 1:3, sep = ""))
                jj <- jj + 3
            }
            if(length(grep("\\bCLlogit\\b", sharedtype[ke])))
            {
                namesh[jj + 1:3] <- paste(evt, paste("CLlogit", 1:3, sep = ""))
                jj <- jj + 3
            }
            if(length(grep("\\bCS\\b", sharedtype[ke])))
            {
                namesh[jj + 1] <- paste(evt, "CS")
                jj <- jj + 1
            }
            if(length(grep("\\bCSpoly\\b", sharedtype[ke])))
            {
                namesh[jj + 1:3] <- paste(evt, paste("CSpoly", 1:3, sep = ""))
                jj <- jj + 3
            }
            if(length(grep("\\bCSlogit\\b", sharedtype[ke])))
            {
                namesh[jj + 1:3] <- paste(evt, paste("CSlogit", 1:3, sep = ""))
                jj <- jj + 3
            }
        }
        names(b)[nrisqtot+nvarxevt+1:nasso] <- namesh
        
        ## if(sharedtype == 'RE'){   #TS
        ##     for(i in 1:nbevt)
        ##     {
        ##         names(b)[nrisqtot+nvarxevt+(nbevt-1)*nea+1:nea] <- paste("event",i," asso",1:nea,sep="")
        ##     }
        ## }
        ## if(sharedtype == 'CL')
        ##     names(b)[nrisqtot+nvarxevt+1:nbevt] <- paste("event",1:nbevt," asso",sep="")
        
        
    }
    
    
    names(b)[nrisqtot+nvarxevt+nasso+1:nef] <- nom.X0[-1][idg[-1]!=0]
    if(ncontr!=0) names(b)[nrisqtot+nvarxevt+nasso+nef+1:ncontr] <- paste("contrast",paste(rep(1:sum(idcontr),each=ny-1),rep(1:(ny-1),sum(idcontr)),sep=""),sep="")
    
    if(idlink[1]==0) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntr[1]]<- c("Linear 1","Linear 2")
    if(idlink[1]==1) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntr[1]]<- paste("Beta",c(1:ntr[1]),sep="")
    if(idlink[1]==2) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntr[1]]<- paste("I-splines",c(1:ntr[1]),sep="")
    if(idlink[1]==3) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntr[1]]<- paste("Thresh",c(1:ntr[1]),sep="")
    if(ny>1)
    {
        for (yk in 2:ny)
        {
            if(idlink[yk]==0) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:(yk-1)])+1:ntr[yk]]<- c("Linear 1","Linear 2")
            if(idlink[yk]==1) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:(yk-1)])+1:ntr[yk]]<- paste("Beta",c(1:ntr[yk]),sep="")
            if(idlink[yk]==2) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:(yk-1)])+1:ntr[yk]]<- paste("I-splines",c(1:ntr[yk]),sep="")
            if(idlink[yk]==3) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:(yk-1)])+1:ntr[yk]]<- paste("Thresh",c(1:ntr[yk]),sep="")
        }
    }
    if(nvc!=0)names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- paste("varcov",c(1:nvc))
    
    if(ncor>0) {names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1:ncor] <- paste("cor",1:ncor,sep="")}
    if(nalea!=0) names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:nalea] <- paste("std.randomY",1:ny,sep="")
    
    names(b)[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny] <- paste("std.err",1:ny)
    namesb <- names(b)
    
    
    
    ## prm fixes
    fix <- rep(0,NPM)
    if(length(posfix))
    {
        if(any(!(posfix %in% 1:NPM))) stop("Indexes in posfix are not correct")
        
        fix[posfix] <- 1
    }
    if(length(posfix)==NPM) stop("No parameter to estimate")
    
    if(!all(partialH %in% 1:NPM)) stop(paste("partialH should contain indices between 1 and",NPM))
    
    # varcov RE
    if(nvc!=0)
    {
        mvc <- matrix(0,nea,nea)
        if(idiag0==0)
        {
            mvc[upper.tri(mvc, diag=TRUE)] <- c(1,b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
            mvc <- t(mvc)
            mvc[upper.tri(mvc, diag=TRUE)] <- c(1,b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
            ch <- chol(mvc)
            
            b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- ch[upper.tri(ch, diag=TRUE)][-1]
        }
        else
        {
            b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- sqrt(b[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
        }            
    }
    
    # fixed prms
    nfix <- sum(fix)
    bfix <- 0
    if(nfix>0)
    {
        bfix <- b[which(fix==1)]
        b <- b[which(fix==0)]
        NPM <- NPM-nfix
    }
    
    
    
    #browser()
    ###estimation
    
    conv3 <- c(convB, convL, convG) #TS: pr reduire le nb d arguments ds appel fct Fortran
    
    ##sharedtype <- ifelse(sharedtype == 'RE',1,2) #recodage 1 pr RE, 2 pr CL  #TS

    sharedtype <- sapply(sharedtype, switch, "RE" = 1, "REpoly" = 1, "RElogit" = 1,
           "CL" = 2, "CLpoly" = 2, "CLlogit" = 2,
           "CS" = 3, "CSpoly" = 3, "CSlogit" = 3,
           4)

    expectancy <- 0
    
    ###############
    ###   MLA   ###
    ###############    
    
    if(maxiter==0)
    {
        vrais <- loglik(b, Y0, X0, tsurv0, tsurv, devt, ind_survint, idea, idg, idcor,
                        idcontr, idsurv, idtdv, typrisq, nz, zi, nbevt, idtrunc,
                        logspecif, ny, ns, nv, nobs, nmes, idiag0, ncor, nalea,
                        NPM, nfix, bfix, epsY, idlink, nbzitr, zitr, uniqueY0, indiceY0,
                        indiceYinf0, nvalSPLORD, fix, methInteg, nMC, dimMC, seqMC, sharedtype,
                        nbXpred, Xpred_Ti, Xpred, Xpredcs_Ti, Xpred_cs, nonlin,
                        centerpoly0, expectancy)
        
        out <- list(conv=2, V=rep(NA, length(b)), best=b, b = b, predRE=NA, predRE_Y=NA,
                    Yobs=NA, resid_m=NA, resid_ss=NA, risqcum_est=NA, risq_est=NA,
                    marker=NA, transfY=NA, gconv=rep(NA,3), niter=0, loglik=vrais)
    }
    else
    {   
        res <- mla(b = b, m = length(b), fn = loglik,
                   clustertype = clustertype, .packages = NULL,
                   epsa = convB, epsb = convL, epsd = convG, 
                   digits = 8, print.info = verbose, blinding = FALSE, 
                   multipleTry = 25, file = "", partialH = partialH, 
                   nproc = nproc, maxiter = maxiter, minimize = FALSE, 
                   Y0 = Y0, X0 = X0, Tentr0 = tsurv0, Tevt0 = tsurv, Devt0 = devt, 
                   ind_survint0 = ind_survint, idea0 = idea, idg0 = idg, idcor0 = idcor, 
                   idcontr0 = idcontr, idsurv0 = idsurv, idtdv0 = idtdv,
                   typrisq0 = typrisq, nz0 = nz, zi0 = zi, nbevt0 = nbevt,
                   idtrunc0 = idtrunc, logspecif0 = logspecif, ny0 = ny, ns0 = ns,
                   nv0 = nv, nobs0 = nobs, nmes0 = nmes, idiag0 = idiag0, 
                   ncor0 = ncor, nalea0 = nalea, npm0 = NPM, nfix0 = nfix, bfix0 = bfix, 
                   epsY0 = epsY, idlink0 = idlink, nbzitr0 = nbzitr, zitr0 = zitr, 
                   uniqueY0 = uniqueY0, indiceY0 = indiceY0, indiceYinf0 = indiceYinf0, nvalSPLORD0 = nvalSPLORD, 
                   fix0 = fix, methInteg0 = methInteg, nMC0 = nMC, dimMC0 = dimMC,
                   seqMC0 = seqMC, idst0 = sharedtype, nXcl0 = nbXpred,
                   Xcl_Ti0 = Xpred_Ti, Xcl_GK0 = Xpred, Xcs_Ti0 = Xpredcs_Ti,
                   Xcs_GK0 = Xpred_cs, nonlin0 = nonlin, centerpoly0 = centerpoly0,
                   expectancy0 = expectancy)
        
        out <- list(conv=res$istop, V=res$v, best=res$b, predRE=NA, predRE_Y=NA, Yobs=NA,
                    resid_m=NA, resid_ss=NA, risqcum_est=NA, risq_est=NA, marker=NA,
                    transfY=NA, gconv=c(res$ca, res$cb, res$rdm), niter=res$ni,
                    loglik=res$fn.value, b = res$b, v = res$v)
    }
    
    
    
    ## creer best a partir de b et bfix
    best <- rep(NA,length(fix))
    best[which(fix==0)] <- out$best
    best[which(fix==1)] <- bfix
    out$best <- best
    NPM <- NPM+nfix
        
    ## item parameters
    thres <- NA
    discrim <- NA
    if(any(idlink == 3))
    {
    # thresholds
    thres <- data.frame(matrix(NA,nrow=max(nbmod)-1,ncol=sum(idlink==3)))
    names(thres) <- nomsY[idlink==3]
    k = 1
    for(yk in 1:ny){
      if(idlink[yk]==3){
        if(yk==1)
          thres[1:(nbmod[yk]-1),k] <- best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntr[yk]]
        if(yk>1)
          thres[1:(nbmod[yk]-1),k] <- best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:(yk-1)])+1:ntr[yk]]
        if((nbmod[yk]-1)>1){
          for(m in 2:(nbmod[yk]-1))
            thres[m,k] <- thres[m-1,k] + thres[m,k]^2
        }
        k = k + 1
      }
    }
    # discrimination
    discrim <- 1 / as.numeric(best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny][idlink==3])
     } 
    
    ## mettre NA pour les variances et covariances non calculees et  0 pr les prm fixes
    if(length(posfix))
    {
        mr <- NPM-length(posfix)
        Vr <- matrix(0,mr,mr)
        Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
        Vr <- t(Vr)
        Vr[upper.tri(Vr,diag=TRUE)] <- out$V[1:(mr*(mr+1)/2)]
        V <- matrix(0,NPM,NPM)
        V[setdiff(1:NPM,posfix),setdiff(1:NPM,posfix)] <- Vr
        V <- V[upper.tri(V,diag=TRUE)]
    }
    else
    {
        V <- out$V
    }
    
    
    ## Creation du vecteur cholesky
    Cholesky <- rep(0,(nea*(nea+1)/2))
    if(idiag0==0 & nvc>0)
    {
        Cholesky[1:(nvc+1)] <- c(1,out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
        ## Construction de la matrice U
        U <- matrix(0,nrow=nea,ncol=nea)
        U[upper.tri(U,diag=TRUE)] <- Cholesky[1:(nvc+1)]
        z <- t(U) %*% U
        out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- z[upper.tri(z,diag=TRUE)][-1]
    }
    if(idiag0==1 & nvc>0)
    {
        id <- 1:nea
        indice <- rep(id+id*(id-1)/2)
        Cholesky[indice] <- c(1,out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
        out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc] <- out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc]**2
    }
    
    ##predictions
    predRE <- matrix(out$predRE,ncol=nea,byrow=T)
    predRE <- data.frame(unique(IND),predRE)
    colnames(predRE) <- c(nom.subject,nom.X0[idea!=0])
    
    if (nalea!=0)
    {
        predRE_Y <- matrix(out$predRE_Y,ncol=ny,byrow=TRUE)
        predRE_Y <- data.frame(unique(IND),predRE_Y)
        colnames(predRE_Y)  <- c(nom.subject,nomsY)
    }
    else
    {
        predRE_Y <- rep(NA,nalea*ns)
    }
    
    
    ##pred
    pred_m <- out$Yobs-out$resid_m
    pred_ss <- out$Yobs - out$resid_ss
    pred <- data.frame(IND,outcome,pred_m,out$resid_m,pred_ss,out$resid_ss,out$Yobs)
    
    colnames(pred)<-c(nom.subject,"Yname","pred_m","resid_m","pred_ss","resid_ss","obs")
    rownames(pred) <- NULL
    
    
    ## risques
    if(nbevt>0)
    {
        risqcum_est <- matrix(out$risqcum_est,nrow=nsim,ncol=nbevt)
        risq_est <- matrix(out$risq_est,nrow=nsim,ncol=nbevt)
        predSurv <- cbind(time,risq_est,risqcum_est)
        
        temp <- paste("event",1:nbevt,".RiskFct",sep="")
        temp1 <- paste("event",1:nbevt,".CumRiskFct",sep="")
        colnames(predSurv) <- c("time",temp,temp1)
        rownames(predSurv) <- 1:nsim
    }
    else
    {
        predSurv <- NA
    }
    
    ##estimlink
    ysim <- matrix(out$marker,nsim,ny)
    transfo <- matrix(out$transfY,nsim,ny)
    estimlink <- as.vector(rbind(ysim,transfo))
    estimlink <- matrix(estimlink,nsim,2*ny)
    colnames(estimlink) <- paste(c("","transf"),rep(nomsY, each=2),sep="")
    
    if(any(idlink==3))
    {
        if(any(2*nbmod[which(idlink==3)] > nsim))
        {
            nsim2 <- max(2*nbmod[which(idlink==3)])
            
            estimlink2 <- matrix(NA, nrow=nsim2, ncol=2*ny)
            estimlink2[1:nsim,] <- estimlink
            
            colnames(estimlink2) <- colnames(estimlink)
            estimlink <- estimlink2
        }
        
        seuils <- function(x)
        {
            n <- length(x)
            if(n>1) res <- c(x[1],x[1]+cumsum(x[-1]^2))
            if(n==1) res <- x[1]
            return(res)
        }
        
        sumntr <- 0
        for (k in 1:ny)
        {
            if(idlink[k]==3)
            {
                estimlink[,2*(k-1)+1] <- NA
                estimlink[,2*(k-1)+2] <- NA
                
                nb <- nbmod[k]-1
                
                marker <- rep(modalites[[k]], each=2)
                seuilsk <- seuils(out$best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1:nb])
                transfY <- rep(seuilsk, each=2)
                transfY <- c(-Inf,transfY,Inf)
                
                estimlink[1:length(marker), 2*(k-1)+1] <- marker
                estimlink[1:length(transfY), 2*(k-1)+2] <- transfY
            }
            
            sumntr <- sumntr + ntr[k]
        }
        
        ## remplacer -Inf et Inf
        hy <- estimlink[,2*(1:ny)]
        maxhy <- max(hy[is.finite(hy)])
        minhy <- min(hy[is.finite(hy)])
        for (k in 1:ny)
        {
            if(idlink[k]==3)
            {
                estimlink[1, 2*(k-1)+2] <- minhy - (maxhy - minhy)/10
                estimlink[2*nbmod[k], 2*(k-1)+2] <- maxhy + (maxhy - minhy)/10
            }
        }
        
    }
    
    N <- rep(NA,12)
    N[1] <- 0 #nprob
    N[2] <- nrisqtot
    N[3] <- nvarxevt
    N[4] <- nasso
    N[5] <- nef
    N[6] <- ncontr
    N[7] <- nvc
    N[8] <- 0 #nw
    N[9] <- ncor
    N[10] <- ntrtot
    N[11] <- nalea
    N[12] <- ny
    N[13] <- nobs
    N[14] <- nbevt
    
    nevent <- rep(0,nbevt)
    for(ke in 1:nbevt)
    {
        nevent[ke] <- length(which(devt==ke))
    }
    
    Nprm <- c(nprisq,ntr)
    
    
    nom.X0[nom.X0=="(Intercept)"] <- "Intercept"
    
    ## noms des variables
    Names <- list(Xnames=nom.X0,Ynames=nomsY,
                  ID=nom.subject,Tnames=noms.surv,
                  TimeDepVar.name=nom.timedepvar,
                  Xvar=ttesLesVar)
    
    names(modalites) <- nomsY
    
    form <- list(fixed=fixed2[-2], random=random, contr=contr,
                 form.commun=form.commun, form.cause=form.cause,
                 form.cor=form.cor)
    
    cost <- proc.time()-ptm
    
    res <-list(ns=ns,idg=idg,idcontr=idcontr,idea=idea,idcor=idcor,
               idsurv=idsurv,idtdv=idtdv,
               loglik=out$loglik,best=out$best,V=V,gconv=out$gconv,conv=out$conv,
               call=cl,niter=out$niter,N=N,nevent=nevent,Nprm=Nprm,
               idiag=idiag0,pred=pred,predRE=predRE,
               predRE_Y=predRE_Y,Names=Names,form=form,cholesky=Cholesky,
               logspecif=logspecif,predSurv=predSurv,typrisq=typrisq,hazardnodes=zi,nz=nz,
               estimlink=estimlink,epsY=epsY,linktype=idlink,linknodes=zitr,nbnodes=nbnodes,nbmod=nbmod,mod=modalites,
               na.action=nayk,AIC=2*(length(out$best)-length(posfix)-out$loglik),BIC=(length(out$best)-length(posfix))*log(ns)-2*out$loglik,data=datareturn,
               #wRandom=wRandom,b0Random=b0Random,
               sharedtype = sharedtype, nonlin = nonlin, centerpoly = centerpoly0,
               posfix=posfix,CPUtime=cost[3], nMC = nMC, b=out$b, v=out$v,
               thres=thres,discrim=discrim,
               rangeTsurv = c(minT, maxT), rangeTlong = rangeTlong, terms = term)
    
    names(res$best) <- namesb
    class(res) <-c("jointLPM")
    
    
    return(res)
}


#' Wrapper to the Fortran subroutine that computes the log-likelihood
#'
#' Computes the log-likelihood of a jointLPM model
#'
#' @param b0 b0
#' @param Y0 Y0
#' @param X0 X0
#' @param Tentr0 Tentr0
#' @param Tevt0 Tevt0
#' @param Devt0 Devt0
#' @param ind_survint0 ind_survint0
#' @param idea0 idea0
#' @param idg0 idg0
#' @param idcor0 idcor0
#' @param idcontr0 idcontr0
#' @param idsurv0 idsurv0
#' @param idtdv0 idtdv0
#' @param typrisq0 typrisq0
#' @param nz0 nz0
#' @param zi0 zi0
#' @param nbevt0 nbevt0
#' @param idtrunc0 idtrunc0
#' @param logspecif0 logspecif0
#' @param ny0 ny0
#' @param ns0 ns0
#' @param nv0 nv0
#' @param nobs0 nobs0
#' @param nmes0 nmes0
#' @param idiag0 idiag0
#' @param ncor0 ncor0
#' @param nalea0 nalea0
#' @param npm0 npm0
#' @param nfix0 nfix0
#' @param bfix0 bfix0
#' @param epsY0 epsY0
#' @param idlink0 idlink0
#' @param nbzitr0 nbzitr0
#' @param zitr0 zitr0
#' @param uniqueY0 uniqueY0
#' @param indiceY0 indiceY0
#' @param indiceYinf0 indiceYinf0
#' @param nvalSPLORD0 nvalSPLORD0
#' @param fix0 fix0
#' @param methInteg0 methInteg0
#' @param nMC0 nMC0
#' @param dimMC0 dimMC0
#' @param seqMC0 seqMC0
#' @param idst0 idst0
#' @param nXcl0 nXcl0
#' @param Xcl_Ti0 Xcl_Ti0
#' @param Xcl_GK0 Xcl_GK0
#' @param Xcs_Ti0 Xcs_Ti0
#' @param Xcs_GK0 Xcs_GK0
#' @param nonlin0 nonlin0
#' @param centerpoly0 centerpoly0
#' @param expectancy0 expectancy0
#' @return the log-likelihood
#' 
#' @export
#' 
loglik <- function(b0,Y0,X0,Tentr0,Tevt0,Devt0,ind_survint0,idea0,idg0,idcor0,idcontr0,
                   idsurv0,idtdv0,typrisq0,nz0,zi0,nbevt0,idtrunc0,logspecif0,ny0,ns0,
                   nv0,nobs0,nmes0,idiag0,ncor0,nalea0,npm0,nfix0,bfix0,
                   epsY0,idlink0,nbzitr0,zitr0,uniqueY0,indiceY0,indiceYinf0,nvalSPLORD0,fix0,
                   methInteg0,nMC0,dimMC0,seqMC0,idst0,nXcl0,Xcl_Ti0,Xcl_GK0,
                   Xcs_Ti0,Xcs_GK0,nonlin0,centerpoly0,expectancy0)
{
    res <- 0
    .Fortran(C_loglik,as.double(Y0),as.double(X0),as.double(Tentr0),as.double(Tevt0),as.integer(Devt0),as.integer(ind_survint0),as.integer(idea0),as.integer(idg0),as.integer(idcor0),as.integer(idcontr0),as.integer(idsurv0),as.integer(idtdv0),as.integer(typrisq0),as.integer(nz0),as.double(zi0),as.integer(nbevt0),as.integer(idtrunc0),as.integer(logspecif0),as.integer(ny0),as.integer(ns0),as.integer(nv0),as.integer(nobs0),as.integer(nmes0),as.integer(idiag0),as.integer(ncor0),as.integer(nalea0),as.integer(npm0),as.double(b0),as.integer(nfix0),as.double(bfix0),as.double(epsY0),as.integer(idlink0),as.integer(nbzitr0),as.double(zitr0),as.double(uniqueY0),as.integer(indiceY0),as.integer(indiceYinf0),as.integer(nvalSPLORD0),as.integer(fix0),as.integer(methInteg0),as.integer(nMC0),as.integer(dimMC0),as.double(seqMC0),as.integer(idst0),as.integer(nXcl0),as.double(Xcl_Ti0),as.double(Xcl_GK0),as.double(Xcs_Ti0),as.double(Xcs_GK0),as.integer(nonlin0),as.double(centerpoly0),as.integer(expectancy0),loglik_res=as.double(res))$loglik_res
}

matrixGK <- function(data, fixed, random = NULL, var.time, idtrunc, T0 = NULL, T, deriv = FALSE, start)
{
    ## ajouter variable temps aux donnees
    data$tmp_name <- NA
    colnames(data)[which(colnames(data) == "tmp_name")] <- var.time
    data[, var.time] <- T

    ## matrices pour risque au temps Ti
    mat_ef <- model.matrix(fixed, data = data)
    mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])  # sans intercept car non-estime
    mat_ea <- NULL
    if(!is.null(random)) mat_ea <- model.matrix(random, data = data)
    Xpred_Ti <- as.matrix(cbind(data[, var.time], mat_ef, mat_ea))

    Xdt_Ti <- NULL
    if(deriv)
    {
        h <- sapply(data[, var.time], function(x) { max(1E-7, 1E-4 * abs(x)) })

        ## t + h
        dplus <- data
        dplus[, var.time] <- dplus[, var.time] + h
        mat_ef <- model.matrix(fixed, data = dplus)
        mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
        mat_ea <- NULL
        if(!is.null(random)) mat_ea <- model.matrix(random, data = dplus)
        Xplus <- cbind(mat_ef, mat_ea)

        ## t - h
        dmoins <- data
        dmoins[, var.time] <- dmoins[, var.time] - h
        mat_ef <- model.matrix(fixed, data = dmoins)
        mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
        mat_ea <- NULL
        if(!is.null(random)) mat_ea <- model.matrix(random, data = dmoins)
        Xmoins <- cbind(mat_ef, mat_ea)
        
        ## derivee par rapport a var.time
        Xdt_Ti <- as.matrix((Xplus - Xmoins) / (2 * h))
        Xdt_Ti <- derivMat(fixed, random, data, var.time)
    }

    ## points de quadrature Gauss-Kronrod
    ptGK_1 <- 0.991455371120812639206854697526329
    ptGK_2 <- 0.949107912342758524526189684047851
    ptGK_3 <- 0.864864423359769072789712788640926
    ptGK_4 <- 0.741531185599394439863864773280788
    ptGK_5 <- 0.586087235467691130294144838258730
    ptGK_6 <- 0.405845151377397166906606412076961
    ptGK_7 <- 0.207784955007898467600689403773245
    ptGK_8 <- 0
    
    ptsGK <- c(ptGK_1, -ptGK_1, ptGK_2, -ptGK_2, ptGK_3, -ptGK_3, ptGK_4, -ptGK_4, ptGK_5, -ptGK_5, ptGK_6, -ptGK_6, ptGK_7, -ptGK_7, ptGK_8) # integration [-1, 1]


    ## 15 lignes par sujet
    data$T0 <- T0
    data$T <- T
    data <- data[rep(1:nrow(data), each = 15), ] 

    ## matrices pour survie 0->T
    data[, var.time] <- (data$T + start) / 2 + (data$T - start) / 2 * ptsGK # integration [start, T]
    mat_ef <- model.matrix(fixed, data = data)
    mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
    mat_ea <- NULL
    if(!is.null(random)) mat_ea <- model.matrix(random, data = data)
    Xpred <- as.matrix(cbind(data[, var.time], mat_ef, mat_ea))

    Xpred0 <- NULL
    if(idtrunc)
    {
        ## matrices pour survie 0->T0
        data[, var.time] <- (data$T0 + start) / 2 + (data$T0 - start) / 2 * ptsGK # integration [start, T0]
        mat_ef <- model.matrix(fixed, data = data)
        mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
        mat_ea <- NULL
        if(!is.null(random)) mat_ea <- model.matrix(random, data = data)
        Xpred0 <- as.matrix(cbind(data[, var.time], mat_ef, mat_ea))
    }


    Xdt <- NULL
    Xdt0 <- NULL
    if(deriv)
    {
        data[, var.time] <- rep(T, each = 15)
        data[, var.time] <- (data$T + start) / 2 + (data$T - start) / 2 * ptsGK
        
        h <- sapply(data[, var.time], function(x) { max(1E-7, 1E-4 * abs(x)) })

        ## t + h
        dplus <- data
        dplus[, var.time] <- dplus[, var.time] + h
        mat_ef <- model.matrix(fixed, data = dplus)
        mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
        mat_ea <- NULL
        if(!is.null(random)) mat_ea <- model.matrix(random, data = dplus)
        Xplus <- cbind(mat_ef, mat_ea)

        ## t - h
        dmoins <- data
        dmoins[, var.time] <- dmoins[, var.time] - h
        mat_ef <- model.matrix(fixed, data = dmoins)
        mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
        mat_ea <- NULL
        if(!is.null(random)) mat_ea <- model.matrix(random, data = dmoins)
        Xmoins <- cbind(mat_ef, mat_ea)

        ## derivee par rapport a var.time
        Xdt <- as.matrix((Xplus - Xmoins) / (2 * h))

        Xdt <- derivMat(fixed, random, data, var.time)

        Xdt0 <- NULL
        if(idtrunc)
        {
            data[, var.time] <- rep(T0, each = 15)
            data[, var.time] <- (data$T0 + start) / 2 + (data$T0 - start) / 2 * ptsGK
            
            h <- sapply(data[, var.time], function(x) { max(1E-7, 1E-4 * abs(x)) })
            
            ## t + h
            dplus <- data
            dplus[, var.time] <- dplus[, var.time] + h
            mat_ef <- model.matrix(fixed, data = dplus)
            mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
            mat_ea <- NULL
            if(!is.null(random)) mat_ea <- model.matrix(random, data = dplus)
            Xplus <- cbind(mat_ef, mat_ea)
            
            ## t - h
            dmoins <- data
            dmoins[, var.time] <- dmoins[, var.time] - h
            mat_ef <- model.matrix(fixed, data = dmoins)
            mat_ef <- as.data.frame(mat_ef[, -1, drop = FALSE])
            mat_ea <- NULL
            if(!is.null(random)) mat_ea <- model.matrix(random, data = dmoins)
            Xmoins <- cbind(mat_ef, mat_ea)
            
            ## derivee par rapport a var.time
            Xdt0 <- as.matrix((Xplus - Xmoins) / (2 * h))
            Xdt0 <- derivMat(fixed, random, data, var.time)
        }
    }

    res <- list(Xpred_Ti = Xpred_Ti, Xpred_cl = cbind(Xpred, Xpred0), Xpredcs_Ti = Xdt_Ti, Xpred_cs = cbind(Xdt, Xdt0))
    
    return(res)
}

# internal function for the 4S method

#### STEP 3 - STAGING
## conditional expectation function : E[sum(Y)] = sum_k{M - sum(P(Yk<=m))_m=0,1,...,m-1}
.expect <- function(lambda, ni, nl, discrim, thres){
  proba <- matrix(NA,nrow=nl-1,ncol=ni) # probability matrix P(Y<=m) with m=0,1,2,..,(nl-2)
  for(k in 1:ni)
    for(l in 1:(nl-1))
      proba[l,k] <- pnorm(-discrim[k]*(lambda-thres[l,k])) # gaussian distrib fct
  probaSum <- colSums(proba) # vector of sums for each item
  probaSum <- sum(probaSum)
  res <- ni*(nl-1) - probaSum
  return(res)
}

! =============== ALNORM : calcul de proba normale ==========




      function alnorm ( x, upper )

!*****************************************************************************80
!
!! ALNORM computes the cumulative density of the standard normal distribution.
!
!  Modified:
!
!    13 January 2008
!
!  Author:
!
!    David Hill
!    FORTRAN90 version by John Burkardt
!
!  Reference:
!
!    David Hill,
!    Algorithm AS 66:
!    The Normal Integral,
!    Applied Statistics,
!    Volume 22, Number 3, 1973, pages 424-427.
!
!  Parameters:
!
!    Input, double precision X, is one endpoint of the semi-infinite interval
!    over which the integration takes place.
!
!    Input, logical UPPER, determines whether the upper or lower
!    interval is to be integrated:
!    .TRUE.  => integrate from X to + Infinity;
!    .FALSE. => integrate from - Infinity to X.
!
!    Output, double precision ALNORM, the integral of the standard normal
!    distribution over the desired interval.
!
      implicit none

      double precision, parameter :: a1 = 5.75885480458D+00
      double precision, parameter :: a2 = 2.62433121679D+00
      double precision, parameter :: a3 = 5.92885724438D+00
      double precision alnorm
      double precision, parameter :: b1 = -29.8213557807D+00
      double precision, parameter :: b2 = 48.6959930692D+00
      double precision, parameter :: c1 = -0.000000038052D+00
      double precision, parameter :: c2 = 0.000398064794D+00
      double precision, parameter :: c3 = -0.151679116635D+00
      double precision, parameter :: c4 = 4.8385912808D+00
      double precision, parameter :: c5 = 0.742380924027D+00
      double precision, parameter :: c6 = 3.99019417011D+00
      double precision, parameter :: con = 1.28D+00
      double precision, parameter :: d1 = 1.00000615302D+00
      double precision, parameter :: d2 = 1.98615381364D+00
      double precision, parameter :: d3 = 5.29330324926D+00
      double precision, parameter :: d4 = -15.1508972451D+00
      double precision, parameter :: d5 = 30.789933034D+00
      double precision, parameter :: ltone = 7.0D+00
      double precision, parameter :: p = 0.398942280444D+00
      double precision, parameter :: q = 0.39990348504D+00
      double precision, parameter :: r = 0.398942280385D+00
      logical up
      logical upper
      double precision, parameter :: utzero = 18.66D+00
      double precision x
      double precision y
      double precision z

      up = upper
      z = x

      if ( z < 0.0D+00 ) then
         up = .not. up
         z = - z
      end if

      if ( ltone < z .and. ( ( .not. up ) .or. utzero < z ) ) then

         if ( up ) then
            alnorm = 0.0D+00
         else
            alnorm = 1.0D+00
         end if

         return

      end if

      y = 0.5D+00 * z * z

      if ( z <= con ) then

         alnorm = 0.5D+00 - z * ( p - q * y  &
         / ( y + a1 + b1/ ( y + a2 + b2 / ( y + a3 ))))

      else

         alnorm = r * exp ( - y )   &
          / ( z + c1 + d1 / ( z + c2 + d2 / ( z + c3 + d3  &
          / ( z + c4 + d4 / ( z + c5 + d5 / ( z + c6 ))))))

      end if

      if ( .not. up ) then
         alnorm = 1.0D+00 - alnorm
      end if

      return
      end function alnorm



! =================================================================
!              Densite d'une beta
!=================================================================

      double precision Function beta_densite(X,a,b)

      implicit none

      double precision :: beta,a,b,gammln,X

      beta=exp(gammln(a+b)-gammln(a)-gammln(b))

      beta_densite=((X)**(a-1))*((1-X)**(b-1))*beta

      return

      end Function beta_densite


! Calcul de gamma (gammln(a))

      double precision Function gammln(xx)

! retourne la valeur ln(gamma(xx)) pour xx>0
      implicit none

      integer::j
      double precision:: ser,stp,tmp,x,y,xx
      double precision,dimension(6)::cof

      save cof,stp

      data cof,stp/76.18009172947146d0,-86.50532032941677d0, &
      24.01409824083091d0, -1.231739572450155d0,.1208650973866179d-2, &
      -.5395239384953d-5,2.5066282746310005d0/

      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0
      do j=1,6
         y=y+1.d0
         ser=ser+cof(j)/y
      end do
      gammln=tmp+log(stp*ser/x)

      return

      end Function gammln







      

      ! =================================================================
!              CDF incomplete d'une beta
!=================================================================


      double precision Function betai(a,b,x)

      implicit none

      double precision :: a,b,x,bt,betaCF,gammln,temp

      if(x.lt.0.d0.or.x.gt.1.d0) then
            betai=999.d0
            return
      end if
      if (x.eq.0..or.X.eq.1.) then
         bt=0.
      else
         bt=exp(gammln(a+b)-gammln(a)-gammln(b)+a*log(x)+b*log(1.-x))
      end if

      if (x.lt.(a+1.)/(a+B+2.)) then
         temp=betaCF(a,b,x)
         if (temp.eq.999.d0) then
            betai=999.d0
            return
         end if
         betai=bt*temp/A
         return
      else
        temp=betaCF(b,a,1.-x)
        if (temp.eq.999.d0) then
            betai=999.d0
            return
         end if
         betai=1.-bt*temp/b
         return
      end if

      end Function betai

! betaCF est utilisï¿½ par betai

      double precision Function betaCF(a,b,x)

      implicit none

      integer ::m,m2
      integer,parameter ::maxit=100
      double precision ::a,b,x,aa,c,d,del,h,qab,qam,qap
      double precision,parameter::eps=3.e-7,fpmin=1.e-30


      qab=a+b
      qap=a+1
      qam=a-1
      c=1.       ! first step of Lentz's method
      d=1.-qab*x/qap
      if (abs(d).lt.fpmin) d=fpmin
      d=1./d
      h=d
      do m=1,maxit
         m2=2*m
         aa=m*(b-m)*x/((qam+m2)*(a+m2))
         d=1.+aa*d        ! one step (the even one) of the recurrence
         if (abs(d).lt.fpmin) d=fpmin
         c=1.+aa/c
         if (abs(c).lt.fpmin) c=fpmin
         d=1./d
         h=h*d*c
         aa=-(a+m)*(qab+m)*x/((a+m2)*(qap+m2))
         d=1.+aa*d        ! next step of the recurrence (the odd one)
         if (abs(d).lt.fpmin) d=fpmin
         c=1.+aa/c
         if (abs(c).lt.fpmin) c=fpmin
         d=1./d
         del=d*c
         h=h*del
         if (abs(del-1.).lt.eps) goto 1
      end do

!  pause 'a or b too big, or maxit too small in betaCF'
            betaCF=999.d0
!            write(*,*)'problem'
            return

 1    betaCF=h
      return
      end Function betaCF




      !C ******************** BGOS ********************************


      SUBROUTINE BGOS(SX,ID,X1,X2,RO)


!C     ID=1:U(0,SX); ID DIFF DE 1 :N(0,SX)

      implicit none
      double precision ::RO,SX
      integer ::ID
      double precision ::F,V1,V2,S,DLS,RO2
      double precision ::X1,X2,runiran
!C     write(*,*)'dans bgos'


 5    CONTINUE

!C     write(*,*)'avant rand :'

!C     X1=RAND()
!C     X2=RAND()

!      X1=UNIRAN()
!      X2=UNIRAN()

      X1 = runiran()
      X2 = runiran()
      
      IF(ID.NE.1) GO TO 10
      F=2.*SQRT(3.)
      X1=(X1-0.5)*F
      X2=(X2-0.5)*F
      GO TO 20
10    CONTINUE
      V1=2.*X1-1
      V2=2.*X2-1
      S=V1*V1+V2*V2
      IF(S.GE.1.) GO TO 5
      DLS=SQRT(-2.*LOG(S)/S)
      X1=V1*DLS
      X2=V2*DLS
20    CONTINUE
      RO2=RO*RO
      IF(ABS(RO).GT.1.E-10) X2=(X1+X2*SQRT(1./RO2-1.))*RO
      X1=X1*SX
      X2=X2*SX

!C      write(*,*) 'X1 ',X1,' X2 ',X2
!C OK, X1 et X2 sont créés

!C      write(*,*)'fin bgos'

      RETURN
      END subroutine bgos


!C ------------------- FIN SUBROUTINE BGOS -----------------




!!!!!!!!!!!!!!!!!!!  DMFSD et DSINV !!!!!!!!!!!!!!!!
      
      
      subroutine dmfsd(a,n,eps,ier)
!
!   FACTORISATION DE CHOLESKY D'UNE MATRICE SDP
!   MATRICE = TRANSPOSEE(T)*T
!   ENTREE : TABLEAU A CONTENANT LA PARTIE SUPERIEURE STOCKEE COLONNE
!            PAR COLONNE DE LA METRICE A FACTORISER
!   SORTIE : A CONTIENT LA PARTIE SUPPERIEURE DE LA MATRICE triangulaire T
!
!   SUBROUTINE APPELE PAR DSINV
!
!   N : DIM. MATRICE
!   EPS : SEUIL DE TOLERANCE
!   IER = 0 PAS D'ERREUR
!   IER = -1 ERREUR
!   IER = K COMPRIS ENTRE 1 ET N, WARNING, LE CALCUL CONTINUE
!
      implicit none

      integer,intent(in)::n
      integer,intent(out)::ier
      double precision,intent(in)::eps
      double precision,dimension(n*(n+1)/2),intent(inout)::A
      double precision :: dpiv,dsum,tol
      integer::i,k,l,kpiv,ind,lend,lanf,lind

!
!   TEST ON WRONG INPUT PARAMETER N
!
      dpiv=0.d0
      if (n-1.lt.0) goto 12
      if (n-1.ge.0) ier=0
!
!   INITIALIZE DIAGONAL-LOOP
!
      kpiv=0
      do k=1,n
          kpiv=kpiv+k
          ind=kpiv
          lend=k-1
!
!   CALCULATE TOLERANCE
!
          tol=dabs(eps*sngl(A(kpiv)))
!
!   START FACTORIZATION-LOOP OVER K-TH ROW
!
         do i=k,n
            dsum=0.d0
            if (lend.lt.0) goto 2
            if (lend.eq.0) goto 4
            if (lend.gt.0) goto 2
!
!   START INNER LOOP
!
2           do l=1,lend
               lanf=kpiv-l
               lind=ind-l
               dsum=dsum+A(lanf)*A(lind)
            end do

!
!   END OF INNEF LOOP
!
!   TRANSFORM ELEMENT A(IND)
!
4           dsum=A(ind)-dsum
            if (i-k.ne.0) goto 10
            if (i-k.eq.0) goto 5
!   TEST FOR NEGATIVE PIVOT ELEMENT AND FOR LOSS OF SIGNIFICANCE
!


5           if (sngl(dsum)-tol.le.0) goto 6
            if (sngl(dsum)-tol.gt.0) goto 9
6           if (dsum.le.0) goto 12
            if (dsum.gt.0) goto 7
7           if (ier.le.0) goto 8
            if (ier.gt.0) goto 9
8           ier=k-1
!
!   COMPUTE PIVOT ELEMENT
!
9           dpiv=dsqrt(dsum)
            A(kpiv)=dpiv
            dpiv=1.D0/dpiv
            goto 11
!
!   CALCULATE TERMS IN ROW
!
10          A(ind)=dsum*dpiv
11          ind=ind+i
         end do
      end do

!
!   END OF DIAGONAL-LOOP
!
      return
12    ier=-1
      return

      end subroutine dmfsd


!------------------------------------------------------------
!                            DSINV
!------------------------------------------------------------


      subroutine dsinv(A,N,EPS,IER,DET)

!
!     INVERSION D'UNE MATRICE SYMETRIQUE DEFINIE POSITIVE :
!
!     MATRICE = TRANSPOSEE(T)*T
!     INERSE(MATRICE) = INVERSE(T)*INVERSE(TRANSPOSEE(T))
!
!     A : TABLEAU CONTENANT LA PARTIE SUPERIEURE DE LA MATRICE A INVERSER
!         STOCKEE COLONNE PAR COLONNE
!     DIM. MATRICE A INVERSER = N
!     DIM. TABLEAU A = N*(N+1)/2
!
!     EPS : SEUIL DE TOLERANCE AU-DESSOUS DUQUEL UN PIVOT EST CONSIDERE
!           COMME NUL
!
!     IER : CODE D'ERREUR
!         IER=0 PAS D'ERREUR
!         IER=-1 ERREUR SUR LA DIM.N OU MATRICE PAS DEFINIE POSITIVE
!         IER=1 PERTE DE SIGNIFICANCE, LE CALCUL CONTINUE
!
      implicit none

      integer,intent(in)::n
      integer,intent(out)::ier
      double precision,intent(in)::eps
      double precision,intent(out),optional::det
      double precision,dimension(n*(n+1)/2),intent(inout)::A
      double precision::din,work
      integer::ind,ipiv,i,j,k,l,min,kend,lhor,lver,lanf

!
!     FACTORIZE GIVEN MATRIX BY MEANS OF SUBROUTINE DMFSD
!     A=TRANSPOSE(T) * T
!

      call dmfsd(A,n,eps,ier)

      det=0.d0

      if (ier.lt.0) goto 9
      if (ier.ge.0) det=0.d0
!
!     INVERT UPPER TRIANGULAR MATRIX T
!     PREPARE INVERSION-LOOP
!
!
! calcul du log du determinant

      do i=1,n
         det=det+dlog(A(i*(i+1)/2))
      end do
      det=2*det
      ipiv=n*(n+1)/2
      ind=ipiv
!
!     INITIALIZE INVERSION-LOOP
!
      do i=1,n
         din=1.d0/A(ipiv)
         A(ipiv)=din
         min=n
         kend=i-1
         lanf=n-kend
         if (kend.le.0) goto 5
         if (kend.gt.0) j=ind
!
!     INITIALIZE ROW-LOOP
!
         do k=1,kend
            work=0.d0
            min=min-1
            lhor=ipiv
            lver=j
!
!     START INNER LOOP
!
            do l=lanf,min
                lver=lver+1
                lhor=lhor+l
                work=work+A(lver)*A(lhor)
            end do
!
!     END OF INNER LOOP
!
            A(j)=-work*din
            j=j-min
         end do

!
!     END OF ROW-LOOP
!
5        ipiv=ipiv-min
         ind=ind-1
      end do

!
!     END OF INVERSION-LOOP
!
!     CALCULATE INVERSE(A) BY MEANS OF INVERSE(T)
!     INVERSE(A) = INVERSE(T) * TRANSPOSE(INVERSE(T))
!     INITIALIZE MULTIPLICATION-LOOP
!
      do i=1,n
         ipiv=ipiv+i
         j=ipiv
!
!     INITIALIZE ROW-LOOP
!
         do k=i,n
            work=0.d0
            lhor=j
!
!     START INNER LOOP
!
            do l=k,n
                lver=lhor+k-i
                work=work+A(lhor)*A(lver)
                lhor=lhor+l
            end do
!
!     END OF INNER LOOP
!
            A(j)=work
            j=j+k
         end do
      end do

!
!     END OF ROW-AND MULTIPLICATION-LOOP
!
9     return
      end subroutine dsinv

#' Summary of a joint latent process model
#' 
#' This function provides a summary of model estimated with the \code{jointLPM} function.
#' 
#' @param object an object inheriting from class \code{jointLPM} for a joint latent process model.
#' @param ... further arguments to be passed to or from other methods. They are ignored in this function.
#' 
#' @return The function is mainly used for its side effects. It returns invisibily a list of two matrices
#' containing the estimates, their standard errors, Wald statistics and associated p-values for the survival
#' submodel (first element of the list) and for the mixed model's fixed effects (second element). 
#' 
#' @author Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
#' 
#' @export
#'
summary.jointLPM <- function(object,...)
{
  x <- object
  
  cat("Joint latent process model with shared random effects", "\n")
  cat("     fitted by maximum likelihood method", "\n")
  
  cl <- x$call
  cl$B <- NULL
  if(is.data.frame(cl$data))
  {
    cl$data <- NULL
    x$call$data <- NULL    
  }
  cat(" \n")
  dput(cl)
  cat(" \n")
  
  posfix <- object$posfix
  
  cat("Statistical Model:", "\n")
  cat(paste("     Dataset:", as.character(as.expression(x$call$data))),"\n")
  cat(paste("     Number of subjects:", x$ns),"\n")
  
  cat(paste("     Number of observations:", x$N[13]),"\n")
  cat(paste("     Number of parameters:", length(x$best))," \n")
  if(length(posfix)) cat(paste("     Number of estimated parameters:", length(x$best)-length(posfix))," \n")
  
  tTable <- list(survModel=NULL, mixedModel=NULL)
  
  nbevt <- x$N[14]
  if(nbevt>0)
  {
    nprisq <- rep(NA,nbevt)
    nrisq <- rep(NA,nbevt)
    for(ke in 1:nbevt)
    {
      if(x$typrisq[ke]==1) nprisq[ke] <- x$nz[ke]-1
      if(x$typrisq[ke]==2) nprisq[ke] <- 2
      if(x$typrisq[ke]==3) nprisq[ke] <- x$nz[ke]+2
      
      
      cat(paste("     Event",ke,": \n"))
      cat(paste("        Number of events: ", x$nevent[ke],"\n",sep=""))
      
      if (x$typrisq[ke]==2)
      {
        cat("        Weibull baseline risk function \n")
      }
      if (x$typrisq[ke]==1)
      {
        cat("        Piecewise constant baseline risk function with nodes \n")
        cat("        ",x$hazardnodes[1:x$nz[ke],ke]," \n")
      }
      if (x$typrisq[ke]==3)
      {
        cat("        M-splines baseline risk function with nodes \n")
        cat("        ",x$hazardnodes[1:x$nz[ke],ke]," \n")
      }
      
    }
  }
  
  
  ny <- x$N[12]
  ntr <- rep(NA,ny)
  numSPL <- 0
  cat("     Link functions: ")
  for (yk in 1:ny)
  {
    if (x$linktype[yk]==0)
    {
      ntr[yk] <- 2
      if (yk>1) cat("                     ")
      cat("Linear for",x$Names$Ynames[yk]," \n")
    }
    if (x$linktype[yk]==1)
    {
      ntr[yk] <- 4
      if (yk>1) cat("                     ")
      cat("Standardised Beta CdF for",x$Names$Ynames[yk]," \n")
    }
    if (x$linktype[yk]==2) 
    {
      numSPL <- numSPL+1
      ntr[yk] <- x$nbnodes[numSPL]+2
      if (yk>1) cat("                     ")
      cat("Quadratic I-splines with nodes", x$linknodes[1:x$nbnodes[numSPL],yk],"for",x$Names$Ynames[yk], "\n")
    }
    if (x$linktype[yk]==3) 
    {
      ntr[yk] <- x$nbmod[yk]-1
      if (yk>1) cat("                     ")
      cat("Thresholds for",x$Names$Ynames[yk], "\n")
    }
  }
  
  cat(" \n")
  cat("Iteration process:", "\n")
  
  if(x$conv==1) cat("     Convergence criteria satisfied")
  if(x$conv==2) cat("     Maximum number of iteration reached without convergence")
  if(x$conv==3) cat("     Convergence with restrained Hessian matrix")
  if(x$conv==4|x$conv==12)
  {
    cat("     The program stopped abnormally. No results can be displayed.\n")
  }
  else
  {
    cat(" \n")
    cat("     Number of iterations: ", x$niter, "\n")
    cat("     Convergence criteria: parameters=", signif(x$gconv[1],2), "\n")
    cat("                         : likelihood=", signif(x$gconv[2],2), "\n")
    cat("                         : second derivatives=", signif(x$gconv[3],2), "\n")
    cat(" \n")
    cat("Goodness-of-fit statistics:", "\n")
    cat(paste("     maximum log-likelihood:", round(x$loglik,2))," \n")
    cat(paste("     AIC:", round(x$AIC,2))," \n")
    cat(paste("     BIC:", round(x$BIC,2))," \n")
    cat(" \n")
    
    
    cat("Maximum Likelihood Estimates:", "\n")
    cat(" \n")
    
    nprob <- x$N[1]
    nrisqtot <- x$N[2]
    nvarxevt <- x$N[3]
    nasso <- x$N[4]
    nef <- x$N[5]
    ncontr <- x$N[6]
    nvc <- x$N[7]
    nw <- x$N[8]
    ncor <- x$N[9]
    ntrtot <- x$N[10]
    nalea <- x$N[11]
    ny <- x$N[12]
    nbevt <- x$N[14]
    NPM <- length(x$best)
    
    
    se <- rep(NA,NPM)
    if (x$conv==1 | x$conv==3)
    {
      ##recuperation des indices de V
      id <- 1:NPM
      indice <- id*(id+1)/2
      se <- sqrt(x$V[indice])
      se[which(is.na(se))] <- 1
      wald <- x$best/se
      pwald <- 1-pchisq(wald**2,1)
      coef <- x$best
    }
    else
    {
      se <- NA
      wald <- NA
      pwald <- NA
      coef <- x$best
      
      sech <- rep(NA,length(coef))
      waldch <- rep(NA,length(coef))
      pwaldch <- rep(NA,length(coef))
    }
    
    if(ncor>0) coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor] <- abs(coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor])
    coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny] <- abs(coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny])
    if(nalea>0) coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:ny] <- abs(coef[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:ny])
    
    if(x$conv!=2)
    {
      coefch <- format(as.numeric(sprintf("%.5f",coef)),nsmall=5,scientific=FALSE)
      sech <- format(as.numeric(sprintf("%.5f",se)),nsmall=5,scientific=FALSE)
      waldch <- format(as.numeric(sprintf("%.3f",wald)),nsmall=3,scientific=FALSE)
      pwaldch <- format(as.numeric(sprintf("%.5f",pwald)),nsmall=5,scientific=FALSE)
    }
    else
    {
      coefch <- format(as.numeric(sprintf("%.5f",coef)),nsmall=5,scientific=FALSE)
    }
    
    
    if(length(posfix))
    {
      coefch[posfix] <- paste(coefch[posfix],"*",sep="")
      sech[posfix] <- ""
      waldch[posfix] <- ""
      pwaldch[posfix] <- ""
    }
    
    ## fct pr determiner la longueur max d'une chaine de caracteres
    ## (avec gestion des NA)
    maxchar <- function(x)
    {
      xx <- na.omit(x)
      if(length(xx))
      {
        res <- max(nchar(xx))
      }
      else
      {
        res <- 2
      }
      return(res)
    }
    
    #browser()
    
    if(nbevt>0)
    {
      cat("\n")
      cat("Parameters in the proportional hazard model:\n" )
      cat("\n")
      
      tmp <- cbind(coefch[c(1:(nrisqtot+nvarxevt), nrisqtot+nvarxevt+1:nasso)],
                   sech[c(1:(nrisqtot+nvarxevt), nrisqtot+nvarxevt+1:nasso)],
                   waldch[c(1:(nrisqtot+nvarxevt), nrisqtot+nvarxevt+1:nasso)],
                   pwaldch[c(1:(nrisqtot+nvarxevt), nrisqtot+nvarxevt+1:nasso)])
      maxch <- apply(tmp,2,maxchar)
      if(any(c(1:(nrisqtot+nvarxevt), nrisqtot+nvarxevt+1:nasso) %in% posfix)) maxch[1] <- maxch[1]-1
      dimnames(tmp) <- list(names(coef)[1:(nrisqtot+nvarxevt+nasso)],
                            c(paste(paste(rep(" ",max(maxch[1]-4,0)),collapse=""),"coef",sep=""),
                              paste(paste(rep(" ",max(maxch[2]-2,0)),collapse=""),"Se",sep=""),
                              paste(paste(rep(" ",max(maxch[3]-4,0)),collapse=""),"Wald",sep=""),
                              paste(paste(rep(" ",max(maxch[4]-7,0)),collapse=""),"p-value",sep="")))
      
      tTable[[1]] <- tmp
      cat("\n")
      print(tmp,quote=FALSE,na.print="")
      cat("\n")
    }
    
    
    
    cat("Fixed effects in the longitudinal model:\n" )
    
    tmp <- matrix(c(0,NA,NA,NA),nrow=1,ncol=4)
    
    if (nef>0)
    {
      tmp2 <- cbind(round(coef[nrisqtot+nvarxevt+nasso+1:nef],5),round(se[nrisqtot+nvarxevt+nasso+1:nef],5),round(wald[nrisqtot+nvarxevt+nasso+1:nef],3),round(pwald[nrisqtot+nvarxevt+nasso+1:nef],5))
      tmp <- rbind(tmp,tmp2)
    }
    interc <- "intercept (not estimated)"
    
    if(nef>0) dimnames(tmp) <- list(c(interc,names(coef)[nrisqtot+nvarxevt+nasso+1:nef]), c("coef", "Se", "Wald", "p-value"))
    else dimnames(tmp) <- list(interc, c("coef", "Se", "Wald", "p-value"))
    cat("\n")
    
    if(ncontr>0)
    {
      indice2 <- 1:NPM*(1:NPM+1)/2
      nom.contr <- x$Names$Xnames[as.logical(x$idcontr)]
      for (i in 1:sum(x$idcontr))
      {
        ##matrice de variance pour test et se du dernier coef
        indtmp <- indice2[(nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))]
        indtmp <- cbind(indtmp-0:(length(indtmp)-1),indtmp)
        indV <- NULL
        for (j in 1:dim(indtmp)[1])
        {
          indV <- c(indV,seq(indtmp[j,1],indtmp[j,2]))
        }
        Vcontr <- matrix(0,ny-1,ny-1)
        Vcontr[upper.tri(Vcontr,diag=TRUE)] <- x$V[indV]
        Vcontr <- t(Vcontr)
        Vcontr[upper.tri(Vcontr)] <- Vcontr[lower.tri(Vcontr)]
        
        vect.gamma <- coef[(nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))]
        if(any(c((nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))) %in% posfix))
        {
          wald.contr <- NA
          p.wald.contr <- NA
        }
        else
        {
          wald.contr <- t(vect.gamma) %*% solve(Vcontr,vect.gamma)
          p.wald.contr <- 1-pchisq(wald.contr,ny-1)
        }
        
        tmp2 <- cbind(round(vect.gamma,5),
                      round(se[(nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))],5),
                      round(wald[(nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))],3),
                      round(pwald[(nrisqtot+nvarxevt+nasso+nef+(i-1)*(ny-1)+1):(nrisqtot+nvarxevt+nasso+nef+i*(ny-1))],5))
        tmp2 <- rbind(rep(NA,4),tmp2)
        
        if(x$conv %in% c(1,3))
        {
          pp <- -sum(na.omit(tmp2[,1]))/sqrt(sum(Vcontr))
          tmp2 <- rbind(tmp2,c(round(-sum(na.omit(tmp2[,1])),5),round(sqrt(sum(Vcontr)),5),round(pp,3),round(1-pchisq(pp*pp,1),5)))
          if(is.na(p.wald.contr)) rownames(tmp2) <- c(paste("Contrasts on ",nom.contr[i],sep=""),x$Ynames)
          else
          {
            if(round(p.wald.contr,5)!=0) rownames(tmp2) <- c(paste("Contrasts on ",nom.contr[i]," (p=",round(p.wald.contr,5),")",sep=""),x$Names$Ynames)
            if(round(p.wald.contr,5)==0) rownames(tmp2) <- c(paste("Contrasts on ",nom.contr[i]," (p<0.00001)",sep=""),x$Names$Ynames)
          }
        }
        if(x$conv==2)
        {
          tmp2 <- rbind(tmp2,c(-sum(na.omit(tmp2[,1])),NA,NA,NA))
          rownames(tmp2) <- c(paste("Contrasts on ",nom.contr[i],sep=""),x$Names$Ynames)
        }
        rownames(tmp2)[nrow(tmp2)] <- paste(rownames(tmp2)[nrow(tmp2)],"**",sep="")
        if(!is.finite(tmp2[nrow(tmp2),3])) tmp2[nrow(tmp2),2:4] <- NA
        tmp <- rbind(tmp,tmp2)
      }
    }
    
    tTable[[2]] <- tmp
    
    if((nef>0) & any(c(nrisqtot+nvarxevt+nasso+1:nef) %in% posfix))
    {      
      col1 <- rep(NA,length(tmp[,1]))
      col1[which(!is.na(tmp[,1]))] <- format(as.numeric(sprintf("%.5f",na.omit(tmp[,1]))),nsmall=5,scientific=FALSE)
      col2 <- rep(NA,length(tmp[,2]))
      col2[which(!is.na(tmp[,2]))] <- format(as.numeric(sprintf("%.5f",na.omit(tmp[,2]))),nsmall=5,scientific=FALSE)
      col3 <- rep(NA,length(tmp[,3]))
      col3[which(!is.na(tmp[,3]))] <- format(as.numeric(sprintf("%.3f",na.omit(tmp[,3]))),nsmall=3,scientific=FALSE)
      col4 <- rep(NA,length(tmp[,4]))
      col4[which(!is.na(tmp[,4]))] <- format(as.numeric(sprintf("%.5f",na.omit(tmp[,4]))),nsmall=5,scientific=FALSE)
      
      pf <- sort(intersect(c(nrisqtot+nvarxevt+nasso+1:(nef+ncontr)),posfix))
      p <- rep(0,length(tmp[,1]))
      p[which(rownames(tmp) %in% c(x$Names$Xnames,x$Names$Ynames[-ny]))] <- c(nrisqtot+nvarxevt+nasso+1:(nef+ncontr))
      col1[which(p %in% pf)] <- paste(col1[which(p %in% pf)],"*",sep="")
      col2[which(p %in% pf)] <- NA
      col3[which(p %in% pf)] <- NA
      col4[which(p %in% pf)] <- NA
      
      tmp <- cbind(col1,col2,col3,col4)
      rownames(tmp) <- rownames(tTable[[2]])
      maxch <- apply(tmp,2,maxchar)
      maxch[1] <- maxch[1]-1
      
      colnames(tmp) <- c(paste(paste(rep(" ",max(maxch[1]-4,0)),collapse=""),"coef",sep=""),
                         paste(paste(rep(" ",max(maxch[2]-2,0)),collapse=""),"Se",sep=""),
                         paste(paste(rep(" ",max(maxch[3]-4,0)),collapse=""),"Wald",sep=""),
                         paste(paste(rep(" ",max(maxch[4]-7,0)),collapse=""),"p-value",sep=""))
      
      print(tmp,quote=FALSE,na.print="")
      cat("\n")
    }
    else
    {
      prmatrix(round(tmp,5),na.print="")
      cat("\n")
    }
    
    
    
    cat("\n")
    cat("Variance-covariance matrix of the random-effects:\n" )
    cat("(the variance of the first random effect is not estimated)\n")
    if(x$idiag==1)
    {
      if (nvc>0) 
      {
        Mat.cov <- diag(c(1,coef[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc]))
      }
      else
      {
        Mat.cov <- matrix(1,ncol=1)
      }
      Mat.cov[lower.tri(Mat.cov)] <- 0
      Mat.cov[upper.tri(Mat.cov)] <- NA
    }
    if(x$idiag==0)
    {
      Mat.cov<-matrix(0,ncol=sum(x$idea),nrow=sum(x$idea))
      if(nvc>0) 
      {
        Mat.cov[upper.tri(Mat.cov,diag=TRUE)]<-c(1,coef[nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc])
        Mat.cov <-t(Mat.cov)
        Mat.cov[upper.tri(Mat.cov)] <- NA
      }
      else Mat.cov[1,1] <- 1
    }
    colnames(Mat.cov) <-x$Names$Xnames[x$idea==1]
    rownames(Mat.cov) <-x$Names$Xnames[x$idea==1]
    
    if(any(posfix %in% c(nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc)))
    {
      Mat.cov <- apply(Mat.cov,2,format,digits=5,nsmall=5)
      Mat.cov[upper.tri(Mat.cov)] <- ""
      pf <- sort(intersect(c(nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc),posfix))
      p <- matrix(0,sum(x$idea),sum(x$idea))
      if(x$idiag==FALSE) p[upper.tri(p,diag=TRUE)] <- c(0,nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc)
      if(x$idiag==TRUE) diag(p) <- c(0,nrisqtot+nvarxevt+nasso+nef+ncontr+1:nvc)
      Mat.cov[which(t(p) %in% pf)] <- paste(Mat.cov[which(t(p) %in% pf)],"*",sep="")
      print(Mat.cov,quote=FALSE)
    }
    else
    {
      prmatrix(round(Mat.cov,5),na.print="")
    }
    cat("\n")
    
    std <- NULL
    nom <- NULL
    if(ncor==2)
    {
      nom <- c(nom,"AR correlation parameter:","AR standard error:")
      std <-rbind(std,c(coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1],sech[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1]),c(coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+2],sech[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+2]))
    }
    if(ncor==1) 
    {
      nom <- c(nom,"BM standard error:")
      std <-rbind(std,c(coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1],sech[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1]))
    }
    if (!is.null(std)) 
    {
      rownames(std) <- nom
      maxch <- apply(std,2,maxchar)
      if(any(c(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1:ncor) %in% posfix)) maxch[1] <- maxch[1]-1
      colnames(std) <- c(paste(paste(rep(" ",max(maxch[1]-4,0)),collapse=""),"coef",sep=""),
                         paste(paste(rep(" ",max(maxch[2]-2,0)),collapse=""),"Se",sep=""))
      
      print(std,quote=FALSE,na.print="")
      cat("\n")
    }
    
    
    std.err <- NULL
    nom <- NULL
    std.err <- rbind(std.err,coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+1:ny])
    nom <- c(nom, "Residual standard error:")
    if(nalea>0)
    {
      std.err <- rbind(std.err,coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:nalea])
      nom <- c(nom, "Standard error of the random effect:")
    }  
    
    rownames(std.err) <- nom
    maxch <- apply(std.err,2,maxchar)
    if(any(c(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+1:(nalea+ny)) %in% posfix))
    {
      if(nalea>0)
      {
        maxch[union(grep("*",std.err[1,]),grep("*",std.err[2,]))] <- maxch[union(grep("*",std.err[1,]),grep("*",std.err[2,]))]-1
      }
      else
      {
        maxch[grep("*",std.err[1,])] <- maxch[grep("*",std.err[1,])]-1
      }
    }
    colnames(std.err) <- sapply(1:ny,function(k) paste(paste(rep(" ",max(0,maxch[k]-maxchar(x$Ynames[k]))),collapse=""),x$Ynames[k],sep=""))
    print(std.err,quote=FALSE,na.print="")
    
    cat("\n")
    
    cat("Parameters of the link functions:\n" )
    
    tmp <- cbind(coefch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntrtot],
                 sech[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntrtot],
                 waldch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntrtot],
                 pwaldch[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntrtot])
    tmp.rownames <- NULL
    for (yk in 1:ny)
    {
      tmp.rownames <- c(tmp.rownames, paste(rep(x$Names$Ynames[yk],ntr[yk]),names(coef[(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:yk])-ntr[yk]+1):(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sum(ntr[1:yk]))]),sep="-"))
    }
    rownames(tmp) <- tmp.rownames
    maxch <- apply(tmp,2,maxchar)
    if(any(c(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+1:ntrtot) %in% posfix)) maxch[1] <- maxch[1]-1
    colnames(tmp) <- c(paste(paste(rep(" ",max(maxch[1]-4,0)),collapse=""),"coef",sep=""),
                       paste(paste(rep(" ",max(maxch[2]-2,0)),collapse=""),"Se",sep=""),
                       paste(paste(rep(" ",max(maxch[3]-4,0)),collapse=""),"Wald",sep=""),
                       paste(paste(rep(" ",max(maxch[4]-7,0)),collapse=""),"p-value",sep=""))
    cat("\n")
    print(tmp,quote=FALSE,na.print="")
    cat("\n")
    
    if(length(posfix))
    {
      cat(" *  coefficient fixed by the user \n \n")
    }
    if(ncontr>0)
    {
      cat(" ** coefficient not estimated but obtained from the others as minus the sum of them \n \n")
    }
    
    return(invisible(tTable))
  }
}

#' Conversion
#'
#' This function converts a jointLPM object to an object of type \code{multlcmm} or
#' \code{Jointlcmm} from lcmm package.
#' The conversion to \code{multlcmm} unable the use of the dedicated postfit and
#' predictions functions implemented in the lcmm package (plot, predictL, predictY,
#' predictlink, predictYcond, fitY, VarExpl).
#' The conversion to \code{Jointlcmm} permits the use of functions cuminc and
#' plot (with options "baselinerisk" or "survival").
#'
#' @param object an object of class \code{jointLPM}
#' @param to character. Either "multlcmm" or "Jointlcmm", indicating to which type the object should be converted.
#' @return an object of class \code{multlcmm} or \code{Jointlcmm}.
#' @export
convert <- function(object, to)
{
    if(!(to %in% c("multlcmm", "Jointlcmm"))) stop("only conversions to multlcmm or Jointlcmm are implemented")

    z <- object$call
    z$maxiter <- 0
    z$verbose <- FALSE
    z$sharedtype <- NULL

    nrisqtot <- object$N[2]
    nvarxevt <- object$N[3]
    nasso <- object$N[4]
    nef <- object$N[5]
    ncontr <- object$N[6]
    nvc <- object$N[7]
    ncor <- object$N[9]
    ntrtot <- object$N[10]
    nalea <- object$N[11]
    ny <- object$N[12]
    
    if(to=="multlcmm")
    {
        z[[1]] <- as.name("multlcmm")
        z$survival <- NULL
        z$hazard <- NULL
        z$hazardrange <- NULL
        z$hazardnodes <- NULL
        z$TimeDepVar <- NULL
        z$logscale <- NULL
        z$startWeibull <- NULL

        index <- c(nrisqtot+nvarxevt+nasso + 1:(nef+ncontr+nvc+ncor),
                   nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea + 1:ny)
        if(nalea>0) index <- c(index, nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot + 1:nalea)
        index <- c(index, nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor + 1:ntrtot)

        B <- object$best[index]
    }
    else
    {
        z[[1]] <- as.name("Jointlcmm")
        z$methInteg <- NULL
        z$nMC <- NULL
        z$fixed <- formula(paste(object$Names$Ynames[1],"~",object$form$fixed[2]))
        z$randomY <- NULL
        if(length(object$call$startWeibull))
        {
           if(object$call$startWeibull != 0) warning("startWeibull is not taking into account")
        }
        if(object$linktype[1]==3)
        {
            warning("the outcome is treated as Gaussian")
            z$link <- "linear"
        }
        if(object$linktype[1]==0) z$link <- "linear"
        
        index <- c(1:(nrisqtot+nvarxevt),
                   nrisqtot+nvarxevt+nasso + 1:nef,
                   nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea + 1,
                   nrisqtot+nvarxevt+nasso+nef+ncontr + 1:(nvc+ncor))
        if(object$linktype[1]==3)
        {
            index <- c(index, nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor + 1:2)
        }
        else
        {
            index <- c(index, nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor + 1:ntrtot)
        }

        ## attention : dans jointLPM Var(u0)=1 alors que dans Jointlcmm Var(E)=1
        ## donc on divise la variance des EA par Var(E). Sinon pb avec chol(varcov).
        if(nvc>0) object$best[nrisqtot+nvarxevt+nasso+nef+ncontr + 1:nvc] <- object$best[nrisqtot+nvarxevt+nasso+nef+ncontr + 1:nvc]/object$best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea + 1]^2
        object$best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea + 1] <- 1/object$best[nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea + 1]^2 
        
        B <- object$best[index]
    }

    z$B <- B
    m <- eval(z)
    m$conv <- object$conv
    
    varcov <- matrix(0, length(object$best), length(object$best))
    varcov[upper.tri(varcov, diag=TRUE)] <- object$V
    varcov <- t(varcov)
    varcov[upper.tri(varcov, diag=TRUE)] <- object$V
    v <- varcov[index,index]
    m$V <- v[upper.tri(v, diag=TRUE)]

    return(m)
}

# internal function for the 4S method

#### STEP 4 - SELECTING
## probability function P : Pim(lambda) = P(Yi = m |lambda)
.Pim <- function(lambda,
                 i,thres,discrim,
                 m,nl){
  # arguments : - lamba = dimension process value
  #             - i = item
  #             - thresh = item threshold parameters
  #             - discrim = item discrimination parameters
  #             - m = modality
  #             - nl = nb of levels
  res <- NA
  discrim_i <- discrim[i]
  # m=0, lower modality
  if(m == 0){
    thres.im <- thres[1,i]
    res <- pnorm(-discrim_i * (lambda - thres.im))
  }
  # m=nl-1, upper modality
  if(m == (nl-1)){
    thres.im <- thres[nl-1,i]
    res <- 1 - pnorm(-discrim_i * (lambda - thres.im))
  }
  # other modalities
  if(m != 0 && m!=(nl-1)){
    thres.im.inf <- thres[m,i]
    thres.im.sup <- thres[m+1,i]
    res <- pnorm(-discrim_i * (lambda - thres.im.sup)) - pnorm(-discrim_i * (lambda - thres.im.inf))
  }
  return(res)  
}

plotAsso <- function(x, xlim = NULL)
{
    sharedtype <- x$sharedtype
    nonlin <- x$nonlin
    nea <- sum(x$idea)
    
    if(sharedtype == 1)
    {
        nomRE <- x$Names$Xnames[x$idea==1]
        simRE <- rangeRE(x)$RE

        if(nonlin[1] == 0)
        {
            asso <- simRE * x$b[sum(x$N[1:3]) + 1:x$N[4]]
        }
        else
        {
            if(nonlin[1] == 1)
            {
                avt <- sum(x$N[1:3])
                asso <- matrix(0, nrow(simRE), ncol(simRE))
                for(q in 1:nea)
                {
                    asso[q, ] <- simRE[q, ] * x$b[avt + 1] + simRE[q, ]^2 * x$b[avt + 2] + simRE[q, ]^3 * x$b[avt + 3]

                    avt <- avt + 3
                }
            }
        }

        par(mfrow = n2mfrow(nea))
        for(q in 1:nea)
            plot(simRE[q, ], asso[q, ], type = "p", col = 1, xlab = nomRE[q], ylab = paste("asso(", nomRE[q], ")", sep = ""))

        res <- list(x = simRE, y = asso)
    }

    if(sharedtype %in% c(2, 4))
    {
        simCL <- rangeCL(x)$CL

        if(nonlin[2] == 0)
        {
            asso <- simCL * x$b[sum(x$N[1:3]) + 1]
        }
        else
        {
            if(nonlin[2] == 1)
            {
                avt <- sum(x$N[1:3])
                asso <- simCL * x$b[avt + 1] + simCL^2 * x$b[avt + 2] + simCL^3 * x$b[avt + 3]
            }
        }

            plot(simCL, asso, type = "p", col = 1, xlab = "CL", ylab = "asso(CL)")

        res <- list(x = simCL, y = asso)
    }

    if(sharedtype %in% c(3, 4))
    {
        simCS <- rangeCS(x)$CS

        avt <- sum(x$N[1:3])
        if(sharedtype == 4)
        {
            if(nonlin[2] == 0)
                avt <- avt + 1
            else if(nonlin[2] == 1)
                avt <- avt + 3         
        }

        if(nonlin[3] == 0)
        {
            asso <- simCS * x$b[avt + 1]
        }
        else
        {
            if(nonlin[3] == 1)
            {
                asso <- simCS * x$b[avt + 1] + simCS^2 * x$b[avt + 2] + simCS^3 * x$b[avt + 3]
            }
        }

            plot(simCS, asso, type = "p", col = 1, xlab = "CS", ylab = "asso(CS)")

        if(sharedtype == 4)
            res <- list(CL = res, CS = list(x = simCS, y = asso))
        else
            res <- list(x = simCS, y = asso)
    }


    
    return(invisible(res))
}



### fct rangeRE

rangeRE <- function(x)
{
    nea <- sum(x$idea)
    nMC <- x$nMC
    if((nea == 0) | (nMC == 0)) stop("nothing to compute in rangeRE")

    ## MC sequence
    sequnif <- spacefillr::generate_sobol_owen_set(nMC, nea)
    seqMC <- apply(sequnif, 2, qnorm)

    ## cholesky of the RE
    ch <- matrix(0, nea, nea)
    if(nea > 1)
    {
        ch[upper.tri(ch, diag = TRUE)] <- c(1, x$b[sum(x$N[1:6]) + 1:x$N[7]])
    }
    else
    {
        ch[1, 1] <- 1
    }
    ch <- t(ch)

    ## simulated RE used in computation
    simRE <- apply(seqMC, 1, function(u) ch %*% u)
    
    ## range
    range_simRE <- apply(simRE, 1, range)

    return(list(range = range_simRE, RE = simRE))
}

rangeCL <- function(x)
{
    nea <- sum(x$idea)
    nMC <- x$nMC
    var.time <- as.character(x$call$var.time)
    
    beta <- c(0, x$b[sum(x$N[1:4]) + 1:x$N[5]])
    nvar <- length(x$Names$Xvar)
    d <- as.data.frame(matrix(0, 100, nvar))
    d[, var.time] <- seq(x$rangeTlong[1], x$rangeTlong[2], length.out = 100)

    Xfixed <- model.matrix(x$form$fixed, data = d)
    mu <- Xfixed %*% beta

    zu <- 0
    if((nea > 0) & (nMC > 0))
    {
        u <- rangeRE(x)$RE
        Xrandom <- model.matrix(x$form$random, data = d)
        zu <- Xrandom %*% u
    }

    cl <- sweep(zu, 1, mu, "+")
    
    return(list(range = range(cl), CL = cl))
}


rangeCS <- function(x)
{
    nea <- sum(x$idea)
    nMC <- x$nMC
    var.time <- as.character(x$call$var.time)
    
    beta <- x$b[sum(x$N[1:4]) + 1:x$N[5]]
    nvar <- length(x$Names$Xvar)
    d <- as.data.frame(matrix(0, 100, nvar))
    d[, var.time] <- seq(x$rangeTlong[1], x$rangeTlong[2], length.out = 100)

    ddt <- derivMat(x$form$fixed, x$form$random, d, var.time)

    #Xfixed <- model.matrix(x$form$fixed, data = d)
    Xfixed <- ddt[, 1:length(beta), drop = FALSE]
    mu <- Xfixed %*% beta

    zu <- 0
    if((nea > 0) & (nMC > 0))
    {
        u <- rangeRE(x)$RE
        #Xrandom <- model.matrix(x$form$random, data = d)
        Xrandom <- ddt[, x$N[5] + 1:nea, drop = FALSE]
        zu <- Xrandom %*% u
    }

    cs <- sweep(zu, 1, mu, "+")
    
    return(list(range = range(cs), CS = cs))
}

#' Standard methods for estimated models
#' 
#' coef and vcov  methods for estimated jointLPM models.
#' 
#' 
#' @aliases coef.jointLPM vcov.jointLPM
#' @param object an object of class \code{jointLPM}
#' @param ...  other arguments. There are ignored in these functions.
#' @return For \code{coef}, the vector of the estimated parameters.
#' 
#' For \code{vcov}, the variance-covariance matrix of the estimated parameters.
#' 
#' @author Viviane Philipps
#' 
#' @name StandardMethods





#' @export
coef.jointLPM <- function(object, ...)
{
    
 if(object$conv %in% c(1,2,3))
 {
     res <- object$best

     if(object$N[7]>0)
     {
         res[sum(object$N[2:6])+1:object$N[7]] <- object$cholesky[-1]
         
         names(res) <- sub("varcov","cholesky",names(res))
     }
 }
 else
 {
  res <- NA
  cat("Output can not be produced since the program stopped abnormally. \n")
 }
 
 return(res)
}


#' @export
vcov.jointLPM <- function(object, ...)
{
    
 if(object$conv %in% c(1,2,3))
 {
  res <- matrix(0,length(object$best),length(object$best))
  res[upper.tri(res,diag=TRUE)] <- object$V
  res <- t(res)
  res[upper.tri(res,diag=TRUE)] <- object$V

  noms <- sub("varcov","cholesky",names(object$best))
  colnames(res) <- noms
  rownames(res) <- noms
 }
 else
 {
  res <- NA
  cat("Output can not be produced since the program stopped abnormally. \n")
 }

 return(res)
}

module modirtsre

  implicit none
  integer,save ::ny,ns,nv,idiag,nvc,nea,ncor &
       ,maxmes,nobs,nef,ncontr,ntrtot,nalea &
       ,nySPL,ntotvalSPL,nyORD,ntotvalORD,npmtot &
       ,nMC,methInteg,nmescur &
       ,nvarxevt,nbevt,logspecif,idtrunc,nvdepsurv,nrisqtot,nxevt,nasso &
       ,nassoCL,nassoCS,nassoRE,expectancy
  integer,dimension(:),allocatable,save::typrisq,nz,nprisq,nevtparx,nxcurr
  integer,dimension(:),allocatable,save::nassoCLevt,nassoCSevt,nassoREevt
  integer,dimension(:),allocatable,save::nonlinCL,nonlinCS,nonlinRE
  double precision,dimension(:),allocatable,save::Y,uniqueY,minY,maxY,rangeY
  double precision,dimension(:,:),allocatable,save ::X,zi
  double precision,dimension(:),allocatable,save::Tsurv0,Tsurv,Tsurvint
  double precision,dimension(:,:),allocatable,save::Tsurv0_st2,Tsurv_st2
  integer,dimension(:),allocatable,save::Devt,ind_survint
  integer,dimension(:),allocatable,save::idtdv,idsurv
  integer,dimension(:),allocatable,save::idea,idg,idcor,idcontr,indiceY,indiceYinf
  integer,dimension(:),allocatable,save::idlink,ntr
  integer,dimension(:,:),allocatable,save::nmes
  integer,dimension(:),allocatable,save::nvalSPL,nvalORD
  double precision,dimension(:),allocatable,save :: seqMC
  double precision,dimension(:),allocatable,save :: epsY
  double precision,dimension(:,:),allocatable,save::zitr
  double precision,dimension(:),allocatable,save::mm,mm1,mm2,im,im1,im2
  double precision,dimension(:),allocatable,save::Tmm,Tmm1,&
       Tmm2,Tmm3,Tim,Tim1,Tim2,Tim3,Tmm0,Tmm01,Tmm02,Tmm03,Tim0, &
       Tim01,Tim02,Tim03,Tmmt,Tmmt1,Tmmt2,Tmmt3,Timt,Timt1,&
       Timt2,Timt3
  double precision,dimension(:,:),allocatable,save::Tmm_st2,Tmm1_st2,&
       Tmm2_st2,Tmm3_st2,Tim_st2,Tim1_st2,Tim2_st2,Tim3_st2,Tmm0_st2,&
       Tmm01_st2,Tmm02_st2,Tmm03_st2,Tim0_st2,Tim01_st2,Tim02_st2,Tim03_st2
!  double precision,dimension(:),allocatable,save::Tmm_est,Tmm1_est &
!       ,Tmm2_est,Tmm3_est,Tim_est,Tim1_est,Tim2_est,Tim3_est
  double precision,dimension(:),allocatable,save::brisq_est
  !  double precision,save::vrais_surv
  integer,dimension(:),allocatable,save::fix
  double precision,dimension(:),allocatable,save::bfix
  integer,dimension(:),allocatable,save::idst        
  integer,dimension(2),save::nXcl        
  integer,dimension(2),save::id_nXcl      
  double precision,dimension(2),save::centerpoly
  double precision,dimension(:,:),allocatable,save::Xcl_Ti,Xcl_GK,Xcl0_GK 
  double precision,dimension(:,:),allocatable,save::Xcs_Ti,Xcs_GK,Xcs0_GK 
  
end module modirtsre




subroutine loglik(Y0,X0,Tentr0,Tevt0,Devt0,ind_survint0 &
     ,idea0,idg0,idcor0,idcontr0,idsurv0,idtdv0 &
     ,typrisq0,nz0,zi0,nbevt0,idtrunc0,logspecif0 &
     ,ny0,ns0,nv0,nobs0,nmes0,idiag0,ncor0,nalea0&
     ,npm0,b0,nfix0,bfix0,epsY0,idlink0,nbzitr0,zitr0,uniqueY0,indiceY0 &
     ,indiceYinf0,nvalSPLORD0,fix0,methInteg0,nMC0,dimMC0,seqMC0 &
     ,idst0,nXcl0,Xcl_Ti0,Xcl_GK0,Xcs_Ti0,Xcs_GK0,nonlin0,centerpoly0 &
     ,expectancy0,loglik_res)

  use modirtsre

  IMPLICIT NONE

  !Declaration des variables en entree
  integer,intent(in)::nv0,ny0,nMC0,methInteg0,dimMC0,nfix0
  integer, intent(in)::ns0,nobs0,idiag0,npm0,ncor0,nalea0
  integer,intent(in)::idtrunc0,logspecif0,nbevt0,expectancy0
  double precision, dimension(ns0),intent(in)::Tentr0,Tevt0
  integer, dimension(ns0),intent(in)::ind_survint0,Devt0
  integer, dimension(nv0),intent(in)::idtdv0,idsurv0
  integer,dimension(nbevt0),intent(in)::typrisq0,nz0
  double precision,dimension(maxval(nz0),nbevt0),intent(in)::zi0    
  double precision,dimension(ny0),intent(in)::epsY0
  integer, dimension(ny0),intent(in)::idlink0,nbzitr0,nvalSPLORD0
  double precision,dimension(maxval(nbzitr0),ny0),intent(in)::zitr0
  integer,dimension(nobs0),intent(in)::indiceY0, indiceYinf0
  double precision,dimension(sum(nvalSPLORD0(:))),intent(in)::uniqueY0
  integer, dimension(nv0),intent(in)::idea0,idg0,idcor0,idcontr0
  integer,dimension(ns0,ny0)::nmes0   
  double precision,dimension(nobs0),intent(in)::Y0
  double precision,dimension(nobs0*nv0),intent(in)::X0
  integer,dimension(npm0+nfix0),intent(in)::fix0
  double precision,dimension(dimMC0*nMC0),intent(in)::seqMC0
  integer,dimension(nbevt0),intent(in)::idst0   
  integer,dimension(2),intent(in)::nXcl0    
  double precision,dimension(2),intent(in)::centerpoly0
  double precision,dimension(ns0,nXcl0(1)),intent(in)::Xcl_Ti0,Xcs_Ti0
  double precision,dimension(15*ns0,nXcl0(2)),intent(in)::Xcl_GK0,Xcs_GK0
  integer,intent(in),dimension(3*nbevt0)::nonlin0
  double precision, dimension(npm0), intent(in) :: b0
  double precision, dimension(nfix0), intent(in) :: bfix0
  
  !Declaration des variables en sortie
  double precision,intent(out)::loglik_res

  !Variables locales
  integer::jtemp,i,j,ier,k,ktemp,yk,k1,k2,mi,nbfix,p
  integer::ke,it,npmtot0
  double precision::eps
  double precision,external::vrais


  !! appel GetRNGstate() de R
  call getrand()
  

  maxmes=0
  do i=1,ns0
     mi=sum(nmes0(i,:))
     if (mi.gt.maxmes) then
        maxmes=mi
     end if
  end do


  allocate(rangeY(ny0),minY(ny0),maxY(ny0),idlink(ny0),ntr(ny0),epsY(ny0))

  nySPL=0
  nyORD=0
  rangeY=0
  epsY=epsY0
  do k=1,ny0
     idlink(k)=idlink0(k)
     minY(k)=zitr0(1,k)
     maxY(k)=zitr0(nbzitr0(k),k)
     if (idlink(k).eq.2) then
        nySPL=nySPL+1
     end if
     if (idlink(k).eq.3) then
        nyORD=nyORD+1
     end if
  end do
  
  if(nySPL.gt.0) then 
     allocate(nvalSPL(nySPL))
     nvalSPL=0
  else
     allocate(nvalSPL(1))
     nvalSPL(1) = 0
  end if

  if(nyORD.gt.0) then
     allocate(nvalORD(nyORD))
     nvalORD=0
  else
     allocate(nvalORD(1))
     nvalORD(1) = 0
  end if

  k1=0
  k2=0
  do k=1,ny0
     if(idlink(k).eq.2) then
        k1=k1+1
        nvalSPL(k1)=nvalSPLORD0(k)
     else if (idlink(k).eq.3) then
        k2=k2+1
        nvalORD(k2)=nvalSPLORD0(k)
     end if
  end do
  ntotvalSPL=sum(nvalSPL(:))
  ntotvalORD=sum(nvalORD(:))

  methInteg = methInteg0
  nMC = nMC0
  
  if(all(idlink.ne.2)) then
     allocate(zitr(1,1))
     allocate(mm(1),mm1(1),mm2(1),im(1),im1(1),im2(1))
     mm(1)=0.d0
     mm1(1)=0.d0
     mm2(1)=0.d0
     im(1)=0.d0
     im1(1)=0.d0
     im2(1)=0.d0
  else
     allocate(zitr(-1:(maxval(nbzitr0)+2),nySPL))
     allocate(mm(ntotvalSPL),mm1(ntotvalSPL),mm2(ntotvalSPL),im(ntotvalSPL),im1(ntotvalSPL),im2(ntotvalSPL))
  end if


  zitr=0.d0  
  k1=0
  k2=0
  do k=1,ny0
     if (idlink(k).eq.0) ntr(k)=2     
     if (idlink(k).eq.1) ntr(k)=4
     if (idlink(k).eq.2) then
        k1=k1+1
        ntr(k)=nbzitr0(k)+2

        zitr(1:nbzitr0(k),k1)=zitr0(1:nbzitr0(k),k)
        zitr(-1,k1)=zitr(1,k1)
        zitr(0,k1)=zitr(1,k1)
        zitr(ntr(k)-1,k1)=zitr(ntr(k)-2,k1)
        zitr(ntr(k),k1)=zitr(ntr(k)-1,k1)
     end if
     if (idlink(k).eq.3) then
        k2 = k2+1
        ntr(k) = nvalORD(k2)-1
     end if
  end do

  ntrtot = sum(ntr)
  allocate(Y(nobs0),X(nobs0,nv0),uniqueY(ntotvalSPL+ntotvalORD) &
       ,idea(nv0),idg(nv0),idcor(nv0),idcontr(nv0),nmes(ns0,ny0) &
       ,indiceY(nobs0),indiceYinf(nobs0))

  allocate(Tsurv0(ns0),Tsurv(ns0),Tsurvint(ns0),ind_survint(ns0),Devt(ns0))
  allocate(Tsurv0_st2(ns0,15),Tsurv_st2(ns0,15))
  allocate(typrisq(nbevt0),nz(nbevt0),nprisq(nbevt0),nevtparx(nv0),nxcurr(nv0))
  allocate(idsurv(nv0),idtdv(nv0))
  allocate(idst(nbevt0), nonlinRE(nbevt0), nonlinCL(nbevt0), nonlinCS(nbevt0))
  allocate(nassoREevt(nbevt0), nassoCLevt(nbevt0), nassoCSevt(nbevt0))

  ! zi : contient noeuds pour hazard (ou min et max si Weibull)
  if(any(typrisq0.eq.3)) then
     allocate(zi(-2:maxval(nz0)+3,nbevt0))
  else
     allocate(zi(maxval(nz0),nbevt0))
  end if
  
  allocate(Xcl_Ti(ns0,nXcl0(1)),Xcl_GK(15*ns0,nXcl0(1)),Xcl0_GK(15*ns0,nXcl0(1)))
  allocate(Xcs_Ti(ns0,nXcl0(1)-1),Xcs_GK(15*ns0,nXcl0(1)-1),Xcs0_GK(15*ns0,nXcl0(1)-1))

  eps=1.d-20

  ! enregistrement pour les modules
  nbevt=nbevt0    
  typrisq=typrisq0
  idtrunc=idtrunc0
  Tsurv0=Tentr0   
  Tsurv=Tevt0    
  devt=devt0    
  ind_survint=ind_survint0
  logspecif=logspecif0 
  Tsurvint=Tsurv
  ny=ny0
  ns=ns0
  nv=nv0
  nobs=nobs0
  ncor=ncor0
  nalea=nalea0
  idiag=idiag0
  nXcl=nXcl0
  centerpoly = centerpoly0
  
  do ke = 1,nbevt
     ! association(s) sur l'evenement ke
     idst(ke) = idst0(ke)
     nonlinRE(ke) = nonlin0(ke)
     nonlinCL(ke) = nonlin0(nbevt + ke)
     nonlinCS(ke) = nonlin0(2 * nbevt + ke)

     ! var expl aux 15 points de quadrature pour niveau courant
     if(idst(ke).ne.1) then ! besoin de Xcl_GK[,1] pour fct_risq_base
        Xcl_Ti=Xcl_Ti0
        Xcl_GK=Xcl_GK0(:,1:nXcl(1))
        if (idtrunc.eq.1) then
           Xcl0_GK=Xcl_GK0(:,(nXcl(1)+1):nXcl(2))
        end if

        ! temps pour les integrales 0->Ti et 0->T0i
        do i=1,ns0  
           do p=1,15
              Tsurv_st2(i,p) = Xcl_GK(15*(i-1)+p,1)
              if (idtrunc.eq.1) then
                 Tsurv0_st2(i,p) = Xcl0_GK(15*(i-1)+p,1) 
              end if
           end do
        end do
     end if

     ! var expl aux 15 points de quadrature pour pente courante
     if(idst(ke).eq.3 .or. idst(ke).eq.4) then
        Xcs_Ti=Xcs_Ti0
        Xcs_GK=Xcs_GK0(:,1:(nXcl(1)-1))
        if (idtrunc.eq.1) then
           Xcs0_GK=Xcs_GK0(:,nXcl(1):(nXcl(2)-2))
        end if
     end if
     
  end do
  
  !     if (verbose==1) write(*,*)'ntotvalSPL',ntotvalSPL

  if (ntotvalSPL+ntotvalORD.gt.0) uniqueY(1:ntotvalSPL+ntotvalORD)=uniqueY0(1:ntotvalSPL+ntotvalORD)

  nmes=0
  Y=0.d0
  X=0.d0
  idea=0
  idg=0
  idcor=0
  idcontr=0
  idsurv=0
  idtdv=0
  ktemp=0

  do k=1,nv
     idsurv(k)=idsurv0(k)
     idtdv(k)=idtdv0(k)
     idea(k)=idea0(k)
     idg(k)=idg0(k)
     idcor(k)=idcor0(k)
     idcontr(k)=idcontr0(k)

     jtemp=0
     DO i=1,ns
        do yk=1,ny            
           if (k.eq.1) then
              nmes(i,yk)=nmes0(i,yk)   !dim(nmes)=ns*ny    
              do j=1,nmes(i,yk)
                 jtemp=jtemp+1
                 Y(jtemp)=Y0(jtemp)
                 indiceY(jtemp)=indiceY0(jtemp)
                 indiceYinf(jtemp)=indiceYinf0(jtemp)
                 ktemp=ktemp+1
                 X(jtemp,k)=X0(ktemp)
              end do
           else
              do j=1,nmes(i,yk)
                 ktemp=ktemp+1
                 jtemp=jtemp+1
                 X(jtemp,k)=X0(ktemp)
              end do
           end if
        end do
     end do
  end do
          
  id_nXcl(1)=sum(idg)-1 !sans intercept
  id_nXcl(2)=sum(idea)
  
  ! definition Tsurvint 
  nvdepsurv=0
  if(sum(ind_survint).gt.0) then
     nvdepsurv=1
     do k=1,nv
        if (idtdv(k).eq.1) then
           it=0
           do i=1,ns
              Tsurvint(i)=X(it+1,k)
              it=it+maxval(nmes(i,:))
           end do
        end if
     end do
  end if


  ! prm fixes
  npmtot0 = npm0+nfix0
  allocate(fix(npmtot0))
  fix=0
  fix(1:npmtot0)=fix0(1:npmtot0)
  nbfix=sum(fix)
  if(nbfix.eq.0) then
     allocate(bfix(1))
  else
     allocate(bfix(nbfix))
  end if
  bfix(1:nbfix)=bfix0(1:nbfix)

  ! creation des parametres

  nprisq=0
  nrisqtot=0

  do ke=1,nbevt

     nz(ke)=nz0(ke) ! nb de noeuds pour hazard (ou 2 pr Weibull)

     if (typrisq(ke).eq.1) then
        nprisq(ke)=nz(ke)-1
     end if
     if (typrisq(ke).eq.2) then
        nprisq(ke)=2
     end if
     if (typrisq(ke).eq.3) then
        nprisq(ke)=nz(ke)+2
     end if

     nrisqtot = nrisqtot+nprisq(ke)  ! nb total de prm pour hazards
     zi(1:nz(ke),ke)=zi0(1:nz(ke),ke)
  end do

  ! nvarxevt = nombre total de coef pour survie (sans prm hazard)
  nxevt=0
  nevtparx=0
  do j=1,nv

!     if(idtdv(j).ne.1) then

        if(idsurv(j).eq.1) then
           nevtparx(j) = 1
           nxevt = nxevt + 1
        end if
        if(idsurv(j).eq.2) then 
           nevtparx(j) = nbevt
           nxevt = nxevt + 1
        end if
!     end if

  end do

  nvarxevt = sum(nevtparx) !+ nvdepsurv

  nea=0
  nef=0
  ncontr=0
  do k=1,nv
     if (idg(k).eq.1) then
        nef = nef + 1
     end if
     nea=nea+idea(k)
     ncontr=ncontr+idcontr(k)*(ny-1) 
  end do
  nef = nef - 1 !intercept pas estime

 nasso = 0
 nassoRE = 0
 nassoCL = 0
 nassoCS = 0
 do ke=1,nbevt
    nassoREevt(ke) = 0
    nassoCLevt(ke) = 0
    nassoCSevt(ke) = 0
    
    if(idst(ke).eq.1) then
       if(nonlinRE(ke).eq.0) then
          nassoRE = nassoRE + nea
          nassoREevt(ke) = nea
       else
          nassoRE = nassoRE + 3 * nea
          nassoREevt(ke) = 3 * nea
       end if
       
    else
       if(idst(ke).eq.2 .or. idst(ke).eq.4) then
          if(nonlinCL(ke).eq.0) then
             nassoCL = nassoCL + 1
             nassoCLevt(ke) = 1
          else
             nassoCL = nassoCL + 3
             nassoCLevt(ke) = 3
          end if
       end if
       
       if(idst(ke).eq.3 .or. idst(ke).eq.4) then
          if(nonlinCS(ke).eq.0) then
             nassoCS = nassoCS + 1
             nassoCSevt(ke) = 1
          else
             nassoCS = nassoCS + 3
             nassoCSevt(ke) = 3
          end if
       end if
    end if
    nasso = nasso + nassoREevt(ke) + nassoCLevt(ke) + nassoCSevt(ke)
 end do

 if (idiag.eq.1) then
     nvc=nea-1
  else if(idiag.eq.0) then
     nvc=(nea+1)*nea/2-1
  end if

  npmtot = nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+ny
  ! points qmc
  if(methInteg.ne.3) then 
     allocate(seqMC(1))
  else
     allocate(seqMC(dimMC0*nMC))
     seqMC = seqMC0(1:dimMC0*nMC) 
  end if



  ! creer base de splines si au moins un hazard splines
  if(any(typrisq.eq.3)) then
     allocate(Tmm(ns*nbevt),Tmm1(ns*nbevt),Tmm2(ns*nbevt),Tmm3(ns*nbevt),        &
              Tim(ns*nbevt),Tim1(ns*nbevt),Tim2(ns*nbevt),Tim3(ns*nbevt),        &
              Tmm0(ns*nbevt),Tmm01(ns*nbevt),Tmm02(ns*nbevt),Tmm03(ns*nbevt),    &
              Tim0(ns*nbevt),Tim01(ns*nbevt),Tim02(ns*nbevt),Tim03(ns*nbevt),    &
              Tmmt(ns*nbevt),Tmmt1(ns*nbevt),Tmmt2(ns*nbevt),Tmmt3(ns*nbevt),    &
              Timt(ns*nbevt),Timt1(ns*nbevt),Timt2(ns*nbevt),Timt3(ns*nbevt),    &
              Tmm_st2(15,ns*nbevt),Tmm1_st2(15,ns*nbevt),Tmm2_st2(15,ns*nbevt),Tmm3_st2(15,ns*nbevt),        &
              Tmm0_st2(15,ns*nbevt),Tmm01_st2(15,ns*nbevt),Tmm02_st2(15,ns*nbevt),Tmm03_st2(15,ns*nbevt))

     do ke=1,nbevt
        if(typrisq(ke).eq.3) then
           call splines_irtsre(ke)
           call splines_irtsre2(ke)
        end if
     end do

  end if



  ! base de splines transfos
  if (any(idlink.eq.2)) then 
     call design_splines_irtsre(ier)
     if (ier.eq.-1) then
        loglik_res=-1.d9
        go to 1589
     end if
  end if

  !vraisemblance (=0) ou temps de sejour (=1)
  expectancy = expectancy0

  ! calcul de la vraisemblance
  loglik_res = vrais(b0,npm0)

  
1589 continue

  !! appel PutRNGstate() de R
  call putrand()

  
  if (any(typrisq.eq.3)) then
     deallocate(Tmm,Tmm1,Tmm2,Tmm3,Tim,Tim1,Tim2,Tim3,Tmm0,    &
          Tmm01,Tmm02,Tmm03,Tim0,Tim01,Tim02,Tim03,Tmmt,Tmmt1,     &
          Tmmt2,Tmmt3,Timt,Timt1,Timt2,Timt3,    &
          Tmm_st2,Tmm1_st2,Tmm2_st2,Tmm3_st2,    &   
          Tmm0_st2,Tmm01_st2,Tmm02_st2,Tmm03_st2)
  endif

  deallocate(Tsurv0,Tsurv,Tsurvint &
       ,Tsurv0_st2, Tsurv_st2 & 
       ,ind_survint,zi,devt,typrisq,nz,nprisq,idsurv,idtdv &
       ,nevtparx,nxcurr)

  deallocate(Y,X,idea,idg,idcor,idcontr,nmes,uniqueY,indiceY,indiceYinf,ntr)


  deallocate(zitr,mm,mm1,mm2,im,im1,im2,minY,maxY,rangeY,idlink,nvalSPL,nvalORD,epsY)

  deallocate(fix,bfix,seqMC)

  deallocate(Xcl_Ti,Xcl_GK,Xcl0_GK, Xcs_Ti,Xcs_GK,Xcs0_GK)
  deallocate(idst, nonlinRE, nonlinCL, nonlinCS, nassoREevt, nassoCLevt, nassoCSevt)
  
  return
  
end subroutine loglik


!-------------------------------------!
!     LOG-LIKELIHOOD all subjects     !
!-------------------------------------!

double precision function vrais(b,m)


  use modirtsre,only:ns,nmes,nmescur

  implicit none

  integer::m,i
  double precision::vrais_i,temp
  double precision,dimension(m)::b
  

  nmescur=0
  vrais=0.d0
  do i=1,ns
  
!     print*,"## NEW SUBJECT i=",i 
  
     temp=vrais_i(b,m,i)  
     
     vrais = vrais + temp
     if (temp.eq.-1.d9 .or. temp/temp.ne.1) then 
        !if (temp/temp.ne.1) write(*,*)"i=",i,"vrais= ",temp
        !if (temp.eq.-1.d9) then 
        vrais = -1.d9
        !print*,"dans vrais i=",i," vrais=",vrais," m=",m," b=",b
        ! if(verbose==1) write(*,*)"i=",i,"vrais= ",temp
        goto 541
     end if
     nmescur = nmescur + sum(nmes(i,:))
  end do
!print*, "vrais = ", vrais  
541 continue
  return

end function vrais



!-------------------------------------!
!      individual LOG-LIKELIHOOD      !
!-------------------------------------!

double precision function vrais_i(b,npm,i) 

  use modirtsre
!  use optim

  IMPLICIT NONE
  integer ::i,j,k,l,m,jj,npm,ll,ii,numSPL,ykord
  integer ::ier,kk,j1,j2,sumMesYk,yk,sumntr,ke,sumnrisq
  integer::nevtxcurr,nxevtcurr,sumnassoCL,sumnassoCS
  double precision,dimension(maxmes,nv) ::X00
  double precision,dimension(maxmes,nea) ::Z
  double precision,dimension(maxmes,(ncontr+sum(idcontr)))::X01
  double precision,dimension(ncontr+sum(idcontr))::b01
  double precision,dimension(nea,nea) ::Ut
  double precision,dimension(maxmes,maxmes) ::VC,Corr
  double precision,dimension(npm) :: b
  double precision,dimension(maxmes*(maxmes+1)/2) ::Vi
  double precision,dimension(nv) :: b0
  double precision,dimension(npmtot)::b1
  double precision,dimension(nxevt*nbevt)::Xevt,bevt
  double precision,dimension(nbevt)::bevtint
  double precision,dimension(maxval(nprisq))::brisq
  double precision,dimension(3)::bassoCL
  double precision,dimension(3)::bassoCS
  double precision,dimension(nef)::beta_ef 

  double precision :: eps,det,som,eta0
  double precision ::Y4,jacobien,beta_densite,ytemp
  double precision,dimension(maxmes) :: mu,Y1,Y2,Y3,tcor
  double precision,dimension(-1:maxval(ntr)-3)::splaa
  double precision::aa1,bb1,dd1,aa,bb,betai,cc1
  double precision,dimension(nea)::ui,usim
  double precision,dimension(maxmes)::wi,wsim
  double precision,dimension(ny)::asim
  double precision::ai,binf,bsup
  double precision,dimension(nbevt)::risq,surv,surv0,survint
  double precision::SX,x22,div,vrais_Y,vrais_surv,varexpsurv
  double precision::surv0_glob,surv_glob,fevt,easurv,alnormbinf
  double precision,external::alnorm, poly, logit, sigmoid
  double precision::pred_cl_Ti 
  double precision::som_T0,som_Ti
  
  ! definir le nombre total de mesures pour le sujet i : nmestot (valable que pour cette fonction)

  ! if (verbose==1) write(*,*)'i',i 
  b1=0.d0
  eps=1.D-20
  l=0
  m=0
  do k=1,npmtot
     if(fix(k).eq.0) then
        l=l+1
        b1(k)=b(l)
     end if
     if(fix(k).eq.1) then
        m=m+1
        b1(k)=bfix(m)
     end if
  end do

  !----------- rappel des parametres utilises ---------



  !        write(*,*)'i',i,nmescur,nmescur + nmes(i)


  Ut=0.d0
  Ut(1,1)=1.d0
  if (nea>1) then 

     If (idiag.eq.1) then
        do j=2,nea
           do k=2,nea
              if (j.eq.k) then
                 Ut(j,k)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+j-1)
              else
                 Ut(j,k)=0.d0
              end if
           end do
        end do
     end if

     If (idiag.eq.0) then
        do j=2,nea
           do k=1,j
              Ut(j,k)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+k-1+j*(j-1)/2)
           end do
        end do
     end if

  end if
  
  vrais_Y=0.d0
  jacobien=0.d0
  ! -------- creation de Vi = ZiGZi'+se*seIni ----------
  ! creation de Zi

  Z=0.d0
  l=0
  do k=1,nv
     if (idea(k).eq.1) then
        l=l+1
        do j=1,sum(nmes(i,:))
           Z(j,l)=dble(X(nmescur+j,k))  
        end do
     end if
  end do

  !matrice Corr variance de BM/AR

  Corr=0.d0
  tcor=0.d0
  if (ncor.gt.0) then
     do k=1,nv
        if (idcor(k).eq.1) then
           do j=1,sum(nmes(i,:))
              tcor(j) = X(nmescur+j,k)
           end do
        end if
     end do
     do j1=1,sum(nmes(i,:))
        do j2=1,sum(nmes(i,:))
           if (ncor.eq.1) then 
              Corr(j1,j2) = Corr(j1,j2)+b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)*min(tcor(j1),tcor(j2))
           else if (ncor.eq.2) then
              Corr(j1,j2) = Corr(j1,j2)+b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor)* &
                   exp(-b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+1)*abs(tcor(j1)-tcor(j2)))
           end if
        end do
     end do

     ! passer en cholesky si on a de l ordinal
     if(any(idlink.eq.3)) then
        jj=0
        Vi=0.d0
        do j=1,sum(nmes(i,:))
           do k=j,sum(nmes(i,:))
              jj=j+k*(k-1)/2
              Vi(jj)=Corr(j,k)
           end do
        end do

        CALL DMFSD(Vi,sum(nmes(i,:)),EPS,IER)

        Corr=0.d0
        do j=1,sum(nmes(i,:))
           do k=1,j
              Corr(j,k)=Vi(k+j*(j-1)/2)
           end do
        end do
     end if
  end if

  ! creation de Y1
  Y1=0.d0
  splaa=0.d0

  sumMesYk = 0
  sumntr=0
  numSPL=0
  do yk=1,ny

     if (idlink(yk).eq.0) then  ! Linear link

        do j=1,nmes(i,yk)
           Y1(sumMesYk+j)=(dble(Y(nmescur+sumMesYk+j))-b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)) &
                /abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))
            
           jacobien = jacobien - log(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))
        end do

     else if (idlink(yk).eq.1) then  ! Beta link


        aa1=exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1))/ &
             (1+exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)))
        bb1=exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2))/ &
             (1+exp(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+2)))
        bb1=aa1*(1.d0-aa1)*bb1

        cc1=abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+3))

        dd1=abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+4))

        aa=aa1*aa1*(1-aa1)/bb1-aa1
        bb=aa*(1-aa1)/aa1

        do j=1,nmes(i,yk)

           ytemp=(dble(Y(nmescur+sumMesYk+j))-minY(yk)+epsY(yk))/(maxY(yk)-minY(yk)+2*epsY(yk))
           Y1(sumMesYk+j)=(betai(aa,bb,ytemp)-cc1)/dd1


           if (Y1(sumMesYk+j).eq.999.d0) then
              vrais_i=-1.d9
              !print*,"-1.d9 Y1=999"
              goto 654
           end if

           jacobien = jacobien + log(abs(beta_densite(ytemp,aa,bb))/dd1)
           jacobien=jacobien-log(abs(maxY(yk)-minY(yk)+2*epsY(yk)))
        end do

     else if (idlink(yk).eq.2) then ! Splines link
        numSPL=numSPL+1

        splaa=0.d0
        eta0=0.d0
        eta0=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)

        do kk=2,ntr(yk)
           splaa(kk-3)=b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+kk)&
                *b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+kk)
        end do
        !if(i==1 .and. id==0 .and. jd==0) print*,"eta0=",eta0,"splaa=",sqrt(splaa)
        do j=1,nmes(i,yk)
           ll=0
           !if(i==1 .and. id==0 .and. jd==0) print*,"Y=",Y(nmescur+sumMesYk+j)
           if (Y(nmescur+sumMesYk+j).eq.zitr(ntr(yk)-2,numSPL)) then
              ll=ntr(yk)-3
           end if

           som=0.d0
           do kk = 2,ntr(yk)-2
              if ((Y(nmescur+sumMesYk+j).ge.zitr(kk-1,numSPL)).and. &
                   (Y(nmescur+sumMesYk+j).lt.zitr(kk,numSPL))) then
                 ll=kk-1
              end if
           end do

           if (ll.lt.1.or.ll.gt.ntr(yk)-3) then          
              vrais_i=-1.d9
              !print*,"-1.d9 ll<1 ou ll>ntr-3",ll!," ntr=",ntr(yk)," numSPL=",numSPL," y=",Y(nmescur+sumMesYk+j)
              goto 654
           end if
           if (ll.gt.1) then
              do ii=2,ll
                 som=som+splaa(ii-3)
              end do
           end if



           Y1(sumMesYk+j)=eta0+som +splaa(ll-2)*im2(indiceY(nmescur+sumMesYk+j)) &
                +splaa(ll-1)*im1(indiceY(nmescur+sumMesYk+j))&
                + splaa(ll)*im(indiceY(nmescur+sumMesYk+j))

           jacobien = jacobien + log(splaa(ll-2)*mm2(indiceY(nmescur+sumMesYk+j)) &
                +splaa(ll-1)*mm1(indiceY(nmescur+sumMesYk+j))&
                +splaa(ll)*mm(indiceY(nmescur+sumMesYk+j)))

           !print*,"jac=",jacobien
           !print*,"ll =",ll
           !print*,"splaa =",splaa(ll-2)
           !print*,"nmescur+sumMesYk+j =",nmescur+sumMesYk+j
           !print*,"indiceY= ",indiceY(nmescur+sumMesYk+j)
           !print*,"mm =",mm(nmescur+sumMesYk+j)
           !print*,"mm1 =",mm1(nmescur+sumMesYk+j)
           !print*,"mm2 =",mm2(nmescur+sumMesYk+j)
           !print*,"Y =",Y(nmescur+sumMesYk+j)

           !                write(*,*)'Y',Y1(sumMesYk+j),sumMesYk,yk,j,jacobien
        end do
     else if (idlink(yk).eq.3) then  ! Threshold link
        do j=1,nmes(i,yk)
           Y1(sumMesYk+j)=Y(nmescur+sumMesYk+j)
           !if(nmescur.lt.15) then
           !   print*,"nmescur=",nmescur," sumMesYk=",sumMesYk," j=",j
           !   print*,"Y=",Y(nmescur+sumMesYk+j), "  Y1=",Y1(sumMesYk+j)
           !end if
        end do
     end if
     sumMesYk=sumMesYk+nmes(i,yk)
     sumntr=sumntr+ntr(yk)
     
  end do !fin boucle yk


  !         if (i.lt.3)then
  !            write(*,*)'nmes',nmes(i),b1((nef+ncontr+nvc+1):npm),nef+ncontr
  !            write(*,*)'Y1',Y1
  !         end if




  ! contribution individuelle a la vraisemblance
  ! print*,"i=",i," -ni*log(2pi)=",-sum(nmes(i,:))*dlog(dble(2*3.14159265)), " log(det)=",det
  ! print*,"Vi=",VC
  ! sans classes latentes : ng=1
  vrais_i=0.d0


  b0=0.d0
  b01=0.d0
  l=0
  m=0
  X00=0.d0
  X01=0.d0
  do k=1,nv
     if (idg(k).ne.0) then
        l=l+1
        do j=1,sum(nmes(i,:))
           X00(j,l)=dble(X(nmescur+j,k))  
        end do
        ! idg ne 0 pour l'intercept forcement donc on met le parm a 0
        if (k.eq.1) then
           b0(l)=0.d0
        else
           b0(l)=b1(nrisqtot+nvarxevt+nasso+l-1)
        end if
     end if

     !contrast : 
     if (idcontr(k).ne.0) then
        m=m+1
        sumMesYk=0
        do yk=1,ny
           ! creation matrice design des contrastes: X01
           do j=1,nmes(i,yk)
              X01(sumMesYk+j,(m-1)*ny+yk) = dble(X(nmescur+sumMesYk+j,k))
           end do
           sumMesYk=sumMesYk+nmes(i,yk)
           ! creation vecteur parms des contrastes: b01
           if (yk<ny) THEN
              b01((m-1)*ny+yk)=b1(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+yk)
           else
              b01((m-1)*ny+ny) =-sum(b1((nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+1) &
                    :(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+(ny-1))))
                   !-sum(b1(nrisqtot+nvarxevt+nasso+nef+(m-1)*(ny-1)+1 &
                   !:nef+(m-1)*(ny-1)+ny-1))
           end if
        end do
     end if
  end do

  som=0.d0
  som_T0=0.d0 !delayed entry
  som_Ti=0.d0
  do l=1,nMC

     vrais_Y=1.d0
     mu=0.d0

!!!!!!!!! MC pour EA et BM/AR !!!!!!!!!

     if(methInteg.eq.1) then 
        ! !!!!!!!!!!!!! MCO !!!!!!!!!!!!!

        ! simuler les effets aleatoires
        if(nea.gt.0) then
           x22=0.d0
           SX=1.d0
           do j=1,nea
              call bgos(SX,0,usim(j),x22,0.d0)
              !print*,"usim=",usim(j)
           end do
           ui=0.d0
           ui=matmul(Ut,usim)
           !print*,"usim=",usim(j)," ui=",ui, " Ut=",Ut, "  nea=",nea
        end if

        ! simuler le BM ou AR
        if(ncor.gt.0) then
           x22=0.d0
           SX=1.d0
           do j=1,sum(nmes(i,:))
              call bgos(SX,0,wsim(j),x22,0.d0)
           end do
           wi=0.d0
           wi=matmul(Corr,wsim)
        end if

     else if(methInteg.eq.2) then 
        ! !!!!!!!!!!!!! MCA !!!!!!!!!!!!!


        if(mod(l,2).eq.0) then
           ! si l est pair on prend l'oppose des precedents
           ui = -ui
           wi = -wi
        else
           ! sinon on simule des nouveaux

           ! simuler les effets aleatoires
           if(nea.gt.0) then
              x22=0.d0
              SX=1.d0
              do j=1,nea
                 call bgos(SX,0,usim(j),x22,0.d0)
              end do
              ui=0.d0
              ui=matmul(Ut,usim)
           end if

           ! simuler le BM ou AR
           if(ncor.gt.0) then
              x22=0.d0
              SX=1.d0
              do j=1,sum(nmes(i,:))
                 call bgos(SX,0,wsim(j),x22,0.d0)
              end do
              wi=0.d0
              wi=matmul(Corr,wsim)
           end if

        end if

     else 
        ! !!!!!!!!!!!!! QMC !!!!!!!!!!!!!

        ! simuler les effets aleatoires
        if(nea.gt.0) then
           usim=0.d0
           do j=1,nea
              usim(j)=seqMC(nMC*(j-1)+l)
           end do
           ui=0.d0
           ui=matmul(Ut,usim)
        end if
        
        ! simuler le BM ou AR
        if(ncor.gt.0) then
           wsim=0.d0
           do j=1,sum(nmes(i,:))
              wsim(j)=seqMC(nMC*(nea+j-1)+l)
           end do
           wi=0.d0
           wi=matmul(Corr,wsim)
        end if


     end if ! fin if methInteg


     ! esperance conditionnelle
     mu = matmul(X00,b0)+matmul(X01,b01)+matmul(Z,ui)

     if(ncor.gt.0) mu = mu+wi
     !if(i.lt.2) then
     !print*,"i=",i," nmes=",nmes(i,1)," nmescur=",nmescur
     !print*,"Xb=",matmul(X00,b0)+matmul(X01,b01)
     !print*,"mu=",mu
     !print*,"ui=",ui, " wi=",wi," b0=",b0, "X00=",X00, "Z=",Z
     !end if
     sumMesYk=0
     sumntr=0
     ykord=0
     do yk =1,ny

        if(idlink(yk).eq.3) then
           !! yk est ordinal
           ykord = ykord + 1

           !! MC pour simuler l'EA specifique au test
           ai=0.d0
           if(nalea.gt.0) then
              if(methInteg.eq.1) then
                 !! MCO
                 call bgos(SX,0,asim(yk),x22,0.d0)
                 ai = abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk))*asim(yk)
              else if(methInteg.eq.2) then
                 !! MCA
                 if(mod(l,2).eq.0) then
                    ! si l est pair on prend l'oppose du precedent
                    ai = -abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk))*asim(yk)
                 else
                    call bgos(SX,0,asim(yk),x22,0.d0)
                    ai = abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk))*asim(yk)
                 end if
              else
                 !! QMC
                 asim(yk) = seqMC(nMC*(nea+sum(nmes(i,:)))+l)
                 ai = abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk))*asim(yk)
              end if
           end if
!if(i.lt.4) print*,"i=",i," avant do j, vrais_Y=",vrais_Y
           do j=1,nmes(i,yk)

              !! on ajoute ai a mu
              if(nalea.gt.0) mu(sumMesYk+j) = mu(sumMesYk+j)+ai

              !! trouver binf et bsup tq binf < lambda + epsilon < bsup

              !! initialiser au premier seuil
              binf = b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+1)
              bsup = binf

              !! si Y>minY ajouter b1(..)^2
              if(indiceY(nmescur+sumMesYk+j).gt.1) then
                 do ll=2,min(indiceY(nmescur+sumMesYk+j),ntr(yk))
                    bsup = bsup + b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+ll)**2
                    if(ll.le.indiceYinf(nmescur+sumMesYk+j)) then
                       binf = binf + b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+sumntr+ll)**2
                    end if
                 end do
              end if
              !!print*,"Y=", Y(nmescur+sumMesYk+j), "binf=",binf, "bsup=",bsup
              !!print*, "indiceY = ", indiceY(nmescur+sumMesYk+j), "indiceY = ",indiceYinf(nmescur+sumMesYk+j)
              !if(i.lt.2)print*,"y=",Y1(sumMesYk+j)," indiceY=",indiceY(nmescur+sumMesYk+j),  &
              !     " Y=",Y(nmescur+sumMesYk+j)," mu=",mu(sumMesYk+j)
              !if(i.lt.2)print*," binf=",binf," bsup=",bsup
              !if(i.lt.2)print*,"se=",abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))
              !if(i.lt.2)print*,nrisqtot, nvarxevt, nasso, nef, ncontr, nvc, ncor, ntrtot, nalea, yk
              !! centrer et standardiser
              binf = (binf - mu(sumMesYk+j))/abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))
              bsup = (bsup - mu(sumMesYk+j))/abs(b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk))

              !if(i.lt.2) print*,"yk=",yk,"j=",j," nvalORD=",nvalORD(ykord)

              alnormbinf = 0.d0
              if(indiceYinf(nmescur+sumMesYk+j).gt.0) then
                 alnormbinf = alnorm(binf,.false.)
              end if
              !print*,"Y=",Y, "indiceY=",indiceY, "indiceYinf=",indiceYinf
              !!print*, "alnormbinf=",alnormbinf, "expect=", expectancy
              !if(i.lt.2) print*,"indice = ", indiceY(nmescur+sumMesYk+j)," binf=",binf," bsup=",bsup
              if(indiceY(nmescur+sumMesYk+j).eq.1) then
                 !! si Y=minY
                  !!print*,"min"
                 !! P(Y = minY) = P(N < binf)
                 vrais_Y = vrais_Y * alnorm(binf,.false.)
              else if(indiceY(nmescur+sumMesYk+j).eq.nvalORD(ykord)) then
                 !! si Y=maxY
                 !! print*,"max"
                 if(expectancy.eq.0) then 
                    !! P(Y = maxY) = P(N > bsup) -> loglik
                    vrais_Y = vrais_Y * (1.d0-alnorm(bsup,.false.))
                 else
                    if(indiceYinf(nmescur+sumMesYk+j).gt.1) then
                       !! P(Y in [m, maxY]) = P(N > binf) -> pour sojournTime
                       vrais_Y = vrais_Y * (1.d0-alnorm(binf,.false.))
                    end if
                 end if
                 !! j'ai verifie dans sojournTime.R que l'on n'a pas Y in [minY, maxY]
              else
                 !! minY < Y < maxY
                 !!print*,"entre min et max"
                 !!print*,""
                 !if(expectancy.eq.0) then
                    vrais_Y = vrais_Y * (alnorm(bsup,.false.)- alnormbinf)
                 !else
                 !   vrais_Y = vrais_Y * alnorm(bsup,.false.)
                 !end if
              end if
              !if(i.lt.2) print*,"vrais_Y=",vrais_Y
           end do


        else
           
           !! yk est continu
           !print*,"MC continu"
           if(nmes(i,yk).gt.0) then
              !! variance de Y|ui,wi
              VC=0.d0
              do j1=1,nmes(i,yk)
                 VC(j1,j1) = b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+nalea+yk)**2 !variance de l'erreur yk
                 if (nalea.eq.ny) then ! intercept aleatoire de yk
                    do j2=1,nmes(i,yk)
                       VC(j1,j2) = VC(j1,j2) + &
                            b1(nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor+ntrtot+yk)**2
                    end do
                 end if
              end do
              
              ! Vi en vecteur
              jj=0
              Vi=0.d0
              do j1=1,nmes(i,yk)
                 do j2=j1,nmes(i,yk)
                    jj=j1+j2*(j2-1)/2
                    Vi(jj)=VC(j1,j2)
                 end do
              end do
              
              ! inversion
              CALL dsinv(Vi,nmes(i,yk),eps,ier,det)
              if (ier.eq.-1) then
                 vrais_i=-1.d9
                 !print*,"-1.d9 dsinv continu MC"
                 !print*,"b=",b
                 !print*,"bfix=",bfix
                 !print*,"fix=",fix
                 goto 654
              end if
              
              ! retransformation du vecteur Vi en matrice :
              VC=0.d0
              do j1=1,nmes(i,yk)
                 do j2=1,nmes(i,yk)
                    if (j2.ge.j1) then
                       VC(j1,j2)=Vi(j1+j2*(j2-1)/2)
                    else
                       VC(j1,j2)=Vi(j2+j1*(j1-1)/2)
                    end if
                 end do
              end do
           
              ! calcul de la vrais
              Y2=0.d0
              Y3=0.d0
              Y4=0.d0
              do j=1,nmes(i,yk)
                 Y2(j) = Y1(sumMesYk+j)-mu(sumMesYk+j)
              end do
              Y3=matmul(VC,Y2)
              Y4=DOT_PRODUCT(Y2,Y3)
              
              div = (dble(2*3.14159265)**(dble(nmes(i,yk))/2))*sqrt(exp(det))
              vrais_Y = vrais_Y * exp(-Y4/2.d0)/div

           end if
        end if
        
        sumMesYk = sumMesYk+nmes(i,yk)
        sumntr = sumntr + ntr(yk)
     end do ! fin boucle yk

     ! partie survie

     if (nbevt.ne.0) then

        ! calcul de brisq en chaque composante et risq, surv et surv0 pour chaque evt
        risq=0.d0
        surv=0.d0
        surv0=0.d0
        survint=0.d0

        sumnrisq=0
        sumnassoCL=0
        sumnassoCS=0
        do ke=1,nbevt

           brisq=0.d0
           if (logspecif.eq.1) then
              do k=1,nprisq(ke)
                 brisq(k)=exp(b1(sumnrisq+k))
              end do
           else
              do k=1,nprisq(ke)
                 brisq(k)=b1(sumnrisq+k)*b1(sumnrisq+k)
              end do
           end if
           
           bassoCL=0.d0 
           do k=1,nassoCLevt(ke)
              bassoCL(k) = b1(nrisqtot+nvarxevt+sumnassoCL+k) !basso = eta = prm estime des EAs partages
           end do

           bassoCS=0.d0 
           do k=1,nassoCSevt(ke)
              bassoCS(k) = b1(nrisqtot+nvarxevt+nassoCL+sumnassoCS+k)
           end do
           
           beta_ef=0.d0  !beta_ef = prms des EFs necessaires pr pred curlev
           do k=1,nef
              beta_ef(k) = b1(nrisqtot+nvarxevt+nasso+k)
           end do

           if (idst(ke).eq.1) then  !bi
              call fct_risq_irtsre(i,ke,brisq,risq,surv,surv0,survint)  
           else !if(idst.eq.2) then   !niv.courant du processus latent
              call fct_risq_irtsre_2(i,ke,brisq,bassoCL,bassoCS,beta_ef,ui,risq,surv,surv0)  
           end if

           sumnrisq = sumnrisq + nprisq(ke)
           sumnassoCL = sumnassoCL + nassoCLevt(ke)
           sumnassoCS = sumnassoCS + nassoCSevt(ke)
        end do

 ! print*,"fct_risq ok"
        ! variables explicatives de la survie
        Xevt=0.d0
        bevt=0.d0
        bevtint=0.d0
        if (nxevt.ne.0) then    ! si varexpsurv

           m=0
           do ke=1,nbevt
              nevtxcurr=0
              do k=1,nv 

                 if (idtdv(k).ne.1) then

                    if (idsurv(k).eq.1) then  
                       m=m+1
                       bevt(m)=b1(nrisqtot+nevtxcurr+1)
                       Xevt(m)=X(nmescur+1,k)
                    else
                       if (idsurv(k).eq.2) then   
                          m=m+1
                          bevt(m)=b1(nrisqtot+nevtxcurr+ke)
                          Xevt(m)=X(nmescur+1,k)
                       end if
                    end if

                 else ! i.e timedepvar

                    if (idsurv(k).eq.1) then  
                       bevtint(ke)=b1(nrisqtot+nevtxcurr+1)
                    else 
                       if (idsurv(k).eq.2) then
                          bevtint(ke)=b1(nrisqtot+nevtxcurr+ke)
                       end if
                    end if
                 end if
                 nevtxcurr=nevtxcurr+nevtparx(k)
              end do
           end do

        end if

        ! vrais survie
        surv_glob=0.d0
        surv0_glob=0.d0
        varexpsurv=0.d0
        nxevtcurr=0
        fevt=0.d0
        pred_cl_Ti=0.d0
        easurv=0.d0
        m=0
        do ke=1,nbevt
        
           ! calculer Xevt * bevt
           varexpsurv=0.d0
           if (nxevt.ne.0) then   !si varexpsurv
              varexpsurv=DOT_PRODUCT(Xevt((nxevtcurr+1):(nxevtcurr+nxevt))&
                   ,bevt((nxevtcurr+1):(nxevtcurr+nxevt)))
           end if
           
           ! effets aleatoires partages 
           if (idst(ke).eq.1) then
              easurv=0.d0
              if(nea.gt.0) then      !si EA
                 if(nonlinRE(ke).eq.0) then ! asso lineaire
                    easurv=DOT_PRODUCT(ui,b1((nrisqtot+nvarxevt+m+1)&
                         :(nrisqtot+nvarxevt+m+nea)))
                    m = m+nea
                 else if(nonlinRE(ke).eq.1) then ! asso polynomiale
                    do k = 1,nea
                       easurv = easurv + poly(ui(k), b1((nrisqtot+nvarxevt+m+1)&
                            :(nrisqtot+nvarxevt+m+3)), 0.d0)
                       m = m+3
                    end do
                 else if (nonlinRE(ke).eq.2) then !  asso logistique
                    do k = 1,nea
                       easurv = easurv + sigmoid(ui(k), b1((nrisqtot+nvarxevt+m+1)&
                         :(nrisqtot+nvarxevt+m+3)))
                       m = m+3
                    end do
                 end if
              end if
           else
              m = m + nassoCLevt(ke) + nassoCSevt(ke)
           end if

           ! avoir evt au temps Ti si Devt=1
           if (Devt(i).eq.ke) then     !si sujet i a evt ke
              if (idst(ke).eq.1) then
                fevt=risq(ke)*exp(varexpsurv+easurv)   !fct de risq         
             else !if (idst(ke).eq.2 .or. idst(ke).eq.4) then
                !! j'ai ajouté l'asso dans fct_risq_irtsre_2
                ! ! niv courant process latent au tps Ti
                ! do ll=1,id_nXcl(1)
                !   pred_cl_Ti = pred_cl_Ti + Xcl_Ti(i,1+ll) * beta_EF(ll)  ! X(t) %*% beta
                ! end do
                ! do ll=1,id_nXcl(2)
                !   pred_cl_Ti = pred_cl_Ti + Xcl_Ti(i,1+nef+ll) * ui(ll)  ! Z(t) %*% ui
                ! end do

                ! if(nonlinCL(ke).eq.0) then
                !    pred_cl_Ti = EXP(pred_cl_Ti*b1(nrisqtot+nvarxevt+sumnassoRE+ke) )  ! exp(predcl*basso)
                ! else if(nonlinCL(ke).eq.1) then
                !    pred_cl_Ti = exp(poly(pred_cl_Ti, bassoCL))
                  
                ! else if(nonlinCL(ke).eq.2) then
                !    pred_cl_Ti = exp(logit(pred_cl_Ti, bassoCL))
                ! end if
                
                fevt=risq(ke)*exp(varexpsurv) !*pred_cl_Ti
             end if
              if (ind_survint(i).eq.1) then
                 fevt=fevt*exp(bevtint(ke))
              end if
           end if

           ! risque cumule jusque Ti
           if (idst(ke).eq.1) then
            Surv_glob=surv_glob + survint(ke)*exp(varexpsurv+easurv) + &
                  exp(bevtint(ke)+varexpsurv+easurv)*(surv(ke)-survint(ke))     
           else !if (idst(ke).eq.2) then
            Surv_glob=surv_glob + surv(ke)*exp(varexpsurv)
         end if
    
           ! troncature : risque cumule au temps T0
           if (idtrunc.eq.1) then
            if (idst(ke).eq.1) then
              surv0_glob=surv0_glob+surv0(ke)*exp(varexpsurv+easurv)    
            else !if (idst(ke).eq.2) then
              surv0_glob=surv0_glob+surv0(ke)*exp(varexpsurv)  
            end if
           end if
           
           nxevtcurr=nxevtcurr+nxevt
        end do

        ! vraisemblance de la partie survie
        vrais_surv = exp(-Surv_glob)
        ! print*,"vrais_surv ok"
       
        if(Devt(i).gt.0) vrais_surv = vrais_surv * fevt

        if (idtrunc.eq.1) then
           !vrais_surv = vrais_surv / exp(-surv0_glob)
           som_T0 = som_T0 + exp(-surv0_glob)   !delayed entry
        end if
        
        som_Ti = som_Ti + vrais_surv
        
        ! vrais totale 
        som = som + vrais_Y * vrais_surv

     else !  pas de survie

        som = som + vrais_Y

     end if
     
    ! if(l.lt.4) print*,"l=", l, " som=",som, " vrais_Y=", vrais_Y, " vrais_surv=", vrais_surv
 !    print*,"fin l=",l
  end do ! fin boucle nMC
  
  vrais_i = vrais_i + log(som) - log(dble(nMC)) + jacobien

  if (idtrunc.eq.1) then    !delayedentry
      vrais_i = vrais_i - log(som_T0) + log(dble(nMC))
  end if  
!  print*,"trunc ok"
!  print*,"i=",i,"som_Ti=",som_Ti,"som_T0=",som_T0," vrais_i=",vrais_i
  
!  print*,"i=",i,"som=",som," vrais_Y=",vrais_Y,"jac=",jacobien," vrais_surv=",vrais_surv," vrais_i=",vrais_i
  654 continue

  return

end function vrais_i


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!      SPLINE BASES      !!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!------------------------------!
! I-SPLINES for link functions !
!------------------------------!

subroutine design_splines_irtsre (ier)

  use modirtsre

  implicit none

  integer ::jj,l,k,ier,yk,q,sumnval,qq,sumqqval
  double precision ::ht,htm,ht2,ht3,h,hh,h2,h3,h2n,hn,hht

  ier=0
  jj=0
  l=0
  q=0
  qq=0
  sumqqval=0
  sumnval=0
  do yk=1,ny
     if (idlink(yk).eq.2) then 
        q=q+1
        do jj=1,nvalSPL(q)      !     ou se trouve la valeur de zi

           do k = 2,ntr(yk)-2
              if ((uniqueY(sumqqval+sumnval+jj).ge.zitr(k-1,q)).and.(uniqueY(sumqqval+sumnval+jj).lt.zitr(k,q))) then
                 l=k-1
              end if
           End do


           if (uniqueY(sumqqval+sumnval+jj).eq.zitr(ntr(yk)-2,q)) then
              l=ntr(yk)-3
           end if

           ht2 = zitr(l+1,q)-uniqueY(sumqqval+sumnval+jj)
           htm= uniqueY(sumqqval+sumnval+jj)-zitr(l-1,q)
           ht = uniqueY(sumqqval+sumnval+jj)-zitr(l,q)
           ht3 = zitr(l+2,q)-uniqueY(sumqqval+sumnval+jj)
           hht = uniqueY(sumqqval+sumnval+jj)-zitr(l-2,q)
           h = zitr(l+1,q)-zitr(l,q)
           hh= zitr(l+1,q)-zitr(l-1,q)
           hn= zitr(l+1,q)-zitr(l-2,q)
           h2n=zitr(l+2,q)-zitr(l-1,q)
           h2= zitr(l+2,q)-zitr(l,q)
           h3= zitr(l+3,q)-zitr(l,q)

           if (uniqueY(sumqqval+sumnval+jj).ne.zitr(ntr(yk)-2,q)) then
              mm2(sumnval+jj) = (3.d0*ht2*ht2)/(hh*h*hn)
              mm1(sumnval+jj) = (3.d0*htm*ht2)/(h2n*hh*h)+(3.d0*ht*ht3)/(h2*h*h2n)
              mm(sumnval+jj)  = (3.d0*ht*ht)/(h3*h2*h)

           end if
           if (uniqueY(sumqqval+sumnval+jj).eq.zitr(ntr(yk)-2,q)) then
              mm2(sumnval+jj) = 0.d0
              mm1(sumnval+jj) = 0.d0
              mm(sumnval+jj)  = 3.d0/h
           end if

           if (mm2(sumnval+jj).lt.0.or.mm1(sumnval+jj).lt.0.or.mm(sumnval+jj).lt.0) then
              ier=-1
              goto 765
           end if

           im2(sumnval+jj)=hht*mm2(sumnval+jj)/(3.d0)+ h2n*mm1(sumnval+jj)/(3.d0) &
                +h3*mm(sumnval+jj)/(3.d0)
           im1(sumnval+jj)=htm*mm1(sumnval+jj)/(3.d0)+h3*mm(sumnval+jj)/(3.d0)
           im(sumnval+jj)=ht*mm(sumnval+jj)/(3.d0)

        end do
        sumnval = sumnval + nvalSPL(q)

     end if

     if(idlink(yk).eq.3) then
        qq = qq + 1
        sumqqval = sumqqval + nvalORD(qq)
     end if

  end do


765 continue

end subroutine design_splines_irtsre
!fin design_splines


!---------------------------------------!
! M-SPLINES for baseline risk functions !
!     in shared random effects case     !
!---------------------------------------!

subroutine splines_irtsre(k)
  use modirtsre
  implicit none

  integer::k
  integer::i,kk,n,l
  double precision::ht,htm,h2t,ht2,ht3,hht,h,hh,h2,h3,h4,h3m,h2n, &
       hn,hh2,hh3



  l=0
  Tmm=0.d0
  Tmm1=0.d0
  Tmm2=0.d0
  Tmm3=0.d0
  Tim=0.d0
  Tim1=0.d0
  Tim2=0.d0
  Tim3=0.d0
  Tmm0=0.d0
  Tmm01=0.d0
  Tmm02=0.d0
  Tmm03=0.d0
  Tim0=0.d0
  Tim01=0.d0
  Tim02=0.d0
  Tim03=0.d0
  Tmmt=0.d0
  Tmmt1=0.d0
  Tmmt2=0.d0
  Tmmt3=0.d0
  Timt=0.d0
  Timt1=0.d0
  Timt2=0.d0
  Timt3=0.d0

  zi(-2,k)=zi(1,k)
  zi(-1,k)=zi(1,k)
  zi(0,k)=zi(1,k)
  zi(nz(k)+1,k)=zi(nz(k),k)
  zi(nz(k)+2,k)=zi(nz(k),k)
  zi(nz(k)+3,k)=zi(nz(k),k)
  
  n=nz(k)+2
  !------------------- Tsurv ---------------------------
  Do i=1,ns

     do kk=2,n-2
        if ((Tsurv(i).ge.zi(kk-1,k)).and.  &
             Tsurv(i).lt.zi(kk,k)) then
           l=kk-1
        end if
     end do

     if (Tsurv(i).eq.zi(n-2,k)) then
        l=n-3
     end if

     ht = Tsurv(i)-zi(l,k)
     htm = Tsurv(i)-zi(l-1,k)
     h2t = Tsurv(i)-zi(l+2,k)
     ht2 = zi(l+1,k)-Tsurv(i)
     ht3 = zi(l+3,k)-Tsurv(i)
     hht = Tsurv(i)-zi(l-2,k)
     h = zi(l+1,k)-zi(l,k)
     hh = zi(l+1,k)-zi(l-1,k)
     h2 = zi(l+2,k)-zi(l,k)
     h3 = zi(l+3,k)-zi(l,k)
     h4 = zi(l+4,k)-zi(l,k)
     h3m = zi(l+3,k)-zi(l-1,k)
     h2n = zi(l+2,k)-zi(l-1,k)
     hn = zi(l+1,k)-zi(l-2,k)
     hh3 = zi(l+1,k)-zi(l-3,k)
     hh2 = zi(l+2,k)-zi(l-2,k)

     if (Tsurv(i).ne.zi(n-2,k)) then

        Tmm3(ns*(k-1)+i) = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))
        Tmm2(ns*(k-1)+i) = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn))  &
             +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))  &
             +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
        Tmm1(ns*(k-1)+i) = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
             +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))   &
             +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
        Tmm(ns*(k-1)+i) = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)

     end if

     if (Tsurv(i).eq.zi(n-2,k)) then

        Tmm3(ns*(k-1)+i) = 0.d0
        Tmm2(ns*(k-1)+i) = 0.d0
        Tmm1(ns*(k-1)+i) = 0.d0
        Tmm(ns*(k-1)+i) = 4.d0/h

     end if

     Tim3(ns*(k-1)+i) = (0.25d0*(Tsurv(i)-zi(l-3,k))*Tmm3(ns*(k-1)+i)) &
          +(0.25d0*hh2*Tmm2(ns*(k-1)+i))        &
          +(0.25d0*h3m*Tmm1(ns*(k-1)+i))+(0.25d0*h4*Tmm(ns*(k-1)+i))
     Tim2(ns*(k-1)+i) = (0.25d0*hht*Tmm2(ns*(k-1)+i))  &
          +(h3m*Tmm1(ns*(k-1)+i)*0.25d0)+(h4*Tmm(ns*(k-1)+i)*0.25d0)
     Tim1(ns*(k-1)+i) = (htm*Tmm1(ns*(k-1)+i)*0.25d0)+(h4*Tmm(ns*(k-1)+i)*0.25d0)
     Tim(ns*(k-1)+i) = ht*Tmm(ns*(k-1)+i)*0.25d0

     !------------------- Tsurv0 --------------------------

     if (idtrunc.eq.1) then

        do kk=2,n-2
           if ((Tsurv0(i).ge.zi(kk-1,k)).and.   &
                Tsurv0(i).lt.zi(kk,k)) then
              l=kk-1
           end if
        end do

        if (Tsurv0(i).eq.zi(n-2,k)) then
           l=n-3
        end if

        ht = Tsurv0(i)-zi(l,k)
        htm = Tsurv0(i)-zi(l-1,k)
        h2t = Tsurv0(i)-zi(l+2,k)
        ht2 = zi(l+1,k)-Tsurv0(i)
        ht3 = zi(l+3,k)-Tsurv0(i)
        hht = Tsurv0(i)-zi(l-2,k)
        h = zi(l+1,k)-zi(l,k)
        hh = zi(l+1,k)-zi(l-1,k)
        h2 = zi(l+2,k)-zi(l,k)
        h3 = zi(l+3,k)-zi(l,k)
        h4 = zi(l+4,k)-zi(l,k)
        h3m = zi(l+3,k)-zi(l-1,k)
        h2n = zi(l+2,k)-zi(l-1,k)
        hn = zi(l+1,k)-zi(l-2,k)
        hh3 = zi(l+1,k)-zi(l-3,k)
        hh2 = zi(l+2,k)-zi(l-2,k)

        if (Tsurv0(i).ne.zi(nz(k)-2,k)) then

           Tmm03(ns*(k-1)+i) = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))

           Tmm02(ns*(k-1)+i) = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn))  &
                +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))   &
                +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
           Tmm01(ns*(k-1)+i) = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
                +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))    &
                +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
           Tmm0(ns*(k-1)+i) = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)

        end if

        if (Tsurv0(i).eq.zi(n-2,k)) then

           Tmm03(ns*(k-1)+i) = 0.d0
           Tmm02(ns*(k-1)+i) = 0.d0
           Tmm01(ns*(k-1)+i) = 0.d0
           Tmm0(ns*(k-1)+i) = 4.d0/h

        end if

        Tim03(ns*(k-1)+i) = (0.25d0*(Tsurv0(i)-zi(l-3,k))*Tmm03(ns*(k-1)+i))  &
             +(0.25d0*hh2*Tmm02(ns*(k-1)+i))           &
             +(0.25d0*h3m*Tmm01(ns*(k-1)+i))+(0.25d0*h4*Tmm0(ns*(k-1)+i))
        Tim02(ns*(k-1)+i) = (0.25d0*hht*Tmm02(ns*(k-1)+i))                  &
             +(h3m*Tmm01(ns*(k-1)+i)*0.25d0)+(h4*Tmm0(ns*(k-1)+i)*0.25d0)
        Tim01(ns*(k-1)+i) = (htm*Tmm01(ns*(k-1)+i)*0.25d0)+(h4*Tmm0(ns*(k-1)+i)*0.25d0)
        Tim0(ns*(k-1)+i) = ht*Tmm0(ns*(k-1)+i)*0.25d0

     end if


     !------------------- Tsurvint --------------------------
     if (ind_survint(i).eq.1) then
        do kk=2,n-2
           if ((Tsurvint(i).ge.zi(kk-1,k)).and. &
                Tsurvint(i).lt.zi(kk,k)) then
              l=kk-1
           end if
        end do

        if (Tsurvint(i).eq.zi(nz(k)-2,k)) then
           l=n-3
        end if

        ht = Tsurvint(i)-zi(l,k)
        htm = Tsurvint(i)-zi(l-1,k)
        h2t = Tsurvint(i)-zi(l+2,k)
        ht2 = zi(l+1,k)-Tsurvint(i)
        ht3 = zi(l+3,k)-Tsurvint(i)
        hht = Tsurvint(i)-zi(l-2,k)
        h = zi(l+1,k)-zi(l,k)
        hh = zi(l+1,k)-zi(l-1,k)
        h2 = zi(l+2,k)-zi(l,k)
        h3 = zi(l+3,k)-zi(l,k)
        h4 = zi(l+4,k)-zi(l,k)
        h3m = zi(l+3,k)-zi(l-1,k)
        h2n = zi(l+2,k)-zi(l-1,k)
        hn = zi(l+1,k)-zi(l-2,k)
        hh3 = zi(l+1,k)-zi(l-3,k)
        hh2 = zi(l+2,k)-zi(l-2,k)

        if (Tsurvint(i).ne.zi(nz(k)-2,k)) then

           Tmmt3(ns*(k-1)+i) = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))
           Tmmt2(ns*(k-1)+i) = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn)) &
                +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))     &
                +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
           Tmmt1(ns*(k-1)+i) = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
                +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))       &
                +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
           Tmmt(ns*(k-1)+i) = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)

        end if

        if (Tsurvint(i).eq.zi(nz(k)-2,k)) then

           Tmmt3(ns*(k-1)+i) = 0.d0
           Tmmt2(ns*(k-1)+i) = 0.d0
           Tmmt1(ns*(k-1)+i) = 0.d0
           Tmmt(ns*(k-1)+i) = 4.d0/h

        end if

        Timt3(ns*(k-1)+i) = (0.25d0*(Tsurvint(i)-zi(l-3,k))*Tmmt3(ns*(k-1)+i)) &
             +(0.25d0*hh2*Tmmt2(ns*(k-1)+i))               &
             +(0.25d0*h3m*Tmmt1(ns*(k-1)+i))+(0.25d0*h4*Tmmt(ns*(k-1)+i))
        Timt2(ns*(k-1)+i) = (0.25d0*hht*Tmmt2(ns*(k-1)+i))                   &
             +(h3m*Tmmt1(ns*(k-1)+i)*0.25d0)+(h4*Tmmt(ns*(k-1)+i)*0.25d0)
        Timt1(ns*(k-1)+i) = (htm*Tmmt1(ns*(k-1)+i)*0.25d0)+(h4*Tmmt(ns*(k-1)+i)*0.25d0)
        Timt(ns*(k-1)+i) = ht*Tmmt(ns*(k-1)+i)*0.25d0

     else
        Timt3(ns*(k-1)+i) =Tim3(ns*(k-1)+i)
        Timt2(ns*(k-1)+i) =Tim2(ns*(k-1)+i)
        Timt1(ns*(k-1)+i) =Tim1(ns*(k-1)+i)
        Timt(ns*(k-1)+i) =Tim(ns*(k-1)+i)
     end if

  end Do

end subroutine splines_irtsre


!---------------------------------------!
! M-SPLINES for baseline risk functions !
!     in shared current level case      !
!---------------------------------------!

!bases de splines pr les 15 noeuds de la quadrature de Gauss-Kronrod
subroutine splines_irtsre2(k)
  use modirtsre
  implicit none

  integer::k
  integer::i,kk,n,l
  integer::p 
  double precision::ht,htm,h2t,ht2,ht3,hht,h,hh,h2,h3,h4,h3m,h2n, &
       hn,hh2,hh3



  l=0
  Tmm_st2=0.d0  !OK pr matrix ?
  Tmm1_st2=0.d0
  Tmm2_st2=0.d0
  Tmm3_st2=0.d0
  !Tim_st2=0.d0
  !Tim1_st2=0.d0
  !Tim2_st2=0.d0
  !Tim3_st2=0.d0
  Tmm0_st2=0.d0
  Tmm01_st2=0.d0
  Tmm02_st2=0.d0
  Tmm03_st2=0.d0
  !Tim0_st2=0.d0
  !Tim01_st2=0.d0
  !Tim02_st2=0.d0
  !Tim03_st2=0.d0

  !zi deja mis en forme a l etape avec splines_irtsre

  n=nz(k)+2
  !------------------- Tsurv ---------------------------
  Do i=1,ns
  
    do p=1,15

     do kk=2,n-2
        if ((Tsurv_st2(i,p).ge.zi(kk-1,k)).and.  &
             Tsurv_st2(i,p).lt.zi(kk,k)) then
           l=kk-1
        end if
     end do

     if (Tsurv_st2(i,p).eq.zi(n-2,k)) then
        l=n-3
     end if

     ht = Tsurv_st2(i,p)-zi(l,k)
     htm = Tsurv_st2(i,p)-zi(l-1,k)
     h2t = Tsurv_st2(i,p)-zi(l+2,k)
     ht2 = zi(l+1,k)-Tsurv_st2(i,p)
     ht3 = zi(l+3,k)-Tsurv_st2(i,p)
     hht = Tsurv_st2(i,p)-zi(l-2,k)
     h = zi(l+1,k)-zi(l,k)
     hh = zi(l+1,k)-zi(l-1,k)
     h2 = zi(l+2,k)-zi(l,k)
     h3 = zi(l+3,k)-zi(l,k)
     h4 = zi(l+4,k)-zi(l,k)
     h3m = zi(l+3,k)-zi(l-1,k)
     h2n = zi(l+2,k)-zi(l-1,k)
     hn = zi(l+1,k)-zi(l-2,k)
     hh3 = zi(l+1,k)-zi(l-3,k)
     hh2 = zi(l+2,k)-zi(l-2,k)

     if (Tsurv_st2(i,p).ne.zi(n-2,k)) then

        Tmm3_st2(p,ns*(k-1)+i) = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))
        Tmm2_st2(p,ns*(k-1)+i) = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn))  &
             +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))  &
             +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
        Tmm1_st2(p,ns*(k-1)+i) = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
             +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))   &
             +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
        Tmm_st2(p,ns*(k-1)+i) = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)

     end if

     if (Tsurv_st2(i,p).eq.zi(n-2,k)) then

        Tmm3_st2(p,ns*(k-1)+i) = 0.d0
        Tmm2_st2(p,ns*(k-1)+i) = 0.d0
        Tmm1_st2(p,ns*(k-1)+i) = 0.d0
        Tmm_st2(p,ns*(k-1)+i) = 4.d0/h

     end if

     !Tim3_st2(p,ns*(k-1)+i) = (0.25d0*(Tsurv_st2(i,p)-zi(l-3,k))*Tmm3_st2(p,ns*(k-1)+i)) &
    !      +(0.25d0*hh2*Tmm2_st2(p,ns*(k-1)+i))        &
    !      +(0.25d0*h3m*Tmm1_st2(p,ns*(k-1)+i))+(0.25d0*h4*Tmm_st2(p,ns*(k-1)+i))
     !Tim2_st2(p,ns*(k-1)+i) = (0.25d0*hht*Tmm2_st2(p,ns*(k-1)+i))  &
    !      +(h3m*Tmm1_st2(p,ns*(k-1)+i)*0.25d0)+(h4*Tmm_st2(p,ns*(k-1)+i)*0.25d0)
    ! Tim1_st2(p,ns*(k-1)+i) = (htm*Tmm1_st2(p,ns*(k-1)+i)*0.25d0)+(h4*Tmm_st2(p,ns*(k-1)+i)*0.25d0)
     !Tim_st2(p,ns*(k-1)+i) = ht*Tmm_st2(p,ns*(k-1)+i)*0.25d0

     !------------------- Tsurv0 --------------------------

     if (idtrunc.eq.1) then

        do kk=2,n-2
           if ((Tsurv0_st2(i,p).ge.zi(kk-1,k)).and.   &
                Tsurv0_st2(i,p).lt.zi(kk,k)) then
              l=kk-1
           end if
        end do

        if (Tsurv0_st2(i,p).eq.zi(n-2,k)) then
           l=n-3
        end if

        ht = Tsurv0_st2(i,p)-zi(l,k)
        htm = Tsurv0_st2(i,p)-zi(l-1,k)
        h2t = Tsurv0_st2(i,p)-zi(l+2,k)
        ht2 = zi(l+1,k)-Tsurv0_st2(i,p)
        ht3 = zi(l+3,k)-Tsurv0_st2(i,p)
        hht = Tsurv0_st2(i,p)-zi(l-2,k)
        h = zi(l+1,k)-zi(l,k)
        hh = zi(l+1,k)-zi(l-1,k)
        h2 = zi(l+2,k)-zi(l,k)
        h3 = zi(l+3,k)-zi(l,k)
        h4 = zi(l+4,k)-zi(l,k)
        h3m = zi(l+3,k)-zi(l-1,k)
        h2n = zi(l+2,k)-zi(l-1,k)
        hn = zi(l+1,k)-zi(l-2,k)
        hh3 = zi(l+1,k)-zi(l-3,k)
        hh2 = zi(l+2,k)-zi(l-2,k)

        if (Tsurv0_st2(i,p).ne.zi(nz(k)-2,k)) then

           Tmm03_st2(p,ns*(k-1)+i) = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))

           Tmm02_st2(p,ns*(k-1)+i) = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn))  &
                +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))   &
                +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
           Tmm01_st2(p,ns*(k-1)+i) = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
                +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))    &
                +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
           Tmm0_st2(p,ns*(k-1)+i) = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)

        end if

        if (Tsurv0_st2(i,p).eq.zi(n-2,k)) then

           Tmm03_st2(p,ns*(k-1)+i) = 0.d0
           Tmm02_st2(p,ns*(k-1)+i) = 0.d0
           Tmm01_st2(p,ns*(k-1)+i) = 0.d0
           Tmm0_st2(p,ns*(k-1)+i) = 4.d0/h

        end if

        !Tim03_st2(p,ns*(k-1)+i) = (0.25d0*(Tsurv0_st2(i,p)-zi(l-3,k))*Tmm03_st2(p,ns*(k-1)+i))  &
        !     +(0.25d0*hh2*Tmm02_st2(p,ns*(k-1)+i))           &
        !     +(0.25d0*h3m*Tmm01_st2(p,ns*(k-1)+i))+(0.25d0*h4*Tmm0_st2(p,ns*(k-1)+i))
        !Tim02_st2(p,ns*(k-1)+i) = (0.25d0*hht*Tmm02_st2(p,ns*(k-1)+i))                  &
        !     +(h3m*Tmm01_st2(p,ns*(k-1)+i)*0.25d0)+(h4*Tmm0_st2(p,ns*(k-1)+i)*0.25d0)
        !Tim01_st2(p,ns*(k-1)+i) = (htm*Tmm01_st2(p,ns*(k-1)+i)*0.25d0)+(h4*Tmm0_st2(p,ns*(k-1)+i)*0.25d0)
        !Tim0_st2(p,ns*(k-1)+i) = ht*Tmm0_st2(p,ns*(k-1)+i)*0.25d0

     end if
     
    end do !end p

  end Do !end i

end subroutine splines_irtsre2


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!      SURVIVAL ELEMENTS      !!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!-----------------------------------------------!
!     baseline and cumulative RISK FUNCTIONS    !
!         in shared random effects case         !
!-----------------------------------------------!

subroutine fct_risq_irtsre(i,k,brisq,risq,surv,surv0,survint)

  use modirtsre
        
  implicit none
  
  integer::i,k
  double precision,dimension(nprisq(k))::brisq
  double precision,dimension(nbevt)::risq,surv,surv0,survint
  
  integer::j,l,ll,kk,ii
  double precision::som
  
  if (typrisq(k).eq.2.and.logspecif.eq.1) then
     
     surv(k)=brisq(1)*(tsurv(i)-zi(1,k))**brisq(2)      !zi(1,k)=depart Weibull
     
     risq(k)=brisq(1)*brisq(2)*(tsurv(i)-zi(1,k))**(brisq(2)-1)
     if (idtrunc.eq.1) then
        surv0(k)=brisq(1)*(tsurv0(i)-zi(1,k))**brisq(2)
     end if
     if (ind_survint(i).eq.1) then
        survint(k)=brisq(1)*(tsurvint(i)-zi(1,k))**brisq(2)
     else
        survint(k)=surv(k)
     end if
     
  end if
  if (typrisq(k).eq.2.and.logspecif.eq.0) then
     
     surv(k)=(brisq(1)*(tsurv(i)-zi(1,k)))**brisq(2)
     
     risq(k)=brisq(1)*brisq(2)*(brisq(1)*(tsurv(i)-zi(1,k)))**(brisq(2)-1)
     if (idtrunc.eq.1) then
        surv0(k)=(brisq(1)*(tsurv0(i)-zi(1,k)))**brisq(2)
     end if
     if (ind_survint(i).eq.1) then
        survint(k)=(brisq(1)*(tsurvint(i)-zi(1,k)))**brisq(2)
     else
        survint(k)=surv(k)
     end if
     
  end if
 
  if (typrisq(k).eq.1) then
     do j=1,nz(k)-1
        som=0.d0
        do l=1,j-1
           som=som+brisq(l)*(zi(l+1,k)-zi(l,k))
        end do
        if (idtrunc.eq.1) then
           if (Tsurv0(i).ge.zi(j,k).and.Tsurv0(i).le.zi(j+1,k)) then
              surv0(k)=som+brisq(j)*(Tsurv0(i)-zi(j,k))
           end if
        end if
        if (Tsurv(i).ge.zi(j,k).and.Tsurv(i).le.zi(j+1,k)) then
           surv(k)=som+brisq(j)*(Tsurv(i)-zi(j,k))
           risq(k)=brisq(j)
        end if
        if (ind_survint(i).eq.1) then
           if (Tsurvint(i).ge.zi(j,k).and.Tsurvint(i).le.zi(j+1,k)) &
                then
              survint(k)=som+brisq(j)*(Tsurvint(i)-zi(j,k))
           end if
        end if
     end do
     if (ind_survint(i).eq.0) then
        survint(k)=surv(k)
     end if
  end if
  
  

  if (typrisq(k).eq.3) then
     !------------ survie et risq pour Tsurv ----------------
     ll=0
     if (Tsurv(i).eq.zi(nz(k),k)) then
        ll=nz(k)-1
     end if
     som=0.d0
     do kk=2,nz(k)
        if ((Tsurv(i).ge.zi(kk-1,k)).and.(Tsurv(i).lt.zi(kk,k))) &
             then
           ll=kk-1
        end if
     end do
     if (ll.gt.1) then
        do ii=1,ll-1
           som=som+brisq(ii)
        end do
     end if
     
     surv(k)=som+brisq(ll)*Tim3(ns*(k-1)+i)+brisq(ll+1)*Tim2(ns*(k-1)+i) &
          +brisq(ll+2)*Tim1(ns*(k-1)+i)+brisq(ll+3)*Tim(ns*(k-1)+i)
     risq(k)=brisq(ll)*Tmm3(ns*(k-1)+i)+brisq(ll+1)*Tmm2(ns*(k-1)+i)     &
          +brisq(ll+2)*Tmm1(ns*(k-1)+i)+brisq(ll+3)*Tmm(ns*(k-1)+i)

     !------------ survie et risq pour Tsurv0 ----------------
     
     if (idtrunc.eq.1) then
        ll=0
        if (Tsurv0(i).eq.zi(nz(k),k)) then
           ll=nz(k)-1
        end if
        som=0.d0
        do kk=2,nz(k)
           if ((Tsurv0(i).ge.zi(kk-1,k)).and.(Tsurv0(i).lt.zi(kk,k))) &
                then
              ll=kk-1
           end if
        end do
        !               if (ll.lt.1.or.ll.gt.nz-1) then
        !                  write(*,*) 'probleme dans fct_risq splines'
        !                  write(*,*) 'll=',ll,'T=',Tsurv0(i)
        !                  stop
        !               end if
        if (ll.gt.1) then
           do ii=1,ll-1
              som=som+brisq(ii)
           end do
        end if
        
        surv0(k)=som+brisq(ll)*Tim03(ns*(k-1)+i)+brisq(ll+1)*Tim02(ns*(k-1)+i) &
             +brisq(ll+2)*Tim01(ns*(k-1)+i)+brisq(ll+3)*Tim0(ns*(k-1)+i)
        
     end if
     
     !------------ survie et risq pour Tsurvint ----------------


     if (ind_survint(i).eq.1) then

        !               write(*,*)'i',i,tsurvint(i),ind_survint(i),tsurv(i),tsurv0(i)
!               write(*,*)timt3(i),Timt2(i),timt1(i)

        ll=0
        if (Tsurvint(i).eq.zi(nz(k),k)) then
           ll=nz(k)-1
        end if
        som=0.d0
        do kk=2,nz(k)
           if((Tsurvint(i).ge.zi(kk-1,k)).and.(Tsurvint(i).lt.zi(kk,k))) &
                then
              ll=kk-1
           end if
        end do
        !               if (ll.lt.1.or.ll.gt.nz-1) then
        !                  write(*,*) 'probleme dans fct_risq splines'
        !                  write(*,*) 'll=',ll,'T=',Tsurvint(i)
        !                  stop
        !               end if
        if (ll.gt.1) then
           do ii=1,ll-1
              som=som+brisq(ii)
           end do
        end if
               
        survint(k)=som+brisq(ll)*Timt3(ns*(k-1)+i)+brisq(ll+1)*Timt2(ns*(k-1)+i) &
             +brisq(ll+2)*Timt1(ns*(k-1)+i)+brisq(ll+3)*Timt(ns*(k-1)+i)
        
     else
        survint(k)=surv(k)
     end if
     
  end if
  
end subroutine fct_risq_irtsre




!-----------------------------------------------!
!     baseline and cumulative RISK FUNCTIONS    !
!    including time-varying linear predictor    !
!          in shared current level case         !
!-----------------------------------------------!


! fct calculant risq de base au tps event et risq cumule (sans varexp) par quadrature de Konrod
subroutine fct_risq_irtsre_2(i, k, brisq, bassoCL, bassoCS, beta_ef, ui, risq, surv, surv0)

  use modirtsre
        
  implicit none
  
  integer::i,k
  double precision,dimension(nprisq(k))::brisq
  double precision,dimension(nbevt)::risq,surv,surv0
  double precision,dimension(nassoCL)::bassoCL
  double precision,dimension(nassoCS)::bassoCS
  double precision,dimension(nef)::beta_ef
  double precision,dimension(nea)::ui
  
  integer::p
  double precision,dimension(8)::wgk
  double precision,dimension(15)::wgk_15
  double precision,dimension(2)::hlgth
  
  double precision,dimension(15)::risq_GK_event,risq_GK_entry
  double precision,dimension(2,15)::pred_GK
  double precision,dimension(15)::pred_GK_event,pred_GK_entry
  double precision,dimension(15)::fct_pred_surv,fct_pred_surv0
  double precision,dimension(15)::fct_pred_surv_pond,fct_pred_surv0_pond
  double precision::asso,cli,csi
  
  double precision,external::fct_risq_base_irtsre_2
  double precision,external::curlev, curslope, poly, logit, sigmoid
  
  hlgth=0.d0
  
  wgk(1)=0.022935322010529224963732008058970d0
  wgk(2)=0.063092092629978553290700663189204d0
  wgk(3)=0.104790010322250183839876322541518d0
  wgk(4)=0.140653259715525918745189590510238d0
  wgk(5)=0.169004726639267902826583426598550d0
  wgk(6)=0.190350578064785409913256402421014d0
  wgk(7)=0.204432940075298892414161999234649d0
  wgk(8)=0.209482141084727828012999174891714d0
  
  wgk_15(1:2)=wgk(1)  !vect de taille 15
  wgk_15(3:4)=wgk(2)
  wgk_15(5:6)=wgk(3)
  wgk_15(7:8)=wgk(4)
  wgk_15(9:10)=wgk(5)
  wgk_15(11:12)=wgk(6)
  wgk_15(13:14)=wgk(7)
  wgk_15(15)=wgk(8)

  hlgth(1)=0.5d+00 * (Tsurv(i) - zi(1,1))  !hlgth_event
  if (idtrunc.eq.1) then
     hlgth(2)=0.5d+00 * (Tsurv0(i) - zi(1,1))   !hlgth_entry
  end if
!  print*,"i=", i, "Tsurv = ", Tsurv(i), "Tsurv0=", Tsurv0(i)
  !!! risque de base au tps d event Ti
  risq(k) = fct_risq_base_irtsre_2(Tsurv(i),i,k,brisq,1,0) !avant dernier argument = 1 pr event ou 2 pr entry, dernier argument = 0 pr tps reel ou numero de point de quadrature GK (necessaire pr base de splines)
!print*,"risq = ", risq(k)
  asso = 0.d0
  if(idst(k).eq.2 .or. idst(k).eq.4) then
     cli = curlev(i,beta_ef,ui)
     if(nonlinCL(k).eq.0) then
        asso = asso + cli * bassoCL(1)
     else if(nonlinCL(k).eq.1) then
        asso = asso + poly(cli, bassoCL, 0.d0)
     else if(nonlinCL(k).eq.2) then
        asso = asso + sigmoid(cli, bassoCL)
     end if
  end if
  if(idst(k).eq.3 .or. idst(k).eq.4) then
     csi = curslope(i,beta_ef,ui)
     if(nonlinCS(k).eq.0) then
        asso = asso + csi * bassoCS(1)
     else if(nonlinCS(k).eq.1) then
        asso = asso + poly(csi, bassoCS, 0.d0)
     else if(nonlinCS(k).eq.2) then
        asso = asso + sigmoid(csi, bassoCS)
     end if
  end if
!print*,"asso = ", asso
  risq(k) = risq(k) * exp(asso)

!!! risque cumule par quadrature de Kronrod

  !risque de base aux differents tps de quadrature
  do p = 1,15
     risq_GK_event(p) = fct_risq_base_irtsre_2(Xcl_GK((i-1)*15+p,1),i,k,brisq,1,p)
    if (idtrunc.eq.1) then
      risq_GK_entry(p) = fct_risq_base_irtsre_2(Xcl0_GK((i-1)*15+p,1),i,k,brisq,2,p)
   end if
 end do
!print*,"risq_GK_event(1)=", risq_GK_event(1)
!print*,"risq_GK_entry(1)=", risq_GK_entry(1)
!print*,"idst = ", idst(k), "nonlinCL = ", nonlinCL(k), "bassoCL = ", bassoCL
  !prediction niveau courant du processus a chq pnt de quadrature
  if(idst(k).eq.2 .or. idst(k).eq.4) then
     call fct_pred_curlev_irtsre_2(i,beta_ef,ui,pred_GK)
     pred_GK_event = pred_GK(1,:)
     if (idtrunc.eq.1) then
        pred_GK_entry = pred_GK(2,:)
     end if
     !multiplication par prm estime et passage a l'exponentiel a chq pnt de quadrature
     if(nonlinCL(k).eq.0) then
        pred_GK_event = pred_GK_event*bassoCL(1)
        if (idtrunc.eq.1) then
           pred_GK_entry = pred_GK_entry*bassoCL(1)
        end if
     else
        ! asso non lineaire de type polynomiale
        if(nonlinCL(k).eq.1) then
           !pred_GK_event = poly15(pred_GK_event, bassoCL, centerpoly(1))
           call poly15(pred_GK_event, bassoCL, centerpoly(1))
           if (idtrunc.eq.1) then
              !pred_GK_entry = poly15(pred_GK_entry, bassoCL, centerpoly(1))
              call poly15(pred_GK_entry, bassoCL, centerpoly(1))
           end if
        else
           ! asso non lineaire de type logistique
           if(nonlinCL(k).eq.2) then
              !pred_GK_event = logit15(pred_GK_event, bassoCL)
              call sigmoid15(pred_GK_event, bassoCL)
              if (idtrunc.eq.1) then
                 !pred_GK_entry = logit15(pred_GK_entry, bassoCL)
                 call sigmoid15(pred_GK_entry, bassoCL)
              end if
           end if
        end if
     end if
     
     !produit des 2 vecteurs (risque de base * assoCL)
     fct_pred_surv = risq_GK_event * exp(pred_GK_event) !produit de 2 vecteurs
     if (idtrunc.eq.1) then
        fct_pred_surv0 = risq_GK_entry * exp(pred_GK_entry)
     end if
  end if

  !prediction pente courante du processus a chq pnt de quadrature
  if(idst(k).eq.3 .or. idst(k).eq.4) then
     call fct_pred_curslope_irtsre_2(i,beta_ef,ui,pred_GK)
     pred_GK_event = pred_GK(1,:)
     if (idtrunc.eq.1) then
        pred_GK_entry = pred_GK(2,:)
     end if
      
     !multiplication par prm estime et passage a l'exponentiel a chq pnt de quadrature
     if(nonlinCS(k).eq.0) then
        pred_GK_event = pred_GK_event*bassoCS(1)
        if (idtrunc.eq.1) then
           pred_GK_entry = pred_GK_entry*bassoCS(1)
        end if
     else
        ! asso non lineaire de type polynomiale
        if(nonlinCS(k).eq.1) then
           !pred_GK_event = poly(pred_GK_event, bassoCS, 15, centerpoly(2))
           call poly15(pred_GK_event, bassoCS, centerpoly(2))
           if (idtrunc.eq.1) then
              !pred_GK_entry = poly(pred_GK_entry, bassoCS, 15, centerpoly(2))
              call poly15(pred_GK_entry, bassoCS, centerpoly(2))
           end if
        else
           ! asso non lineaire de type logistique
           if(nonlinCS(k).eq.2) then
              !pred_GK_event = logit(pred_GK_event, bassoCS, 15)
              call sigmoid15(pred_GK_event, bassoCS)
              if (idtrunc.eq.1) then
                 !pred_GK_entry = logit(pred_GK_entry, bassoCS, 15)
                 call sigmoid15(pred_GK_entry, bassoCS)
              end if
           end if
        end if
     end if
  end if
  
  ! produit risque de base et toutes les assos, pour les 15 points
  if(idst(k).eq.3) then
     fct_pred_surv = risq_GK_event * exp(pred_GK_event)
     if (idtrunc.eq.1) then
        fct_pred_surv0 = risq_GK_entry * exp(pred_GK_entry)
     end if
  else if(idst(k).eq.4) then
     fct_pred_surv = fct_pred_surv * exp(pred_GK_event)
     if (idtrunc.eq.1) then
        fct_pred_surv0 = fct_pred_surv0 * exp(pred_GK_entry)
     end if
  end if

  
  !ponderation
  fct_pred_surv_pond = 0.d0
  fct_pred_surv0_pond = 0.d0
  do p=1,15
     fct_pred_surv_pond(p) = wgk_15(p) * fct_pred_surv(p)
    if (idtrunc.eq.1) then
      fct_pred_surv0_pond(p) = wgk_15(p) * fct_pred_surv0(p)
    end if
  end do

  !somme
  surv(k) = SUM(fct_pred_surv_pond)
  if (idtrunc.eq.1) then
    surv0(k) = SUM(fct_pred_surv0_pond)
  end if

  !multiplier par hlgth -> resultat de l integral = risq cumule (sans varexp)
  surv(k) = surv(k) * hlgth(1)
  if (idtrunc.eq.1) then
    surv0(k) = surv0(k) * hlgth(2)
  else
    surv0(k) = 0
  end if

end subroutine fct_risq_irtsre_2


!-----------------------------------------------!
!             baseline RISK FUNCTION            !
!          in shared current level case         !
!-----------------------------------------------!

!!! fct du risq de base : renvoit une valeur
double precision function fct_risq_base_irtsre_2(t,i,k,brisq,arg,p)  !t time, i subject, k event, brisq prm de base, arg = 1 pr event ou 2 pr entry

  use modirtsre
        
  implicit none
  
  integer::i,k,arg,p
  double precision::t 
  double precision,dimension(nprisq(k))::brisq
  
  integer::j,ll,kk
  double precision::risq0 

  risq0=0.d0
  if (typrisq(k).eq.2.and.logspecif.eq.1) then     !Weibull & exponentiel
     
     risq0=brisq(1)*brisq(2)*(t-zi(1,k))**(brisq(2)-1)   !fct risq base au tps t, pq tsurv(i)-zi(1,k) ? zi(1,k)=depart Weibull
     
  end if
  
  if (typrisq(k).eq.2.and.logspecif.eq.0) then         !Weibull & quadratique

     risq0=brisq(1)*brisq(2)*(brisq(1)*(t-zi(1,k)))**(brisq(2)-1)
  end if
 
  if (typrisq(k).eq.1) then        !piecewise
     do j=1,nz(k)-1
        if (t.ge.zi(j,k).and.t.le.zi(j+1,k)) then
           risq0=brisq(j)
        end if
     end do
  end if
  
  if (typrisq(k).eq.3) then          !splines
     ll=0
     if (t.eq.zi(nz(k),k)) then
        ll=nz(k)-1
     end if
     do kk=2,nz(k)
        if ((t.ge.zi(kk-1,k)).and.(t.lt.zi(kk,k))) &
             then
           ll=kk-1
        end if
     end do

     if(p.eq.0) then
        if (arg.eq.1) then
          risq0=brisq(ll)*Tmm3(ns*(k-1)+i)+brisq(ll+1)*Tmm2(ns*(k-1)+i)     &
            +brisq(ll+2)*Tmm1(ns*(k-1)+i)+brisq(ll+3)*Tmm(ns*(k-1)+i)
        else if (arg.eq.2) then
          risq0=brisq(ll)*Tmm03(ns*(k-1)+i)+brisq(ll+1)*Tmm02(ns*(k-1)+i)     &
            +brisq(ll+2)*Tmm01(ns*(k-1)+i)+brisq(ll+3)*Tmm0(ns*(k-1)+i)
        end if
     else
        if (arg.eq.1) then
          risq0=brisq(ll)*Tmm3_st2(p,ns*(k-1)+i)+brisq(ll+1)*Tmm2_st2(p,ns*(k-1)+i)     &
            +brisq(ll+2)*Tmm1_st2(p,ns*(k-1)+i)+brisq(ll+3)*Tmm_st2(p,ns*(k-1)+i)
        else if (arg.eq.2) then
          risq0=brisq(ll)*Tmm03_st2(p,ns*(k-1)+i)+brisq(ll+1)*Tmm02_st2(p,ns*(k-1)+i)     &
            +brisq(ll+2)*Tmm01_st2(p,ns*(k-1)+i)+brisq(ll+3)*Tmm0_st2(p,ns*(k-1)+i)
        end if
     end if
  end if
  
  
  fct_risq_base_irtsre_2 = risq0
  
  
end function fct_risq_base_irtsre_2


!-----------------------------------------------!
!    latent process current level predictions   !
!             at each GK time points            !
!          in shared current level case         !
!-----------------------------------------------!

!!! fct de prediction du niv courant  : renvoie une matrice de 2 lignes/event,entree et 15 colonnes/pnts qua 
subroutine fct_pred_curlev_irtsre_2(i,beta_ef,ui,pred_GK)

  use modirtsre

  implicit none

  integer::i
  double precision,dimension(nef)::beta_ef
  double precision,dimension(nea)::ui
  double precision,dimension(2,15)::pred_GK
  
  integer::p,ll
  
  pred_GK = 0.d0

  ! X(t) %*% beta : necessite X au tps de quadrature t
  do p=1,15
    do ll=1,id_nXcl(1)
       pred_GK(1,p) = pred_GK(1,p) + Xcl_GK((i-1)*15+p,ll+1) * beta_ef(ll)
      !intercept pas estime
      if (idtrunc.eq.1) then
        pred_GK(2,p) = pred_GK(2,p) + Xcl0_GK((i-1)*15+p,ll+1) * beta_ef(ll)
      end if
   end do
  end do

  ! Z(t) %*% ui : necessite Z au tps de quadrature t
  do p=1,15
    do ll=1,id_nXcl(2)
      pred_GK(1,p) = pred_GK(1,p) + Xcl_GK((i-1)*15+p,1+nef+ll) * ui(ll)    !ui=vect de taille nea
      if (idtrunc.eq.1) then
         pred_GK(2,p) = pred_GK(2,p) + Xcl0_GK((i-1)*15+p,1+nef+ll)* ui(ll)
      end if
    end do
  end do

end subroutine fct_pred_curlev_irtsre_2


!!! (subroutine) prediction de la pente courante
!! resultat dans pred_GK une matrice de 2 lignes (event,entree) et 15 colonnes (pnts qua)
! idem fct_pred_curlev_irtsre_2 mais utilise les matrice Xcs au lieu de Xcl
subroutine fct_pred_curslope_irtsre_2(i, beta_ef, ui, pred_GK)

  use modirtsre

  implicit none

  integer::i
  double precision,dimension(nef)::beta_ef
  double precision,dimension(nea)::ui
  double precision,dimension(2,15)::pred_GK
  
  integer::p,ll
  
  pred_GK = 0.d0
  
  ! X'(t) %*% beta : necessite X au tps de quadrature t
  do p=1,15
    do ll=1,id_nXcl(1)
       pred_GK(1,p) = pred_GK(1,p) + Xcs_GK((i-1)*15+p,ll) * beta_ef(ll)
      !intercept pas estime
      if (idtrunc.eq.1) then
        pred_GK(2,p) = pred_GK(2,p) + Xcs0_GK((i-1)*15+p,ll) * beta_ef(ll)
      end if
   end do

  end do

  ! Z'(t) %*% ui : necessite Z au tps de quadrature t
  do p=1,15
    do ll=1,id_nXcl(2)
      pred_GK(1,p) = pred_GK(1,p) + Xcs_GK((i-1)*15+p,nef+ll) * ui(ll)    !ui=vect de taille nea
      if (idtrunc.eq.1) then
        pred_GK(2,p) = pred_GK(2,p) + Xcs0_GK((i-1)*15+p,nef+ll)* ui(ll)
      end if
    end do
  end do

end subroutine fct_pred_curslope_irtsre_2



!! fct curlev : prediction du niveau courant au temps de survie Ti
double precision function curlev(i, beta, ui)

  use modirtsre, only : nef, nea, id_nXcl, Xcl_Ti
  
  implicit none

  integer :: i, j
  double precision, dimension(nef) :: beta
  double precision, dimension(nea) :: ui

  curlev = 0.d0
  do j = 1, id_nXcl(1) ! id_nXcl(1) = nef
     curlev = curlev + Xcl_Ti(i, 1 + j) * beta(j)
  end do
  
  do j = 1, id_nXcl(2) ! id_nXcl(2) = nea
     curlev = curlev + Xcl_Ti(i, 1 + nef + j) * ui(j)
  end do
 
end function curlev




!! fct curslope : prediction de la pente courant au temps de survie Ti
double precision function curslope(i, beta, ui)

  use modirtsre, only : nef, nea, id_nXcl, Xcs_Ti
  
  implicit none

  integer :: i, j
  double precision, dimension(nef) :: beta
  double precision, dimension(nea) :: ui

  curslope = 0.d0
  do j = 1, id_nXcl(1) ! id_nXcl(1) = nef
     curslope = curslope + Xcs_Ti(i, j) * beta(j)
  end do
  
  do j = 1, id_nXcl(2) ! id_nXcl(2) = nea
     curslope = curslope + Xcs_Ti(i, nef + j) * ui(j)
  end do
  
end function curslope




!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!           POSTFIT           !!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!-----------------------------------------------!
!       estimated baseline risk functions       !
!-----------------------------------------------!

!! fct_risq_estime
!TS: no linear predictor
subroutine fct_risq_estime_irtsre(k,brisq,time,nsim,risq,surv)

  use modirtsre
        
  implicit none
  
  integer::i,k,nsim
  double precision,dimension(nprisq(k))::brisq
  double precision,dimension(nsim)::time
  double precision,dimension(nsim,nbevt)::risq,surv
  
  integer::j,l,ll,kk,ii
  double precision::som
  double precision::ht,htm,h2t,ht2,ht3,hht,h,hh,h2,h3,h4,h3m,h2n,hn,hh3,hh2
  double precision::Tmm3_est,Tmm2_est,Tmm1_est,Tmm_est,Tim3_est,Tim2_est,Tim1_est,Tim_est

  if(typrisq(k).eq.3) then
     zi(-2,k)=zi(1,k)
     zi(-1,k)=zi(1,k)
     zi(0,k)=zi(1,k)
     zi(nz(k)+1,k)=zi(nz(k),k)
     zi(nz(k)+2,k)=zi(nz(k),k)
     zi(nz(k)+3,k)=zi(nz(k),k)
  end if
        
  l=0
  do i=1,nsim
     
     if (typrisq(k).eq.2.and.logspecif.eq.1) then
        
        surv(i,k)=brisq(1)*(time(i)-zi(1,k))**brisq(2)     
        risq(i,k)=brisq(1)*brisq(2)*(time(i)-zi(1,k))**(brisq(2)-1)
        
     end if
     if (typrisq(k).eq.2.and.logspecif.eq.0) then
     
        surv(i,k)=(brisq(1)*(time(i)-zi(1,k)))**brisq(2)
        risq(i,k)=brisq(1)*brisq(2)*(brisq(1)*(time(i)-zi(1,k)))**(brisq(2)-1)
     
     end if
 

     if (typrisq(k).eq.1) then
        do j=1,nz(k)-1
           som=0.d0
           do l=1,j-1
              som=som+brisq(l)*(zi(l+1,k)-zi(l,k))
           end do
           if (time(i).ge.zi(j,k).and.time(i).le.zi(j+1,k)) then
              surv(i,k)=som+brisq(j)*(time(i)-zi(j,k))
              risq(i,k)=brisq(j)
           end if
        end do
     end if
  

     if (typrisq(k).eq.3) then
        ll=0
        if (time(i).eq.zi(nz(k),k)) then
           ll=nz(k)-1
        end if
        som=0.d0
        do kk=2,nz(k)
           if ((time(i).ge.zi(kk-1,k)).and.(time(i).lt.zi(kk,k))) &
                then
              ll=kk-1
           end if
        end do
        if (ll.gt.1) then
           do ii=1,ll-1
              som=som+brisq(ii)
           end do
        end if

        ht = time(i)-zi(ll,k)
        htm = time(i)-zi(ll-1,k)
        h2t = time(i)-zi(ll+2,k)
        ht2 = zi(ll+1,k)-time(i)
        ht3 = zi(ll+3,k)-time(i)
        hht = time(i)-zi(ll-2,k)
        h = zi(ll+1,k)-zi(ll,k)
        hh = zi(ll+1,k)-zi(ll-1,k)
        h2 = zi(ll+2,k)-zi(ll,k)
        h3 = zi(ll+3,k)-zi(ll,k)
        h4 = zi(ll+4,k)-zi(ll,k)
        h3m = zi(ll+3,k)-zi(ll-1,k)
        h2n = zi(ll+2,k)-zi(ll-1,k)
        hn = zi(ll+1,k)-zi(ll-2,k)
        hh3 = zi(ll+1,k)-zi(ll-3,k)
        hh2 = zi(ll+2,k)-zi(ll-2,k)
        
     
        if (time(i).ne.zi(nz(k),k)) then
           
           Tmm3_est = ((4.d0*ht2*ht2*ht2)/(h*hh*hn*hh3))
           Tmm2_est = ((4.d0*hht*ht2*ht2)/(hh2*hh*h*hn))  &
                +((-4.d0*h2t*htm*ht2)/(hh2*h2n*hh*h))  &
                +((4.d0*h2t*h2t*ht)/(hh2*h2*h*h2n))
           Tmm1_est = (4.d0*(htm*htm*ht2)/(h3m*h2n*hh*h)) &
                +((-4.d0*htm*ht*h2t)/(h3m*h2*h*h2n))   &
                +((4.d0*ht3*ht*ht)/(h3m*h3*h2*h))
           Tmm_est = 4.d0*(ht*ht*ht)/(h4*h3*h2*h)
           
        end if
        
        if (time(i).eq.zi(nz(k),k)) then
           
           Tmm3_est = 0.d0
           Tmm2_est = 0.d0
           Tmm1_est = 0.d0
           Tmm_est = 4.d0/h
           
        end if
        
        Tim3_est = (0.25d0*(time(i)-zi(ll-3,k))*Tmm3_est) &
             +(0.25d0*hh2*Tmm2_est)        &
             +(0.25d0*h3m*Tmm1_est)+(0.25d0*h4*Tmm_est)
        Tim2_est = (0.25d0*hht*Tmm2_est)  &
             +(h3m*Tmm1_est*0.25d0)+(h4*Tmm_est*0.25d0)
        Tim1_est = (htm*Tmm1_est*0.25d0)+(h4*Tmm_est*0.25d0)
        Tim_est = ht*Tmm_est*0.25d0
        
        surv(i,k)=som+brisq(ll)*Tim3_est+brisq(ll+1)*Tim2_est &
             +brisq(ll+2)*Tim1_est+brisq(ll+3)*Tim_est
        risq(i,k)=brisq(ll)*Tmm3_est+brisq(ll+1)*Tmm2_est     &
             +brisq(ll+2)*Tmm1_est+brisq(ll+3)*Tmm_est
        
     end if

  end do
  
end subroutine fct_risq_estime_irtsre


!-----------------------------------------------!
!       estimated and transformed outcomes      !
!-----------------------------------------------!

subroutine transfos_estimees_irtsre(b,npm,nsim,marker,transfY)
  
  use modirtsre
  
  implicit none
  
  integer::kk,nsim,npm,j,k,yk,sumntr,numSPL,l,avantntr
  double precision,dimension(nsim*ny)::marker,transfY
  double precision,dimension(maxval(ntr))::splaa
  double precision,dimension(maxval(ntr))::Xspl
  double precision,dimension(nsim)::mmm,mmm1,mmm2,iim,iim1,iim2
  double precision::aa1,bb1,dd1,aa,bb,betai,eps,pas,ytemp,cc1
  double precision, dimension(npm)::b,b1
  double precision ::ht,htm,ht2,ht3,hht,h,hh,h2,h3,h2n,hn
  
  
  b1=0.d0
  eps=1.D-20
  do k=1,npm
     b1(k)=b(k)
  end do

  marker=0.d0
  transfY=0.d0

  avantntr = nrisqtot+nvarxevt+nasso+nef+ncontr+nvc+ncor
  
  sumntr = 0
  numSPL =0
  do yk=1,ny
     
     pas=(maxY(yk)-minY(yk))/dble(nsim-1)
     j=1
     marker((yk-1)*nsim+1)=minY(yk)
     do while(j.lt.nsim)
        j=j+1
        marker((yk-1)*nsim+j)=marker((yk-1)*nsim+j-1)+pas
     end do
     marker(yk*nsim)=maxY(yk)


     if (idlink(yk).eq.2) then
        numSPL = numSPL+1
        splaa=0.d0

        splaa(1)=b1(avantntr+sumntr+1)
        do kk=2,ntr(yk)
           splaa(kk)=b1(avantntr+sumntr+kk)*b1(avantntr+sumntr+kk)
        end do
        
        !calcul de H(y) 
        do j=1,nsim
           ! ou se trouve la valeur
           l=0
           
           do k = 2,ntr(yk)-2
              if ((marker((yk-1)*nsim+j).ge.zitr(k-1,numSPL)).and.(marker((yk-1)*nsim+j).lt.zitr(k,numSPL))) then
                 l=k-1
              end if
           end do
           
           if (marker((yk-1)*nsim+j).eq.zitr(ntr(yk)-2,numSPL)) then
              l=ntr(yk)-3
           end if
           
           ht2 = zitr(l+1,numSPL)-marker((yk-1)*nsim+j)
           htm= marker((yk-1)*nsim+j)-zitr(l-1,numSPL)
           ht = marker((yk-1)*nsim+j)-zitr(l,numSPL)
           ht3 = zitr(l+2,numSPL)-marker((yk-1)*nsim+j)
           hht = marker((yk-1)*nsim+j)-zitr(l-2,numSPL)
           h = zitr(l+1,numSPL)-zitr(l,numSPL)
           hh= zitr(l+1,numSPL)-zitr(l-1,numSPL)
           hn= zitr(l+1,numSPL)-zitr(l-2,numSPL)
           h2n=zitr(l+2,numSPL)-zitr(l-1,numSPL)
           h2= zitr(l+2,numSPL)-zitr(l,numSPL)
           h3= zitr(l+3,numSPL)-zitr(l,numSPL)
           
           if (marker((yk-1)*nsim+j).ne.zitr(ntr(yk)-2,numSPL)) then
              mmm2(j) = (3.d0*ht2*ht2)/(hh*h*hn)
              mmm1(j) = (3.d0*htm*ht2)/(h2n*hh*h)+(3.d0*ht*ht3)/(h2*h*h2n)
              mmm(j)  = (3.d0*ht*ht)/(h3*h2*h)
           end if
           if (marker((yk-1)*nsim+j).eq.zitr(ntr(yk)-2,numSPL)) then
              mmm2(j) = 0.d0
              mmm1(j) = 0.d0
              mmm(j)  = 3.d0/h
           end if
           
           iim2(j)=hht*mmm2(j)/(3.d0)+ h2n*mmm1(j)/(3.d0) &
                +h3*mmm(j)/(3.d0)
           iim1(j)=htm*mmm1(j)/(3.d0)+h3*mmm(j)/(3.d0)
           
           iim(j)=ht*mmm(j)/(3.d0)
           
           !-------- transformation et IC de la transformation :
           
           Xspl=0.d0
           Xspl(1)=1
           do k=2,l
              Xspl(k)=1
           end do
           Xspl(l+1)=iim2(j)
           Xspl(l+2)=iim1(j)
           Xspl(l+3)=iim(j)
           transfY((yk-1)*nsim+j)= dot_product(Xspl,splaa)
        end do
        !fin H(y)

     else if (idlink(yk).eq.1) then
        
        aa1=exp(b1(avantntr+sumntr+1))/ &
             (1+exp(b1(avantntr+sumntr+1)))
        bb1=exp(b1(avantntr+sumntr+2))/ &
             (1+exp(b1(avantntr+sumntr+2)))
        bb1=aa1*(1.d0-aa1)*bb1
        cc1=b1(avantntr+sumntr+3)
        dd1=abs(b1(avantntr+sumntr+4))
        
        aa=aa1*aa1*(1-aa1)/bb1-aa1
        bb=aa*(1-aa1)/aa1
        
        do j=1,nsim
           ytemp=(marker((yk-1)*nsim+j)-minY(yk)+epsY(yk))/(maxY(yk)-minY(yk)+2*epsY(yk))
           transfY((yk-1)*nsim+j)=(betai(aa,bb,ytemp)-cc1)/dd1
           if (transfY((yk-1)*nsim+j).eq.999.d0) then
              !                    write(*,*)'problem'
           end if
           
        end do
        
        
     else if (idlink(yk).eq.0) then
        
        do j=1,nsim
           transfY((yk-1)*nsim+j)=(marker((yk-1)*nsim+j)-b1(avantntr+sumntr+1)) &
                /abs(b1(avantntr+sumntr+2))
        end do

     end if

     !! cas idlink = 3 fait dans R
     
     sumntr = sumntr + ntr(yk)
     
  end do

end subroutine transfos_estimees_irtsre


!!  associations non lineaires :

  !! polynomes de degre 3
double precision function poly(x, coef, center)

  implicit none

  double precision::x
  double precision, dimension(3)::coef
  double precision::center

  poly = coef(1) * x + coef(2) * (x - center)**2 + coef(3) * (x - center)**3
  
end function poly


subroutine poly15(x, coef, center)

  implicit none

  integer::j
  double precision,dimension(15)::x
  double precision, dimension(3)::coef
  double precision::center
  double precision, external:: poly

  do j = 1, 15
     !x(j) = coef(1) * x(j) + coef(2) * (x(j) - center)**2 + coef(3) * (x(j) - center)**3
     x(j) = poly(x(j), coef, center)
  end do
  
end subroutine poly15

  
!! logistique
double precision function logit(x, coef)

  implicit none

  double precision::x
  double precision, dimension(3)::coef

  logit = coef(1) /(1 + exp(-coef(2) * (x - coef(3))))
  
end function logit
  
subroutine logit15(x, coef)

  implicit none

  integer::j
  double precision,dimension(15)::x
  double precision, dimension(3)::coef
  double precision, external :: logit

  do j = 1, 15
     x(j) = logit(x(j), coef)
  end do

end subroutine logit15

!! sigmoid basee sur tangente hyperbolique
double precision function sigmoid(x, coef)

  implicit none

  double precision::x
  double precision, dimension(3)::coef

  !!sigmoid = coef(1) * (exp(x) - exp(-x)) / (coef(2)**2 * exp(x) + coef(3)**2 * exp(-x)) 
!!  sigmoid = coef(1) * (exp(x) - exp(-x)) / (exp(coef(2)) * exp(x) + exp(coef(3)) * exp(-x))

  sigmoid = coef(1) / (1 + exp(-(x - coef(2)) / exp(coef(3)))) - coef(1) / (1 + exp(coef(2) / exp(coef(3))))
end function sigmoid
  
subroutine sigmoid15(x, coef)

  implicit none

  integer::j
  double precision,dimension(15)::x
  double precision, dimension(3)::coef
  double precision, external :: sigmoid

  do j = 1, 15
     x(j) = sigmoid(x(j), coef)
  end do

end subroutine sigmoid15



!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!       LIFE EXPECTANCY       !!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!! pour calculer les esperances de vie !!!
!! -> on se sert de la fonction de vraisemblance, eventuellement seulement sur une partie des items, et avec expectancy=1
!TS: pas modif pr niv courant partage

!subroutine proba_irtsre

#' createX0
#'
#' Create the matrix containing all covariates appearing in the formula
#' 
#'
#' Caution 1 : 'data' should not contain any missing values. There is no check for that within the function.
#' Caution 2 : if interaction terms are present, the names are sorted. For example x:age becomes age:x in X0.
#' Caution 3 : intercept is not systematically in the first column. It will only if form[[1]] contains an intercept.
#' 
#' @param form a list of formula
#' @param data a data frame containing all variables mentionned in form
#' @return a list containing
#'   \item{X0}{the pooled (with cbind) model matrix. The number of lines is the same as in 'data'}
#'  \item{idform}{a list of the same length as 'form' specifying which term of X0 appear in each formula}
#' 
#'@export
createX0 <- function(form, data)
{   
    nform <- length(form)

    ## combine all model.matrix
    X0 <- NULL
    namesxk <- vector("list", nform)
    for(k in 1:nform)
    {
        xk <- model.matrix(delete.response(terms(form[[k]])), data=data)
        if(ncol(xk) == 0) next;

        z <- strsplit(colnames(xk), split=":", fixed=TRUE)
        z <- lapply(z, sort)
        colnames(xk) <- sapply(z, function(x) paste(x, collapse=":"))
        namesxk[[k]] <- colnames(xk)
        
        X0 <- cbind(X0, xk)
    }
 
    ## remove duplicated columns
    if(!is.null(X0))
    {
        unames <- unique(colnames(X0))
        X0 <- X0[,unames, drop=FALSE]
        z.X0 <- strsplit(unames, split=":", fixed=TRUE)
        z.X0 <- lapply(z.X0, sort)
        namesX0 <- sapply(z.X0, function(x) paste(x, collapse=":"))
    }
    else
    {
        X0 <- matrix(0, nrow(data), 0)
    }
    
    ## indicator of presence in each formula
    idform <- vector("list", nform)
    for(k in 1:nform)
    {
        if(!is.null(namesxk[[k]]))
        {
            idform[[k]] <- (namesX0 %in% namesxk[[k]]) + 0
        }
        else
        {
            idform[[k]] <- rep(0, ncol(X0))
        }
    }

    return(list(X0=X0, idform=idform))
}

#' Random effects prediction
#'
#' The random effects of a \code{jointLPM} model are predicted as the mode of 
#' the conditional distribution of the random effects given the subject's
#' longitudinal and survival observations.
#'
#' @param x a \code{jointLPM} object
#' @param data a data.frame object containing the data from the predictions are
#'  to be computed
#' @param control optional list of arguments to be passed to the
#' \code{marqLevAlg} optimization function.
#' @param variance logical indicating whether the variance of the estimated
#' predictions should be returned. Default ot FALSE.
#'
#' @return a matrix containing the predicted random effects (in columns) for
#' each subject (in rows). If \code{variance = TRUE}, returns a list containing
#' the previous matrix and a vector containing the upper part of each variance
#' matrix.
#' @export
predictRE <- function(x, data, control = NULL, variance = FALSE)
{
    data <- as.data.frame(data)

    ## loglik arguments
    Fargs <- createFargs(x, data)

    ## no cor, no randomY
    if(Fargs$ncor0 > 0) stop("No implemented yet with cor")
    if(Fargs$nalea0 > 0) stop("No implemented yet with randomY")

    ## prepare result
    ns <- Fargs$ns0
    nea <- sum(Fargs$idea0)
    predRE <- matrix(NA, ns, 1 + nea)
    colnames(predRE) <- c(x$Names$ID, x$Names$Xnames[which(x$idea == 1)])
    varRE <- rep(NA, ns * nea * (nea + 1) / 2)

    ## id in (clean) data
    id <- as.numeric(Fargs$ID)
    Fargs <- Fargs[setdiff(names(Fargs), "ID")]

    ## remove unused arguments
    Fargs$ns0 <- NULL
    Fargs$methInteg0 <- NULL
    Fargs$nMC0 <- NULL
    Fargs$dimMC0 <- NULL
    Fargs$seqMC0 <- NULL
    Fargs$expectancy0 <- NULL
    if(Fargs$nfix0 > 0) # we don't need to distingish b from bfix
    {
        Fargs$npm0 <- Fargs$npm0 + Fargs$nfix0
        btot <- rep(NA, Fargs$npm0)
        btot[which(Fargs$fix0 == 0)] <- Fargs$b0
        btot[which(Fargs$fix0 == 1)] <- Fargs$bfix0
        Fargs$b0 <- btot
    }
    Fargs$fix0 <- NULL
    Fargs$nfix0 <- NULL
    Fargs$bfix0 <- NULL

    args_i <- Fargs

    ## optimization options
    defaultmla <- list(b = rep(0, nea), gr = NULL, hess = NULL, maxiter = 30, epsa = 0.0001, epsb = 0.0001, epsd = 0.0001, partialH = NULL, digits = 8, print.info = FALSE, blinding = TRUE, multipleTry = 25, nproc = 1, clustertype = NULL, file = "", .packages = NULL)
    controlmla <- control[which(names(control) %in% names(defaultmla))]
    defaultmla[names(controlmla)] <- controlmla
    
    args_mla <- c(list(m = nea, fn = REconddensity, minimize = FALSE),
                  defaultmla)

    ## different initial values for each subject
    matinit <- FALSE
    if(!is.null(controlmla$b))
    {
        binit <- controlmla$b
        if(is.matrix(binit))
        {
            if(nrow(binit) != ns) stop("control$b should be a vector or a matrix with as many rows as subjects")
            if(ncol(binit) != nea) stop("control$b should be a vector or a matrix with as many columns as random effects")

            matinit <- TRUE
        }
    }
    
    nmes <- Fargs$nmes0
#browser()
    ## loop over the subjects
    sumnmes <- 0
    sumvarRE <- 0
    for(i in 1:ns)
    {
        ## data of subject i
        args_i$Y0 <- Fargs$Y0[sumnmes + 1:sum(nmes[i, ])]
        args_i$X0 <- Fargs$X0[sumnmes + 1:sum(nmes[i, ]), ]
        args_i$Tentr0 <- Fargs$Tentr0[i]
        args_i$Tevt0 <- Fargs$Tevt0[i]
        args_i$Devt0 <- Fargs$Devt0[i]
        args_i$ind_survint0 <- Fargs$ind_survint0[i]
        args_i$nmes0 <- Fargs$nmes0[i, ]
        args_i$indiceY0 <- Fargs$indiceY0[sumnmes + 1:sum(nmes[i, ])]

        if(Fargs$idst0 %in% c(2, 4))
        {
            args_i$Xcl_Ti0 <- Fargs$Xcl_Ti0[i, ]
            args_i$Xcl_GK0 <- Fargs$Xcl_GK0[15 * (i - 1) + 1:15, ]
        }
        
        if(Fargs$idst0 %in% c(3, 4))
        {
            args_i$Xcs_Ti0 <- Fargs$Xcs_Ti0[i, ]
            args_i$Xcs_GK0 <- Fargs$Xcs_GK0[15 * (i - 1) + 1:15, ]
        }

        if(matinit)
            args_mla$b <- binit[i, ]

        ## find the mode of the distribution
        res_i <- do.call(marqLevAlg::mla, c(args_mla, args_i))

        ## save results
        if(res_i$istop %in% c(1, 3))
            predRE[i, ] <- c(id[i], res_i$b)
        else
            predRE[i, 1] <- id[i]

        ## variance
        if(variance == TRUE)
        {
            varRE[sumvarRE + 1:(nea * (nea + 1) / 2)] <- res_i$v
        }

        sumnmes <- sumnmes + sum(nmes[i, ])
        sumvarRE <- sumvarRE + nea * (nea + 1) / 2
    }

    if(variance == TRUE)
        return(list(predRE = predRE, varRE = varRE))
    else
        return(predRE)
}

#' REconddendity
#'
#' Wrapper to the Fortran subroutine computing the conditional density of the random effects
#'
#' @param ui0 ui0
#' @param Y0 Y0
#' @param X0 X0
#' @param Tentr0 Tentr0
#' @param Tevt0 Tevt0
#' @param Devt0 Devt0
#' @param ind_survint0 ind_survint0
#' @param idea0 idea0
#' @param idg0 idg0
#' @param idcor0 idcor0
#' @param idcontr0 idcontr0
#' @param idsurv0 idsurv0
#' @param idtdv0 idtdv0
#' @param typrisq0 typrisq0
#' @param nz0 nz0
#' @param zi0 zi0
#' @param nbevt0 nbevt0
#' @param idtrunc0 idtrunc0
#' @param logspecif0 logspecif0
#' @param ny0 ny0
#' @param nv0 nv0
#' @param nobs0 nobs0
#' @param nmes0 nmes0
#' @param idiag0 idiag0
#' @param ncor0 ncor0
#' @param nalea0 nalea0
#' @param npm0 npm0
#' @param b0 b0
#' @param epsY0 epsY0
#' @param idlink0 idlink0
#' @param nbzitr0 nbzitr0
#' @param zitr0 zitr0
#' @param uniqueY0 uniqueY0
#' @param indiceY0 indiceY0
#' @param nvalSPLORD0 nvalSPLORD0
#' @param idst0 idst0
#' @param nXcl0 nXcl0
#' @param Xcl_Ti0 Xcl_Ti0
#' @param Xcl_GK0 Xcl_GK0
#' @param Xcs_Ti0 Xcs_Ti0
#' @param Xcs_GK0 Xcs_GK0
#' @param nonlin0 nonlin0
#' @param centerpoly0 centerpoly0
#' @return the log-density
#' @export
REconddensity <- function(ui0,Y0,X0,Tentr0,Tevt0,Devt0,ind_survint0,idea0,idg0,idcor0,idcontr0,
                   idsurv0,idtdv0,typrisq0,nz0,zi0,nbevt0,idtrunc0,logspecif0,ny0,
                   nv0,nobs0,nmes0,idiag0,ncor0,nalea0,npm0,b0,
                   epsY0,idlink0,nbzitr0,zitr0,uniqueY0,indiceY0,nvalSPLORD0,
                   idst0,nXcl0,Xcl_Ti0,Xcl_GK0,
                   Xcs_Ti0,Xcs_GK0,nonlin0,centerpoly0)
{
    res <- 0
    .Fortran(C_reconddensity,as.double(ui0),as.double(Y0),as.double(X0),as.double(Tentr0),as.double(Tevt0),as.integer(Devt0),as.integer(ind_survint0),as.integer(idea0),as.integer(idg0),as.integer(idcor0),as.integer(idcontr0),as.integer(idsurv0),as.integer(idtdv0),as.integer(typrisq0),as.integer(nz0),as.double(zi0),as.integer(nbevt0),as.integer(idtrunc0),as.integer(logspecif0),as.integer(ny0),as.integer(nv0),as.integer(nobs0),as.integer(nmes0),as.integer(idiag0),as.integer(ncor0),as.integer(nalea0),as.integer(npm0),as.double(b0),as.double(epsY0),as.integer(idlink0),as.integer(nbzitr0),as.double(zitr0),as.double(uniqueY0),as.integer(indiceY0),as.integer(nvalSPLORD0),as.integer(idst0),as.integer(nXcl0),as.double(Xcl_Ti0),as.double(Xcl_GK0),as.double(Xcs_Ti0),as.double(Xcs_GK0),as.integer(nonlin0),as.double(centerpoly0),loglik_res=as.double(res))$loglik_res
}

#' Likelihood arguments
#'
#' Create the arguments to be passed to the \code{loglik} function
#'
#' @param x a \code{jointLPM} object
#' @param data a data.frame containing the estimation dataset
#' @return a named list
#'@export
createFargs <- function(x, data)
{
    namesargs <- c("b0", "Y0", "X0", "Tentr0", "Tevt0", "Devt0", "ind_survint0",
                   "idea0", "idg0", "idcor0", "idcontr0", "idsurv0", "idtdv0",
                   "typrisq0", "nz0", "zi0", "nbevt0", "idtrunc0", "logspecif0",
                   "ny0", "ns0", "nv0", "nobs0", "nmes0", "idiag0", "ncor0",
                   "nalea0", "npm0", "nfix0", "bfix0", "epsY0", "idlink0",
                   "nbzitr0", "zitr0", "uniqueY0", "indiceY0", "nvalSPLORD0",
                   "fix0", "methInteg0", "nMC0", "dimMC0", "seqMC0", "idst0",
                   "nXcl0", "Xcl_Ti0", "Xcl_GK0", "Xcs_Ti0", "Xcs_GK0",
                   "nonlin0", "centerpoly0", "expectancy0")

    res <- vector("list", 1 + length(namesargs))
    names(res) <- c("ID", namesargs)

    best <- coef(x)
    posfix <- x$posfix
    fix <- rep(0, length(best))
    if(length(posfix))
    {
        res$b0 <- best[setdiff(1:length(best), posfix)]
        res$bfix0 <- best[posfix]
        res$nfix0 <- length(posfix)
        fix[posfix] <- 1
        
    }
    else
    {
        res$b0 <- best
        res$bfix0 <- 0
        res$nfix0 <- 0
    }
    res$fix0 <- fix
    res$npm0 <- length(res$b0)

    res$idea0 <- x$idea
    res$idg0 <- x$idg
    res$idcor0 <- x$idcor
    res$idcontr0 <- x$idcontr
    res$idsurv0 <- x$idsurv
    res$idtdv0 <- x$idtdv
    res$typrisq0 <- x$typrisq
    res$nz0 <- x$nz
    res$zi0 <- x$hazardnodes
    res$nbevt0 <- length(x$nevent)
    res$idtrunc0 <- ifelse(length(x$call$survival[[2]]) == 4, 1, 0)
    res$logspecif0 <- x$logspecif
    res$ny0 <- x$N[12]
    res$nv0 <- length(x$Names$Xnames)
    res$idiag0 <- x$idiag
    res$ncor0 <- x$N[9]
    res$nalea0 <- x$N[11]
    res$epsY0 <- x$epsY
    res$idlink0 <- x$linktype

    nbzitr <- rep(2, res$ny0)
    nbzitr[which(res$idlink0 == 2)] <- x$nbnodes
    res$nbzitr0 <- nbzitr
    
    res$zitr0 <- x$linknodes

    methInteg <- 3
    if(!is.null(x$call$methInteg))
    {
        if(as.character(x$call$methInteg) == "MCO") methInteg <- 1
        if(as.character(x$call$methInteg) == "MCA") methInteg <- 2
    }
    res$methInteg0 <- methInteg
    
    res$nMC0 <- x$nMC
    res$idst0 <- x$sharedtype
    res$nonlin0 <- x$nonlin
    res$centerpoly0 <- x$centerpoly
    res$expectancy0 <- 0

    if(!is.null(data))
    {
        resNA <- removeNA(x$terms, data)
        dataWithoutNA <- resNA$newdata

        nobsparY <- resNA$nmes
        uniqueY0 <- NULL
        indiceY0 <- NULL
        nvalSPLORD <- rep(0, res$ny0)
        sumnobsparY <- 0
        nb <- 0
        for(k in 1:res$ny0)
        {
            if((res$idlink0 != 2) & (res$idlink0 != 3))
            {
                indiceY0 <- c(indiceY0, rep(0, nobsparY[k]))
                next
            }

            yk <- dataWithoutNA[sumnobsparY + 1:nobsparY[k],]
            uniqueTemp <- sort(unique(yk))
            permut <- order(order(yk))

            if(length(as.vector(table(yk))) == length(uniqueTemp))
            {
                indice <- rep(1:length(uniqueTemp), as.vector(table(yk)))
                if(res$idlink0[k] == 2)
                {
                    indiceTemp <- nb + indice[permut]
                }
                else
                {
                    indiceTemp <- indice[permut]
                }
                
                nb <- nb + length(uniqueTemp)
                
                uniqueY0 <- c(uniqueY0, uniqueTemp)
                indiceY0 <- c(indiceY0, indiceTemp)
                nvalSPLORD[k] <- length(uniqueTemp)
            }
            else
            {
                uniqueY0 <- c(uniqueY0, yk)
                indiceY0 <- c(indiceY0, ifelse(res$idlink0[k] == 2, nb, 0) + c(1:length(yk)))
                nb <- nb + length(yk)
                nvalSPLORD[k] <- length(yk)
            }

            sumnobsparY <- sumnobsparY + nobsparY[k]
        }

        if(is.null(uniqueY0)) uniqueY0 <- 0        
        res$uniqueY0 <- uniqueY0
        res$indiceY0 <- indiceY0[order(dataWithoutNA[, x$Names$ID])]
        dataWithoutNA <- dataWithoutNA[order(dataWithoutNA[, x$Names$ID]),]
        res$Y0 <- as.vector(dataWithoutNA[, x$Names$Ynames])
        
        id <- unique(dataWithoutNA[, x$Names$ID])
        listnmes <- tapply(data[, x$Names$Ynames, drop = FALSE], data[, x$Names$ID], function(d) apply(d, 2, function(x) length(which(!is.na(x)))))
        if(length(x$Names$Ynames) > 1)
            res$nmes0 <- t(simplify2array(listnmes))
        else
            res$nmes0 <- matrix(listnmes, ncol = 1)
        
        formulas <- list(x$terms$fixed.right,
                         x$terms$random,
                         x$terms$surv.commun,
                         x$terms$surv.cause,
                         x$terms$form.cor)
        
        resX0 <- createX0(formulas, dataWithoutNA)
        res$X0 <- resX0$X0
        colnames(res$X0)[which(colnames(res$X0) == "(Intercept)")] <- "intercept"
        if(any(colnames(res$X0) != x$Names$Xnames)) stop("X0 !")

        res$nobs0 <- nrow(res$X0)
        maxmes <- max(res$nmes0)
        res$ns0 <- length(id)
        res$ID <- id

        if(res$nbevt0 > 0)
        {
            dataSurv <- dataWithoutNA[, c(x$Names$ID, x$Names$Tnames, x$Names$TimeDepVar.name)]
            dataSurv <- dataSurv[!duplicated(dataSurv[, x$Names$ID]),]

            if(res$idtrunc0 == 1)
            {
                res$Tentr0 <- dataSurv[, 2]
                res$Tevt0 <- dataSurv[, 3]
                res$Devt0 <- dataSurv[, 4]
            }
            else
            {
                res$Tentr0 <- rep(0, res$ns0)
                res$Tevt0 <- dataSurv[, 2]
                res$Devt0 <- dataSurv[, 3]
            }
            if(is.null(x$Names$TimeDepVar.name))
            {
                res$ind_survint0 <- rep(0, res$ns0)
            }
            else
            {
                Tint <- dataSurv[, x$Names$TimeDepVar.name]
                res$ind_survint0 <- (Tint < res$Tevt0) + 0
            }
            
            deriv <- ifelse(res$idst0 > 2, TRUE, FALSE)
            data_tmp <- dataWithoutNA[!duplicated(dataWithoutNA[, x$Names$ID]), setdiff(x$Names$Xvar, as.character(x$call$var.time)), drop = FALSE]
            X_GK <- matrixGK(data_tmp, x$terms$fixed.right, x$terms$random, as.character(x$call$var.time), res$idtrunc0, res$Tentr0, res$Tevt0, deriv, res$zi0[1,1])

            res$Xcl_Ti0 <- X_GK$Xpred_Ti 
            res$Xcl_GK0 <- X_GK$Xpred_cl 
            res$Xcs_Ti0 <- X_GK$Xpredcs_Ti 
            res$Xcs_GK0 <- X_GK$Xpred_cs 
            res$nXcl0 <- c(ncol(res$Xcl_Ti0), ncol(res$Xcl_GK0))

            if(is.null(res$Xcl_GK0)) res$Xcl_GK0 <- 0
            if(is.null(res$Xcs_Ti0)) res$Xcs_Ti0 <- 0
            if(is.null(res$Xcs_GK0)) res$Xcs_GK0 <- 0
        }
        else
        {
            res$Tentr0 <- rep(0, res$ns0)
            res$Tevt0 <- rep(0, res$ns0)
            res$Devt0 <- rep(0, res$ns0)
            res$ind_survint0 <- rep(0, res$ns0)
            res$nXcl0 <- c(1, 1)
            res$Xcl_Ti0 <- rep(0, res$ns0)
            res$Xcl_GK0 <- rep(0, 15 * res$ns0)
            res$Xcs_Ti0 <- rep(0, res$ns0)
            res$Xcs_GK0 <- rep(0, 15 * res$ns0)
        }
    }
    else
    {
        maxmes <- 20
    }
    
    seqMC <- 0
    dimMC <- 0
    if(methInteg==3)
    {
        dimMC <- sum(res$idea0) + res$nalea0
        if(res$ncor0 > 0) dimMC <- dimMC + maxmes # maxmes dep des donnees
        if((dimMC > 0) & (res$nMC0 > 0))
        {
            sequnif <- spacefillr::generate_sobol_owen_set(res$nMC0, dimMC)
            seqMC <- apply(sequnif, 2, qnorm)
        }
    }
    res$dimMC0 <- dimMC
    res$seqMC0 <- seqMC

    return(res)
}

#' Difference of sojourn times
#'
#' This function computes the difference between two sojourn times.
#'
#' @param x1 a sojournTime object, called with \code{draws = TRUE} and
#' \code{returndraws = TRUE}, computed for a specified seed.
#' @param x2 a sojournTime object, called with \code{draws = TRUE} and
#' \code{returndraws = TRUE}, computed with the same seed as \code{x1}.
#' 
#' @return returns the median, the 2.5\% and 97.5\% quantiles, the mean,
#' the standard deviation of the difference between the two sojourn times
#' and the number of removed draws (eventually due to computational issues) 

#' @export
diffSojournTime <- function(x1, x2)
{
    if(!inherits(x1, "sojournTime") | !inherits(x2, "sojournTime")) stop("Use only with sojournTime objects")
    if(!inherits(x1, "draws") | !inherits(x2, "draws")) stop("Please use returndraws = TRUE in the sojournTime call")
    if(!length(attr(x1, "seed")) | !length(attr(x1, "seed"))) stop("Please specify the seed in the sojournTime call")
    if(attr(x1, "seed") != attr(x2, "seed")) stop("The seeds should be the same in both sojournTime objects")

    res <- x1 - x2
    res_50 <- quantile(res, probs=0.5, na.rm=TRUE)
    res_2.5 <- quantile(res, probs=0.025, na.rm=TRUE)
    res_97.5 <- quantile(res, probs=0.975, na.rm=TRUE)
    
    res_mean <- mean(res, na.rm=TRUE)
    res_sd <- sd(res, na.rm=TRUE)
    nn <- length(which(is.na(res)))

    result <- c(res_50, res_2.5, res_97.5, res_mean, res_sd, nn)
    class(result) <- "diffSojournTime"

    return(result)
}

#' @export
coef.jointLPM <- function(object, ...)
{
    ## vector of parameters
    b <- object$best

    avt <- sum(object$N[1:6])
    nvc <- object$N[7]
    nea <- sum(object$idea)
    idiag <- object$idiag
    
    ## replace varcov with cholesky
    if(nvc > 0)
    {
        if(idiag == 0)
        {
            b[avt + 1:nvc] <- object$cholesky[-1]
        }
        else
        {
            id <- 1:nea
            indice <- c(id + id * (id - 1) / 2)
            b[avt + 1:nvc] <- object$cholesky[indice[-1]]
        }

        names(b) <- sub("varcov", "cholesky", names(b))
    }

    return(b)
}

#' Computation of sojourn time under specific condition 
#'
#' This function computes posterior computations from a joint shared 
#' random-effect model with ordinal longitudinal outcomes, aka a joint 
#' item response theory model. Specifically, the function computes the 
#' expected time spent before reaching a given level of impairment specified
#' by one or multiple items for a covariate profile. 
#'
#' 1. Lifetime expected sojourn time: the expected time before reaching 
#' level k at item(s) Y is the integral over time t, from 0 to infinity, 
#' of P(Y(t) <= k, T > t), i.e., int_0^infty P(Y(t) <= k, T > t) dt.
#'
#' 2. Residual expected sojourn time from a time s: conditionally on being
#'  below m at item(s) Y at time s and being alive at time s, the 
#'  sojourn time below level k at item(s) Y is computed as: 
#'  int_s^infty P(Y(t) <= k, T > t | Y(s) <= m, T > s) dt = 
#'  int_s^infty P(Y(t) <= k, T > t, Y(s) <= m) dt / P(Y(s) <= m, T > s)
#' 
#' 
#' @param x an object of class \code{jointLPM} representing of joint shared 
#' random effects model with binary or ordinal longitudinal outcome(s)
#' @param maxState a list specifying the items and the corresponding levels 
#' defining the maximum state for the computation of the sojourn time. For instance 
#' \code{maxState=list(Y = 3)} will compute the expected sojourn time corresponding to 
#' an impairment of Y lower or equal to 3.
#' @param condState an optional list specifying the initial state at start time 
#' (argument \code{startTime}) from which to compute the residual sojourn time.
#' The state can be defined either as a single value or as an interval.
#' For example \code{condState=list(Y = 2)} means that the state at start time
#' is equal to 2, whereas \code{condState=list(Y = c(1, 2))} means that the
#' state at start time is between 1 and 2 (both included).
#' @param newdata a one line data frame specifying the covariate profile for
#' which the sojourn time is computed.
#' @param var.time a character string specifying the name of the time variable 
#' in the longitudinal submodel. Note that this time covariate should not be 
#' included in newdata.
#' @param startTime a numeric value specifying the time from which the residual 
#' sojourn time is computed (the lower bound of the integral over time). 
#' Default to 0 for the expected sojourn time over lifetime.
#' @param nMC an integer giving the number of Monte Carlo simulations used to 
#' compute the integral over the random effects.
#' @param upperTime a numeric specifying the upper bound of the integral over time. 
#' Default to 150 (150 years as an approximation of infinity). 
#' @param subdivisions passed to the \code{integrate} function.
#' @param rel.tol passed to the \code{integrate} function.
#' @param draws logical indicating if 95\% confidence interval should be computed.
#'  Default to FALSE.
#' @param ndraws integer giving the number of draws to be used to compute the 
#' 95\% confidence interval. Default to 2000.
#' @param returndraws logical indicating if the \code{ndraws} results should be 
#' returned. Default to FALSE.
#' @param cl either a cluster created with \code{makeCluster} or an integer 
#' specifying the number of cores that should be used for computation. 
#' Only used with draws = TRUE.
#' @param seed integer only used with \code{draws = TRUE} to set the random seed.
#'
#' @return if \code{draws = FALSE}, returns a single value. 
#' If \code{draws = TRUE} and \code{returndraws = FALSE}, returns the median, 
#' the 2.5\% and 97.5\% quantiles, the mean, the standard deviation and the 
#' number of removed draws (eventually due to computational issues). 
#' If \code{draws = TRUE} and \code{returndraws = TRUE}, returns the 
#' \code{ndraws} values.
#' 
#' @examples
#' library(lcmm)
#' paq <- paquid[which(paquid$age_init < paquid$agedem), ]
#' paq$age65 <- (paq$age - 65) / 10
#' paq$ageinit65 <- (paq$age_init - 65) / 10
#' paq$agedem65 <- (paq$agedem - 65) / 10
#' 
#' #### Estimation of the joint model with one ordinal longitudinal item
#' \dontrun{
#' M2 <- jointLPM(fixed = HIER ~ age65 * male,
#'                 random = ~ age65,
#'                 subject = "ID", 
#'                 link = "thresholds",
#'                 survival = Surv(ageinit65, agedem65, dem) ~ male,
#'                 sharedtype = 'RE',
#'                 var.time = "age65",
#'                 data = paq, 
#'                 methInteg = "QMC", 
#'                 nMC = 1000,
#'                 B = c(0.6, 2.399, -0.409, 2.076, 6.338, 0.994, -0.223, -0.005,
#'                      -0.299, 0.174, 0.523, 1.044, 1.064, 0.506))
#'
#' #### Computation of the expected lifetime sojourn time with HIER impairment
#' up to 2 (HIER = 2)
#' #### (=int_0^150 P(HIER(t) <= 2, T > t) dt)
#' sojournTime(M2, list(HIER = 2), newdata = data.frame(male = 0), 
#' var.time = "age65")
#'
#' #### Computation of the expected residual time with maximum HIER impairment
#'  of 2 (HIER = 2), given the impairment was 1 at time 0.5
#' #### (=int_0.5^150 P(HIER(t) <= 2, T > t | HIER(0.5) <= 1, T > 0.5) dt)
#' sojournTime(M2, list(HIER = 2), condState = list(HIER = 1), startTime = 0.5,
#' newdata = data.frame(male = 0), var.time = "age65")
#' }
#' 
#' @author Viviane Philipps and Cecile Proust-Lima
#' 
#' @export

sojournTime <- function(x, maxState, condState=NULL, newdata, var.time, 
                        startTime=0, nMC=1000, upperTime=150, subdivisions=100L, 
                        rel.tol=.Machine$double.eps^0.25, draws=FALSE, 
                        ndraws=2000, returndraws=FALSE, cl=NULL, seed=NULL)
{
    if(missing(x)) stop("the model (argument x) is missing")
    if(!inherits(x,"jointLPM")) stop("use only with jointLPM model")
    if(missing(maxState)) stop("argument maxState is missing")
    if(missing(var.time)) stop("argument var.time is missing")
    if(!(var.time %in% x$Names$Xvar)) stop("var.time does not appear in the model")
    if(missing(newdata) & length(setdiff(x$Names$Xvar,var.time))) stop("argument newdata is missing")
    if(startTime < min(x$hazardnodes[1, ])) warning("startTime is outside the hazard range")
    if(any(x$typrisq != 2) & (upperTime > max(x$hazardnodes, na.rm = TRUE))) warning("upperTime is outside the hazard range")
    

    if((x$conv != 1) & (draws != FALSE))
    {
        draws <- FALSE
        warning("No confidence interval will be computed since the model did 
                not converge correctly.")
    }

    if(length(setdiff(x$Names$Xvar,var.time)))
    {
        if(!all(setdiff(x$Names$Xvar,var.time) %in% 
                colnames(newdata))) stop(paste("newdata should include variables",
                                               paste(setdiff(x$Names$Xvar, var.time),
                                                     collapse=" ")))
        if(any(x$idtdv==1))
        {
            if(is.na(newdata[,x$Names$TimeDepVar.name])) newdata[,x$Names$TimeDepVar.name] <- Inf
        }

        if(nrow(newdata) > 1) stop("newdata should only contain one line")
        
        newdata1 <- na.omit(newdata[1,setdiff(x$Names$Xvar,c(var.time)),drop=FALSE])
    }
    else
    {
        newdata1 <- data.frame(fakeCovariateX = 0)
    }


    ny <- x$N[12]
    idlink <- x$linktype

    nmes <- rep(0,ny)
    nmescond <- rep(0,ny)
    Yevent <- rep(NA,ny)
    indiceYevent <- rep(NA,ny)
    Ycond <- rep(NA,ny)
    indiceYcond <- rep(NA,ny) 
    indiceYinf <- rep(NA, 2 * ny) 
    nvalSPLORD <- rep(1,ny)
    uniqueY <- NULL
    indic_s1t2 <- rep(NA, 2*ny)
    for(k in 1:ny)
    {        
        if(x$Names$Ynames[k] %in% names(maxState))
        {
            if(idlink[k] != 3) stop("maxState should only include ordinal outcomes")
            Yevent[k] <- maxState[[x$Names$Ynames[k]]]
            nmes[k] <- nmes[k] + 1
            modality <- which(x$mod[[k]] == Yevent[k])
            if(!length(modality)) stop(paste("maxState for outcome", x$Names$Ynames[k], "is not correct"))
            indiceYevent[k] <- modality
            indic_s1t2[2*(k-1)+2] <- 2
            indiceYinf[2*(k-1)+2] <- 0
        }

        if(idlink[k]==3) nvalSPLORD[k] <- length(x$mod[[k]])
        if(idlink[k]==3) uniqueY <- c(uniqueY,x$mod[[k]])
        if(idlink[k]==2) uniqueY <- c(uniqueY,x$linknodes[1,k])
        
        if(x$Names$Ynames[k] %in% names(condState))
        {
            if(idlink[k] != 3) stop("condState should only include ordinal outcomes")
            Ycondvect <- condState[[x$Names$Ynames[k]]]

            modality <- which(x$mod[[k]] == Ycondvect[1]) - 1
            if(!length(modality)) stop(paste("condState for outcome", x$Names$Ynames[k], "is not correct"))
            indiceYinf[2*(k-1)+1] <- modality

            Ycond[k] <- Ycondvect[length(Ycondvect)]
            nmes[k] <- nmes[k] + 1 
            nmescond[k] <- nmescond[k] + 1
            modality <- which(x$mod[[k]] == Ycond[k])
            if(!length(modality)) stop(paste("condState for outcome", x$Names$Ynames[k], "is not correct"))
            indiceYcond[k] <- modality
            indic_s1t2[2*(k-1)+1] <- 1
            if(length(Ycondvect) > 2) stop(paste("condState for outcome", x$Names$Ynames[k], "should be of length 1 or 2"))
        }
    }
    
    Y <- na.omit(as.vector(rbind(Ycond,Yevent)))
    indiceY <- na.omit(as.vector(rbind(indiceYcond,indiceYevent)))
    indic_s1t2 <- na.omit(indic_s1t2)
    indiceYinf <- na.omit(indiceYinf)
    
    fixed <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$fixed[2])
    random <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$random[2])
    contr <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$contr[2])
    surv <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$form.commun[2])
    survcause <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$form.cause[2])
    cor <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",x$form$form.cor[2])

    fixed <- formula(paste("~",fixed))
    random <- formula(paste("~",random))
    contr <- formula(paste("~",contr))
    surv <- formula(paste("~",surv))
    survcause <- formula(paste("~",survcause))
    cor <- formula(paste("~",cor))

    Tentr <- 0
    Devt <- 0
    idea <- x$idea
    idg <- x$idg
    idcor <- x$idcor
    idcontr <- x$idcontr
    idsurv <- x$idsurv
    idtdv <- x$idtdv
    typrisq <- x$typrisq
    nz <- x$nz
    zi <- x$hazardnodes
    nbevt <- length(x$nevent)
    idtrunc <- 0
    logspecif <- x$logspecif
    nv <- length(x$Names$Xnames)
    nobs <- length(Y)
    nea <- sum(idea)
    nvc <- x$N[7]
    idiag <- x$idiag
    ncor <- x$N[9]
    nalea <- x$N[11]
    epsY <- x$epsY
    nbzitr <- rep(2,ny)
    nbzitr[which(idlink==2)] <- x$nbnodes
    zitr <- x$linknodes
    fix <- rep(0,length(x$best))
    posfix <- eval(x$call$posfix)
    if(length(posfix)) fix[posfix] <- 1
    methInteg <- 3
    dimMC <- nea + nalea + sum(nmes)*as.numeric(ncor>0)
    if((dimMC > 0) & (nMC > 0))
    {
        sequnif <- spacefillr::generate_sobol_owen_set(nMC, dimMC)
        seqMC <- apply(sequnif, 2, qnorm)
    }
    npmtot <- length(x$best)
    btot <- x$best
    b <- btot[which(fix == 0)]
    bfix <- btot[which(fix == 1)]
    npm <- length(b)
    nfix <- length(bfix)

    if(nvc>0)
    {
        ## remplacer varcov par cholesky dans btot

        if(idiag==1)
        {
            btot[sum(x$N[1:6]) + 1:nvc] <- sqrt(btot[sum(x$N[1:6]) + 1:nvc])
        }
        else
        {
            btot[sum(x$N[1:6]) + 1:nvc] <- x$cholesky[-1]
        }
    }

    sharedtype <- x$sharedtype
    
    
    fctprob <- function(t, s, x, newdata, Y, fixed, random, contr, surv, survcause, cor,
                        indic_s1t2, Tentr,Devt,
                        idea,idg,idcor,idcontr,idsurv,idtdv,
                        typrisq,nz,zi,nbevt,idtrunc,logspecif,
                        ny,nv,nobs,nea,nmes,idiag,ncor,nalea,
                        epsY,idlink,nbzitr,zitr,uniqueY,indiceY,indiceYinf,
                        nvalSPLORD,fix,methInteg,nMC,dimMC,seqMC,npm,b,nfix,bfix,computeSurv)
    {
        ## The function computes P(T > t, Y(t) = 0, Y(s) = 0) ##
        
        ## to compute P(T > t):
        Tevt <- t
        ind_survint <- 0
        if(any(idtdv==1)) ind_survint <- as.numeric(newdata[1,x$Names$TimeDepVar.name] < t)

        ## for sharedtype = "CL"
        if(any(sharedtype > 1))
        {
            deriv <- FALSE
            if(any(sharedtype %in% c(3, 4))) deriv <- TRUE
            X_GK <- matrixGK(newdata, fixed, random, "t", idtrunc, 0, t, deriv, zi[1,1])
            Xcl_GK <- X_GK$Xpred_cl
            Xcl_Ti <- 0 # don't need to compute instantaneous hazard
            nXcl <- c(ncol(Xcl_GK), ncol(Xcl_GK))
            Xcs_Ti <- 0
            Xcs_GK <- 0
            if(any(sharedtype %in% c(3, 4))) Xcs_GK <- X_GK$Xpred_cs
        }
        else # sharedtype ="RE"
        {
            nXcl <- c(0, 0)
            Xcl_Ti <- 0
            Xcl_GK <- 0
            Xcs_Ti <- 0
            Xcs_GK <- 0
        }
       
        ## to compute P(Y(t)=0, Y(s) = 0):
        time <- ifelse(indic_s1t2==1, s, t)
        ##cat("time=", time, "\n")
        newdata <- data.frame(newdata, t=time, row.names=NULL)
                
        Xfixed <- model.matrix(fixed, data=newdata)
        Xrandom <- model.matrix(random, data=newdata)
        Xcontr <- model.matrix(contr,data=newdata)
        Xsurv <- model.matrix(surv,data=newdata)
        Xsurvcause <- model.matrix(survcause,data=newdata)
        Xcor <- model.matrix(cor,data=newdata)   
        
        X0 <- cbind(Xfixed, Xrandom, Xsurv, Xsurvcause, Xcor)

        Xnames <- x$Names$Xnames
        Xnames[1] <- "(Intercept)"
        Xnames <- gsub(paste("\\b",var.time,"\\b",sep=""),"t",Xnames)
        X0 <- X0[,Xnames,drop=FALSE]


        if(computeSurv == 0)
        {
            nbevt <- 0
            
            btot <- rep(NA, length(fix))
            btot[which(fix == 0)] <- b
            btot[which(fix == 1)] <- bfix

            bsansSurv <- btot[-c(1:sum(x$N[1:4]))]
            fixsansSurv <- fix[-c(1:sum(x$N[1:4]))]
            
            b <- bsansSurv[which(fixsansSurv == 0)]
            bfix <- bsansSurv[which(fixsansSurv == 1)]
        }
        
        nonlin <- x$nonlin 
        centerpoly <- x$centerpoly

        expectancy <- 1
        proba <- 0

        ## compute log P(T > t, Y(t) = 0, Y(s) = 0):
        res <- .Fortran(C_loglik,
                        as.double(Y),
                        as.double(X0),
                        as.double(Tentr),
                        as.double(Tevt),
                        as.integer(Devt),
                        as.integer(ind_survint),
                        as.integer(idea),
                        as.integer(idg),
                        as.integer(idcor),
                        as.integer(idcontr),
                        as.integer(idsurv),
                        as.integer(idtdv),
                        as.integer(typrisq),
                        as.integer(nz),
                        as.double(zi),
                        as.integer(nbevt),
                        as.integer(idtrunc),
                        as.integer(logspecif),
                        as.integer(ny),
                        as.integer(1),
                        as.integer(nv),
                        as.integer(nobs),
                        as.integer(nmes),
                        as.integer(idiag),
                        as.integer(ncor),
                        as.integer(nalea),
                        as.integer(npm),
                        as.double(b),
                        as.integer(nfix),
                        as.double(bfix),
                        as.double(epsY),
                        as.integer(idlink),
                        as.integer(nbzitr),
                        as.double(zitr),
                        as.double(uniqueY),
                        as.integer(indiceY),
                        as.integer(indiceYinf),
                        as.integer(nvalSPLORD),
                        as.integer(fix),
                        as.integer(methInteg),
                        as.integer(nMC),
                        as.integer(dimMC),
                        as.double(seqMC),
                        as.integer(sharedtype),
                        as.integer(nXcl),
                        as.double(Xcl_Ti),
                        as.double(Xcl_GK),
                        as.double(Xcs_Ti),
                        as.double(Xcs_GK),
                        as.integer(nonlin),
                        as.double(centerpoly),
                        as.integer(expectancy),
                        res=as.double(proba))$res

        if(!is.na(res)){ if(res == -1E+9) res <- NA }

        return(exp(res))
    }
    fctprobVect <- Vectorize(fctprob,"t")
    
    ## doone : computes the result for a set of parameters
    doone <- function(bdraw)
    {
        bdrawest <- bdraw[which(fix == 0)]
        bdrawfix <- bdraw[which(fix == 1)]
       ## b <- btot + Chol %*% bdraw
        res1 <- list(value=NA)
        try(res1 <- integrate(f=fctprobVect, lower=startTime, upper=upperTime, s=startTime, x=x,
                          newdata=newdata1, Y=Y,
                          fixed=fixed, random=random, contr=contr, surv=surv,
                          survcause=survcause, cor=cor,
                          indic_s1t2=indic_s1t2, Tentr=Tentr,Devt=Devt,
                          idea=idea,idg=idg,idcor=idcor,idcontr=idcontr,idsurv=idsurv,
                          idtdv=idtdv,
                          typrisq=typrisq,nz=nz,zi=zi,nbevt=nbevt,idtrunc=idtrunc,
                          logspecif=logspecif,
                          ny=ny,nv=nv,nobs=nobs,nea=nea,nmes=nmes,idiag=idiag,ncor=ncor,
                          nalea=nalea,
                          epsY=epsY,idlink=idlink,nbzitr=nbzitr,zitr=zitr,
                          uniqueY=uniqueY,indiceY=indiceY,indiceYinf=indiceYinf,
                          nvalSPLORD=nvalSPLORD,fix=fix,methInteg=methInteg,nMC=nMC,
                          dimMC=dimMC,seqMC=seqMC,npm=npm,b=bdrawest,nfix=nfix,bfix=bdrawfix,
                          computeSurv= 1,
                          rel.tol=rel.tol, subdivisions=subdivisions))

        ##if(inherits(res1, "try-error")) print(bdraw)
        
        result <- res1$value

        ##browser()
        if((startTime > zi[1, 1]) | length(na.omit(Ycond)))
        {
            nobscond <- length(na.omit(Ycond))
            Ycond <- na.omit(Ycond)
            indiceYcond <- na.omit(indiceYcond)
            indiceYinfcond <- indiceYinf[which(indic_s1t2 == 1)]
            
            computeSurv <- 0
            if(startTime > zi[1, 1]) computeSurv <- 1
            
            ## compute P(Y(s) = 0, T > s):
            res2 <- fctprob(t=startTime, s=startTime, x=x, newdata=newdata1, Y=Ycond,
                            fixed=fixed, random=random, contr=contr, surv=surv,
                            survcause=survcause, cor=cor,
                            indic_s1t2=rep(2, nobscond), Tentr=Tentr,Devt=Devt,
                            idea=idea,idg=idg,idcor=idcor,idcontr=idcontr,idsurv=idsurv,
                            idtdv=idtdv,
                            typrisq=typrisq,nz=nz,zi=zi,nbevt=nbevt,idtrunc=idtrunc,
                            logspecif=logspecif,
                            ny=ny,nv=nv,nobs=nobscond,nea=nea,nmes=nmescond,idiag=idiag,
                            ncor=ncor,nalea=nalea,
                            epsY=epsY,idlink=idlink,nbzitr=nbzitr,zitr=zitr,uniqueY=uniqueY,
                            indiceY=indiceYcond,indiceYinf=indiceYinfcond,
                            nvalSPLORD=nvalSPLORD,fix=fix,methInteg=methInteg,nMC=nMC,
                            dimMC=dimMC,seqMC=seqMC,
                            npm=npm,b=bdrawest,nfix=nfix,bfix=bdrawfix, computeSurv=computeSurv)

            result <- result/res2            
        }
 
        return(result)
    }
##browser()
    
    if(!isTRUE(draws))
    {
        ## compute the result for parameters btot
        result <- doone(bdraw=btot)
        class(result) <- "sojournTime"
    }
    else
    {
        ndraws <- as.integer(ndraws)
        
        posfix <- eval(x$call$posfix)
        
        if(ndraws>0)
        {
            Mat <- matrix(0,ncol=npmtot,nrow=npmtot)
            Mat[upper.tri(Mat,diag=TRUE)]<- x$V
            if(length(posfix))
            {
                Mat2 <- Mat[-posfix,-posfix]
                Chol2 <- chol(Mat2)
                Chol <- matrix(0,npmtot,npmtot)
                Chol[setdiff(1:npmtot,posfix),setdiff(1:npmtot,posfix)] <- Chol2
                Chol <- t(Chol)
            }
            else
            {
                Chol <- chol(Mat)
                Chol <- t(Chol)
            }
        }

        if(!is.null(seed)) set.seed(seed)
        bdraw <- replicate(ndraws, btot + as.vector(Chol %*% rnorm(npmtot))) # ndraws colonnes

        ## compute the result for the ndraws sets of parameters
        if(!is.null(cl))
        {
            ncl <- NULL
            if(!inherits(cl,"cluster"))
            {
                if(!is.numeric(cl)) stop("argument cl should be either 
                                         a cluster or a numeric value 
                                         indicating the number of cores")
                
                ncl <- cl
                cl <- makeCluster(ncl)
            }
      
            ## set different seeds
            clusterSetRNGStream(cl)
            
            ## export other arguments
            clusterExport(cl, list("fctprobVect", "startTime", "upperTime", "x",
                          "newdata1", "Y",
                          "fixed", "random", "contr", "surv",
                          "survcause", "cor",
                          "indic_s1t2", "Tentr","Devt",
                          "idea", "idg", "idcor", "idcontr", "idsurv",
                          "idtdv",
                          "typrisq", "nz", "zi", "nbevt", "idtrunc",
                          "logspecif",
                          "ny", "nv", "nobs", "nea", "nmes", "idiag", "ncor",
                          "nalea",
                          "epsY", "idlink", "nbzitr", "zitr",
                          "uniqueY", "indiceY", "indiceYinf",
                          "nvalSPLORD", "fix", "methInteg", "nMC",
                          "dimMC", "seqMC", "npmtot", "btot",
                          "rel.tol", "subdivisions",
                          "fctprob", "Ycond", "indiceYcond"), envir = environment())
            
            ## get and export loaded packages
            pck <- .packages()
            dir0 <- find.package()
            dir <- sapply(1:length(pck),function(k){gsub(pck[k],"",dir0[k])})
            clusterExport(cl,list("pck","dir"),envir=environment())
            clusterEvalQ(cl,sapply(1:length(pck),function(k){require(pck[k],lib.loc=dir[k],character.only=TRUE)}))
            
            ## faire les ndraws repliques
            res <- parApply(cl,X=bdraw, MARGIN=2, FUN=doone)
            
            if(!is.null(ncl)) stopCluster(cl)            
        }
        else
        {
            res <- apply(bdraw, 2, doone)
        }
        
        if(returndraws)
        {
            class(res) <- c("sojournTime", "draws")
            if(!is.null(seed)) attr(res, "seed") <- seed
            return(res)
        }

        ## return the quantiles
        res_50 <- quantile(res, probs=0.5, na.rm=TRUE)
        res_2.5 <- quantile(res, probs=0.025, na.rm=TRUE)
        res_97.5 <- quantile(res, probs=0.975, na.rm=TRUE)

        res_mean <- mean(res, na.rm=TRUE)
        res_sd <- sd(res, na.rm=TRUE)
        nn <- length(which(is.na(res)))

        result <- c(res_50, res_2.5, res_97.5, res_mean, res_sd, nn)
        if(!is.null(seed)) attr(result, "seed") <- seed  
        class(result) <- c("sojournTime", "IC")
    }

    return(result)
}






##     proba <- 0

##     .Fortran(C_proba_irtsre,as.double(Y),as.double(X),as.double(Tentr),as.double(t),as.integer(Devt),as.integer(ind_survint),
##      as.integer(idea),as.integer(idg),as.integer(idcor),as.integer(idcontr),as.integer(idsurv),as.integer(idtdv),
##      as.integer(typrisq),as.integer(nz),as.double(zi),as.integer(nbevt),as.integer(idtrunc),as.integer(logspecif),
##      as.integer(ny),as.integer(nv),as.integer(nobs),as.integer(nea),as.integer(nmes),as.integer(idiag),as.integer(ncor),as.integer(nalea),
##      as.double(epsY),as.integer(idlink),as.integer(nbzitr),as.double(zitr),as.integer(uniqueY),as.integer(indiceY),
##      as.integer(nvalSPLORD),as.integer(fix),as.integer(methInteg),as.integer(nMC),as.integer(dimMC),as.double(seqMC),as.integer(npmtot),as.double(btot),res=as.double(proba))$res
## }

#' Fisher information function, used in Step 4 _ Selecting from the 4Smethod
#'
#' This function computes the contribution if each item during stages, based on 
#' the Fisher information. 
#' 
#' @param D an object of class \code{jointLPM} representing the estimated 
#' dimension model with the items
#' @param proj a vector
#'
#' @return a list of two matrices
#' 
#' ###@examples
#' 
#' @author Tiphaine Saulnier, Viviane Philipps and Cecile Proust-Lima
#' 
#' @references
#' preprint :  arXiv:2407.08278
#' 
#' @export

IRT4Sselecting <- function(D,proj)
{
  if(missing(D)) stop("the model (argument D) is missing")
  if(!inherits(D,"jointLPM")) stop("use only with jointLPM model")
  if(missing(proj)) stop("the vector (argument proj) is missing")
  
  ns <- length(proj) + 1  # nb of stages
  ni <- D$N[12]  # nb of items
  nl <- max(D$nbmod)  # nb of modalities
    
  thres <- D$thres
  discrim <- D$discrim
  
  
  ### raw information
  raw_info <- matrix(NA, ncol = ns, nrow = ni)
  rownames(raw_info) <- D$Names$Ynames
  colnames(raw_info) <- as.roman(1:ns)
  
  for(i in 1:ni){ # item
    
    for(s in 1:ns){ # stage
      
      # boundaries of stage s
      if(s == 1){ # first stage
        proj.s.inf <- -Inf 
        proj.s.sup <- proj[1]
      }
      if(s == ns){ # last stage
        proj.s.inf <- proj[ns-1] 
        proj.s.sup <- +Inf
      }
      if(s != 1 && s != ns){ # other stages
        proj.s.inf <- proj[s-1] 
        proj.s.sup <- proj[s]
      }
      
      E1 <- 0 # first element = sum_i(integral)
      E2 <- 0 # second element = sum_i(substraction)
      for(m in 0:(nl-1)){ # modality
        E1 <- E1 + integrate(f=.integrate_info,
                             lower=proj.s.inf, upper=proj.s.sup,
                             i, thres, discrim, m, nl)$value
        E2 <- E2 + .Pim_prime(lambda=proj.s.sup,i,thres,discrim,m,nl) - .Pim_prime(lambda=proj.s.inf,i,thres,discrim,m,nl)
      }  
      
      raw_info[i,s] <- E1 - E2
      
    }
    
  }
  
  ### % of information
  percent_info <- raw_info
  
  for(i in 1:ni)
    percent_info[i,] <- percent_info[i,] / colSums(raw_info)
  percent_info <- percent_info * 100
  
  
  
  res <- list(raw_info=raw_info,percent_info=percent_info)
  
  return(res)
  
}
  

---
title: "Joint Latent Process Models"
output: 
  rmarkdown::html_vignette:
    toc: true # table of content true
    toc_depth : 2  
vignette: >
  %\VignetteIndexEntry{Introduction}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

The R package JLPM implements in the jointLPM function the estimation of a joint shared random effects model.

&nbsp;

The longitudinal data ($y_1, y_2, \dots, y_K$) can be continuous or ordinal and are modelled using a mixed model.

&nbsp;


For the continuous Gaussian case:

\[
  \forall k \in 1,\dots,K \hspace{1cm}  y_k(t_{ijk}) = X_i(t_{ijk})\beta + Z_i(t_{ijk})u_i + \varepsilon_{ijk} 
\]

For the continuous non Gaussien case, a transformation $H_k$ is estimated for each outcome:

\[
  \forall k \in 1,\dots,K \hspace{1cm}  H_k(y_k(t_{ijk}), \eta_k) = X_i(t_{ijk})\beta + Z_i(t_{ijk})u_i + \varepsilon_{ijk} 
\]


For the ordinal case, mixed models are combined to the Item Response Theory:

\[
  \forall k \in 1,\dots,K \hspace{1cm} \mathbb{P}( y_k(t_{ijk}) = m) \Leftrightarrow \mathbb{P}(\eta_{m-1} <  X_i(t_{ijk})\beta + Z_i(t_{ijk})u_i + \varepsilon_{ijk} < \eta_m)
\]

where  $X_i(t_{ijk})$ and $Z_i(t_{ijk})$ are vectors of covariates measured at time $t_{ijk}$ for subject $i$, $\beta$ is the vector of fixed effects, $u_i \sim \mathcal{N}(0,B)$,  $\varepsilon_{ijk} \sim \mathcal{N}(0,\sigma_k^2)$, $\eta_k$ are the parameters of the link function $H_k$ or the thresholds associated to the outcome $y_k$.

Note that even with multiple longitudinal outcomes, a univariate mixed model is estimated.

&nbsp;

The time-to-event data are modelled in a proportional hazard model where different associations between the longitudinal outcome and the event can be included.


&nbsp;

With an association through the random effects of the longitudinal model:

\[
  \alpha_i(t) = \alpha_0(t) \exp(\tilde{X}_i \gamma + \delta u_i)
\]


With an association through the current level of the longitudinal model:

\[
  \alpha_i(t) = \alpha_0(t) \exp(\tilde{X}_i \gamma + \delta (X_i(t)\beta + Z_i(t)u_i))
\]


With an association through the current slope of the longitudinal model:

\[
  \alpha_i(t) = \alpha_0(t, \omega) \exp(\tilde{X}_i \gamma + \delta (
  \frac{d}{dt}X_i(t)\beta + \frac{d}{dt}Z_i(t)u_i))
\]


  with $\alpha_0(t, \omega)$ the baseline risk function at time $t$, parameterized with $\omega$,$\tilde{X}_i$ a vector of covariates,  $\gamma$ the fixed effects, $\delta$ the association parameter.

&nbsp;

The parameters $\beta, B, \sigma_k, \eta_k, \omega, \gamma, \delta$ are estimated using a Marquardt-Levenberg algorithm.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictRE.R
\name{predictRE}
\alias{predictRE}
\title{Random effects prediction}
\usage{
predictRE(x, data, control = NULL, variance = FALSE)
}
\arguments{
\item{x}{a \code{jointLPM} object}

\item{data}{a data.frame object containing the data from the predictions are
to be computed}

\item{control}{optional list of arguments to be passed to the
\code{marqLevAlg} optimization function.}

\item{variance}{logical indicating whether the variance of the estimated
predictions should be returned. Default ot FALSE.}
}
\value{
a matrix containing the predicted random effects (in columns) for
each subject (in rows). If \code{variance = TRUE}, returns a list containing
the previous matrix and a vector containing the upper part of each variance
matrix.
}
\description{
The random effects of a \code{jointLPM} model are predicted as the mode of 
the conditional distribution of the random effects given the subject's
longitudinal and survival observations.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sojournTime.R
\name{sojournTime}
\alias{sojournTime}
\title{Computation of sojourn time under specific condition}
\usage{
sojournTime(
  x,
  maxState,
  condState = NULL,
  newdata,
  var.time,
  startTime = 0,
  nMC = 1000,
  upperTime = 150,
  subdivisions = 100L,
  rel.tol = .Machine$double.eps^0.25,
  draws = FALSE,
  ndraws = 2000,
  returndraws = FALSE,
  cl = NULL,
  seed = NULL
)
}
\arguments{
\item{x}{an object of class \code{jointLPM} representing of joint shared 
random effects model with binary or ordinal longitudinal outcome(s)}

\item{maxState}{a list specifying the items and the corresponding levels 
defining the maximum state for the computation of the sojourn time. For instance 
\code{maxState=list(Y = 3)} will compute the expected sojourn time corresponding to 
an impairment of Y lower or equal to 3.}

\item{condState}{an optional list specifying the initial state at start time 
(argument \code{startTime}) from which to compute the residual sojourn time.
The state can be defined either as a single value or as an interval.
For example \code{condState=list(Y = 2)} means that the state at start time
is equal to 2, whereas \code{condState=list(Y = c(1, 2))} means that the
state at start time is between 1 and 2 (both included).}

\item{newdata}{a one line data frame specifying the covariate profile for
which the sojourn time is computed.}

\item{var.time}{a character string specifying the name of the time variable 
in the longitudinal submodel. Note that this time covariate should not be 
included in newdata.}

\item{startTime}{a numeric value specifying the time from which the residual 
sojourn time is computed (the lower bound of the integral over time). 
Default to 0 for the expected sojourn time over lifetime.}

\item{nMC}{an integer giving the number of Monte Carlo simulations used to 
compute the integral over the random effects.}

\item{upperTime}{a numeric specifying the upper bound of the integral over time. 
Default to 150 (150 years as an approximation of infinity).}

\item{subdivisions}{passed to the \code{integrate} function.}

\item{rel.tol}{passed to the \code{integrate} function.}

\item{draws}{logical indicating if 95\% confidence interval should be computed.
Default to FALSE.}

\item{ndraws}{integer giving the number of draws to be used to compute the 
95\% confidence interval. Default to 2000.}

\item{returndraws}{logical indicating if the \code{ndraws} results should be 
returned. Default to FALSE.}

\item{cl}{either a cluster created with \code{makeCluster} or an integer 
specifying the number of cores that should be used for computation. 
Only used with draws = TRUE.}

\item{seed}{integer only used with \code{draws = TRUE} to set the random seed.}
}
\value{
if \code{draws = FALSE}, returns a single value. 
If \code{draws = TRUE} and \code{returndraws = FALSE}, returns the median, 
the 2.5\% and 97.5\% quantiles, the mean, the standard deviation and the 
number of removed draws (eventually due to computational issues). 
If \code{draws = TRUE} and \code{returndraws = TRUE}, returns the 
\code{ndraws} values.
}
\description{
This function computes posterior computations from a joint shared 
random-effect model with ordinal longitudinal outcomes, aka a joint 
item response theory model. Specifically, the function computes the 
expected time spent before reaching a given level of impairment specified
by one or multiple items for a covariate profile.
}
\details{
1. Lifetime expected sojourn time: the expected time before reaching 
level k at item(s) Y is the integral over time t, from 0 to infinity, 
of P(Y(t) <= k, T > t), i.e., int_0^infty P(Y(t) <= k, T > t) dt.

2. Residual expected sojourn time from a time s: conditionally on being
 below m at item(s) Y at time s and being alive at time s, the 
 sojourn time below level k at item(s) Y is computed as: 
 int_s^infty P(Y(t) <= k, T > t | Y(s) <= m, T > s) dt = 
 int_s^infty P(Y(t) <= k, T > t, Y(s) <= m) dt / P(Y(s) <= m, T > s)
}
\examples{
library(lcmm)
paq <- paquid[which(paquid$age_init < paquid$agedem), ]
paq$age65 <- (paq$age - 65) / 10
paq$ageinit65 <- (paq$age_init - 65) / 10
paq$agedem65 <- (paq$agedem - 65) / 10

#### Estimation of the joint model with one ordinal longitudinal item
\dontrun{
M2 <- jointLPM(fixed = HIER ~ age65 * male,
                random = ~ age65,
                subject = "ID", 
                link = "thresholds",
                survival = Surv(ageinit65, agedem65, dem) ~ male,
                sharedtype = 'RE',
                var.time = "age65",
                data = paq, 
                methInteg = "QMC", 
                nMC = 1000,
                B = c(0.6, 2.399, -0.409, 2.076, 6.338, 0.994, -0.223, -0.005,
                     -0.299, 0.174, 0.523, 1.044, 1.064, 0.506))

#### Computation of the expected lifetime sojourn time with HIER impairment
up to 2 (HIER = 2)
#### (=int_0^150 P(HIER(t) <= 2, T > t) dt)
sojournTime(M2, list(HIER = 2), newdata = data.frame(male = 0), 
var.time = "age65")

#### Computation of the expected residual time with maximum HIER impairment
 of 2 (HIER = 2), given the impairment was 1 at time 0.5
#### (=int_0.5^150 P(HIER(t) <= 2, T > t | HIER(0.5) <= 1, T > 0.5) dt)
sojournTime(M2, list(HIER = 2), condState = list(HIER = 1), startTime = 0.5,
newdata = data.frame(male = 0), var.time = "age65")
}

}
\author{
Viviane Philipps and Cecile Proust-Lima
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IRT4Sselecting.R
\name{IRT4Sselecting}
\alias{IRT4Sselecting}
\title{Fisher information function, used in Step 4 _ Selecting from the 4Smethod}
\usage{
IRT4Sselecting(D, proj)
}
\arguments{
\item{D}{an object of class \code{jointLPM} representing the estimated 
dimension model with the items}

\item{proj}{a vector}
}
\value{
a list of two matrices

###@examples
}
\description{
This function computes the contribution if each item during stages, based on 
the Fisher information.
}
\references{
preprint :  arXiv:2407.08278
}
\author{
Tiphaine Saulnier, Viviane Philipps and Cecile Proust-Lima
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictRE.R
\name{REconddensity}
\alias{REconddensity}
\title{REconddendity}
\usage{
REconddensity(
  ui0,
  Y0,
  X0,
  Tentr0,
  Tevt0,
  Devt0,
  ind_survint0,
  idea0,
  idg0,
  idcor0,
  idcontr0,
  idsurv0,
  idtdv0,
  typrisq0,
  nz0,
  zi0,
  nbevt0,
  idtrunc0,
  logspecif0,
  ny0,
  nv0,
  nobs0,
  nmes0,
  idiag0,
  ncor0,
  nalea0,
  npm0,
  b0,
  epsY0,
  idlink0,
  nbzitr0,
  zitr0,
  uniqueY0,
  indiceY0,
  nvalSPLORD0,
  idst0,
  nXcl0,
  Xcl_Ti0,
  Xcl_GK0,
  Xcs_Ti0,
  Xcs_GK0,
  nonlin0,
  centerpoly0
)
}
\arguments{
\item{ui0}{ui0}

\item{Y0}{Y0}

\item{X0}{X0}

\item{Tentr0}{Tentr0}

\item{Tevt0}{Tevt0}

\item{Devt0}{Devt0}

\item{ind_survint0}{ind_survint0}

\item{idea0}{idea0}

\item{idg0}{idg0}

\item{idcor0}{idcor0}

\item{idcontr0}{idcontr0}

\item{idsurv0}{idsurv0}

\item{idtdv0}{idtdv0}

\item{typrisq0}{typrisq0}

\item{nz0}{nz0}

\item{zi0}{zi0}

\item{nbevt0}{nbevt0}

\item{idtrunc0}{idtrunc0}

\item{logspecif0}{logspecif0}

\item{ny0}{ny0}

\item{nv0}{nv0}

\item{nobs0}{nobs0}

\item{nmes0}{nmes0}

\item{idiag0}{idiag0}

\item{ncor0}{ncor0}

\item{nalea0}{nalea0}

\item{npm0}{npm0}

\item{b0}{b0}

\item{epsY0}{epsY0}

\item{idlink0}{idlink0}

\item{nbzitr0}{nbzitr0}

\item{zitr0}{zitr0}

\item{uniqueY0}{uniqueY0}

\item{indiceY0}{indiceY0}

\item{nvalSPLORD0}{nvalSPLORD0}

\item{idst0}{idst0}

\item{nXcl0}{nXcl0}

\item{Xcl_Ti0}{Xcl_Ti0}

\item{Xcl_GK0}{Xcl_GK0}

\item{Xcs_Ti0}{Xcs_Ti0}

\item{Xcs_GK0}{Xcs_GK0}

\item{nonlin0}{nonlin0}

\item{centerpoly0}{centerpoly0}
}
\value{
the log-density
}
\description{
Wrapper to the Fortran subroutine computing the conditional density of the random effects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert.R
\name{convert}
\alias{convert}
\title{Conversion}
\usage{
convert(object, to)
}
\arguments{
\item{object}{an object of class \code{jointLPM}}

\item{to}{character. Either "multlcmm" or "Jointlcmm", indicating to which type the object should be converted.}
}
\value{
an object of class \code{multlcmm} or \code{Jointlcmm}.
}
\description{
This function converts a jointLPM object to an object of type \code{multlcmm} or
\code{Jointlcmm} from lcmm package.
The conversion to \code{multlcmm} unable the use of the dedicated postfit and
predictions functions implemented in the lcmm package (plot, predictL, predictY,
predictlink, predictYcond, fitY, VarExpl).
The conversion to \code{Jointlcmm} permits the use of functions cuminc and
plot (with options "baselinerisk" or "survival").
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/removeNA.R
\name{removeNA}
\alias{removeNA}
\title{removeNA}
\usage{
removeNA(form, data)
}
\arguments{
\item{form}{a list of formula}

\item{data}{a data frame containing all variables mentionned in form}
}
\value{
a list containing
 \item{newdata}{a data frame created from data, containing only the variables used in the formula, and without any missing values}
 \item{Y}{the names of the response variables}
 \item{YlinesNA}{the lines removed from each response variables}
 \item{nmes}{the number of measures for each response variable}
 \item{X}{the names of the covariates}
 \item{XlinesNA}{the lines removed from data because of missing values in the covariates}
}
\description{
Remove missing values
}
\details{
Caution : if several response variables are included in 'form', then the resulting data frame will contain one column (named outcome) grouping all these response variables. The irst lines will refer to the first response variables, a second block to the second, etc. All covariates will be repeated for each response variable.
}
\examples{
 d <- data.frame(y1=c(3,6,4,2), y2=c(6.5,NA,9.5,4.5), x=c(0,0,1,0))
 removeNA(form=list(y1+y2~x), data=d)
## 1      3.0 0   | first block referring to y1
## 2      6.0 0   |
## 3      4.0 1   |
## 4      2.0 0   |
## 11     6.5 0     | second block referring to y2
## 31     9.5 1     |
## 41     4.5 0     |

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JLPM-package.R
\name{JLPM-package}
\alias{JLPM-package}
\title{Estimation of joint latent process models}
\description{
Functions for the estimation of joint latent process models (JLPM).
Continuous and ordinal outcomes are handled for the longitudinal part,
whereas the survival part considers multiple competing events.
The likelihood is computed using Monte-Carlo integration. Estimation is achieved
by maximizing the log-likelihood using a robust iterative algorithm.
}
\details{
Please report to the JLPM-team any question or suggestion regarding the package
via github only (https://github.com/VivianePhilipps/JLPM/issues).
}
\references{
Saulnier, Philipps, Meissner, Rascol, Pavy-Le-Traon, Foubert-Samier, Proust-Lima (2022).
Joint models for the longitudinal analysis of measurement scales in the presence 
of informative dropout, Methods; 203:142-51. 

Philipps, Hejblum, Prague, Commenges, Proust-Lima (2021).
Robust and efficient optimization using a Marquardt-Levenberg algorithm with 
R package marqLevAlg, The R Journal 13:2.
}
\author{
Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/predictRE.R
\name{createFargs}
\alias{createFargs}
\title{Likelihood arguments}
\usage{
createFargs(x, data)
}
\arguments{
\item{x}{a \code{jointLPM} object}

\item{data}{a data.frame containing the estimation dataset}
}
\value{
a named list
}
\description{
Create the arguments to be passed to the \code{loglik} function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WaldMult.R
\name{WaldMult}
\alias{WaldMult}
\title{WaldMult 
Performs multivariate Wald tests.}
\usage{
WaldMult(
  Mod,
  coef,
  vcov,
  pos = NULL,
  contrasts = NULL,
  A = NULL,
  name = NULL,
  value = 0
)
}
\arguments{
\item{Mod}{an object for which the \code{coef} and \code{vcov} methods are diefined.}

\item{coef}{if \code{Mod} is missing, the vector of estimated parameters}

\item{vcov}{if \code{Mod} is missing, the matrix of variance of coef}

\item{pos}{vector containing the indices of the parameters involved in
the test. Required if A is not specified.}

\item{contrasts}{optional vector containing the the c1, ..., cm value in
case 2 (see details). Should be of same lengtht as pos.}

\item{A}{matrix A of case 3 (see details). A should have as many columns as
parameters in coef.}

\item{name}{characters string giving the name of the test printed in the
output (the row names of the output). By default, the name's test is the
null hypothesis.}

\item{value}{the value against which to test. By default, value=0.}
}
\value{
If contrasts is NULL, the function returns a matrix with 1 row and
2 columns containing the value of the Wald test's statistic and
the associated p-value.

If contrasts is not NULL, the function returns a matrix with 1 row
and 4 columns containing the value of the coefficient (dot product
of pos and contrasts), his standard deviation, the value of the
Wald test's statistic and the associated p-value.

If A is specified, the function returns a matrix with m rows and
2 columns containing the value of the Wald test's statistic and
the associated p-value.
}
\description{
Given a vector of estimated parameters (coef = (theta_1, ... theta_n)') and
their associated variance matrix (vcov = Var(coef)), the WaldMult function
performs either the test of the null hypothesis :
}
\details{
1. H0 : theta_m = (theta_j1, ..., theta_jm)' = 0
   ie, tests if a subset of m parameters are simultaneously equal to zero.
   The Wald statistic is w = theta_m' * Var(theta_m)^(-1) * theta_m
   and the associated p-value is obtained by p = P(chi2_(dof=m) > w)

2. H0 : theta = c1*theta_j1 + ... + cm*theta_jm = 0
   ie, tests if a combination of m parameters is equal to zero.
   The Wald statistic is w = theta^2 / Var(theta)
                           = (c1*theta_j1 + ... + cm*theta_jm)^2 / Var(c1*theta_j1 + ... + cm*theta_jm)
   and the associated p-value is obtained by p = P(chi2_(dof=1) > w)

3. H0 : Theta = A*coef = 0 with A a matrix with m lines and n columns.
   ie, tests if multiple combinations of parameters are simultaneously equal to zero.
   The Wald statistic is w = Theta * Var(Theta)^(-1) * Theta'
                           = (A*coef)' * (A*vcov*A')^(-1) * (A*coef)
   and the associated p-value is obtained by p = P(chi2_(dof=m) > w)

The function can be applied to any object having a \code{coef}
}
\examples{
require(lcmm)
m <- hlme(Y ~ Time + X2, random = ~ 1, subject = "ID", data = data_hlme)
summary(m)

mm <- hlme(Y ~ -1 + Time + factor(X2), random = ~ 1, subject = "ID", data = data_hlme)
summary(mm)

## Retrieve from model mm the difference between X2 levels as in model m :
WaldMult(Mod = mm, pos = c(2, 3), contrasts = c(-1, 1))
## or
WaldMult(coef = coef(mm), vcov = vcov(mm), A = matrix(c(0, -1, 1, 0, 0), 1, 5))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffSojournTime.R
\name{diffSojournTime}
\alias{diffSojournTime}
\title{Difference of sojourn times}
\usage{
diffSojournTime(x1, x2)
}
\arguments{
\item{x1}{a sojournTime object, called with \code{draws = TRUE} and
\code{returndraws = TRUE}, computed for a specified seed.}

\item{x2}{a sojournTime object, called with \code{draws = TRUE} and
\code{returndraws = TRUE}, computed with the same seed as \code{x1}.}
}
\value{
returns the median, the 2.5\% and 97.5\% quantiles, the mean,
the standard deviation of the difference between the two sojourn times
and the number of removed draws (eventually due to computational issues)
}
\description{
This function computes the difference between two sojourn times.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.jointLPM.R
\name{print.jointLPM}
\alias{print.jointLPM}
\title{Brief summary of a joint latent process model}
\usage{
\method{print}{jointLPM}(x, ...)
}
\arguments{
\item{x}{an object inheriting from class \code{jointLPM} for a joint latent process model.}

\item{...}{further arguments to be passed to or from other methods. They are ignored in this function.}
}
\value{
the function is used for its side effects, no value is returned.
}
\description{
This function provides a brief summary of model estimated with the \code{jointLPM} function.
}
\author{
Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ForInternalUse.R
\name{ForInternalUse}
\alias{ForInternalUse}
\alias{.expect}
\alias{.Pim}
\alias{.Pim_prime}
\alias{.integrate_info}
\title{For internal use only ...}
\description{
For internal use only ...
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createX0.R
\name{createX0}
\alias{createX0}
\title{createX0}
\usage{
createX0(form, data)
}
\arguments{
\item{form}{a list of formula}

\item{data}{a data frame containing all variables mentionned in form}
}
\value{
a list containing
  \item{X0}{the pooled (with cbind) model matrix. The number of lines is the same as in 'data'}
 \item{idform}{a list of the same length as 'form' specifying which term of X0 appear in each formula}
}
\description{
Create the matrix containing all covariates appearing in the formula
}
\details{
Caution 1 : 'data' should not contain any missing values. There is no check for that within the function.
Caution 2 : if interaction terms are present, the names are sorted. For example x:age becomes age:x in X0.
Caution 3 : intercept is not systematically in the first column. It will only if form[[1]] contains an intercept.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.jointLPM.R
\name{summary.jointLPM}
\alias{summary.jointLPM}
\title{Summary of a joint latent process model}
\usage{
\method{summary}{jointLPM}(object, ...)
}
\arguments{
\item{object}{an object inheriting from class \code{jointLPM} for a joint latent process model.}

\item{...}{further arguments to be passed to or from other methods. They are ignored in this function.}
}
\value{
The function is mainly used for its side effects. It returns invisibily a list of two matrices
containing the estimates, their standard errors, Wald statistics and associated p-values for the survival
submodel (first element of the list) and for the mixed model's fixed effects (second element).
}
\description{
This function provides a summary of model estimated with the \code{jointLPM} function.
}
\author{
Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standardMethods.R
\name{StandardMethods}
\alias{StandardMethods}
\alias{coef.jointLPM}
\alias{vcov.jointLPM}
\title{Standard methods for estimated models}
\usage{
\method{coef}{jointLPM}(object, ...)
}
\arguments{
\item{object}{an object of class \code{jointLPM}}

\item{...}{other arguments. There are ignored in these functions.}
}
\value{
For \code{coef}, the vector of the estimated parameters.

For \code{vcov}, the variance-covariance matrix of the estimated parameters.
}
\description{
coef and vcov  methods for estimated jointLPM models.
}
\author{
Viviane Philipps
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/jointLPM.R
\name{jointLPM}
\alias{jointLPM}
\title{Estimation of latent process joint models for multivariate longitudinal 
outcomes and time-to-event data.}
\usage{
jointLPM(
  fixed,
  random,
  subject,
  idiag = FALSE,
  cor = NULL,
  link = "linear",
  intnodes = NULL,
  epsY = 0.5,
  randomY = FALSE,
  var.time,
  survival = NULL,
  hazard = "Weibull",
  hazardrange = NULL,
  hazardnodes = NULL,
  TimeDepVar = NULL,
  logscale = FALSE,
  startWeibull = 0,
  sharedtype = "RE",
  centerpoly = 0,
  methInteg = "QMC",
  nMC = 1000,
  data,
  subset = NULL,
  na.action = 1,
  B,
  posfix = NULL,
  maxiter = 100,
  convB = 1e-04,
  convL = 1e-04,
  convG = 1e-04,
  partialH = NULL,
  nsim = 100,
  range = NULL,
  verbose = TRUE,
  returndata = FALSE,
  nproc = 1,
  clustertype = NULL
)
}
\arguments{
\item{fixed}{a two-sided linear formula object for specifying the
fixed-effects in the linear mixed model at the latent process level. The
response outcomes are separated by \code{+} on the left of \code{~} and the
covariates are separated by \code{+} on the right of the \code{~}. For
identifiability purposes, the intercept specified by default should not be
removed by a \code{-1}. Variables on which a contrast above the different
outcomes should also be estimated are included with \code{contrast()}.}

\item{random}{a one-sided formula for the random-effects in the
latent process mixed model. Covariates with a random-effect are separated
by \code{+}. An intercept should always be included for identifiability.}

\item{subject}{name of the covariate representing the grouping structure.}

\item{idiag}{optional logical for the variance-covariance structure of the
random-effects. If \code{FALSE}, a non structured matrix of
variance-covariance is considered (by default). If \code{TRUE} a diagonal
matrix of variance-covariance is considered.}

\item{cor}{optional indicator for inclusion of an autocorrelated Gaussian
process in the linear mixed model at the latent process level. Option
\code{BM(time)} indicates a brownian motion with parameterized variance while
option \code{AR(time)} specifies an autoregressive process in continuous time
with parameterized variance and correlation intensity. In both cases, \code{time}
is the variable representing the measurement times. By default, no autocorrelated
Gaussian process is added.}

\item{link}{optional vector of families of parameterized link functions defining
the measurement models (one by outcome). Option "linear" (by default) specifies a linear
link function. Other possibilities include "beta" for estimating a link
function from the family of Beta cumulative distribution functions, "Splines" 
for approximating the link function by I-splines and "thresholds" for ordinal
outcomes modelled by cumulative probit models. For splines case, the number of 
nodes and the nodes location should be also specified. The number of nodes is 
first entered followed by \code{-}, then the location is specified with "equi", 
"quant" or "manual" for respectively equidistant
nodes, nodes at quantiles of the marker distribution or interior nodes
entered manually in argument \code{intnodes}. It is followed by \code{-splines}.
For example, "7-equi-splines" means
I-splines with 7 equidistant nodes, "6-quant-splines" means I-splines with 6
nodes located at the quantiles of the marker distribution and
"9-manual-splines" means I-splines with 9 nodes, the vector of 7 interior
nodes being entered in the argument \code{intnodes}.}

\item{intnodes}{optional vector of interior nodes. This argument is only
required for a I-splines link function with nodes entered manually.}

\item{epsY}{optional positive real used to rescale the marker in
(0,1) when the beta link function is used. By default, epsY=0.5.}

\item{randomY}{optional logical for including an outcome-specific random
intercept. If \code{FALSE} no outcome-specific random intercept is added
(default). If \code{TRUE} independent outcome-specific random intercepts
with parameterized variance are included.}

\item{var.time}{name of the variable representing the measurement times.}

\item{survival}{two-sided formula object. The left side of the formula corresponds 
to a Surv() object for right-censored (\code{Surv(EntryTime,Time,Indicator)})
and possibly left-truncated (\code{Surv(EntryTime,Time,Indicator)}).
Multiple causes of event can be considered 
in the Indicator (0 for censored, k for event of cause k). The right side of the 
formula specifies the covariates to include in the survival model.
Cause-specific covariate effect are specified with \code{cause()}.For example,  
Surv(Time,Indicator) ~ X1 + cause(X2) indicates a common effect of X1 and a cause-specific effect of X2.}

\item{hazard}{optional family of hazard function assumed for the survival model. 
By default, "Weibull" specifies a Weibull baseline risk function. Other possibilities 
are "piecewise" for a piecewise constant risk function or "splines" for a cubic M-splines 
baseline risk function. For these two latter families, the number of nodes and the 
location of the nodes should be specified as well, separated by -. The number of 
nodes is entered first followed by -, then the location is specified with "equi", 
"quant" or "manual" for respectively equidistant nodes, nodes at quantiles of the 
times of event distribution or interior nodes entered manually in argument hazardnodes. 
It is followed by - and finally "piecewise" or "splines" indicates the family of 
baseline risk function considered. Examples include "5-equi-splines" for M-splines 
with 5 equidistant nodes, "6-quant-piecewise" for piecewise constant risk over 5 
intervals and nodes defined at the quantiles of the times of events distribution 
and "9-manual-splines" for M-splines risk function with 9 nodes, the vector of 7 
interior nodes being entered in the argument hazardnodes. In the presence of competing 
events, a vector of hazards should be provided such as hazard=c("Weibull","5-quant-splines") 
with 2 causes of event, the first one modelled by a Weibull baseline cause-specific 
risk function and the second one by splines.}

\item{hazardrange}{optional vector indicating the range of the survival times 
(that is the minimum and maximum). By default, the range is defined according 
to the minimum and maximum observed values of the survival times. The option 
should be used only for piecewise constant and Splines hazard functions.}

\item{hazardnodes}{optional vector containing interior nodes if splines or piecewise 
is specified for the baseline hazard function in hazard.}

\item{TimeDepVar}{optional vector containing an intermediate time
corresponding to a change in the risk of event. This time-dependent
covariate can only take the form of a time variable with the assumption that
there is no effect on the risk before this time and a constant effect on the
risk of event after this time (example: initiation of a treatment to account
for).}

\item{logscale}{optional boolean indicating whether an exponential
(logscale=TRUE) or a square (logscale=FALSE -by default) transformation is
used to ensure positivity of parameters in the baseline risk functions. See
details section}

\item{startWeibull}{optional numeric with Weibull hazard functions only.
Indicates the shift in the Weibull distribution.}

\item{sharedtype}{indicator of shared random function type. \code{'RE'} indicates
an association through the random effects included in the linear mixed model.
\code{'CL'} defines a association through the predicted current level of the latent process.}

\item{centerpoly}{for polynomial associations only, value(s) to center degree 2 and 3 polynomial terms.}

\item{methInteg}{character indicating the type of integration to compute the
log-likelihood. 'MCO' for ordinary Monte Carlo, 'MCA' for antithetic Monte Carlo,
'QMC' for quasi Monte Carlo. Default to "QMC".}

\item{nMC}{integer, number of Monte Carlo simulations. Default to 1000.}

\item{data}{data frame containing all variables named in \code{fixed},
\code{random}, \code{cor}, \code{survival} and \code{subject}.}

\item{subset}{optional vector giving the subset of observations in
\code{data} to use. By default, all lines.}

\item{na.action}{Integer indicating how NAs are managed. The default is 1
for 'na.omit'. The alternative is 2 for 'na.fail'. Other options such as
'na.pass' or 'na.exclude' are not implemented in the current version.}

\item{B}{optional specification for the initial values for the parameters.
Initial values should be entered in the order detailed in \code{details} section.}

\item{posfix}{Optional vector giving the indices in vector B of the
parameters that should not be estimated. Default to NULL, all parameters are
estimated.}

\item{maxiter}{optional maximum number of iterations for the Marquardt
iterative algorithm. By default, maxiter=100.}

\item{convB}{optional threshold for the convergence criterion based on the
parameter stability. By default, convB=0.0001.}

\item{convL}{optional threshold for the convergence criterion based on the
log-likelihood stability. By default, convL=0.0001.}

\item{convG}{optional threshold for the convergence criterion based on the
derivatives. By default, convG=0.0001.}

\item{partialH}{optional vector giving the indices in vector B of parameters that
can be dropped from the Hessian matrix to define convergence criteria.}

\item{nsim}{number of points used to plot the estimated link functions. By
default, nsim=100.}

\item{range}{optional vector indicating the range of the outcomes (that is
the minimum and maximum). By default, the range is defined according to the
minimum and maximum observed values of the outcome. The option should be
used only for Beta and Splines transformations.}

\item{verbose}{logical indicating if information about computation should be
reported. Default to TRUE.}

\item{returndata}{logical indicating if data used for computation should be
returned. Default to FALSE, data are not returned.}

\item{nproc}{number of cores for parallel computation.}

\item{clustertype}{the type of cluster that should internally be created.
See \code{parallel::makeCluster} for possible values.}
}
\value{
An object of class "jointLPM" is returned containing some internal information
used in related functions. Users may investigate the following elements : 
\item{ns}{number of grouping units in the dataset} 
\item{loglik}{log-likelihood of the model} 
\item{best}{vector of parameter estimates in the same order as specified in 
\code{B} and detailed in section \code{details}}
\item{V}{vector containing the upper triangle matrix of variance-covariance
estimates of \code{best} with exception for variance-covariance parameters
of the random-effects for which \code{V} contains the variance-covariance
estimates of the Cholesky transformed parameters displayed in \code{cholesky}} 
\item{gconv}{vector of convergence criteria: 1. on the parameters, 2. on the 
likelihood, 3. on the derivatives} 
\item{conv}{status of convergence: =1 if the convergence criteria were satisfied, 
=2 if the maximum number of iterations was reached, =4 or 5 if a problem occured 
during optimisation} 
\item{call}{the matched call} 
\item{niter}{number of Marquardt iterations} 
\item{nevent}{number of occured event}
\item{pred}{table of individual predictions and residuals in the underlying
latent process scale; it includes marginal predictions (pred_m), marginal
residuals (resid_m), subject-specific predictions (pred_ss) and
subject-specific residuals (resid_ss) and finally the transformed
observations in the latent process scale (obs).}
\item{predRE}{table containing individual predictions of the random-effects : 
a column per random-effect, a line per subject.}
\item{predRE_Y}{table containing individual predictions of the outcome-specific
random intercept}
\item{predSurv}{table containing the predicted baseline risk function and
the predicted cumulative baseline risk function }
\item{cholesky}{vector containing the estimates of the Cholesky transformed
parameters of the variance-covariance matrix of the random-effects}
\item{estimlink}{table containing the simulated values of each outcome and
the corresponding estimated link function} 
\item{epsY}{definite positive reals used to rescale the markers in (0,1) when 
the beta link function is used. By default, epsY=0.5.} 
\item{AIC}{the Akaike's information criterion}
\item{BIC}{the Bayesian information criterion}
\item{CPUtime}{the runtime in seconds}
\item{data}{the original data set (if returndata is TRUE)}
}
\description{
This function fits extended joint models with shared random effects.
The longitudinal submodel handles multiple continuous longitudinal outcomes
(Gaussian or  non-Gaussian, curvilinear) as well as 
ordinal longitudinal outcomes in a mixed effect framework. The model assumes 
that all the outcomes 
measure the same underlying latent process defined as their common
factor, and each outcome is related to this latent common factor by a
outcome-specific measurement model whose nature depends on the type of the 
outcome (linear model for Gaussian outcome, curvilinear model for 
non-Gaussian outcome, probit model for binary outcome, cumulative probit 
model for ordinal outcome).
At the latent process level, the model assumes a linear mixed model.
The survival submodel handles right-censored (possibly left-truncated) 
time-to-event with competing causes
The association between the longitudinal and the survival data is captured 
by including 
the random effect from the mixed model or the predicted current level 
of the underlying process as linear predictors in the cause-specific 
proportional hazard survival model.
Parameters of the measurement models, of the latent process mixed model and of the 
survival model are simultaneously estimated using a maximum likelihood method,
through a Marquardt-Levenberg optimization algorithm.
}
\details{
A. THE MEASUREMENT MODELS

\code{jointLPM} function estimates one measurement model per outcome to link
each outcome Y_k(t) with the underlying latent common factor L(t) they measure.
To fix the latent process dimension, we chose to constrain the latent 
process dimension with the intercept of the mixed model fixed at 0 and the 
standard error of the first random effect fixed at 1. The nature of each 
measurement
model adapts to the type of the outcome it models. 

1. For continuous Gaussian outcomes, linear models are used and require 2 parameters for 
the transformation (Y(t) - b1)/b2

2. For continuous non-Gaussian outcomes, curvilinear models use 
parameterized link functions to link outcomes to the latent process. 
With the "beta" link function, 4 parameters are required for the
following transformation: [ h(Y(t)',b1,b2) - b3]/b4 where h is the Beta CDF
with canonical parameters c1 and c2 that can be derived from b1 and b2 as
c1=exp(b1)/[exp(b2)*(1+exp(b1))] and c2=1/[exp(b2)*(1+exp(b1))], and Y(t)'
is the rescaled outcome i.e. Y(t)'= [ Y(t) - min(Y(t)) + epsY ] / [
max(Y(t)) - min(Y(t)) +2*epsY ].
With the "splines" link function, n+2 parameters are required for the
following transformation b_1 + b_2*I_1(Y(t)) + ... + b_(n+2)*I_(n+1)(Y(t)),
where I_1,...,I_(n+1) is the basis of quadratic I-splines. To constrain the
parameters to be positive, except for b_1, the program estimates b_k^* (for
k=2,...,n+2) so that b_k=(b_k^*)^2.

3. For ordinal outcomes (and binary outcomes), cumulative probit models are used. For a
(n+1)-level outcome, the model consist of determining n thresholds t_k in the 
latent process scale which correspond to the outcome level changes. Then,
Y(t) = n' <=> t_n' < L(t) + e <= t_(n'+1) with e the standard error of the outcome.
To ensure that t_1 < t_2 < ... < t_n, the program estimates t'_1, t'_2, ..., t'_n
such that t_1=t'_1, t_2=t_1+(t'_2)^2, t_3=t_2+(t'_3)^2, ...


B. THE SURVIVAL MODEL

a. BASELINE RISK FUNCTIONS

For the baseline risk functions, the following parameterizations are considered. 

1. With the "Weibull" function: 2 parameters are necessary w_1 and w_2 so that 
the baseline risk function a_0(t) = w_1^2*w_2^2*(w_1^2*t)^(w_2^2-1) if logscale=FALSE  
and \cr
a_0(t) = exp(w_1)*exp(w_2)(t)^(exp(w_2)-1) if logscale=TRUE.

2. with the "piecewise" step function and nz nodes (y_1,...y_nz), nz-1 parameters 
are necesssary p_1,...p_nz-1 so that the baseline risk function a_0(t) = p_j^2 
for y_j < t =< y_j+1 if logscale=FALSE and a_0(t) = exp(p_j) for y_j < t =< y_j+1 if logscale=TRUE.

3. with the "splines" function and nz nodes (y_1,...y_nz), nz+2 parameters 
are necessary s_1,...s_nz+2 so that the baseline risk function a_0(t) = sum_j s_j^2 M_j(t) 
if logscale=FALSE and a_0(t) = sum_j exp(s_j) M_j(t) if logscale=TRUE where M_j is the basis of cubic M-splines.
Two parameterizations of the baseline risk function are proposed (logscale=TRUE or FALSE) 
because in some cases, especially when the instantaneous risks are very close to 0, 
some convergence problems may appear with one parameterization or the other. 
As a consequence, we recommend to try the alternative parameterization 
(changing logscale option) 
when a model does not converge (maximum number of iterations reached) although
  convergence criteria based on the parameters and likelihood are small.


b. ASSOCIATION BETWEEN LONGITUDINAL AND SURVIVAL DATA

The association between the longitudinal and the survival data is captured by including 
a function of the elements from the latent process mixed model as a predictor in the survival model.
 We implemented so far two association structures,
that should be specified through \code{sharedtype} argument.

1. the random effect from the latent process linear mixed model (\code{sharedtype='RE'}):
the q random effects modeling the individual deviation in the longitudinal model
 are also included
in the survival model, so that a q-vector of parameters measures the association
between the risk of event and the longitudinal outcome(s).

2. the predicted current level of the underlying process (\code{sharedtype='CL'}):
the predicted latent process defined by the mixed model at time t is included as
time-dependent covariate in the risk of event model for time t.
The association between the longitudinal process and the risk of event
is then quantified by a unique parameter.

 
C. THE VECTOR OF PARAMETERS B

The parameters in the vector of initial values \code{B} or in the vector of
maximum likelihood estimates \code{best} are included in the following
order:
(1) parameters for the baseline risk function: 2 parameters for each Weibull, 
nz-1 for each piecewise constant risk and nz+2 for each splines risk. In the 
presence of competing events, the number of parameters should be adapted to 
the number of causes of event;
(2) for all covariates in survival, one parameter 
is required. Covariates parameters should be included in the same order as in survival.
In the presence of cause-specific effects, the number of parameters should be
multiplied by the number of causes;
(3) parameter(s) of association between the longitudinal 
and the survival process: for \code{sharedtype='RE'}, one parameter per random effect
and per cause of event is 
required; for \code{sharedtype='CL'}, one parameter per cause of event is required;
(4) for all covariates  in fixed, one parameter is required. Parameters should
be included in the same  order as in fixed;
(5)for all covariates included with \code{contrast()} in \code{fixed}, one
supplementary parameter per outcome is required excepted for the last
outcome for which the parameter is not estimated but deduced from the others;
(6) the variance of each random-effect specified in random 
(excepted the intercept which is constrained to 1) 
if idiag=TRUE and the inferior triangular variance-covariance matrix of all 
the random-effects if idiag=FALSE;
(7) if \code{cor} is specified, the standard error of the Brownian motion or 
the standard error and the correlation parameter of the autoregressive process;
(8) parameters of each measurement model: 2 for "linear", 4 for "beta", 
n+2 for "splines" with n nodes, n for "thresholds" for a (n+1)-level outcome; 
(9) if \code{randomY=TRUE}, the standard 
error of the outcome-specific random intercept (one per outcome); 
(10) the outcome-specific standard errors (one per outcome)



C. CAUTIONS REGARDING THE USE OF THE PROGRAM

Some caution should be made when using the program. Convergence criteria are
very strict as they are based on the derivatives of the log-likelihood in
addition to the parameter and log-likelihood stability. In some cases, the
program may not converge and reach the maximum number of iterations fixed at
100 by default. In this case, the user should check that parameter estimates at the
last iteration are not on the boundaries of the parameter space.

If the parameters are on the boundaries of the parameter space, the
identifiability of the model is critical. This may happen especially with
splines parameters that may be too close to 0 (lower boundary). When
identifiability of some parameters is suspected, the program can be run
again from the former estimates by fixing the suspected parameters to their
value with option posfix. This usually solves the problem. An alternative is
to remove the parameters of the Beta of Splines link function from the
inverse of the Hessian with option partialH. If not, the program should be 
run again with other initial values, with a higher maximum number of iterations 
or less strict convergence tolerances.

To reduce the computation time, this program can be carried out in parallel mode,
ie. using multiple cores which number can be specified with argument \code{nproc}.
}
\examples{
#### Examples with paquid data from R-package lcmm
library(lcmm)
paq <- paquid[which(paquid$age_init<paquid$agedem),]
paq$age65 <- (paq$age-65)/10

#### Example with one Gaussian marker :
## We model the cognitive test IST according to age, sexe and eduction level. We assume
## a Weibull distribution for the time to dementia and link the longitudinal and survival
## data using the random effects.
## We provide here the call to the jointLPM function without optimization (maxiter=0). The
## results should therefore not be interpreted.
M0 <- jointLPM(fixed = IST~age65*(male+CEP),
                random=~age65,
                idiag=FALSE,
                subject="ID",
                link="linear",
                survival=Surv(age_init,agedem,dem)~male,
                sharedtype='RE',
                hazard="Weibull",
                data=paq,
                var.time="age65",
                maxiter=0)
M0$best ## these are the initial values of each of the 15 parameters

\donttest{ 
## Estimation with one Gaussian marker
## We remove the maxiter=0 option to estimate the model. We specify initial values
## to reduce the runtime, but this can take several minutes.
binit1 <- c(0.1039, 5.306, -0.1887, -1.0355, -4.3817, -1.0543, -0.1161, 0.8588,
0.0538, -0.1722, -0.2224, 0.3296, 30.7768, 4.6169, 0.7396)
M1 <- jointLPM(fixed = IST~age65*(male+CEP),
                random=~age65,
                idiag=FALSE,
                subject="ID",
                link="linear",
                survival=Surv(age_init,agedem,dem)~male,
                sharedtype='RE',
                hazard="Weibull",
                data=paq,
                var.time="age65",
                B=binit1)
## Optimized the parameters to be interpreted :
summary(M1)


#### Estimation with one ordinal marker :
## We consider here the 4-level hierarchical scale of dependence HIER and use "thresholds"
## to model it as an ordinal outcome. We assume an association between the current level
## of dependency and the risk of dementia through the option sharedtype="CL".
## We use a parallel optimization on 2 cores to reduce computation time.
binit2 <- c(0.0821, 2.4492, 0.1223, 1.7864, 0.0799, -0.2864, 0.0055, -0.0327, 0.0017,
0.3313, 0.9763, 0.9918, -0.4402)
M2 <- jointLPM(fixed = HIER~I(age-65)*male,
                random = ~I(age-65),
                subject = "ID", 
                link = "thresholds",
                survival = Surv(age_init,agedem,dem)~male,
                sharedtype = 'CL',
                var.time = "age",
                data = paq, 
                methInteg = "QMC", 
                nMC = 1000,
                B=binit2,
                nproc=2)
summary(M2)

}

}
\references{
Saulnier, Philipps, Meissner, Rascol, Pavy-Le-Traon, Foubert-Samier, Proust-Lima (2022).
Joint models for the longitudinal analysis of measurement scales in the presence 
of informative dropout, Methods; 203:142-51. 

Philipps, Hejblum, Prague, Commenges, Proust-Lima (2021).
Robust and efficient optimization using a Marquardt-Levenberg algorithm with 
R package marqLevAlg, The R Journal 13:2.
}
\author{
Viviane Philipps, Tiphaine Saulnier and Cecile Proust-Lima
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/IRT4Sstaging.R
\name{IRT4Sstaging}
\alias{IRT4Sstaging}
\title{Projection function, used in Step 3 _ Staging from the 4Smethod}
\usage{
IRT4Sstaging(D, Dss, nsim = 1000, bounds = c(-10, 10))
}
\arguments{
\item{D}{an object of class \code{jointLPM} representing the estimated 
dimension model with the items}

\item{Dss}{an object of class \code{jointLPM} representing the estimated proxy 
model between the dimension scores and the stages}

\item{nsim}{number of points used in the numerical integration (Monte-Carlo). 
nsim should be relatively important (nsim=1000 by default).}

\item{bounds}{dimension scale boundaries between which are computed a grid of
the dum of the items}
}
\value{
a vector of the estimated stage transition thresholds

###@examples
}
\description{
This function determines the stage thresholds in the latent scale of the
dimension process. Specifically, this function predicts the dimension scores
corresponding to stage changes and deducts the corresponding thresholds in the
dimension scale based on a correspondence between the predicted scores and
the expected sum of the items. The output is a vector of the estimated stage transition
thresholds.
}
\references{
preprint :  arXiv:2407.08278
}
\author{
Tiphaine Saulnier, Viviane Philipps and Cecile Proust-Lima
}
